@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_anim_c =
- *   "$Id: anim.c,v 1.16 2003/09/13 05:01:26 mwedel Exp $";
+ *   "$Id: anim.c,v 1.22 2006/03/18 15:05:31 ryo_saeba Exp $";
  */
 
 /*
@@ -31,7 +31,7 @@
 #include <global.h>
 #include <stdio.h>
 
-void free_all_anim() {
+void free_all_anim(void) {
     int i;
 
     for (i=0; i<=num_animations; i++) {
@@ -41,7 +41,7 @@ void free_all_anim() {
     free(animations);
 }
 
-void init_anim() {
+void init_anim(void) {
     char buf[MAX_BUF];
     FILE *fp;
     static int anim_init=0;
@@ -68,8 +68,7 @@ void init_anim() {
     sprintf(buf,"%s/animations", settings.datadir);
     LOG(llevDebug,"Reading animations from %s...", buf);
     if ((fp=fopen(buf,"r")) ==NULL) {
-	perror("Can not open animations file\n");
-	fprintf(stderr,"Filename=%s\n", buf);
+	LOG(llevError, "Cannot open animations file %s: %s\n", buf, strerror_local(errno));
 	exit(-1);
     }
     while (fgets(buf, MAX_BUF-1, fp)!=NULL) {
@@ -109,30 +108,31 @@ void init_anim() {
 	    }
 
 	} else {
-	    if (!(faces[num_frames++] = FindFace(buf,0)))
-		LOG(llevDebug,"Could not find face %s for animation %s\n",
-		    buf, animations[num_animations].name);
+        if (!(faces[num_frames++] = find_face(buf,0)))
+		    LOG(llevDebug,"Could not find face %s for animation %s\n",
+		        buf, animations[num_animations].name);
 	}
     }
     fclose(fp);
     LOG(llevDebug,"done. got (%d)\n", num_animations);
 }
 
-static int anim_compare(Animations *a, Animations *b) {
+static int anim_compare(const Animations *a, const Animations *b) {
     return strcmp(a->name, b->name);
 }
 
-/* Tries to find the animation id that matches name.  Returns an integer match 
+/**
+ * Tries to find the animation id that matches name.  Returns an integer match 
  * 0 if no match found (animation 0 is initialized as the 'bug' face
  */
-int find_animation(char *name)
+int find_animation(const char *name)
 {
     Animations search, *match;
 
     search.name = name;
 
     match = (Animations*)bsearch(&search, animations, (num_animations+1), 
-		sizeof(Animations), (int (*)())anim_compare);
+		sizeof(Animations), (int (*)(const void*, const void*))anim_compare);
 
 
     if (match) return match->num;
@@ -140,7 +140,7 @@ int find_animation(char *name)
     return 0;
 }
 
-/*
+/**
  * animate_object(object) updates the face-variable of an object.
  * If the object is the head of a multi-object, all objects are animated.
  * op is the object to animate.
