@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_button_c =
- *   "$Id: button.c,v 1.26 2005/07/15 13:51:48 ryo_saeba Exp $";
+ *   "$Id: button.c,v 1.31 2006/02/09 00:48:36 akirschbaum Exp $";
  */
 
 /*
@@ -171,8 +171,22 @@ void update_button(object *op) {
 	tmp = ol->ob;
 	if (tmp->type==BUTTON) {
 	    for(ab=tmp->above,tot=0;ab!=NULL;ab=ab->above)
-		if(!QUERY_FLAG(ab,FLAG_FLYING)) /* Why was nrof removed? */
+		/* Bug? The pedestal code below looks for the head of
+		 * the object, this bit doesn't.  I'd think we should check
+		 * for head here also.  Maybe it also makese sense to 
+		 * make the for ab=tmp->above loop common, and alter
+		 * behaviour based on object within that loop?
+		 */
+
+		/* Basically, if the move_type matches that on what the
+		 * button wants, we count it.  The second check is so that
+		 * objects don't move (swords, etc) will count.  Note that
+		 * this means that more work is needed to make buttons
+		 * that are only triggered by flying objects.
+		 */
+		if ((ab->move_type & tmp->move_on) || ab->move_type==0 ) 
 		    tot+=ab->weight*(ab->nrof?ab->nrof:1)+ab->carrying;
+
 	    tmp->value=(tot>=tmp->weight)?1:0;
 	    if(tmp->value)
 		any_down=1;
@@ -180,8 +194,8 @@ void update_button(object *op) {
 	    tmp->value = 0;
 	    for(ab=tmp->above; ab!=NULL; ab=ab->above) {
 		head = ab->head ? ab->head : ab;
-		if ( (!QUERY_FLAG(head,FLAG_FLYING) || 
-		      QUERY_FLAG(tmp,FLAG_FLY_ON)) &&
+		/* Same note regarding move_type for buttons above apply here. */
+		if  ( ((head->move_type & tmp->move_on) || ab->move_type==0) &&
 		     (head->race==tmp->slaying ||
 		      ((head->type==SPECIAL_KEY) && (head->slaying==tmp->slaying)) || 
 		      (!strcmp (tmp->slaying, "player") && 
@@ -252,7 +266,7 @@ void animate_turning(object *op) /* only one part objects */
 }
 
 #define ARCH_SACRIFICE(xyz) ((xyz)->slaying)
-#define NROF_SACRIFICE(xyz) ((xyz)->stats.food)
+#define NROF_SACRIFICE(xyz) ((uint32)(xyz)->stats.food)
 
 /* Returns true if the sacrifice meets the needs of the altar.
  *
@@ -265,7 +279,7 @@ void animate_turning(object *op) /* only one part objects */
  * objects on the same space that take the same sacrifice.
  */
  
-int check_altar_sacrifice (object *altar, object *sacrifice)
+int check_altar_sacrifice (const object *altar, const object *sacrifice)
 {
   if ( ! QUERY_FLAG (sacrifice, FLAG_ALIVE)
       && ! QUERY_FLAG (sacrifice, FLAG_IS_LINKED)
@@ -273,7 +287,8 @@ int check_altar_sacrifice (object *altar, object *sacrifice)
   {
       if ((ARCH_SACRIFICE(altar) == sacrifice->arch->name ||
           ARCH_SACRIFICE(altar) == sacrifice->name ||
-	  ARCH_SACRIFICE(altar) == sacrifice->slaying)
+	  ARCH_SACRIFICE(altar) == sacrifice->slaying ||
+	  (!strcmp(ARCH_SACRIFICE(altar),query_base_name(sacrifice,0))))
 	  && NROF_SACRIFICE(altar) <= (sacrifice->nrof?sacrifice->nrof:1))
 		return 1;
       if (strcmp (ARCH_SACRIFICE(altar), "money") == 0
@@ -360,114 +375,126 @@ void trigger_move (object *op, int state) /* 1 down and 0 up */
  */
 int check_trigger (object *op, object *cause)
 {
-  object *tmp;
-  int push = 0, tot = 0;
-  int in_movement = op->stats.wc || op->speed;
+    object *tmp;
+    int push = 0, tot = 0;
+    int in_movement = op->stats.wc || op->speed;
 
-  switch (op->type) {
-    case TRIGGER_BUTTON:
-    if (op->weight > 0) {
-        if (cause) {
-          for (tmp = op->above; tmp; tmp = tmp->above)
-            if ( ! QUERY_FLAG (tmp, FLAG_FLYING))
-	      tot += tmp->weight * (tmp->nrof ? tmp->nrof : 1)
-                     + tmp->carrying;
-          if (tot >= op->weight)
-            push = 1;
-          if (op->stats.ac == push)
-            return 0;
-          op->stats.ac = push;
-          SET_ANIMATION (op, push);
-          update_object (op, UP_OBJ_FACE);
-          if (in_movement || ! push)
-            return 0;
-        }
-        trigger_move (op, push);
-    }
-    return 0;
+    switch (op->type) {
+	case TRIGGER_BUTTON:
+	    if (op->weight > 0) {
+		if (cause) {
+		    for (tmp = op->above; tmp; tmp = tmp->above)
+			/* Comment reproduced from update_buttons(): */
+			/* Basically, if the move_type matches that on what the
+			 * button wants, we count it.  The second check is so that
+			 * objects that don't move (swords, etc) will count.  Note that
+			 * this means that more work is needed to make buttons
+			 * that are only triggered by flying objects.
+			 */
 
-    case TRIGGER_PEDESTAL:
-        if (cause) {
-          for (tmp = op->above; tmp; tmp = tmp->above) {
-            object *head = tmp->head ? tmp->head : tmp;
-            if ((!QUERY_FLAG(head,FLAG_FLYING) || QUERY_FLAG(op,FLAG_FLY_ON))
-	        && (head->race==op->slaying ||
-                (!strcmp (op->slaying, "player") && head->type == PLAYER)))
-              {
-                push = 1;
-                break;
-              }
-	  }
-          if (op->stats.ac == push)
-            return 0;
-          op->stats.ac = push;
-          SET_ANIMATION (op, push);
-          update_object(op,UP_OBJ_FACE);
-          if (in_movement || ! push)
-            return 0;
-        }
-        trigger_move (op, push);
-        return 0;
+			if ((tmp->move_type & op->move_on) || tmp->move_type==0) {
+			    tot += tmp->weight * (tmp->nrof ? tmp->nrof : 1)
+				+ tmp->carrying;
+			}
+		    if (tot >= op->weight)
+		    push = 1;
+		    if (op->stats.ac == push)
+			return 0;
+		    op->stats.ac = push;
+		    SET_ANIMATION (op, push);
+		    update_object (op, UP_OBJ_FACE);
+		    if (in_movement || ! push)
+			return 0;
+		}
+		trigger_move (op, push);
+	    }
+	    return 0;
 
-    case TRIGGER_ALTAR:
-        if (cause) {
-          if (in_movement)
-            return 0;
-          if (operate_altar (op, &cause)) {
-            SET_ANIMATION (op, 1);
-            update_object(op,UP_OBJ_FACE);
+	case TRIGGER_PEDESTAL:
+	    if (cause) {
+		for (tmp = op->above; tmp; tmp = tmp->above) {
+		    object *head = tmp->head ? tmp->head : tmp;
+
+		    /* See comment in TRIGGER_BUTTON about move_types */
+		    if (((head->move_type & op->move_on) || head->move_type==0)
+		      && (head->race==op->slaying ||
+		      (!strcmp (op->slaying, "player") && head->type == PLAYER))) {
+			push = 1;
+			break;
+		    }
+		}
+		if (op->stats.ac == push)
+		    return 0;
+		op->stats.ac = push;
+		SET_ANIMATION (op, push);
+		update_object(op,UP_OBJ_FACE);
+		if (in_movement || ! push)
+		    return 0;
+	    }
+	    trigger_move (op, push);
+	    return 0;
+
+	case TRIGGER_ALTAR:
+	    if (cause) {
+		if (in_movement)
+		    return 0;
+		if (operate_altar (op, &cause)) {
+		    SET_ANIMATION (op, 1);
+		    update_object(op,UP_OBJ_FACE);
 	    
-	    if (op->last_sp >= 0) {
-	      trigger_move (op, 1);
-	      if (op->last_sp > 0)
-		op->last_sp = -op->last_sp;
-	    }
-	    else {
-	      /* for trigger altar with last_sp, the ON/OFF
-		 status (-> +/- value) is "simulated": */
-	      op->value = !op->value;
-	      trigger_move (op, 1);
-	      op->last_sp = -op->last_sp;
-	      op->value = !op->value;
-	    }
-            return cause == NULL;
-          } else {
-            return 0;
-          }
-        } else {
-          SET_ANIMATION (op, 0);
-          update_object(op,UP_OBJ_FACE);
+		    if (op->last_sp >= 0) {
+			trigger_move (op, 1);
+			if (op->last_sp > 0)
+			op->last_sp = -op->last_sp;
+		    }
+		    else {
+			/* for trigger altar with last_sp, the ON/OFF
+			 * status (-> +/- value) is "simulated": 
+			 */
+			op->value = !op->value;
+			trigger_move (op, 1);
+			op->last_sp = -op->last_sp;
+			op->value = !op->value;
+		    }
+		    return cause == NULL;
+		} else {
+		    return 0;
+		}
+	    } else {
+		SET_ANIMATION (op, 0);
+		update_object(op,UP_OBJ_FACE);
 	  
-          /* If trigger_altar has "last_sp > 0" set on the map,
-             it will push the connected value only once per sacrifice.
-             Otherwise (default), the connected value will be
-             pushed twice: First by sacrifice, second by reset! -AV */
-          if (!op->last_sp)
-            trigger_move (op, 0);
-          else {
-            op->stats.wc = 0;
-            op->value = !op->value;
-            op->speed = 0;
-            update_ob_speed(op);
-          }
-        }
-        return 0;
+		/* If trigger_altar has "last_sp > 0" set on the map,
+		 * it will push the connected value only once per sacrifice.
+		 * Otherwise (default), the connected value will be
+		 * pushed twice: First by sacrifice, second by reset! -AV 
+		 */
+		if (!op->last_sp)
+		    trigger_move (op, 0);
+		else {
+		    op->stats.wc = 0;
+		    op->value = !op->value;
+		    op->speed = 0;
+		    update_ob_speed(op);
+		}
+	    }
+	    return 0;
 
-    case TRIGGER:
-        if (cause) {
-          if (in_movement)
-            return 0;
-          push = 1;
-        }
-        SET_ANIMATION (op, push);
-        update_object(op,UP_OBJ_FACE);
-        trigger_move (op, push);
-	return 1;
+	case TRIGGER:
+	    if (cause) {
+		if (in_movement)
+		    return 0;
+		push = 1;
+	    }
+	    SET_ANIMATION (op, push);
+	    update_object(op,UP_OBJ_FACE);
+	    trigger_move (op, push);
+	    return 1;
 
-    default:
-	LOG(llevDebug, "Unknown trigger type: %s (%d)\n", op->name, op->type);
-        return 0;
-  }
+	default:
+	    LOG(llevDebug, "Unknown trigger type: %s (%d)\n", op->name, op->type);
+	    return 0;
+    }
 }
 
 void add_button_link(object *button, mapstruct *map, int connected) {
@@ -537,7 +564,7 @@ void remove_button_link(object *op) {
  * Return the first objectlink in the objects linked to this one
  */
 
-objectlink *get_button_links(object *button) {
+objectlink *get_button_links(const object *button) {
   oblinkpt *obp;
   objectlink *ol;
 
@@ -554,7 +581,7 @@ objectlink *get_button_links(object *button) {
  * Made as a separate function to increase efficiency
  */
 
-int get_button_value(object *button) {
+int get_button_value(const object *button) {
   oblinkpt *obp;
   objectlink *ol;
 
@@ -595,7 +622,7 @@ void do_mood_floor(object *op, object *op2) {
 		if(QUERY_FLAG(tmp, FLAG_FRIENDLY)) { 
 			CLEAR_FLAG(tmp, FLAG_FRIENDLY);
 			remove_friendly_object(tmp);
-			tmp->move_type = 0;
+			tmp->attack_movement = 0;
 			/* lots of checks here, but want to make sure we don't
 			 * dereference a null value
 			 */
@@ -635,7 +662,7 @@ void do_mood_floor(object *op, object *op2) {
 		tmp->stats.exp = 0;
 		SET_FLAG(tmp, FLAG_FRIENDLY);
                 add_friendly_object (tmp);
-		tmp->move_type = PETMOVE;
+		tmp->attack_movement = PETMOVE;
 		break;		
 
 	  default:
@@ -651,7 +678,7 @@ void do_mood_floor(object *op, object *op2) {
  *		hp = match object type (excpt type '0'== PLAYER)
  */
 
-object * check_inv_recursive(object *op, object *trig)
+object * check_inv_recursive(object *op, const object *trig)
 {
     object *tmp,*ret=NULL;
 
@@ -702,7 +729,7 @@ object *match;
  * map.  All it really does it much sure the object id link that is set
  * matches what the object has.
  */
-void verify_button_links(mapstruct *map) {
+void verify_button_links(const mapstruct *map) {
     oblinkpt *obp;
     objectlink *ol;
 
