@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_arch_c =
- *   "$Id: exp.c,v 1.8 2004/03/26 21:59:25 ryo_saeba Exp $";
+ *   "$Id: exp.c,v 1.11 2006/06/05 06:28:55 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -90,7 +90,7 @@ float exp_prot_mult[NROFATTACKS+2] = {
  * and not anyplace in the code.
  */
 	
-int new_exp(object *ob) {
+int new_exp(const object *ob) {
   double	att_mult, prot_mult, spec_mult;
   double	exp;
   int		i;
@@ -134,7 +134,7 @@ int new_exp(object *ob) {
  * Returns true if the monster specified has any innate abilities.
  */
 
-int has_ability(object *ob) {
+int has_ability(const object *ob) {
     object *tmp;
 
     for(tmp=ob->inv;tmp!=NULL;tmp=tmp->below)
@@ -149,7 +149,7 @@ int has_ability(object *ob) {
  * are errors, the table is likely in an inconsistent
  * state.
  */
-void init_experience()
+void init_experience(void)
 {
     char buf[MAX_BUF], *cp;
     int lastlevel=0, comp;
@@ -231,12 +231,12 @@ void init_experience()
  * format of the exp_table is correct.
  */
 
-void dump_experience()
+void dump_experience(void)
 {
     int i;
 
     for (i=1; i<= settings.max_level; i++) {
-	fprintf(logfile,"%4d %20lld\n", i, levels[i]);
+	fprintf(logfile,"%4d %20"  FMT64  "\n", i, levels[i]);
     }
     exit(0);
 }
