@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_friend_c =
- *   "$Id: friend.c,v 1.6 2002/07/15 04:25:39 mwedel Exp $";
+ *   "$Id: friend.c,v 1.9 2006/02/09 00:48:36 akirschbaum Exp $";
  */
 
 /*
@@ -101,7 +101,7 @@ void remove_friendly_object(object *op) {
  * Dumps all friendly objects.  Invoked in DM-mode with the G key.
  */
 
-void dump_friendly_objects() {
+void dump_friendly_objects(void) {
     objectlink *ol;
 
     for(ol=first_friendly_object;ol!=NULL;ol=ol->next)
@@ -112,7 +112,7 @@ void dump_friendly_objects() {
  * It traverses the friendly list removing objects that should not be here
  * (ie, do not have friendly flag set, freed, etc)
  */
-void clean_friendly_list() {
+void clean_friendly_list(void) {
     objectlink *this, *prev=NULL, *next;
     int count=0;
 
@@ -137,3 +137,17 @@ void clean_friendly_list() {
 	LOG(llevDebug,"clean_friendly_list: Removed %d bogus links\n", count);
 }
 
+/* Checks if the given object is already in the friendly list or not
+ * Lauwenmark - 31/07/05
+ */
+int is_friendly(const object* op)
+{
+    objectlink *ol;
+
+    for(ol=first_friendly_object;ol!=NULL;ol=ol->next)
+        if (ol->ob == op)
+            return 1;
+
+    return 0;
+}
+
