@@ -1,4 +1,31 @@
-  
+/*
+ * static char *rcsid_holy_c =
+ *   "$Id: holy.c,v 1.7 2006/06/05 06:28:55 mwedel Exp $";
+ */
+
+/*
+    CrossFire, A Multiplayer game for X-windows
+
+    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
+    Copyright (C) 1992 Frank Tore Johansen
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+
+    The authors can be reached via e-mail at crossfire-devel@real-time.com
+*/
+ 
 /* Started file Sept 1996 - initialization of gods in form of a
  * linked list -b.t.
  */ 
@@ -7,7 +34,9 @@
 #include <living.h>
 #include <spells.h>
 
-static godlink *init_godslist() {
+static void add_god_to_list (archetype *god_arch);
+
+static godlink *init_godslist(void) {
     godlink *gl = (godlink *) malloc(sizeof(godlink));
     if(gl==NULL)
       fatal(OUT_OF_MEMORY);
@@ -23,7 +52,7 @@ static godlink *init_godslist() {
 /* init_gods() - this takes a look at all of the archetypes to find
  * the objects which correspond to the GODS (type GOD) */
    
-void init_gods () {
+void init_gods (void) {
   archetype *at=NULL;
    
   LOG(llevDebug, "Initializing gods...");
@@ -33,8 +62,7 @@ void init_gods () {
   LOG(llevDebug,"done.\n");
 }
   
-/* add_god_to_list()- called only from init_gods */
-void add_god_to_list (archetype *god_arch) {
+static void add_god_to_list (archetype *god_arch) {
   godlink *god;
   
   if(!god_arch) {
@@ -59,11 +87,18 @@ void add_god_to_list (archetype *god_arch) {
 #endif
 }
 
+#if 0
+/* This is no longer used - disabling it from compilation to
+ * see if it breaks anything.  If it doesn't, it should get
+ * removed in the near future.
+ * MSW 2006-06-02
+ */
+
 /* baptize_altar() - (cosmetically) change the name to that of the
  * god in question, then set the title for later use. -b.t.
  */
  
-int baptize_altar(object *op) {
+static int baptize_altar(object *op) {
    char buf[MAX_BUF];
    
    /* if the title field is pre-set, then that altar is
@@ -85,6 +120,7 @@ int baptize_altar(object *op) {
    }
    return 0;
 }
+#endif
  
 godlink * get_rand_god ( void ) {
   godlink *god=first_god;
@@ -110,7 +146,7 @@ object *pntr_to_god_obj(godlink *godlnk) {
   return god;
 }
 
-void free_all_god() {
+void free_all_god(void) {
     godlink *god, *godnext;
 
     LOG(llevDebug,"Freeing god information\n");
@@ -122,7 +158,7 @@ void free_all_god() {
 }
 
 
-void dump_gods() {
+void dump_gods(void) {
     godlink *glist;
   
     fprintf(stderr,"\n");
