@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_image_c =
- *   "$Id: image.c,v 1.11 2005/03/19 09:14:56 mwedel Exp $";
+ *   "$Id: image.c,v 1.21 2006/03/18 15:05:31 ryo_saeba Exp $";
  */
 
 /*
@@ -32,7 +32,7 @@
 
 New_Face *new_faces;
 
-/* bmappair and xbm are used when looking for the image id numbers
+/** bmappair and xbm are used when looking for the image id numbers
  * of a face by name.  xbm is sorted alphabetically so that bsearch
  * can be used to quickly find the entry for a name.  the number is
  * then an index into the new_faces array.
@@ -43,53 +43,52 @@ New_Face *new_faces;
  * xbm, it may not.  At current time, these do in fact match because
  * the bmaps file is created in a sorted order.
  */
-
 struct bmappair {
     char *name;
     unsigned int number;
 };
-
+/** The xbm array (which contains name and number information, and
+ * is then sorted) contains nroffiles entries. 
+ */
 static struct bmappair *xbm=NULL;
 
-/* only used in loader.c, to go from the numeric image id (which is
- * used throughout the program) backto the standard name.
- */
-
-MapLook blank_look;
-
-/* Following can just as easily be pointers, but 
+/** Following can just as easily be pointers, but 
  * it is easier to keep them like this.
  */
-New_Face *blank_face, *dark_faces[3], *potion_face, *empty_face,
-    *smooth_face;
+New_Face *blank_face, *dark_faces[3], *empty_face, *smooth_face;
 
 
-/* nroffiles is the actual number of bitmaps defined.
- * nrofpixmaps is the higest numbers bitmap that is loaded.  With
+/** nroffiles is the actual number of bitmaps defined. */
+static int nroffiles = 0;
+
+/** nrofpixmaps is the number of bitmaps loaded.  With
  * the automatic generation of the bmaps file, this is now equal
  * to nroffiles.
- *
- * The xbm array (which contains name and number information, and
- * is then sorted) contains nroffiles+1 entries.  the xbm_names
- * array (which is used for converting the numeric face to
- * a name) contains nrofpixmaps+1 entries.
  */
-int nroffiles = 0, nrofpixmaps=0;
+int nrofpixmaps = 0;
 
+/**
+ * id is the face to smooth, smooth is the 16x2 face used to smooth id.
+ */
 struct smoothing {
     uint16 id;
     uint16 smooth;
 };
 
+/**
+ * Contains all defined smoothing entries. smooth is an array of nrofsmooth
+ * entries. It is sorted by smooth[].id.
+ */
 static struct smoothing *smooth=NULL;
 int nrofsmooth=0;
-/* the only thing this table is used for now is to
+
+/** the only thing this table is used for now is to
  * translate the colorname in the magicmap field of the
  * face into a numeric index that is then sent to the
  * client for magic map commands.  The order of this table
  * must match that of the NDI colors in include/newclient.h.
  */
-char *colorname[NUM_COLORS] = {
+static const char *const colorname[] = {
 "black",		/* 0  */
 "white",		/* 1  */
 "blue",			/* 2  */
@@ -105,10 +104,10 @@ char *colorname[NUM_COLORS] = {
 "khaki"			/* 12 */ 
 };
 
-static int compar (struct bmappair *a, struct bmappair *b) {
+static int compar (const struct bmappair *a, const struct bmappair *b) {
     return strcmp (a->name, b->name);
 }
-static int compar_smooth (struct smoothing *a, struct smoothing *b) {
+static int compar_smooth (const struct smoothing *a, const struct smoothing *b) {
     if (a->id<b->id)
         return -1;
     if (b->id<a->id)
@@ -117,30 +116,26 @@ static int compar_smooth (struct smoothing *a, struct smoothing *b) {
 }
 
 
-/*
+/**
  * Returns the matching color in the coloralias if found,
  * 0 otherwise.  Note that 0 will actually be black, so there is no
  * way the calling function can tell if an error occurred or not
- *
- * CF 0.91.7: relocated from loader.c file - this perhaps can be
- * declared static.
  */
 
-char find_color(char *name) {
-  int i;
-  for(i=0;i<NUM_COLORS;i++)
+static uint8 find_color(const char *name) {
+  uint8 i;
+  for(i=0;i<sizeof(colorname)/sizeof(*colorname);i++)
     if(!strcmp(name,colorname[i]))
       return i;
   LOG(llevError,"Unknown color: %s\n",name);
   return 0;
 }
 
-/* This reads the lib/faces file, getting color and visibility information.
- * it is called by ReadBmapNames.
+/** This reads the lib/faces file, getting color and visibility information.
+ * it is called by read_bmap_names.
  */
 
-
-static void ReadFaceData()
+static void read_face_data(void)
 {
     char buf[MAX_BUF], *cp;
     New_Face *on_face=NULL;
@@ -149,8 +144,7 @@ static void ReadFaceData()
     sprintf(buf,"%s/faces", settings.datadir);
     LOG(llevDebug,"Reading faces from %s...",buf);
     if ((fp=fopen(buf,"r"))==NULL) {
-	perror("Can't open faces file");
-	printf("buf = %s\n", buf);
+	LOG(llevError, "Cannot open faces file %s: %s\n", buf, strerror_local(errno));
 	exit(-1);
     }
     while (fgets(buf, MAX_BUF, fp)!=NULL) {
@@ -164,7 +158,7 @@ static void ReadFaceData()
 	     cp = buf + 5;
 	     cp[strlen(cp)-1] = '\0';	/* remove newline */
 
-	     if ((tmp=FindFace(cp,-1))==-1) {
+	     if ((tmp=find_face(cp,-1))==-1) {
 		LOG(llevError,"Could not find face %s\n", cp);
 		continue;
 	     }
@@ -200,23 +194,23 @@ static void ReadFaceData()
     fclose(fp);
 }
 
-/* This reads the bmaps file to get all the bitmap names and
+/** This reads the bmaps file to get all the bitmap names and
  * stuff.  It only needs to be done once, because it is player
  * independent (ie, what display the person is on will not make a
  * difference.)
  */
 
-int ReadBmapNames () {
+void read_bmap_names(void) {
     char buf[MAX_BUF], *p, *q;
     FILE *fp;
     int value, nrofbmaps = 0, i;
+    size_t l;
 
     bmaps_checksum=0;
     sprintf (buf,"%s/bmaps", settings.datadir);
     LOG(llevDebug,"Reading bmaps from %s...",buf);
     if ((fp=fopen(buf,"r"))==NULL) {
-	perror("Can't open bmaps file");
-	printf("buf = %s\n", buf);
+	LOG(llevError, "Cannot open bmaps file %s: %s\n", buf, strerror_local(errno));
 	exit(-1);
     }
     
@@ -226,10 +220,10 @@ int ReadBmapNames () {
 	    nrofbmaps++;
     rewind(fp);
     
-    xbm = (struct bmappair *) malloc(sizeof(struct bmappair) * (nrofbmaps + 1));
-    memset (xbm, 0, sizeof (struct bmappair) * (nrofbmaps + 1));
+    xbm = (struct bmappair *) malloc(sizeof(struct bmappair) * nrofbmaps);
+    memset (xbm, 0, sizeof (struct bmappair) * nrofbmaps);
     
-    while(fgets (buf, MAX_BUF, fp)!=NULL) {
+    while(nroffiles < nrofbmaps && fgets (buf, MAX_BUF, fp) != NULL) {
 	if (*buf == '#')
 	    continue;
 	
@@ -255,24 +249,23 @@ int ReadBmapNames () {
 	ROTATE_RIGHT(bmaps_checksum);
 	bmaps_checksum += (value >> 8) & 0xff;
 	bmaps_checksum &= 0xffffffff;
-	for (i=0; i<strlen(q); i++) {
+	for (l=0; l<strlen(q); l++) {
 	    ROTATE_RIGHT(bmaps_checksum);
-	    bmaps_checksum += q[i];
+	    bmaps_checksum += q[l];
 	    bmaps_checksum &= 0xffffffff;
 	}
-	
 
 	xbm[nroffiles].number = value;
 	nroffiles++;
-	if(value > nrofpixmaps)
-	    nrofpixmaps = value;
+	if(value >= nrofpixmaps)
+	    nrofpixmaps = value+1;
     }
     fclose(fp);
 
     LOG(llevDebug,"done (got %d/%d/%d)\n",nrofpixmaps,nrofbmaps,nroffiles);
 
-    new_faces = (New_Face *)malloc(sizeof(New_Face) * (nrofpixmaps+1));
-    for (i = 0; i <= nrofpixmaps; i++) {
+    new_faces = (New_Face *)malloc(sizeof(New_Face) * nrofpixmaps);
+    for (i = 0; i < nrofpixmaps; i++) {
 	new_faces[i].name = "";
 	new_faces[i].number = i;
 	new_faces[i].visibility=0;
@@ -282,11 +275,9 @@ int ReadBmapNames () {
 	new_faces[xbm[i].number].name = xbm[i].name;
     }
 
-    nrofpixmaps++;
+    qsort (xbm, nroffiles, sizeof(struct bmappair), (int (*)(const void*, const void*))compar);
 
-    qsort (xbm, nrofbmaps, sizeof(struct bmappair), (int (*)())compar);
-
-    ReadFaceData();
+    read_face_data();
 
     for (i = 0; i < nrofpixmaps; i++) {
 	if (new_faces[i].magicmap==255) {
@@ -300,25 +291,19 @@ int ReadBmapNames () {
     /* Actually forcefully setting the colors here probably should not
      * be done - it could easily create confusion.
      */
-    blank_face = &new_faces[FindFace(BLANK_FACE_NAME, 0)];
-    blank_look.face = blank_face;
-    blank_look.flags = 0;
+    blank_face = &new_faces[find_face(BLANK_FACE_NAME, 0)];
     blank_face->magicmap = find_color ("khaki") | FACE_FLOOR;
 
-    empty_face = &new_faces[FindFace(EMPTY_FACE_NAME, 0)];
+    empty_face = &new_faces[find_face(EMPTY_FACE_NAME, 0)];
 
-    dark_faces[0] = &new_faces[FindFace (DARK_FACE1_NAME,0)];
-    dark_faces[1] = &new_faces[FindFace (DARK_FACE2_NAME,0)];
-    dark_faces[2] = &new_faces[FindFace (DARK_FACE3_NAME,0)];
+    dark_faces[0] = &new_faces[find_face(DARK_FACE1_NAME,0)];
+    dark_faces[1] = &new_faces[find_face(DARK_FACE2_NAME,0)];
+    dark_faces[2] = &new_faces[find_face(DARK_FACE3_NAME,0)];
 
-    potion_face  = &new_faces[FindFace(POTION_FACE_NAME,0)];
-
-    smooth_face = &new_faces[FindFace(SMOOTH_FACE_NAME,0)];
-
-    return nrofpixmaps;
+    smooth_face = &new_faces[find_face(SMOOTH_FACE_NAME,0)];
 }
 
-/* This returns an the face number of face 'name'.  Number is constant
+/** This returns an the face number of face 'name'.  Number is constant
  * during an invocation, but not necessarily between versions (this
  * is because the faces are arranged in alphabetical order, so
  * if a face is removed or added, all faces after that will now
@@ -331,63 +316,28 @@ int ReadBmapNames () {
  * (needed in client, so that it will know to request that image
  * from the server)
  */
-int FindFace (char *name, int error) {
-    int i;
+int find_face(const char *name, int error) {
     struct bmappair *bp, tmp;
     char *p;
 
-
-    /* Using actual numbers for faces is a very bad idea.  This is because
-     * each time the archetype file is rebuilt, all the face numbers
-     * change.
-     */
-    if ((i = atoi(name))) {
-	LOG(llevError,"Warning: Integer face name used: %s\n", name);
-	return i;
-    }
-
     if ((p = strchr (name, '\n')))
 	*p = '\0';
 
-    tmp.name = name;
+    tmp.name = (char *)name;
     bp = (struct bmappair *)bsearch 
-	(&tmp, xbm, nroffiles, sizeof(struct bmappair), (int (*)())compar);
+	(&tmp, xbm, nroffiles, sizeof(struct bmappair), (int (*)(const void*, const void*))compar);
 
     return bp ? bp->number : error;
 }
-/* According to docs, faces are saved in an array
- * The number associated with each face is it's
- * position in the array. Right, this should
- * be easy to convert a facenumber to a facename.
- * Hope this is so. Tchize
- * Damn, this is not, have to work around a little. Tchize
- */
-char* FindFaceName (int facenbr, char* error) {
-    int i=0;
-    if ( (facenbr>=nroffiles) ||(facenbr<0)) 
-        return error;
-    /*xbm[facenbr] is 'near' the face we are seeking*/
-    if (xbm[facenbr].number==facenbr)
-        return xbm[facenbr].name;
-    for (i=1;;i++){
-        if ( ((facenbr-i)<0) && ((facenbr+i)>nroffiles))
-            return error;
-        if (xbm[facenbr-i].number==facenbr){
-            return xbm[facenbr-i].name;
-        }
-        if (xbm[facenbr+i].number==facenbr){
-            return xbm[facenbr+i].name;
-        }
-    }
-}
-/* Reads the smooth file to know how to smooth datas.
+
+/** Reads the smooth file to know how to smooth datas.
  * the smooth file if made of 2 elements lines.
  * lines starting with # are comment
  * the first element of line is face to smooth
  * the next element is the 16x2 faces picture
  * used for smoothing
  */
-int ReadSmooth () {
+int read_smooth(void) {
     char buf[MAX_BUF], *p, *q;
     FILE *fp;
     int smoothcount = 0;
@@ -396,8 +346,7 @@ int ReadSmooth () {
     sprintf (buf,"%s/smooth", settings.datadir);
     LOG(llevDebug,"Reading smooth from %s...",buf);
     if ((fp=fopen(buf,"r"))==NULL) {
-	perror("Can't open smooth file");
-	printf("buf = %s\n", buf);
+	LOG(llevError, "Cannot open smooth file %s: %s\n", strerror_local(errno));
 	exit(-1);
     }
 
@@ -410,7 +359,7 @@ int ReadSmooth () {
     smooth = (struct smoothing *) malloc(sizeof(struct smoothing) * (smoothcount));
     memset (smooth, 0, sizeof (struct smoothing) * (smoothcount));
 
-    while(fgets (buf, MAX_BUF, fp)!=NULL) {
+    while(nrofsmooth < smoothcount && fgets (buf, MAX_BUF, fp)!=NULL) {
         if (*buf == '#')
             continue;
         p=strchr(buf,' ');
@@ -418,31 +367,43 @@ int ReadSmooth () {
             continue;
         *p='\0';
         q=buf;
-        smooth[nrofsmooth].id=FindFace(q,0);
+        smooth[nrofsmooth].id=find_face(q,0);
         q=p+1;
-        smooth[nrofsmooth].smooth=FindFace(q,0);
+        smooth[nrofsmooth].smooth=find_face(q,0);
         nrofsmooth++;
     }
     fclose(fp);
 
     LOG(llevDebug,"done (got %d smooth entries)\n",nrofsmooth);
-    qsort (smooth, nrofsmooth, sizeof(struct smoothing), (int (*)())compar_smooth);
+    qsort (smooth, nrofsmooth, sizeof(struct smoothing), (int (*)(const void*, const void*))compar_smooth);
     return nrofsmooth;
 }
 
-int FindSmooth (uint16 face, uint16* smoothed) {
+/**
+ * Find the smooth face for a given face.
+ *
+ * @param face the face to find the smoothing face for
+ *
+ * @param smoothed return value: set to smooth face
+ *
+ * @return 1=smooth face found, 0=no smooth face found
+ */
+int find_smooth(uint16 face, uint16* smoothed) {
     struct smoothing *bp, tmp;
 
     tmp.id = face;
     bp = (struct smoothing *)bsearch 
-        (&tmp, smooth, nrofsmooth, sizeof(struct smoothing), (int (*)())compar_smooth);
+        (&tmp, smooth, nrofsmooth, sizeof(struct smoothing), (int (*)(const void*, const void*))compar_smooth);
     (*smoothed)=0;
     if (bp)
          (*smoothed)=bp->smooth;
     return bp ? 1 : 0;
 }
 
-void free_all_images()
+/**
+ * Deallocates memory allocated by read_bmap_names() and read_smooth().
+ */
+void free_all_images(void)
 {
     int i;
 
@@ -450,4 +411,5 @@ void free_all_images()
 	free(xbm[i].name);
     free(xbm);
     free(new_faces);
+    free(smooth);
 }
