@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_info_c =
- *    "$Id: info.c,v 1.5 2003/03/08 05:35:32 mwedel Exp $";
+ *    "$Id: info.c,v 1.9 2006/06/05 06:28:55 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -37,10 +37,11 @@
  * Dump to standard out the abilities of all monsters.
  */
 
-void dump_abilities() {
+void dump_abilities(void) {
   archetype *at;
   for(at = first_archetype; at; at=at->next) {
-    char *ch, *gen_name = "";
+    char *ch;
+    const char *gen_name = "";
     archetype *gen;
 
     if(!QUERY_FLAG(&at->clone,FLAG_MONSTER))
@@ -58,7 +59,7 @@ void dump_abilities() {
     }
 
     ch = describe_item(&at->clone, NULL);
-    printf("%-16s|%6lld|%4d|%3d|%s|%s|%s\n",at->clone.name,at->clone.stats.exp,
+    printf("%-16s|%6" FMT64 "|%4d|%3d|%s|%s|%s\n",at->clone.name,at->clone.stats.exp,
            at->clone.stats.hp,at->clone.stats.ac,ch,at->name,gen_name);
   }
 }
@@ -67,10 +68,10 @@ void dump_abilities() {
  * As dump_abilities(), but with an alternative way of output.
  */
 
-void print_monsters() {
+void print_monsters(void) {
     archetype *at;
     object *op;
-    char   attbuf[32];
+    char   attbuf[34];
     int i;
 
     printf("               |     |   |    |    |      attack       |                        resistances                                                                       |\n");
@@ -85,7 +86,7 @@ void print_monsters() {
 		   op->stats.wc,attbuf);
 	    for (i=0; i<NROFATTACKS; i++)
 		printf("%4d", op->resist[i]);
-	    printf("|%8lld|%9d|\n",op->stats.exp, new_exp(op));
+	    printf("|%8" FMT64 "|%9d|\n",op->stats.exp, new_exp(op));
     }
     free_object(op);
   }
