@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *   "$Id: init.c,v 1.41 2005/07/15 13:51:48 ryo_saeba Exp $";
+ *   "$Id: init.c,v 1.49 2006/03/18 15:05:31 ryo_saeba Exp $";
  */
 
 /*
@@ -31,7 +31,7 @@
 #include <global.h>
 #include <object.h>
 
-/* You unforunately need to looking in include/global.h to see what these
+/** You unforunately need to looking in include/global.h to see what these
  * correspond to.
  */
 struct Settings settings = {
@@ -52,10 +52,13 @@ CONFDIR,
 DATADIR, 
 LOCALDIR,
 PLAYERDIR, MAPDIR, ARCHETYPES,REGIONS,TREASURES, 
-UNIQUE_DIR, TMPDIR,
+UNIQUE_DIR, TEMPLATE_DIR,
+TMPDIR,
 STAT_LOSS_ON_DEATH,
 PK_LUCK_PENALTY,
-USE_PERMANENT_EXPERIENCE,
+PERMANENT_EXPERIENCE_RATIO,
+DEATH_PENALTY_RATIO,
+DEATH_PENALTY_LEVEL,
 BALANCED_STAT_LOSS,
 NOT_PERMADETH,
 SIMPLE_EXP,
@@ -91,14 +94,15 @@ ARMOR_MAX_ENCHANT,
 ARMOR_WEIGHT_REDUCTION,
 ARMOR_WEIGHT_LINEAR,
 ARMOR_SPEED_IMPROVEMENT,
-ARMOR_SPEED_LINEAR
+ARMOR_SPEED_LINEAR,
+1, /* no_player_stealing */
 };
 
-/* perhaps not the best place for this, but needs to be
+/** perhaps not the best place for this, but needs to be
  * in some file in the common area so that standalone
  * programs, like the random map generator, can be built.
  */
-char *spellpathnames[NRSPELLPATHS] = {
+const char* const spellpathnames[NRSPELLPATHS] = {
  "Protection",
  "Fire",
  "Frost",
@@ -122,14 +126,14 @@ char *spellpathnames[NRSPELLPATHS] = {
 };
 
 
-/* This loads the emergency map information from a
+/** This loads the emergency map information from a
  * .emergency file in the map directory.  Doing this makes
  * it easier to switch between map distributions (don't need
  * to recompile.  Note that there is no reason I see that
  * this could not be re-loaded during play, but it seems
  * like there should be little reason to do that.
  */
-static void init_emergency_mappath()
+static void init_emergency_mappath(void)
 {
     char filename[MAX_BUF], tmpbuf[MAX_BUF];
     FILE    *fp;
@@ -164,7 +168,7 @@ static void init_emergency_mappath()
 }
     
 
-/*
+/**
  * It is vital that init_library() is called by any functions
  * using this library.
  * If you want to lessen the size of the program using the library,
@@ -173,15 +177,15 @@ static void init_emergency_mappath()
  * init_hash_table if you are doing any object loading.
  */
 
-void init_library() {
+void init_library(void) {
     init_environ();
     init_globals();
     init_hash_table();
     init_objects();
     init_vars();
     init_block();
-    ReadBmapNames ();
-    ReadSmooth();
+    read_bmap_names ();
+    read_smooth();
     init_anim();    /* Must be after we read in the bitmaps */
     init_archetypes();	/* Reads all archetypes from file */
     init_dynamic ();
@@ -192,11 +196,11 @@ void init_library() {
 }
 
 
-/* init_environ initializes values from the environmental variables.
+/** init_environ initializes values from the environmental variables.
  * it needs to be called very early, since command line options should
  * overwrite these if specified.
  */
-void init_environ() {
+void init_environ(void) {
     char *cp;
 
     cp=getenv("CROSSFIRE_LIBDIR");
@@ -213,17 +217,19 @@ void init_environ() {
     if (cp) settings.treasures=cp;
     cp=getenv("CROSSFIRE_UNIQUEDIR");
     if (cp) settings.uniquedir=cp;
+    cp=getenv("CROSSFIRE_TEMPLATEDIR");
+    if (cp) settings.templatedir=cp;
     cp=getenv("CROSSFIRE_TMPDIR");
     if (cp) settings.tmpdir=cp;
 }
     
 
-/*
+/**
  * Initialises all global variables.
  * Might use environment-variables as default for some of them.
  */
 
-void init_globals() {
+void init_globals(void) {
     if (settings.logfilename[0] == 0) {
 	logfile = stderr;
     }
@@ -232,6 +238,9 @@ void init_globals() {
 		settings.logfilename);
 	logfile = stderr;
     }
+    else {
+	setvbuf(logfile, NULL, _IOLBF, 0);
+    }
     exiting = 0;
     first_player=NULL;
     first_friendly_object=NULL;
@@ -255,13 +264,13 @@ void init_globals() {
     init_defaults();
 }
 
-/*
+/**
  * Sets up and initialises the linked list of free and used objects.
  * Allocates a certain chunk of objects and puts them on the free list.
  * Called by init_library();
  */
 
-void init_objects() {
+void init_objects(void) {
   int i;
 /* Initialize all objects: */
   objects=NULL;
@@ -288,18 +297,20 @@ void init_objects() {
 #endif
 }
 
-/*
+/**
  * Initialises global variables which can be changed by options.
  * Called by init_library().
  */
 
-void init_defaults() {
+void init_defaults(void) {
   editor=0;
   nroferrors=0;
 }
 
-
-void init_dynamic () {
+/**
+ * Initializes first_map_path from the archetype collection.
+ */
+void init_dynamic (void) {
     archetype *at = first_archetype;
     while (at) {
 	if (at->clone.type == MAP && EXIT_PATH (&at->clone)) {
@@ -314,12 +325,12 @@ void init_dynamic () {
 
 unsigned long todtick;
 
-/*
+/**
  * Write out the current time to the file so time does not
  * reset every time the server reboots.
  */
 
-void write_todclock()
+void write_todclock(void)
 {
     char filename[MAX_BUF];
     FILE *fp;
@@ -333,12 +344,12 @@ void write_todclock()
     fclose(fp);
 }
 
-/*
+/**
  * Initializes the gametime and TOD counters
  * Called by init_library().
  */
 
-void init_clocks()
+void init_clocks(void)
 {
     char filename[MAX_BUF];
     FILE *fp;
@@ -362,14 +373,13 @@ void init_clocks()
     fclose(fp);
 }
 
-/*
+attackmess_t attack_mess[NROFATTACKMESS][MAXATTACKMESS];
+
+/**
  * Initializes the attack messages.
  * Called by init_library().
  */
-
-attackmess_t attack_mess[NROFATTACKMESS][MAXATTACKMESS];
-
-void init_attackmess(){
+void init_attackmess(void){
     char buf[MAX_BUF];
     char filename[MAX_BUF];
     char *cp, *p;
@@ -418,9 +428,9 @@ void init_attackmess(){
 	    attack_mess[mess][level].level = atoi(buf);
 	    p = strtok(NULL, "=");
 	    if (p != NULL)
-	        attack_mess[mess][level].buf1 = strdup(p);
+	        attack_mess[mess][level].buf1 = strdup_local(p);
 	    else
-	      attack_mess[mess][level].buf1 = strdup("");
+	      attack_mess[mess][level].buf1 = strdup_local("");
 	    mode = 2;
 	    continue;
 	} else if (mode==2) {
@@ -428,9 +438,9 @@ void init_attackmess(){
 	    attack_mess[mess][level].level = atoi(buf);
 	    p = strtok(NULL, "=");
 	    if (p != NULL)
-	        attack_mess[mess][level].buf2 = strdup(p);
+	        attack_mess[mess][level].buf2 = strdup_local(p);
 	    else
-	        attack_mess[mess][level].buf2 = strdup("");
+	        attack_mess[mess][level].buf2 = strdup_local("");
 	    mode = 3;
 	    continue;
 	} else if (mode==3) {
@@ -438,9 +448,9 @@ void init_attackmess(){
 	    attack_mess[mess][level].level = atoi(buf);
 	    p = strtok(NULL, "=");
 	    if (p != NULL)
-	        attack_mess[mess][level].buf3 = strdup(p);
+	        attack_mess[mess][level].buf3 = strdup_local(p);
 	    else
-	        attack_mess[mess][level].buf3 = strdup("");
+	        attack_mess[mess][level].buf3 = strdup_local("");
 	    mode = 1;
 	    level++;
 	    total++;
