@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_item_c =
- *   "$Id: item.c,v 1.48 2005/07/15 13:51:48 ryo_saeba Exp $";
+ *   "$Id: item.c,v 1.63 2006/06/05 06:28:55 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -52,7 +52,7 @@
  * Basically, for the use/nonuse, the code does something like:
  * "This item goes %s\n", with the use/nonuse values filling in the %s
  */
-Body_Locations body_locations[NUM_BODY_LOCATIONS] = {
+body_locations_struct body_locations[NUM_BODY_LOCATIONS] = {
 {"body_range",	"in your range slot",	"in a human's range slot"},
 {"body_arm",	"on your arm",		"on a human's arm"},
 {"body_torso",	"on your body",		"on a human's torso"},
@@ -92,30 +92,138 @@ static char levelnumbers_10[11][20] = {
   "seventieth","eightieth","ninetieth"
 };
 
-/*
-materialtype material[NROFMATERIALS] = {
-  *  		  P  M  F  E  C  C  A  D  W  G  P S P T F  C D D C C G H B  I *
-   *		  H  A  I  L  O  O  C  R  E  H  O L A U E  A E E H O O O L  N *
-   *		  Y  G  R  E  L  N  I  A  A  O  I O R R A  N P A A U D L I  T *
-   *		  S  I  E  C  D  F  D  I  P  S  S W A N R  C L T O N   Y N  R *
-   *		  I  C     T     U     N  O  T  O   L      E E H S T P   D  N *
-  {"paper", 	{15,10,17, 9, 5, 7,13, 0,20,15, 0,0,0,0,0,10,0,0,0,0,0,0,0,0}},
-  {"metal", 	{ 2,12, 3,12, 2,10, 7, 0,20,15, 0,0,0,0,0,10,0,0,0,0,0,0,0,0}},
-  {"glass", 	{14,11, 8, 3,10, 5, 1, 0,20,15, 0,0,0,0,0, 0,0,0,0,0,0,0,0,0}},
-  {"leather", 	{ 5,10,10, 3, 3,10,10, 0,20,15, 0,0,0,0,0,12,0,0,0,0,0,0,0,0}},
-  {"wood", 	{10,11,13, 2, 2,10, 9, 0,20,15, 0,0,0,0,0,12,0,0,0,0,0,0,0,0}},
-  {"organics", 	{ 3,12, 9,11, 3,10, 9, 0,20,15, 0,0,0,0,0, 0,0,0,0,0,0,0,0,0}},
-  {"stone", 	{ 2, 5, 2, 2, 2, 2, 1, 0,20,15, 0,0,0,0,0, 5,0,0,0,0,0,0,0,0}},
-  {"cloth", 	{14,11,13, 4, 4, 5,10, 0,20,15, 0,0,0,0,0, 5,0,0,0,0,0,0,0,0}},
-  {"adamant", 	{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,0,0,0,0, 0,0,0,0,0,0,0,0,0}},
-  {"liquid", 	{ 0, 8, 9, 6,17, 0,15, 0,20,15,12,0,0,0,0,11,0,0,0,0,0,0,0,0}},
-  {"soft metal",{ 6,12, 6,14, 2,10, 1, 0,20,15, 0,0,0,0,0,10,0,0,0,0,0,0,0,0}},
-  {"bone", 	{10, 9, 4, 5, 3,10,10, 0,20,15, 0,0,0,0,0, 2,0,0,0,0,0,0,0,0}},
-  {"ice", 	{14,11,16, 5, 0, 5, 6, 0,20,15, 0,0,0,0,0, 7,0,0,0,0,0,0,0,0}}
+/* The following is a large table of item types, the fields are:
+ * item number, item name, item name (plural), and two numbers that are the skills
+ * used to identify them. Anytime a new item type is added or removed, this list
+ * should be altered to reflect that. The defines for the numerical values are in 
+ * define.h 
+ */
+static const typedata item_types[] = {
+{PLAYER, "player", "players", 0, 0},
+{ROD, "rod", "rods", SK_THAUMATURGY, 0},
+{TREASURE, "treasure", "treasure", 0, 0},
+{POTION, "potion", "potions", SK_ALCHEMY, 0},
+{FOOD, "food", "food", SK_WOODSMAN, 0},
+{POISON, "poison", "poisons", SK_ALCHEMY, 0},
+{BOOK, "book", "books", SK_LITERACY, 0},
+{CLOCK, "clock", "clocks", 0, 0},
+{LIGHTNING, "lightning", "lightning", 0, 0},
+{ARROW, "arrow", "arrows", SK_BOWYER, 0},
+{BOW, "bow", "bows", SK_BOWYER, 0},
+{WEAPON, "weapon", "weapons", SK_SMITHERY, 0},
+{ARMOUR, "armour", "armour", SK_SMITHERY, 0},
+{PEDESTAL, "pedestal", "pedestals", 0, 0},
+{ALTAR, "altar", "altars", 0, 0},
+{LOCKED_DOOR, "locked door", "locked doors", 0, 0},
+{SPECIAL_KEY, "special key", "special keys", 0, 0},
+{MAP, "map", "maps", 0, 0},
+{DOOR, "door", "doors", 0, 0},
+{KEY, "key", "keys", 0, 0},
+{TIMED_GATE, "timed_gate", "timed_gates", 0, 0},
+{TRIGGER, "trigger", "triggers", 0, 0},
+{GRIMREAPER, "grimreaper", "grimreapers", 0, 0},
+{MAGIC_EAR, "magic ear", "magic ears", 0, 0},
+{TRIGGER_BUTTON, "trigger button", "trigger buttons", 0, 0},
+{TRIGGER_ALTAR, "trigger altar", "trigger altars", 0, 0},
+{TRIGGER_PEDESTAL, "trigger pedestal", "trigger pedestals", 0, 0},
+{SHIELD, "shield", "shields", SK_SMITHERY, 0},
+{HELMET, "helmet", "helmets", SK_SMITHERY, 0},
+{HORN, "horn", "horns", SK_THAUMATURGY, 0},
+{MONEY, "money", "money", 0, 0},
+{CLASS, "class", "classes", 0, 0},
+{GRAVESTONE, "gravestone", "gravestones", 0, 0},
+{AMULET, "amulet", "amulets", SK_JEWELER, 0},
+{PLAYERMOVER, "player mover", "player movers", 0, 0},
+{TELEPORTER, "teleporter", "teleporters", 0, 0},
+{CREATOR, "creator", "creators", 0, 0},
+{SKILL, "skill", "skills", 0, 0},
+{EXPERIENCE, "experience", "experience", 0, 0},
+{EARTHWALL, "earthwall", "earthwalls", 0, 0},
+{GOLEM, "golem", "golems", 0, 0},
+{THROWN_OBJ, "projectile", "projectiles", 0, 0},
+{BLINDNESS, "blindness", "blindness", 0, 0},
+{GOD, "god", "gods", 0, 0},
+{DETECTOR, "detector", "detectors", 0, 0},
+{TRIGGER_MARKER, "trigger marker", "trigger markers", 0, 0},
+{DEAD_OBJECT, "dead object", "dead objects", 0, 0},
+{DRINK, "drink", "drinks", SK_WOODSMAN, SK_ALCHEMY},
+{MARKER, "marker", "markers", 0, 0},
+{HOLY_ALTAR, "holy altar", "holy altars", 0, 0},
+{PLAYER_CHANGER, "player changer", "player changers", 0, 0},
+{BATTLEGROUND, "battleground", "battlegrounds", 0, 0},
+{PEACEMAKER, "peacemaker", "peacemakers", 0, 0},
+{GEM, "gem", "gems", SK_JEWELER, 0},
+{FIREWALL, "firewall", "firewalls", 0, 0},
+{ANVIL, "anvil", "anvils", 0, 0},
+{CHECK_INV, "inventory checker", "inventory checkers", 0, 0},
+{MOOD_FLOOR, "mood floor", "mood floors", 0, 0},
+{EXIT, "exit", "exits", 0, 0},
+{ENCOUNTER, "encounter", "encounters", 0, 0},
+{SHOP_FLOOR, "shop floor", "shop floors", 0, 0},
+{SHOP_MAT, "shop mat", "shop mats", 0, 0},
+{RING, "ring", "rings", SK_JEWELER, 0},
+{FLOOR, "floor", "floors", 0, 0},
+{FLESH, "flesh", "flesh", SK_WOODSMAN, 0},
+{INORGANIC, "inorganic", "inorganics", SK_ALCHEMY, 0},
+{SKILL_TOOL, "skill tool", "skill tools", 0, 0},
+{LIGHTER, "lighter", "lighters", 0, 0},
+{TRAP_PART, "trap part", "trap parts", 0, 0},
+{WALL, "wall", "walls", 0, 0},
+{LIGHT_SOURCE, "light source", "light sources", 0, 0},
+{MISC_OBJECT, "bric-a-brac", "bric-a-brac", 0, 0},
+{MONSTER, "monster", "monsters", 0, 0},
+{SPAWN_GENERATOR, "monster generator", "monster generators", 0, 0},
+{LAMP, "lamp", "lamps", 0, 0},
+{DUPLICATOR, "duplicator", "duplicators", 0, 0},
+{TOOL, "tool", "tools", 0, 0},
+{SPELLBOOK, "spellbook", "spellbooks", SK_LITERACY, 0},
+{BUILDFAC, "building facility", "building facilities", 0, 0},
+{CLOAK, "cloak", "cloaks", SK_SMITHERY, 0},
+{SPINNER, "spinner", "spinners", 0, 0},
+{GATE, "gate", "gates", 0, 0},
+{BUTTON, "button", "buttons", 0, 0},
+{CF_HANDLE, "cf handle", "cf handles", 0, 0},
+{HOLE, "hole", "holes", 0, 0},
+{TRAPDOOR, "trapdoor", "trapdoors", 0, 0},
+{SIGN, "sign", "signs", 0, 0},
+{BOOTS, "boots", "boots", SK_SMITHERY, 0},
+{GLOVES, "gloves", "gloves", SK_SMITHERY, 0},
+{SPELL, "spell", "spells", 0, 0},
+{SPELL_EFFECT, "spell effect", "spell effects", 0, 0},
+{CONVERTER, "converter", "converters", 0, 0},
+{BRACERS, "bracers", "bracers", SK_SMITHERY, 0},
+{POISONING, "poisoning", "poisonings", 0, 0},
+{SAVEBED, "savebed", "savebeds", 0, 0},
+{POISONCLOUD, "poison cloud", "poison clouds", 0, 0},
+{FIREHOLES, "fireholes", "fireholes", 0, 0},
+{WAND, "wand", "wands", SK_THAUMATURGY, 0},
+{SCROLL, "scroll", "scrolls", SK_LITERACY, 0},
+{DIRECTOR, "director", "directors", 0, 0},
+{GIRDLE, "girdle", "girdles", SK_SMITHERY, 0},
+{FORCE, "force", "forces", 0, 0},
+{POTION_EFFECT, "potion effect", "potion effects", 0, 0},
+{CLOSE_CON, "closed container", "closed container", 0, 0},
+{CONTAINER, "container", "containers", SK_ALCHEMY, 0},
+{ARMOUR_IMPROVER, "armour improver", "armour improvers", 0, 0},
+{WEAPON_IMPROVER, "weapon improver", "weapon improvers", 0, 0},
+{SKILLSCROLL, "skillscroll", "skillscrolls", 0, 0},
+{DEEP_SWAMP, "deep swamp", "deep swamps", 0, 0},
+{IDENTIFY_ALTAR, "identify altar", "identify altars", 0, 0},
+{MENU, "inventory list", "inventory lists", 0, 0},
+{RUNE, "rune", "runes", 0, 0},
+{TRAP, "trap", "traps", 0, 0},
+{POWER_CRYSTAL, "power_crystal", "power_crystals", 0, 0},
+{CORPSE, "corpse", "corpses", 0, 0},
+{DISEASE, "disease", "diseases", 0, 0},
+{SYMPTOM, "symptom", "symptoms", 0, 0},
+{BUILDER, "item builder", "item builders", 0, 0},
+{MATERIAL, "building material", "building materials", 0, 0},
+{ITEM_TRANSFORMER, "item_transformer", "item_transformers", 0, 0},
 };
-*/
 
-/* This curve may be too steep.  But the point is that there should
+const int item_types_size=sizeof(item_types)/sizeof(*item_types);
+
+/** This curve may be too steep.  But the point is that there should
  * be tough choices - there is no real point to this if everyone can
  * wear whatever they want with no worries.  Perhaps having the steep
  * curve is good (maybe even steeper), but allowing players to
@@ -138,14 +246,14 @@ int get_power_from_ench(int ench)
     return enc_to_item_power[ench];
 }
 
-/* This takes an object 'op' and figures out what its item_power
+/** This takes an object 'op' and figures out what its item_power
  * rating should be.  This should only really be used by the treasure
  * generation code, and when loading legacy objects.  It returns
  * the item_power it calculates.
  * If flag is 1, we return the number of enchantment, and not the
  * the power.  This is used in the treasure code.
  */
-int calc_item_power(object *op, int flag)
+int calc_item_power(const object *op, int flag)
 {
     int i, tmp, enc;
 
@@ -209,13 +317,40 @@ int calc_item_power(object *op, int flag)
 
 }
 
-/* describe_resistance generates the visible naming for resistances.
- * returns a static array of the description.  This can return
+/** returns the typedata that has a number equal to itemtype, if there 
+ * isn't one, returns NULL */
+
+const typedata *get_typedata(int itemtype) {
+    int i;
+    for (i=0;i<item_types_size;i++) 
+	if (item_types[i].number==itemtype) return &item_types[i];
+    return NULL;
+}
+
+/** returns the typedata that has a name equal to itemtype, if there 
+ * isn't one, return the plural name that matches, if there still isn't
+ * one return NULL */
+
+const typedata *get_typedata_by_name(const char *name) {
+    int i;
+    for (i=0;i<item_types_size;i++) 
+	if (!strcmp(item_types[i].name, name)) return &item_types[i];
+    for (i=0;i<item_types_size;i++) 
+	if (!strcmp(item_types[i].name_pl, name)) {
+	    LOG(llevInfo, 
+		"get_typedata_by_name: I have been sent the plural %s, the singular form %s is preffered\n",
+		name, item_types[i].name);
+	    return &item_types[i];
+	}
+    return NULL;
+}
+/** Generates the visible naming for resistances.
+ * Returns a static array of the description.  This can return
  * a big buffer.
- * if newline is true, we don't put parens around the description
+ * If newline is true, we don't put parens around the description
  * but do put a newline at the end.  Useful when dumping to files
  */
-char *describe_resistance(object *op, int newline)
+char *describe_resistance(const object *op, int newline)
 {
     static char buf[VERY_BIG_BUF];
     char    buf1[VERY_BIG_BUF];
@@ -236,15 +371,15 @@ char *describe_resistance(object *op, int newline)
 }
 
 
-/*
+/**
  * query_weight(object) returns a character pointer to a static buffer
  * containing the text-representation of the weight of the given object.
  * The buffer will be overwritten by the next call to query_weight().
  */
 
-char *query_weight(object *op) {
+const char *query_weight(const object *op) {
   static char buf[10];
-  sint32 i=op->nrof?op->nrof*op->weight:op->weight+op->carrying;
+  sint32 i=(op->nrof?op->nrof:1)*op->weight+op->carrying;
 
   if(op->weight<0)
     return "      ";
@@ -255,7 +390,7 @@ char *query_weight(object *op) {
   return buf;
 }
 
-/*
+/**
  * Returns the pointer to a static buffer containing
  * the number requested (of the form first, second, third...)
  */
@@ -276,7 +411,7 @@ char *get_levelnumber(int i) {
 }
 
 
-/*
+/**
  * get_number(integer) returns the text-representation of the given number
  * in a static buffer.  The buffer might be overwritten at the next
  * call to get_number().
@@ -293,7 +428,7 @@ char *get_number(int i) {
   }
 }
 
-/*
+/**
  *  Returns pointer to static buffer containing ring's or amulet's
  *  abilities
  *  These are taken from old query_name(), but it would work better
@@ -301,11 +436,11 @@ char *get_number(int i) {
  *  caller would handle FULL_RING_DESCRIPTION definition.
  *  Or make FULL_RING_DESCRIPTION standard part of a game and let
  *  client handle names.
- */
-/* Aug 95 modified this slightly so that Skill tools don't have magic bonus
+ * 
+ * Aug 95 modified this slightly so that Skill tools don't have magic bonus
  * from stats.sp - b.t.
  */
-char *ring_desc (object *op) 
+static const char *ring_desc (const object *op) 
 {
     static char buf[VERY_BIG_BUF];
     int attr, val,len;
@@ -315,13 +450,13 @@ char *ring_desc (object *op)
     if (! QUERY_FLAG(op, FLAG_IDENTIFIED))
 	return buf;
 
-    for (attr=0; attr<7; attr++) {
+    for (attr=0; attr<NUM_STATS; attr++) {
 	if ((val=get_attr_value(&(op->stats),attr))!=0) {
 	    sprintf (buf+strlen(buf), "(%s%+d)", short_stat_name[attr], val);
 	}
     }
     if(op->stats.exp)
-	sprintf(buf+strlen(buf), "(speed %+lld)", op->stats.exp);
+	sprintf(buf+strlen(buf), "(speed %+" FMT64 ")", op->stats.exp);
     if(op->stats.wc)
 	sprintf(buf+strlen(buf), "(wc%+d)", op->stats.wc);
     if(op->stats.dam)
@@ -367,11 +502,11 @@ char *ring_desc (object *op)
     return buf;
 }
 
-/*
+/**
  * query_short_name(object) is similar to query_name, but doesn't 
  * contain any information about object status (worn/cursed/etc.)
  */
-char *query_short_name(object *op) 
+const char *query_short_name(const object *op) 
 {
     static char buf[HUGE_BUF];
     char buf2[HUGE_BUF];
@@ -417,7 +552,7 @@ char *query_short_name(object *op)
       case RING:
 	if (!op->title) {
 	    /* If ring has a title, full description isn't so useful */ 
-	    char *s = ring_desc(op);
+	    const char *s = ring_desc(op);
 	    if (s[0]) {
 		safe_strcat (buf, " ", &len, HUGE_BUF);
 		safe_strcat(buf, s, &len, HUGE_BUF);
@@ -434,7 +569,7 @@ char *query_short_name(object *op)
     return buf;
 }
 
-/*
+/**
  * query_name(object) returns a character pointer pointing to a static
  * buffer which contains a verbose textual representation of the name
  * of the given object.
@@ -445,7 +580,7 @@ char *query_short_name(object *op)
  * use several names much easier (don't need to store them to temp variables.)
  *
  */
-char *query_name(object *op) {
+char *query_name(const object *op) {
     static char buf[5][HUGE_BUF];
     static int use_buf=0;
     int len=0;
@@ -539,7 +674,7 @@ char *query_name(object *op) {
     return buf[use_buf];
 }
 
-/*
+/**
  * query_base_name(object) returns a character pointer pointing to a static
  * buffer which contains a verbose textual representation of the name
  * of the given object.  The buffer will be overwritten at the next
@@ -548,10 +683,12 @@ char *query_name(object *op) {
  * and sending to client.
  * If plural is set, we generate the plural name of this.
  */
-char *query_base_name(object *op, int plural) {
+const char *query_base_name(const object *op, int plural) {
     static char buf[MAX_BUF], buf2[MAX_BUF];
     int len;
+#ifdef NEW_MATERIAL_CODE
     materialtype_t *mt;
+#endif
 
     if((!plural && !op->name) || (plural && !op->name_pl))
 	return "(null)";
@@ -559,11 +696,13 @@ char *query_base_name(object *op, int plural) {
     if(!op->nrof && !op->weight && !op->title && !is_magical(op)) 
 	return op->name; /* To speed things up (or make things slower?) */
 
+#ifdef NEW_MATERIAL_CODE
     if ((IS_ARMOR(op) || IS_WEAPON(op)) && op->materialname)
 	mt = name_to_material(op->materialname);
+    else
+	mt = NULL;
 
-#ifdef NEW_MATERIAL_CODE
-    if ((IS_ARMOR(op) || IS_WEAPON(op)) && op->materialname && mt &&
+    if (mt &&
 	op->arch->clone.materialname != mt->name &&
 	!(op->material & M_SPECIAL)) {
 	strcpy(buf, mt->description);
@@ -615,7 +754,7 @@ char *query_base_name(object *op, int plural) {
       case RING:
 	if (!op->title) {
 	    /* If ring has a title, full description isn't so useful */ 
-	    char *s = ring_desc(op);
+	    const char *s = ring_desc(op);
 	    if (s[0]) {
 		safe_strcat (buf, " ", &len, MAX_BUF);
 		safe_strcat (buf, s, &len, MAX_BUF);
@@ -631,14 +770,14 @@ char *query_base_name(object *op, int plural) {
     return buf;
 }
 
-/* Break this off from describe_item - that function was way
+/** Break this off from describe_item - that function was way
  * too long, making it difficult to read.  This function deals
  * with describing the monsters & players abilities.  It should only
  * be called with monster & player objects.  Returns a description
  * in a static buffer.
  */
 
-static char *describe_monster(object *op) {
+static char *describe_monster(const object *op) {
     char buf[MAX_BUF];
     static char retbuf[VERY_BIG_BUF];
     int i;
@@ -681,8 +820,6 @@ static char *describe_monster(object *op) {
     }
     if(QUERY_FLAG(op,FLAG_UNDEAD))
 	strcat(retbuf,"(undead)");
-    if(QUERY_FLAG(op,FLAG_CAN_PASS_THRU))
-	strcat(retbuf,"(pass through doors)");
     if(QUERY_FLAG(op,FLAG_SEE_INVISIBLE))
 	strcat(retbuf,"(see invisible)");
     if(QUERY_FLAG(op,FLAG_USE_WEAPON))
@@ -780,7 +917,7 @@ static char *describe_monster(object *op) {
 }
 
 
-/*
+/**
  * Returns a pointer to a static buffer which contains a
  * description of the given object.
  * If it is a monster, lots of information about its abilities
@@ -807,7 +944,7 @@ static char *describe_monster(object *op) {
  * item (eg, debug dump or the like)
  */
 
-char *describe_item(object *op, object *owner) {
+char *describe_item(const object *op, const object *owner) {
     char buf[MAX_BUF];
     static char retbuf[VERY_BIG_BUF];
     int identified,i;
@@ -824,7 +961,6 @@ char *describe_item(object *op, object *owner) {
 	strcpy(retbuf,"(unidentified)");
 	identified = 0;
     }
-
     switch(op->type) {
 	case BOW:
 	case ARROW:
@@ -843,6 +979,32 @@ char *describe_item(object *op, object *owner) {
 	case SKILL_TOOL:
 	    break;  /* We have more information to do below this switch */
 
+	case POWER_CRYSTAL:
+	    if (op->stats.maxsp>1000){ /*higher capacity crystals*/
+	        i = (op->stats.maxsp%100)/10;
+	        if (i)
+	            snprintf(buf,MAX_BUF,"(capacity %d.%dk). It is ",op->stats.maxsp/100,i);
+                else
+	            snprintf(buf,MAX_BUF,"(capacity %dk). It is ",op->stats.maxsp/100);
+	    }else
+	        snprintf(buf,MAX_BUF,"(capacity %d). It is ",op->stats.maxsp);
+	    strcat(retbuf,buf);
+	    i = (op->stats.sp*10)/op->stats.maxsp;
+	    if (op->stats.sp==0)
+	        strcat(retbuf,"empty.");
+	    else if (i==0)
+	        strcat(retbuf,"almost empty.");
+	    else if (i<3)
+	        strcat(retbuf,"partially filled.");
+	    else if (i<6)
+	        strcat(retbuf,"half full.");
+	    else if (i<9)
+	        strcat(retbuf,"well charged.");
+	    else if (op->stats.sp == op->stats.maxsp)
+	        strcat(retbuf,"fully charged.");
+	    else
+	        strcat(retbuf,"almost full.");
+	    break;
 	case FOOD:
 	case FLESH:
 	case DRINK:
@@ -892,7 +1054,7 @@ char *describe_item(object *op, object *owner) {
     if (identified || QUERY_FLAG(op,FLAG_BEEN_APPLIED)) {
 	int attr,val;
 
-	for (attr=0; attr<7; attr++) {
+	for (attr=0; attr<NUM_STATS; attr++) {
 	    if ((val=get_attr_value(&(op->stats),attr))!=0) {
 		sprintf(buf, "(%s%+d)", short_stat_name[attr], val);
 		strcat(retbuf,buf);
@@ -900,7 +1062,7 @@ char *describe_item(object *op, object *owner) {
 	}
 
 	if(op->stats.exp) {
-	    sprintf(buf,"(speed %+lld)",op->stats.exp);
+	    sprintf(buf,"(speed %+" FMT64 ")",op->stats.exp);
 	    strcat(retbuf,buf);
 	}
 
@@ -944,11 +1106,21 @@ char *describe_item(object *op, object *owner) {
 	}
 	if(QUERY_FLAG(op,FLAG_XRAYS))
 	    strcat(retbuf,"(xray-vision)");
-	if(QUERY_FLAG(op,FLAG_FLYING))
-	    strcat(retbuf,"(levitate)");
 	if(QUERY_FLAG(op,FLAG_SEE_IN_DARK))
 	    strcat(retbuf,"(infravision)");
 
+	/* levitate was what is was before, so we'll keep it */
+	if (op->move_type & MOVE_FLY_LOW)
+	    strcat(retbuf,"(levitate)");
+
+	if (op->move_type & MOVE_FLY_HIGH)
+	    strcat(retbuf,"(fly)");
+
+	if (op->move_type & MOVE_SWIM)
+	    strcat(retbuf,"(swim)");
+
+	/* walking is presumed as 'normal', so doesn't need mentioning */
+
 	if(op->item_power) {
 	    sprintf(buf,"(item_power %+d)",op->item_power);
 	    strcat(retbuf,buf);
@@ -1057,14 +1229,14 @@ char *describe_item(object *op, object *owner) {
     return retbuf;
 }
 
-/* Return true if the item is magical.  A magical item is one that
+/** Return true if the item is magical.  A magical item is one that
  * increases/decreases any abilities, provides a resistance,
  * has a generic magical bonus, or is an artifact.
  * This function is used by detect_magic to determine if an item
  * should be marked as magical.
  */
 
-int is_magical(object *op) {
+int is_magical(const object *op) {
     int i;
 
     /* living creatures are considered non magical */
@@ -1082,9 +1254,9 @@ int is_magical(object *op) {
 		 return 1;
 
     /* Check for stealty, speed, flying, or just plain magic in the boots */
+    /* Presume any boots that hvae a move_type are special. */
     if (op->type== BOOTS && 
-	(QUERY_FLAG(op, FLAG_STEALTH) || QUERY_FLAG(op, FLAG_FLYING) ||
-	 op->stats.exp))
+	((QUERY_FLAG(op, FLAG_STEALTH) || op->move_type ||  op->stats.exp)))
 		return 1;
 
     /* Take care of amulet/shield that reflects spells/missiles */
@@ -1121,7 +1293,7 @@ int is_magical(object *op) {
 	    return 1;
 
     /* Check to see if it increases/decreases any stats */
-    for (i=0; i<7; i++) 
+    for (i=0; i<NUM_STATS; i++) 
 	    if (get_attr_value(&(op->stats),i)!=0) return 1;
 
     /* If it doesn't fall into any of the above categories, must
@@ -1130,12 +1302,12 @@ int is_magical(object *op) {
     return 0;
 }
 
-/* need_identify returns true if the item should be identified.  This 
+/** need_identify returns true if the item should be identified.  This 
  * function really should not exist - by default, any item not identified
  * should need it.
  */
 
-int need_identify(object *op) {
+int need_identify(const object *op) {
     switch(op->type) {
 	case RING:
 	case WAND:
@@ -1181,7 +1353,7 @@ int need_identify(object *op) {
 }
 
 
-/*
+/**
  * Supposed to fix face-values as well here, but later.
  */
 
