@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_living_c =
- *   "$Id: living.c,v 1.71 2005/08/10 13:38:31 ryo_saeba Exp $";
+ *   "$Id: living.c,v 1.78 2006/04/06 21:18:34 tchize Exp $";
  */
 
 /*
@@ -35,7 +35,7 @@
  */
 #define ADD_EXP(exptotal, exp) {exptotal += exp; if (exptotal > MAX_EXPERIENCE) exptotal = MAX_EXPERIENCE; }
 
-static int con_bonus[MAX_STAT + 1]={
+static const int con_bonus[MAX_STAT + 1]={
   -6,-5,-4,-3,-2,-1,-1,0,0,0,0,1,2,3,4,5,6,7,8,9,10,12,14,16,18,20,
   22,25,30,40,50
 };
@@ -44,12 +44,12 @@ static int con_bonus[MAX_STAT + 1]={
  * because Pow can now be the stat that controls spellpoint
  * advancement. -b.t.
  */
-static int sp_bonus[MAX_STAT + 1]={
+static const int sp_bonus[MAX_STAT + 1]={
   -10,-10,-9,-8,-7,-6,-5,-4,-3,-2,-1,0,1,2,3,4,5,6,7,8,9,10,12,15,20,25,
   30,40,50,70,100
 };
 
-static int grace_bonus[MAX_STAT +1] = {
+static const int grace_bonus[MAX_STAT +1] = {
     -10,-10,-9,-8,-7,-6,-5,-4,-3,-2,-1,0,1,2,3,4,5,6,7,8,9,10,12,15,20,25,
   30,40,50,70,100
 };
@@ -70,18 +70,18 @@ static int grace_bonus[MAX_STAT +1] = {
  * it is 1-diff
  */
 
-float cha_bonus[MAX_STAT + 1]={10.0, 10.0, 9.0, 8.0, 7.0, 6.0, /*<-5*/
+const float cha_bonus[MAX_STAT + 1]={10.0, 10.0, 9.0, 8.0, 7.0, 6.0, /*<-5*/
     5.0, 4.5, 4.0, 3.5, 3.0, /*<-10*/ 2.9, 2.8, 2.7, 2.6, 2.5, /*<-15*/
     2.4, 2.3, 2.2, 2.1, 2.0, /*<-20*/ 1.95, 1.90, 1.85, 1.80, 1.75, /*25 */
     1.70, 1.65, 1.60, 1.55, 1.50 /*30 */
 };
 
-int dex_bonus[MAX_STAT + 1]={
+const int dex_bonus[MAX_STAT + 1]={
   -4,-3,-2,-2,-1,-1,-1,0,0,0,0,0,0,0,1,1,1,2,2,2,3,3,3,4,4,4,5,5,6,6,7
 };
 
 /* speed_bonus uses dex as its stat */
-float speed_bonus[MAX_STAT + 1]={
+const float speed_bonus[MAX_STAT + 1]={
   -0.4, -0.4, -0.3, -0.3, -0.2, -0.2, -0.2, -0.1, -0.1, -0.1, -0.05, 0, 0, 0,
   0.05, 0.1, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.8, 1.0, 1.2, 1.4,
   1.6, 1.8, 2.0, 2.5, 3.0
@@ -90,15 +90,16 @@ float speed_bonus[MAX_STAT + 1]={
 /* dam_bonus, thaco_bonus, max_carry, weight limit all are based on
  * strength.
  */
-int dam_bonus[MAX_STAT + 1]={
+const int dam_bonus[MAX_STAT + 1]={
   -2,-2,-2,-1,-1,-1,0,0,0,0,0,0,1,1,1,2,2,2,3,3,3,4,4,5,5,6,6,7,8,10,15
 };
-int thaco_bonus[MAX_STAT + 1]={
+
+const int thaco_bonus[MAX_STAT + 1]={
   -2,-2,-1,-1,0,0,0,0,0,0,0,0,0,0,1,1,1,2,2,2,3,3,3,4,4,5,5,6,7,8,10
 };
 
 /* Max you can carry before you start getting extra speed penalties */
-int max_carry[MAX_STAT + 1]={
+const int max_carry[MAX_STAT + 1]={
   2,4,7,11,16,22,29,37,46,56,67,79,92,106,121,137,154,172,191,211,232,254,277,
   301,326,352,400,450,500,600,1000
 };
@@ -110,7 +111,7 @@ int max_carry[MAX_STAT + 1]={
  * before, you need to start someplace.
  */
 
-uint32 weight_limit[MAX_STAT+ 1] = {
+const uint32 weight_limit[MAX_STAT+ 1] = {
     200000,  /* 0 */
     250000,300000,350000,400000,500000,	    /* 5*/
     600000,700000,800000,900000,1000000,    /* 10 */
@@ -120,17 +121,20 @@ uint32 weight_limit[MAX_STAT+ 1] = {
     3250000,3500000,3750000,4000000,4500000  /*30 */ 
 };
 
-int learn_spell[MAX_STAT + 1]={
+const int learn_spell[MAX_STAT + 1]={
   0,0,0,1,2,4,8,12,16,25,36,45,55,65,70,75,80,85,90,95,100,100,100,100,100,
   100,100,100,100,100,100
 };
-int cleric_chance[MAX_STAT + 1]={
+
+const int cleric_chance[MAX_STAT + 1]={
   100,100,100,100,90,80,70,60,50,40,35,30,25,20,15,14,13,12,11,10,9,8,7,6,5,4,3,2,1,0,0
 };
-int turn_bonus[MAX_STAT + 1]={
+
+const int turn_bonus[MAX_STAT + 1]={
   -1,-1,-1,-1,-1,-1,-1,-1,0,0,0,1,1,1,2,2,2,3,3,3,4,4,5,5,6,7,8,9,10,12,15
 };
-int fear_bonus[MAX_STAT + 1]={
+
+const int fear_bonus[MAX_STAT + 1]={
   3,3,3,3,2,2,2,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
 };
 
@@ -171,7 +175,7 @@ extern sint64 *levels;
  * for safety, savethrow should not be accessed directly anymore,
  * and instead did_make_save should be used instead.
  */
-static int savethrow[MAX_SAVE_LEVEL+1]={
+static const int savethrow[MAX_SAVE_LEVEL+1]={
   18,
   18,17,16,15,14,14,13,13,12,12,12,11,11,11,11,10,10,10,10, 9,
    9, 9, 9, 9, 8, 8, 8, 8, 8, 8, 7, 7, 7, 7, 7, 7, 7, 6, 6, 6,
@@ -181,7 +185,7 @@ static int savethrow[MAX_SAVE_LEVEL+1]={
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1
 };
 
-char *attacks[NROFATTACKS] = {
+const char *const attacks[NROFATTACKS] = {
   "physical", "magical", "fire", "electricity", "cold", "confusion",
   "acid", "drain", "weaponmagic", "ghosthit", "poison", "slow",
   "paralyze", "turn undead", "fear", "cancellation", "depletion", "death",
@@ -189,7 +193,7 @@ char *attacks[NROFATTACKS] = {
   "life stealing"
 };
 
-static char *drain_msg[NUM_STATS] = {
+static const char *const drain_msg[NUM_STATS] = {
   "Oh no! You are weakened!",
   "You're feeling clumsy!",
   "You feel less healthy",
@@ -198,7 +202,7 @@ static char *drain_msg[NUM_STATS] = {
   "Watch out, your mind is going!", 
   "Your spirit feels drained!"
 };
-char *restore_msg[NUM_STATS] = {
+const char *const restore_msg[NUM_STATS] = {
   "You feel your strength return.",
   "You feel your agility return.",
   "You feel your health return.",
@@ -207,7 +211,7 @@ char *restore_msg[NUM_STATS] = {
   "You feel your memory return.", 
   "You feel your spirits return."
 };
-char *gain_msg[NUM_STATS] = {
+const char *const gain_msg[NUM_STATS] = {
 	"You feel stronger.",
 	"You feel more agile.",
 	"You feel healthy.",
@@ -216,7 +220,7 @@ char *gain_msg[NUM_STATS] = {
 	"You feel smarter.", 
 	"You feel more potent."
 };
-char *lose_msg[NUM_STATS] = {
+const char *const lose_msg[NUM_STATS] = {
 	"You feel weaker!",
 	"You feel clumsy!",
 	"You feel less healthy!",
@@ -226,11 +230,11 @@ char *lose_msg[NUM_STATS] = {
 	"You feel less potent!"
 };
 
-char *statname[NUM_STATS] = {
+const char *const statname[NUM_STATS] = {
   "strength", "dexterity", "constitution", "wisdom", "charisma", "intelligence","power" 
 };
 
-char *short_stat_name[NUM_STATS] = {
+const char *const short_stat_name[NUM_STATS] = {
   "Str", "Dex", "Con", "Wis", "Cha", "Int","Pow" 
 };
 
@@ -306,7 +310,7 @@ change_attr_value(living *stats,int attr,sint8 value) {
  */
 
 sint8
-get_attr_value(living *stats,int attr) {
+get_attr_value(const living *stats,int attr) {
   switch(attr) {
   case STR:
     return(stats->Str);
@@ -342,6 +346,13 @@ void check_stat_bounds(living *stats) {
 
 #define ORIG_S(xyz,abc)	(op->contr->orig_stats.abc)
 
+/* Rather than having a whole bunch of if (flag) new_draw.. else new_draw,
+ * make this macro to clean those up.  Not usuable outside change_abil
+ * function since some of the values passed to new_draw_info are hardcoded.
+ */
+#define DIFF_MSG(flag, msg1, msg2) \
+    new_draw_info(NDI_UNIQUE, 0, op, (flag>0)?msg1:msg2);
+
 /* return 1 if we sucessfully changed a stat, 0 if nothing was changed. */
 /* flag is set to 1 if we are applying the object, -1 if we are removing
  * the object.
@@ -355,15 +366,16 @@ void check_stat_bounds(living *stats) {
  * that gives them that ability.
  */
 int change_abil(object *op, object *tmp) {
-  int flag=QUERY_FLAG(tmp,FLAG_APPLIED)?1:-1,i,j,success=0;
-  object refop;
-  char message[MAX_BUF];
-  int potion_max=0;
+    int flag=QUERY_FLAG(tmp,FLAG_APPLIED)?1:-1,i,j,success=0;
+    object refop;
+    char message[MAX_BUF];
+    int potion_max=0;
   
-  /* remember what object was like before it was changed.  note that
-   * refop is a local copy of op only to be used for detecting changes
-   * found by fix_player.  refop is not a real object */
-  memcpy(&refop, op, sizeof(object));
+    /* remember what object was like before it was changed.  note that
+     * refop is a local copy of op only to be used for detecting changes
+     * found by fix_player.  refop is not a real object 
+     */
+    memcpy(&refop, op, sizeof(object));
 
     if(op->type==PLAYER) {
 	if (tmp->type==POTION) {
@@ -405,232 +417,214 @@ int change_abil(object *op, object *tmp) {
 	} /* end of potion handling code */
     }
 
-  /* reset attributes that fix_player doesn't reset since it doesn't search
-   * everything to set */
-  if(flag == -1)
-    op->attacktype&=~tmp->attacktype,
-    op->path_attuned&=~tmp->path_attuned,
-    op->path_repelled&=~tmp->path_repelled,
-    op->path_denied&=~tmp->path_denied;
-
-  /* call fix_player since op object could have whatever attribute due
-   * to multiple items.  if fix_player always has to be called after
-   * change_ability then might as well call it from here
-   */
-  fix_player(op);
-
-  /* Fix player won't add the bows ability to the player, so don't
-   * print out message if this is a bow.
-   */
-  if(tmp->attacktype & AT_CONFUSION && tmp->type != BOW) {
-    success=1;
-    if(flag>0)
-      new_draw_info(NDI_UNIQUE, 0, op,"Your hands begin to glow red.");
-    else
-      new_draw_info(NDI_UNIQUE, 0, op,"Your hands stop glowing red.");
-  }
-  if ( QUERY_FLAG(op,FLAG_LIFESAVE) != QUERY_FLAG(&refop,FLAG_LIFESAVE)){
-    success=1;
-    if(flag>0) {
-      new_draw_info(NDI_UNIQUE, 0, op,"You feel very protected.");
-    } else {
-      new_draw_info(NDI_UNIQUE, 0, op,"You don't feel protected anymore.");
-    }
-  }
-  if ( QUERY_FLAG(op,FLAG_REFL_MISSILE) != QUERY_FLAG(&refop,FLAG_REFL_MISSILE)){
-    success=1;
-    if(flag>0) {
-      new_draw_info(NDI_UNIQUE, 0, op,"A magic force shimmers around you.");
-    } else {
-      new_draw_info(NDI_UNIQUE, 0, op,"The magic force fades away.");
-    }
-  }
-  if ( QUERY_FLAG(op,FLAG_REFL_SPELL) != QUERY_FLAG(&refop,FLAG_REFL_SPELL)){
-    success=1;
-    if(flag>0) {
-      new_draw_info(NDI_UNIQUE, 0, op,"You feel more safe now, somehow.");
-    } else {
-      new_draw_info(NDI_UNIQUE, 0, op,"Suddenly you feel less safe, somehow.");
-    }
-  }
-  if(QUERY_FLAG(tmp,FLAG_FLYING)) {
-    if(flag>0) {
-      success=1;
-      /* if were already flying then now flying higher */
-      if ( QUERY_FLAG(op,FLAG_FLYING) == QUERY_FLAG(&refop,FLAG_FLYING))
-        new_draw_info(NDI_UNIQUE, 0, op,"You float a little higher in the air.");
-      else {
-        new_draw_info(NDI_UNIQUE, 0, op,"You start to float in the air!.");
-        SET_FLAG(op,FLAG_FLYING);
-        if(op->speed>1)
-          op->speed=1;
-      }
-    } else {
-      success=1;
-      /* if were already flying then now flying lower */
-      if ( QUERY_FLAG(op,FLAG_FLYING) == QUERY_FLAG(&refop,FLAG_FLYING))
-        new_draw_info(NDI_UNIQUE, 0, op,"You float a little lower in the air.");
-      else {
-        new_draw_info(NDI_UNIQUE, 0, op,"You float down to the ground.");
-	check_walk_on (op, op);
-      }
-    }
-  }
-
-  /* becoming UNDEAD... a special treatment for this flag. Only those not
-   * originally undead may change their status */ 
-  if(!QUERY_FLAG(&op->arch->clone,FLAG_UNDEAD)) 
-    if ( QUERY_FLAG(op,FLAG_UNDEAD) != QUERY_FLAG(&refop,FLAG_UNDEAD)) {
-      success=1;
-      if(flag>0) {
-        if(op->race) free_string(op->race); 
-	op->race=add_string("undead");
-        new_draw_info(NDI_UNIQUE, 0, op,"Your lifeforce drains away!");
-      } else {
-        if(op->race) free_string(op->race); 
-        if(op->arch->clone.race) 
-	   op->race=add_string(op->arch->clone.race);  
-        new_draw_info(NDI_UNIQUE, 0, op,"Your lifeforce returns!");
-      }
+    /* reset attributes that fix_player doesn't reset since it doesn't search
+     * everything to set 
+     */
+    if(flag == -1) {
+	op->attacktype&=~tmp->attacktype;
+	op->path_attuned&=~tmp->path_attuned;
+	op->path_repelled&=~tmp->path_repelled;
+	op->path_denied&=~tmp->path_denied;
+	/* Presuming here that creatures only have move_type,
+	 * and not the other move_ fields.
+	 */
+	op->move_type &= ~tmp->move_type;
     }
 
-  if ( QUERY_FLAG(op,FLAG_STEALTH) != QUERY_FLAG(&refop,FLAG_STEALTH)){
-    success=1;
-    if(flag>0) {
-      new_draw_info(NDI_UNIQUE, 0, op,"You walk more quietly.");
-    } else {
-      new_draw_info(NDI_UNIQUE, 0, op,"You walk more noisily.");
+    /* call fix_player since op object could have whatever attribute due
+     * to multiple items.  if fix_player always has to be called after
+     * change_ability then might as well call it from here
+     */
+    fix_player(op);
+
+    /* Fix player won't add the bows ability to the player, so don't
+     * print out message if this is a bow.
+     */
+    if(tmp->attacktype & AT_CONFUSION && tmp->type != BOW) {
+	success=1;
+	DIFF_MSG(flag, "Your hands begin to glow red.",
+		 "Your hands stop glowing red.");
     }
-  }
-  if ( QUERY_FLAG(op,FLAG_MAKE_INVIS) != QUERY_FLAG(&refop,FLAG_MAKE_INVIS)){
-    success=1;
-    if(flag>0) {
-      new_draw_info(NDI_UNIQUE, 0, op,"You become transparent.");
-    } else {
-      new_draw_info(NDI_UNIQUE, 0, op,"You can see yourself.");
+    if ( QUERY_FLAG(op,FLAG_LIFESAVE) != QUERY_FLAG(&refop,FLAG_LIFESAVE)){
+	success=1;
+	DIFF_MSG(flag, "You feel very protected.", 
+		 "You don't feel protected anymore.");
     }
-  }
-  /* blinded you can tell if more blinded since blinded player has minimal
-   * vision */
-  if(QUERY_FLAG(tmp,FLAG_BLIND)) {
-    success=1;
-    if(flag>0) {
-      if(QUERY_FLAG(op,FLAG_WIZ))
-        new_draw_info(NDI_UNIQUE, 0, op,"Your mortal self is blinded.");
-      else { 
-        new_draw_info(NDI_UNIQUE, 0, op,"You are blinded.");
-        SET_FLAG(op,FLAG_BLIND);
+    if ( QUERY_FLAG(op,FLAG_REFL_MISSILE) != QUERY_FLAG(&refop,FLAG_REFL_MISSILE)){
+	success=1;
+	DIFF_MSG(flag, "A magic force shimmers around you.",
+		 "The magic force fades away.");
+    }
+    if ( QUERY_FLAG(op,FLAG_REFL_SPELL) != QUERY_FLAG(&refop,FLAG_REFL_SPELL)){
+	success=1;
+	DIFF_MSG(flag, "You feel more safe now, somehow.",
+		 "Suddenly you feel less safe, somehow.");
+    }
+    /* movement type has changed.  We don't care about cases where
+     * user has multiple items giving the same type appled like we
+     * used to - that is more work than what we gain, plus messages
+     * can be misleading (a little higher could be miscontrued from
+     * from fly high)
+     */
+    if (tmp->move_type && op->move_type != refop.move_type) {
+	success=1;
+
+	/* MOVE_FLY_HIGH trumps MOVE_FLY_LOW - changing your move_fly_low
+	 * status doesn't make a difference if you are flying high
+	 */
+	if (tmp->move_type & MOVE_FLY_LOW && !(op->move_type & MOVE_FLY_HIGH)) {
+	    DIFF_MSG(flag, "You start to float in the air!.", "You float down to the ground.");
+	}
+
+	if (tmp->move_type & MOVE_FLY_HIGH) {
+	    /* double conditional - second case covers if you have move_fly_low -
+	     * in that case, you don't actually land
+	     */
+	    DIFF_MSG(flag, "You soar into the air air!.", 
+		     (op->move_type&MOVE_FLY_LOW ? "You fly lower in the air":
+		      "You float down to the ground."));
+	}
+	if (tmp->move_type & MOVE_SWIM)
+	    DIFF_MSG(flag,"You feel ready for a swim", "You no longer feel like swimming");
+
+	/* Changing move status may mean you are affected by things you weren't before */
+	check_move_on(op, op);
+    }
+
+    /* becoming UNDEAD... a special treatment for this flag. Only those not
+     * originally undead may change their status 
+     */ 
+    if(!QUERY_FLAG(&op->arch->clone,FLAG_UNDEAD)) 
+	if ( QUERY_FLAG(op,FLAG_UNDEAD) != QUERY_FLAG(&refop,FLAG_UNDEAD)) {
+	    success=1;
+	    if(flag>0) {
+		if(op->race) free_string(op->race); 
+		op->race=add_string("undead");
+		new_draw_info(NDI_UNIQUE, 0, op,"Your lifeforce drains away!");
+	    } else {
+		if(op->race) free_string(op->race); 
+		if(op->arch->clone.race) 
+		    op->race=add_string(op->arch->clone.race);  
+		else
+		    op->race = NULL;
+		new_draw_info(NDI_UNIQUE, 0, op,"Your lifeforce returns!");
+	    }
+	}
+
+    if ( QUERY_FLAG(op,FLAG_STEALTH) != QUERY_FLAG(&refop,FLAG_STEALTH)){
+	success=1;
+	DIFF_MSG(flag, "You walk more quietly.", "You walk more noisily.");
+    }
+    if ( QUERY_FLAG(op,FLAG_MAKE_INVIS) != QUERY_FLAG(&refop,FLAG_MAKE_INVIS)){
+	success=1;
+	DIFF_MSG(flag, "You become transparent.", "You can see yourself.");
+    }
+    /* blinded you can tell if more blinded since blinded player has minimal
+     * vision 
+     */
+    if(QUERY_FLAG(tmp,FLAG_BLIND)) {
+	success=1;
+	if(flag>0) {
+	    if(QUERY_FLAG(op,FLAG_WIZ))
+		new_draw_info(NDI_UNIQUE, 0, op,"Your mortal self is blinded.");
+	    else { 
+		new_draw_info(NDI_UNIQUE, 0, op,"You are blinded.");
+		SET_FLAG(op,FLAG_BLIND);
+		if(op->type==PLAYER)
+		    op->contr->do_los=1;
+	    }
+	} else {
+	    if(QUERY_FLAG(op,FLAG_WIZ))
+		new_draw_info(NDI_UNIQUE, 0, op,"Your mortal self can now see again.");
+	    else {
+		new_draw_info(NDI_UNIQUE, 0, op,"Your vision returns.");
+		CLEAR_FLAG(op,FLAG_BLIND);
+		if(op->type==PLAYER)
+		    op->contr->do_los=1;
+	    }
+	}
+    }
+
+    if ( QUERY_FLAG(op,FLAG_SEE_IN_DARK) != QUERY_FLAG(&refop,FLAG_SEE_IN_DARK)){
+	success=1;
         if(op->type==PLAYER)
-          op->contr->do_los=1;
-      }  
-    } else {
-      if(QUERY_FLAG(op,FLAG_WIZ))
-        new_draw_info(NDI_UNIQUE, 0, op,"Your mortal self can now see again.");
-      else {
-        new_draw_info(NDI_UNIQUE, 0, op,"Your vision returns.");
-        CLEAR_FLAG(op,FLAG_BLIND);
-        if(op->type==PLAYER)
-          op->contr->do_los=1;
-      }  
-    }  
-  }
-
-  if ( QUERY_FLAG(op,FLAG_SEE_IN_DARK) != QUERY_FLAG(&refop,FLAG_SEE_IN_DARK)){
-    success=1;
-    if(flag>0) {
-        new_draw_info(NDI_UNIQUE, 0, op,"Your vision is better in the dark.");
-        if(op->type==PLAYER)
-          op->contr->do_los=1;
-    } else {
-        new_draw_info(NDI_UNIQUE, 0, op,"You see less well in the dark.");
-        if(op->type==PLAYER)
-          op->contr->do_los=1;
-    }  
-  }  
-
-  if ( QUERY_FLAG(op,FLAG_XRAYS) != QUERY_FLAG(&refop,FLAG_XRAYS)){
-    success=1;
-    if(flag>0) {
-      if(QUERY_FLAG(op,FLAG_WIZ))
-        new_draw_info(NDI_UNIQUE, 0, op,"Your vision becomes a little clearer.");
-      else {
-        new_draw_info(NDI_UNIQUE, 0, op,"Everything becomes transparent.");
-        if(op->type==PLAYER)
-          op->contr->do_los=1;
-      }
-    } else {
-      if(QUERY_FLAG(op,FLAG_WIZ))
-        new_draw_info(NDI_UNIQUE, 0, op,"Your vision becomes a bit out of focus.");
-      else {
-        new_draw_info(NDI_UNIQUE, 0, op,"Everything suddenly looks very solid.");
-        if(op->type==PLAYER)
-          op->contr->do_los=1;
-      }
+	    op->contr->do_los=1;
+	DIFF_MSG(flag, "Your vision is better in the dark.", "You see less well in the dark.");
     }
-  }
-  if(tmp->stats.luck) {
-    success=1;
-    if(flag*tmp->stats.luck>0) {
-      new_draw_info(NDI_UNIQUE, 0, op,"You feel more lucky.");
-    } else {
-      new_draw_info(NDI_UNIQUE, 0, op,"You feel less lucky.");
-    }
-  }
-  if(tmp->stats.hp && op->type==PLAYER) {
-    success=1;
-    if(flag*tmp->stats.hp>0)
-      new_draw_info(NDI_UNIQUE, 0, op,"You feel much more healthy!");
-    else
-      new_draw_info(NDI_UNIQUE, 0, op,"You feel much less healthy!");
-  }
-  if(tmp->stats.sp && op->type==PLAYER && tmp->type!=SKILL) {
-    success=1;
-    if(flag*tmp->stats.sp>0)
-      new_draw_info(NDI_UNIQUE, 0, op,"You feel one with the powers of magic!");
-    else
-      new_draw_info(NDI_UNIQUE, 0, op,"You suddenly feel very mundane.");
-  }
-  /* for the future when artifacts set this -b.t. */
-  if(tmp->stats.grace && op->type==PLAYER) {
-    success=1;
-     if(flag*tmp->stats.grace>0)
-      new_draw_info(NDI_UNIQUE, 0, op,"You feel closer to your god!");
-    else 
-      new_draw_info(NDI_UNIQUE, 0, op,"You suddenly feel less holy.");
-  } 
-  if(tmp->stats.food && op->type==PLAYER) {
-    success=1;
-    if(tmp->stats.food*flag>0)
-      new_draw_info(NDI_UNIQUE, 0, op,"You feel your digestion slowing down.");
-    else
-      new_draw_info(NDI_UNIQUE, 0, op,"You feel your digestion speeding up.");
-  }
 
-  /* Messages for changed resistance */
-  for (i=0; i<NROFATTACKS; i++) {
-    if (i==ATNR_PHYSICAL) continue;	/* Don't display about armour */
+    if ( QUERY_FLAG(op,FLAG_XRAYS) != QUERY_FLAG(&refop,FLAG_XRAYS)){
+	success=1;
+	if(flag>0) {
+	    if(QUERY_FLAG(op,FLAG_WIZ))
+		new_draw_info(NDI_UNIQUE, 0, op,"Your vision becomes a little clearer.");
+	    else {
+		new_draw_info(NDI_UNIQUE, 0, op,"Everything becomes transparent.");
+		if(op->type==PLAYER)
+		    op->contr->do_los=1;
+	    }
+	} else {
+	    if(QUERY_FLAG(op,FLAG_WIZ))
+		new_draw_info(NDI_UNIQUE, 0, op,"Your vision becomes a bit out of focus.");
+	    else {
+		new_draw_info(NDI_UNIQUE, 0, op,"Everything suddenly looks very solid.");
+		if(op->type==PLAYER)
+		    op->contr->do_los=1;
+	    }
+	}
+    }
+
+    if(tmp->stats.luck) {
+	success=1;
+	DIFF_MSG(flag*tmp->stats.luck, "You feel more lucky.", "You feel less lucky.");
+    }
+
+    if(tmp->stats.hp && op->type==PLAYER) {
+	success=1;
+	DIFF_MSG(flag*tmp->stats.hp, "You feel much more healthy!",
+		 "You feel much less healthy!");
+    }
+
+    if(tmp->stats.sp && op->type==PLAYER && tmp->type!=SKILL) {
+	success=1;
+	DIFF_MSG(flag*tmp->stats.sp, "You feel one with the powers of magic!",
+		 "You suddenly feel very mundane.");
+    }
+
+    /* for the future when artifacts set this -b.t. */
+    if(tmp->stats.grace && op->type==PLAYER) {
+	success=1;
+	DIFF_MSG(flag*tmp->stats.grace, "You feel closer to your god!", 
+		 "You suddenly feel less holy.");
+    }
+
+    if(tmp->stats.food && op->type==PLAYER) {
+	success=1;
+	DIFF_MSG(flag*tmp->stats.food, "You feel your digestion slowing down.", 
+		 "You feel your digestion speeding up.");
+    }
+
+    /* Messages for changed resistance */
+    for (i=0; i<NROFATTACKS; i++) {
+	if (i==ATNR_PHYSICAL) continue;	/* Don't display about armour */
     
-    if (op->resist[i] != refop.resist[i]) {
-      success=1;
-      if (op->resist[i] > refop.resist[i])
-	sprintf(message, "Your resistance to %s rises to %d%%.",
-	       change_resist_msg[i], op->resist[i]);
-      else
-	sprintf(message, "Your resistance to %s drops to %d%%.",
-	       change_resist_msg[i], op->resist[i]);
+	if (op->resist[i] != refop.resist[i]) {
+	    success=1;
+	    if (op->resist[i] > refop.resist[i])
+		sprintf(message, "Your resistance to %s rises to %d%%.",
+			change_resist_msg[i], op->resist[i]);
+	    else
+		sprintf(message, "Your resistance to %s drops to %d%%.",
+			change_resist_msg[i], op->resist[i]);
       
-      new_draw_info(NDI_UNIQUE|NDI_BLUE, 0, op, message);
+	    new_draw_info(NDI_UNIQUE|NDI_BLUE, 0, op, message);
+	}
     }
-  }
 
-     if(tmp->type!=EXPERIENCE && !potion_max) {
+    if(tmp->type!=EXPERIENCE && !potion_max) {
 	for (j=0; j<NUM_STATS; j++) {
 	    if ((i=get_attr_value(&(tmp->stats),j))!=0) {
 		success=1;
-		if (i * flag > 0)
-		    new_draw_info(NDI_UNIQUE, 0, op, gain_msg[j]);
-		else
-		    new_draw_info(NDI_UNIQUE, 0, op, lose_msg[j]);
+		DIFF_MSG(i * flag, gain_msg[j], lose_msg[j]);
 	    }
 	}
     }
@@ -810,8 +804,6 @@ void fix_player(object *op) {
 	op->slaying=NULL;
     }
     if(!QUERY_FLAG(op,FLAG_WIZ)) {
-        if ( ! QUERY_FLAG (&op->arch->clone, FLAG_FLYING))
-	    CLEAR_FLAG(op, FLAG_FLYING);
 	CLEAR_FLAG(op, FLAG_XRAYS);
 	CLEAR_FLAG(op, FLAG_MAKE_INVIS);
     }
@@ -832,6 +824,7 @@ void fix_player(object *op) {
     op->path_repelled=op->arch->clone.path_repelled;
     op->path_denied=op->arch->clone.path_denied;
     op->glow_radius=op->arch->clone.glow_radius;
+    op->move_type = op->arch->clone.move_type;
     op->chosen_skill = NULL;
 
     /* initializing resistances from the values in player/monster's
@@ -982,6 +975,7 @@ void fix_player(object *op) {
 	    op->path_repelled|=tmp->path_repelled;
 	    op->path_denied|=tmp->path_denied;
 	    op->stats.luck+=tmp->stats.luck;
+	    op->move_type |= tmp->move_type;
 
 	    if(QUERY_FLAG(tmp,FLAG_LIFESAVE))       SET_FLAG(op,FLAG_LIFESAVE);
 	    if(QUERY_FLAG(tmp,FLAG_REFL_SPELL))	    SET_FLAG(op,FLAG_REFL_SPELL);
@@ -999,12 +993,6 @@ void fix_player(object *op) {
 		op->invisible=1;
 	    }
 
-	    if(QUERY_FLAG(tmp,FLAG_FLYING)) {
-		SET_FLAG(op,FLAG_FLYING);
-		if(!QUERY_FLAG(op,FLAG_WIZ))
-		    max=1;
-	    }
-
 	    if(tmp->stats.exp && tmp->type!=SKILL) {
 		if(tmp->stats.exp > 0) {
 		    added_speed+=(float)tmp->stats.exp/3.0;
@@ -1361,7 +1349,21 @@ void fix_player(object *op) {
     else if (ac<-120) ac=-120;
     op->stats.ac=ac;
 
+    /* if for some reason the creature doesn't have any move type,
+     * give them walking as a default.
+     * The second case is a special case - to more closely mimic the
+     * old behaviour - if your flying, your not walking - just
+     * one or the other.
+     */
+    if (op->move_type == 0) op->move_type = MOVE_WALK;
+    else if (op->move_type & (MOVE_FLY_LOW | MOVE_FLY_HIGH)) op->move_type &= ~MOVE_WALK;
+
     update_ob_speed(op);
+
+    /* It is quite possible that a player's spell costing might have changed,
+     * so we will check that now.
+     */
+    if (op->type == PLAYER) esrv_update_spells(op->contr);
 }
 
 /*
@@ -1372,7 +1374,7 @@ void fix_player(object *op) {
  * false otherwise.
  */
 
-int allowed_class(object *op) {
+int allowed_class(const object *op) {
   return op->stats.Dex>0&&op->stats.Str>0&&op->stats.Con>0&&
          op->stats.Int>0&&op->stats.Wis>0&&op->stats.Pow>0&&
 	 op->stats.Cha>0;
@@ -1388,7 +1390,7 @@ int allowed_class(object *op) {
  * as soon as clients support this!
  * Please, anyone, write support for 'ext_title'.
  */
-void set_dragon_name(object *pl, object *abil, object *skin) {
+void set_dragon_name(object *pl, const object *abil, const object *skin) {
   int atnr=-1;  /* attacknumber of highest level */
   int level=0;  /* highest level */
   int i;
@@ -1500,12 +1502,18 @@ void dragon_level_gain(object *who) {
 object *give_skill_by_name(object *op, const char *skill_name)
 {
     object *skill_obj;
+    archetype *skill_arch;
 
-    skill_obj = get_archetype_by_skill_name(skill_name, SKILL);
-    if (!skill_obj) {
+    skill_arch = get_archetype_by_skill_name(skill_name, SKILL);
+    if (!skill_arch) {
 	LOG(llevError, "add_player_exp: couldn't find skill %s\n", skill_name);
 	return NULL;
     }
+    skill_obj = arch_to_object(skill_arch);
+    if (!skill_obj) {
+        LOG(llevError, "add_player_exp: couldn't instanciate skill %s\n", skill_name);
+        return NULL;
+    }
     /* clear the flag - exp goes into this bucket, but player
      * still doesn't know it.
      */
@@ -1565,10 +1573,10 @@ void player_lvl_adj(object *who, object *op) {
 	}
 	player_lvl_adj(who,op); /* To decrease more levels */
     }
+    /* check if the spell data has changed */
+    esrv_update_spells(who->contr);
 }
 
-
-
 /*
  * Returns how much experience is needed for a player to become
  * the given level.  level should really never exceed max_level
@@ -1591,8 +1599,10 @@ void calc_perm_exp(object *op)
 {
     int p_exp_min;
     
-    /* Ensure that our permanent experience minimum is met. */
-    p_exp_min = (int)(PERM_EXP_MINIMUM_RATIO * (float)(op->stats.exp));
+    /* Ensure that our permanent experience minimum is met.
+     * permenent_exp_ratio is an integer percentage, we divide by 100 
+     * to get the fraction   */
+    p_exp_min = (int)(settings.permanent_exp_ratio * (float)(op->stats.exp)/100);
 
     if (op->perm_exp < p_exp_min)
         op->perm_exp = p_exp_min;
@@ -1658,7 +1668,7 @@ static void add_player_exp(object *op, sint64 exp, const char *skill_name, int f
     if (exp_to_add > limit) exp_to_add=limit;
 
     ADD_EXP(op->stats.exp, (float) exp_to_add * (skill_obj? skill_obj->expmul:1));
-    if (settings.use_permanent_experience) {
+    if (settings.permanent_exp_ratio) {
 	ADD_EXP(op->perm_exp, (float) exp_to_add * PERM_EXP_GAIN_RATIO * (skill_obj? skill_obj->expmul:1));
 	calc_perm_exp(op);
     }
@@ -1669,7 +1679,7 @@ static void add_player_exp(object *op, sint64 exp, const char *skill_name, int f
 	limit=(levels[skill_obj->level+1]-levels[skill_obj->level])/2;
 	if (exp_to_add > limit) exp_to_add=limit;
 	ADD_EXP(skill_obj->stats.exp, exp_to_add);
-	if (settings.use_permanent_experience) {
+	if (settings.permanent_exp_ratio) {
 	    skill_obj->perm_exp += exp_to_add * PERM_EXP_GAIN_RATIO;
 	    calc_perm_exp(skill_obj);
 	}
@@ -1685,12 +1695,12 @@ static void add_player_exp(object *op, sint64 exp, const char *skill_name, int f
  * the 'exp' value passed should be positive - this is the
  * amount that should get subtract from the player.
  */
-sint64 check_exp_loss(object *op, sint64 exp)
+sint64 check_exp_loss(const object *op, sint64 exp)
 {
     sint64 del_exp;
 
     if (exp > op->stats.exp) exp = op->stats.exp;
-    if (settings.use_permanent_experience) {
+    if (settings.permanent_exp_ratio) {
 	del_exp = (op->stats.exp - op->perm_exp) * PERM_EXP_MAX_LOSS_RATIO;
 	if (del_exp < 0) del_exp = 0;
 	if (exp > del_exp) exp=del_exp;
@@ -1698,7 +1708,7 @@ sint64 check_exp_loss(object *op, sint64 exp)
     return exp;
 }
 
-sint64 check_exp_adjust(object *op, sint64 exp)
+sint64 check_exp_adjust(const object *op, sint64 exp)
 {
     if (exp<0) return check_exp_loss(op, exp);
     else return MIN(exp, MAX_EXPERIENCE - op->stats.exp);
@@ -1811,39 +1821,40 @@ void change_exp(object *op, sint64 exp, const char *skill_name, int flag) {
     }
 }
 
-/* Applies a death penalty experience.  20% or 3 levels, whichever is
- *  less experience lost. 
+/* Applies a death penalty experience, the size of this is defined by the 
+ * settings death_penalty_percentage and death_penalty_levels, and by the
+ * amount of permenent experience, whichever gives the lowest loss.
  */
 
 void apply_death_exp_penalty(object *op) {
     object *tmp;
     sint64 loss;
-    sint64 loss_20p;  /* 20 percent experience loss */
-    sint64 loss_3l;   /* 3 level experience loss */
+    sint64 percentage_loss;  /* defined by the setting 'death_penalty_percent' */
+    sint64 level_loss;   /* defined by the setting 'death_penalty_levels */
 
     for(tmp=op->inv;tmp;tmp=tmp->below)
 	if(tmp->type==SKILL && tmp->stats.exp) { 
 
-	    loss_20p = tmp->stats.exp * 0.20;
-	    loss_3l = tmp->stats.exp - levels[MAX(0,tmp->level -3)];
+	    percentage_loss = tmp->stats.exp * settings.death_penalty_ratio/100;
+	    level_loss = tmp->stats.exp - levels[MAX(0,tmp->level - settings.death_penalty_level)];
 
 	    /* With the revised exp system, you can get cases where
-	     * losing 3 levels would still require that you have more
-	     * exp than you current have - this is true if the levels
+	     * losing several levels would still require that you have more
+	     * exp than you currently have - this is true if the levels
 	     * tables is a lot harder.
 	     */
-	    if (loss_3l < 0) loss_3l = 0;
+	    if (level_loss < 0) level_loss = 0;
 
-	    loss = check_exp_loss(tmp, MIN(loss_3l, loss_20p));
+	    loss = check_exp_loss(tmp, MIN(level_loss, percentage_loss));
 
 	    tmp->stats.exp -= loss;
 	    player_lvl_adj(op,tmp);
 	}
 
-    loss_20p = op->stats.exp * 0.20;
-    loss_3l = op->stats.exp - levels[MAX(0,op->level -3)];
-    if (loss_3l < 0) loss_3l = 0;
-    loss = check_exp_loss(op, MIN(loss_3l, loss_20p));
+    percentage_loss = op->stats.exp * settings.death_penalty_ratio/100;
+    level_loss = op->stats.exp - levels[MAX(0,op->level - settings.death_penalty_level)];
+    if (level_loss < 0) level_loss = 0;
+    loss = check_exp_loss(op, MIN(level_loss, percentage_loss));
 
     op->stats.exp -= loss;
     player_lvl_adj(op,NULL);
@@ -1856,7 +1867,7 @@ void apply_death_exp_penalty(object *op) {
  * resistance to particular attacktype.
  * Returns 1 if op makes his save, 0 if he failed
  */
-int did_make_save(object *op, int level, int bonus)
+int did_make_save(const object *op, int level, int bonus)
 {
     if (level > MAX_SAVE_LEVEL) level = MAX_SAVE_LEVEL;
 
