@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_los_c =
- *   "$Id: los.c,v 1.13 2005/07/15 13:51:49 ryo_saeba Exp $";
+ *   "$Id: los.c,v 1.18 2006/02/17 21:29:47 akirschbaum Exp $";
  */
 
 /*
@@ -49,6 +49,8 @@ typedef struct blstr {
 
 blocks block[MAP_CLIENT_X][MAP_CLIENT_Y];
 
+static void expand_lighted_sight(object *op);
+
 /*
  * Used to initialise the array used by the LOS routines.
  * What this sets if that x,y blocks the view of bx,by
@@ -59,7 +61,7 @@ blocks block[MAP_CLIENT_X][MAP_CLIENT_Y];
  * be blocked.
  */
 
-void set_block(int x,int y,int bx, int by) {
+static void set_block(int x, int y, int bx, int by) {
     int index=block[x][y].index,i;
 
     /* Due to flipping, we may get duplicates - better safe than sorry.
@@ -72,7 +74,7 @@ void set_block(int x,int y,int bx, int by) {
     block[x][y].y[index]=by;
     block[x][y].index++;
 #ifdef LOS_DEBUG
-    fprintf(stderr,"setblock: added %d %d -> %d %d (%d)\n", x, y, bx, by,
+    LOG(llevDebug, "setblock: added %d %d -> %d %d (%d)\n", x, y, bx, by,
 	    block[x][y].index);
 #endif
 }
@@ -87,7 +89,7 @@ void set_block(int x,int y,int bx, int by) {
  * sightline.
  */
 
-void init_block() {
+void init_block(void) {
     int x,y, dx, dy, i;
     static int block_x[3] = {-1, -1, 0}, block_y[3] = {-1, 0, -1};
 
@@ -177,7 +179,7 @@ static void set_wall(object *op,int x,int y) {
 	if (ax < 0 || ax>=op->contr->socket.mapx ||
 	    ay < 0 || ay>=op->contr->socket.mapy) continue;
 #if 0
-	fprintf(stderr,"blocked %d %d -> %d %d\n",
+	LOG(llevDebug, "blocked %d %d -> %d %d\n",
 		dx, dy, ax, ay);
 #endif
 	/* we need to adjust to the fact that the socket
@@ -212,7 +214,7 @@ static void check_wall(object *op,int x,int y) {
 	return;
 
 #if 0
-    fprintf(stderr,"check_wall, ax,ay=%d, %d  x,y = %d, %d  blocksview = %d, %d\n",
+    LOG(llevDebug, "check_wall, ax,ay=%d, %d  x,y = %d, %d  blocksview = %d, %d\n",
 	    ax, ay, x, y, op->x + x - MAP_CLIENT_X/2, op->y + y - MAP_CLIENT_Y/2);
 #endif
 
@@ -251,7 +253,7 @@ void clear_los(object *op) {
  * This is somewhat suboptimal, would be better to improve the formula.
  */
 
-void expand_sight(object *op) 
+static void expand_sight(object *op) 
 {
     int i,x,y, dx, dy;
 
@@ -292,14 +294,14 @@ void expand_sight(object *op)
  * check the op->glow_radius instead of calling this.
  */
 
-int has_carried_lights(object *op) {
+int has_carried_lights(const object *op) {
     /* op may glow! */
     if(op->glow_radius>0) return 1;
 
     return 0;
 }
  
-void expand_lighted_sight(object *op)
+static void expand_lighted_sight(object *op)
 {
     int x,y,darklevel,ax,ay, basex, basey, mflags, light, x1, y1;
     mapstruct *m=op->map;
@@ -339,10 +341,10 @@ void expand_lighted_sight(object *op)
      * value) than to recalculate them down below.
      */
     for (x=(op->x - op->contr->socket.mapx/2 - MAX_LIGHT_RADII), basex=-MAX_LIGHT_RADII;
-      x < (op->x + op->contr->socket.mapx/2 + MAX_LIGHT_RADII); x++, basex++) {
+      x <= (op->x + op->contr->socket.mapx/2 + MAX_LIGHT_RADII); x++, basex++) {
 
 	for (y=(op->y - op->contr->socket.mapy/2 - MAX_LIGHT_RADII), basey=-MAX_LIGHT_RADII;
-	  y < (op->y + op->contr->socket.mapy/2 + MAX_LIGHT_RADII); y++, basey++) {
+	  y <= (op->y + op->contr->socket.mapy/2 + MAX_LIGHT_RADII); y++, basey++) {
 	    m = op->map;
 	    nx = x;
 	    ny = y;
@@ -357,12 +359,12 @@ void expand_lighted_sight(object *op)
 	    light = GET_MAP_LIGHT(m, nx, ny);
 	    if (light != 0) {
 #if 0
-		fprintf(stderr,"expand_lighted_sight: Found light at x=%d, y=%d, basex=%d, basey=%d\n", 
+		LOG(llevDebug, "expand_lighted_sight: Found light at x=%d, y=%d, basex=%d, basey=%d\n", 
 			x, y, basex, basey);
 #endif
-		for (ax=basex - light; ax<basex+light; ax++) {
+		for (ax=basex - light; ax<=basex+light; ax++) {
 		    if (ax<0 || ax>=op->contr->socket.mapx) continue;
-		    for (ay=basey - light; ay<basey+light; ay++) {
+		    for (ay=basey - light; ay<=basey+light; ay++) {
 			if (ay<0 || ay>=op->contr->socket.mapy) continue;
 
 			/* If the space is fully blocked, do nothing.  Otherwise, we
@@ -411,7 +413,7 @@ void expand_lighted_sight(object *op)
  * really need for any reasonable game play.
  */
 
-void blinded_sight (object *op) {
+static void blinded_sight(object *op) {
     int x,y;
 
     for (x = 0; x < op->contr->socket.mapx; x++)
@@ -494,7 +496,7 @@ void update_all_map_los(mapstruct *map) {
  * map is the map that changed, x and y are the coordinates.
  */
 
-void update_all_los(mapstruct *map, int x, int y) {
+void update_all_los(const mapstruct *map, int x, int y) {
     player *pl;
 
     for(pl=first_player;pl!=NULL;pl=pl->next) {
@@ -586,7 +588,7 @@ void print_los(object *op) {
  * check if any players are nearby, and edit their LOS array.
  */
 
-void make_sure_seen(object *op) {
+void make_sure_seen(const object *op) {
     player *pl;
 
     for (pl = first_player; pl; pl = pl->next)
@@ -605,7 +607,7 @@ void make_sure_seen(object *op) {
  * players within its range
  */
 
-void make_sure_not_seen(object *op) { 
+void make_sure_not_seen(const object *op) { 
     player *pl;
     for (pl = first_player; pl; pl = pl->next)
 	if (pl->ob->map == op->map &&
