@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_player_c =
- *   "$Id: player.c,v 1.16 2004/08/17 07:38:42 mwedel Exp $";
+ *   "$Id: player.c,v 1.18 2006/02/09 00:48:36 akirschbaum Exp $";
  */
 
 /*
@@ -49,6 +49,8 @@ void free_player(player *pl) {
     /* Clear item stack */
     if (pl->stack_items) free( pl->stack_items );
 
+    free(pl->socket.faces_sent);
+
     CFREE(pl);
 }
 
@@ -70,7 +72,7 @@ int atnr_is_dragon_enabled(int attacknr) {
  * returns true if the adressed object 'ob' is a player
  * of the dragon race.
  */
-int is_dragon_pl(object* op) {
+int is_dragon_pl(const object* op) {
   if (op != NULL && op->type == PLAYER && op->arch != NULL
       && op->arch->clone.race != NULL &&
       strcmp(op->arch->clone.race, "dragon")==0)
