@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_player_c =
- *   "$Id: re-cmp.c,v 1.11 2005/01/12 08:03:59 mwedel Exp $";
+ *   "$Id: re-cmp.c,v 1.13 2005/10/28 19:08:53 akirschbaum Exp $";
  */
 
 
@@ -36,11 +36,11 @@
 
 /*   P r o t o t y p e s
  */
-char *re_cmp(char *, char *);
-static Boolean re_cmp_step(char *, char *, int, int);
+const char *re_cmp(const char *, const char *);
+static Boolean re_cmp_step(const char *, const char *, unsigned, int);
 static void re_init(void);
 static Boolean re_match_token(uchar, selection *);
-static char *re_get_token(selection *, char *);
+static const char *re_get_token(selection *, const char *);
 #ifdef DEBUG2
 static void re_dump_sel(selection *);
 #endif
@@ -49,7 +49,7 @@ static void re_dump_sel(selection *);
  */
 static Boolean		re_init_done = False;
 static selection	*re_token[RE_TOKEN_MAX];
-static char		*re_substr[RE_TOKEN_MAX];
+static const char	*re_substr[RE_TOKEN_MAX];
 static unsigned int	re_token_depth;
 
 /*   E x t e r n a l   f u n c t i o n
@@ -59,9 +59,9 @@ static unsigned int	re_token_depth;
  * Return values: NULL - no match or error in regexp.
  *                pointer to beginning of matching string
  */
-char *
-re_cmp(char *str, char *regexp) {
-    char *next_regexp;
+const char *
+re_cmp(const char *str, const char *regexp) {
+    const char *next_regexp;
     Boolean once = False;
     Boolean matched;
 
@@ -85,6 +85,7 @@ re_cmp(char *str, char *regexp) {
     re_token_depth = 0;
     re_substr[0] = next_regexp;
 
+    matched = False;
     while (*str != '\0' && !(matched = re_match_token(*str, re_token[0])))
 	str++;
     
@@ -161,17 +162,17 @@ re_cmp(char *str, char *regexp) {
  */
 
 static Boolean
-re_cmp_step(char *str, char *regexp, int slot, int matches) {
+re_cmp_step(const char *str, const char *regexp, unsigned slot, int matches) {
     /* str	- string to match
      * regexp	- pattern
      * slot	- number of the token which under consideration
      * matches	- how many times the token has matched
      */
-    char *next_regexp;
+    const char *next_regexp;
     Boolean matched;
 
 #ifdef DEBUG
-/*    fprintf(stderr, "['%s', '%s', %d, %d]\n", str, regexp, slot, matches);*/
+/*    fprintf(stderr, "['%s', '%s', %u, %d]\n", str, regexp, slot, matches);*/
 #endif
 
     if (*regexp == 0) {
@@ -275,8 +276,8 @@ re_match_token(uchar c, selection *sel) {
  * Return values: NULL	syntax error
  *                pointer to first character past token.
  */
-static char *
-re_get_token(selection *sel, char *regexp) {
+static const char *
+re_get_token(selection *sel, const char *regexp) {
 
 #ifdef SAFE_CHECKS
 #   define exit_if_null	if (*regexp == 0) return NULL
