@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_readable_c =
- *   "$Id: readable.c,v 1.19 2005/07/16 21:10:37 akirschbaum Exp $";
+ *   "$Id: readable.c,v 1.31 2006/04/06 21:18:34 tchize Exp $";
  */
 
 /*
@@ -80,9 +80,9 @@
 
 /* 'title' and 'titlelist' are used by the readable code */
 typedef struct titlestruct {
-        char *name;     /* the name of the book */
-        char *authour;  /* the name of the book authour */
-        char *archname;  /* the archetype name of the book */
+        const char *name;     /* the name of the book */
+        const char *authour;  /* the name of the book authour */
+        const char *archname;  /* the archetype name of the book */
         int level;       /* level of difficulty of this message */ 
         int size;       /* size of the book message */ 
         int msg_index;  /* an index value derived from book message */
@@ -100,7 +100,7 @@ typedef struct titleliststruct {
 
 typedef struct namebytype
   {
-      char   *name;		/* generic name to call artifacts of this type */
+      const char *name;		/* generic name to call artifacts of this type */
       int     type;		/* matching type */
   }
 arttypename;
@@ -154,7 +154,7 @@ static uint32 spellpathdef[NRSPELLPATHS] =
     PATH_LIGHT
 };
 
-static char *path_book_name[] =
+static const char* const path_book_name[] =
 {
     "codex",
     "compendium",
@@ -164,7 +164,7 @@ static char *path_book_name[] =
 };
 
 /* used by spellpath texts */
-static char *path_author[] =
+static const char* const path_author[] =
 {
     "aether",
     "astral byways",
@@ -209,7 +209,7 @@ static arttypename art_name_array[] =
     {"Body Armour", ARMOUR}
 };
 
-static char *art_book_name[] =
+static const char* const art_book_name[] =
 {
     "collection",
     "file",
@@ -225,7 +225,7 @@ static char *art_book_name[] =
 };
 
 /* used by artifact texts */
-static char *art_author[] =
+static const char* const art_author[] =
 {
     "ancient things",
     "artifacts",
@@ -240,7 +240,7 @@ static char *art_author[] =
  * Monster book information
  */
 
-static char *mon_book_name[] =
+static const char* const mon_book_name[] =
 {
     "beastuary",
     "catalog",
@@ -259,7 +259,7 @@ static char *mon_book_name[] =
 
 
 /* used by monster beastuary texts */
-static char *mon_author[] =
+static const char* const mon_author[] =
 {
     "beasts",
     "creatures",
@@ -280,7 +280,7 @@ static char *mon_author[] =
  * God book information
  */
 
-static char *gods_book_name[] =
+static const char* const gods_book_name[] =
 {
     "devotional",
     "devout notes",
@@ -295,7 +295,7 @@ static char *gods_book_name[] =
 };
 
 /* used by gods texts */
-static char *gods_author[] =
+static const char* const gods_author[] =
 {
     "cults",
     "joy",
@@ -316,7 +316,7 @@ static char *gods_author[] =
  * Alchemy (formula) information
  */
 
-static char *formula_book_name[] =
+static const char* const formula_book_name[] =
 {
     "cookbook",
     "formulary",
@@ -329,7 +329,7 @@ static char *formula_book_name[] =
 };
 
 /* this isn't used except for empty books */
-static char *formula_author[] =
+static const char* const formula_author[] =
 {
     "Albertus Magnus",
     "alchemy",
@@ -352,7 +352,7 @@ static char *formula_author[] =
  */
 
 /* used by msg file and 'generic' books */
-static char *light_book_name[] =
+static const char* const light_book_name[] =
 {
     "calendar",
     "datebook",
@@ -369,7 +369,7 @@ static char *light_book_name[] =
     "transcript"
 };
 
-static char *heavy_book_name[] =
+static const char* const heavy_book_name[] =
 {
     "catalog",
     "compendium",
@@ -384,7 +384,7 @@ static char *heavy_book_name[] =
 
 
 /* used by 'generic' books */
-static char *book_author[] =
+static const char* const book_author[] =
 {
     "Abdulah",
     "Al'hezred",
@@ -430,7 +430,7 @@ static char *book_author[] =
     "Zagy"
 };
 
-static char *book_descrpt[] =
+static const char* const book_descrpt[] =
 {
     "ancient",
     "cryptic",
@@ -540,7 +540,7 @@ static int max_titles[6] =
  *****************************************************************************/
 
 static titlelist *
-get_empty_booklist ()
+get_empty_booklist (void)
 {
     titlelist *bl = (titlelist *) malloc (sizeof (titlelist));
     if (bl == NULL)
@@ -552,7 +552,7 @@ get_empty_booklist ()
 }
 
 static title  *
-get_empty_book ()
+get_empty_book (void)
 {
     title  *t = (title *) malloc (sizeof (title));
     if (t == NULL)
@@ -597,7 +597,7 @@ get_titlelist (int i)
  */
 
 int 
-nstrtok (char *buf1, char *buf2)
+nstrtok (const char *buf1, const char *buf2)
 {
     char   *tbuf, sbuf[12], buf[MAX_BUF];
     int     number = 0;
@@ -621,7 +621,7 @@ nstrtok (char *buf1, char *buf2)
  */
 
 char   *
-strtoktolin (char *buf1, char *buf2)
+strtoktolin (const char *buf1, const char *buf2)
 {
     int     maxi, i = nstrtok (buf1, buf2);
     char   *tbuf, buf[MAX_BUF], sbuf[12];
@@ -648,7 +648,7 @@ strtoktolin (char *buf1, char *buf2)
 }
 
 int 
-book_overflow (char *buf1, char *buf2, int booksize)
+book_overflow (const char *buf1, const char *buf2, int booksize)
 {
 
     if (buf_overflow (buf1, buf2, BOOK_BUF - 2)		/* 2 less so always room for trailing \n */
@@ -758,7 +758,9 @@ init_book_archive (void)
 
     if ((fp = open_and_uncompress (fname, 0, &comp)) != NULL)
       {
-	  int     i = 0, value, type = 0;
+	  int value, type = 0;
+	  size_t i;
+
 	  while (fgets (buf, MAX_BUF, fp) != NULL)
 	    {
 		if (*buf == '#')
@@ -809,12 +811,9 @@ init_book_archive (void)
 		  }
 	    }
 	  LOG (llevDebug, " book archives(used/avail): ");
-	  bl = booklist;
-	  while (bl && max_titles[i])
+	  for (bl = booklist, i = 0; bl != NULL && i < sizeof(max_titles)/sizeof(*max_titles); bl = bl->next, i++)
 	    {
 		LOG (llevDebug, "(%d/%d)", bl->number, max_titles[i]);
-		bl = bl->next;
-		i++;
 	    }
 	  LOG (llevDebug, "\n");
 	  close_and_delete (fp, comp);
@@ -896,7 +895,7 @@ init_readable (void)
  */
 
 static title  *
-find_title (object *book, int msgtype)
+find_title (const object *book, int msgtype)
 {
     title  *t = NULL;
     titlelist *tl = get_titlelist (msgtype);
@@ -1030,7 +1029,7 @@ add_author (object *op, int msgtype)
  */
 
 static int 
-unique_book (object *book, int msgtype)
+unique_book (const object *book, int msgtype)
 {
     title  *test;
 
@@ -1050,7 +1049,7 @@ unique_book (object *book, int msgtype)
 /* add_book_to_list() */
 
 static void 
-add_book_to_list (object *book, int msgtype)
+add_book_to_list (const object *book, int msgtype)
 {
     titlelist *tl = get_titlelist (msgtype);
     title  *t;
@@ -1117,7 +1116,7 @@ change_book (object *book, int msgtype)
 		    object *tmpbook;
 
 		    /* alter book properties */
-		    if ((tmpbook = get_archetype (t->archname)) != NULL)
+		    if ((tmpbook = create_archetype (t->archname)) != NULL)
 		      {
 			  if (tmpbook->msg)
 			      free_string (book->msg);
@@ -1135,8 +1134,8 @@ change_book (object *book, int msgtype)
 	      else
 		{
 		    int     numb, maxnames = max_titles[msgtype];
-		    char *old_title;
-		    char *old_name;
+		    const char *old_title;
+		    const char *old_name;
 
                     old_title = book->title ? add_string(book->title) : NULL;
                     old_name = add_string(book->name);
@@ -1312,7 +1311,7 @@ get_random_mon (int level)
  */
 
 char   *
-mon_desc (object *mon)
+mon_desc (const object *mon)
 {
     static char retbuf[HUGE_BUF];
 
@@ -1401,8 +1400,7 @@ mon_info_msg (int level, int booksize)
  * of 1-6 artifacts drawn sequentially from the artifact list.
  */
 
-char   *
-artifact_msg (int level, int booksize)
+static const char *artifact_msg (int level, int booksize)
 {
     artifactlist *al = NULL;
     artifact *art;
@@ -1622,7 +1620,7 @@ void make_formula_book(object *book, int level) {
 	    break;
     }
 
-    if (!formula) {
+    if (!formula || formula->arch_names <= 0) {
 	book->msg = add_string(" <indecipherable text>\n");
 	new_text_name(book, 4);
 	add_author(book,4);
@@ -1632,28 +1630,13 @@ void make_formula_book(object *book, int level) {
 	 * of information on the booklevel and the spellevel
 	 * of the formula. */
 
-	char   *op_name = NULL;
+	const char   *op_name = formula->arch_name[RANDOM()%formula->arch_names];
 	archetype *at;
-	int     nindex = nstrtok (formula->arch_name, ",");
 
 	/* preamble */
 	sprintf(retbuf, "Herein is described a project using %s: \n", 
 		 formula->skill?formula->skill:"an unknown skill");
 
-	/* construct name of object to be made */
-	if (nindex > 1)
-	{
-	    int     rnum = RANDOM () % nindex;
-	    op_name = strtok (formula->arch_name, ",");
-	    while (rnum)
-	    {
-		op_name = strtok (NULL, ",");
-		rnum--;
-	    }
-	}
-	else
-	    op_name = formula->arch_name;
-
 	if ((at = find_archetype (op_name)) != (archetype *) NULL)
 	    op_name = at->clone.name;
 	else
@@ -1761,12 +1744,14 @@ msgfile_msg (int level, int booksize)
  * of a random god. Used by the book hack. b.t.
  */
 
-char   *
+const char   *
 god_info_msg (int level, int booksize)
 {
     static char retbuf[BOOK_BUF];
-    char   *name = NULL, buf[BOOK_BUF];
-    int     i, introlen;
+    const char   *name = NULL;
+    char buf[BOOK_BUF];
+    int     i;
+    size_t  introlen;
     object *god = pntr_to_god_obj (get_rand_god ());
 
     if (!god)
@@ -1799,14 +1784,14 @@ god_info_msg (int level, int booksize)
 	  sprintf (buf, " ");
 	  if (level == 2 && RANDOM () % 2)
 	    {			/* enemy god */
-		char   *enemy = god->title;
+		const char   *enemy = god->title;
 		if (enemy)
 		    sprintf (buf, "The gods %s and %s are enemies.\n ---\n",
 			     name, enemy);
 	    }
 	  if (level == 3 && RANDOM () % 2)
 	    {			/* enemy race, what the god's holy word effects */
-		char   *enemy = god->slaying;
+		const char   *enemy = god->slaying;
 		if (enemy && !(god->path_denied & PATH_TURNING))
 		    if ((i = nstrtok (enemy, ",")) > 0)
 		      {
@@ -1840,7 +1825,7 @@ god_info_msg (int level, int booksize)
 	    }
 	  if (level == 5 && RANDOM () % 2)
 	    {			/* aligned race, summoning  */
-		char   *race = god->race;	/* aligned race */
+		const char   *race = god->race;	/* aligned race */
 		if (race && !(god->path_denied & PATH_SUMMON))
 		    if ((i = nstrtok (race, ",")) > 0)
 		      {
@@ -2041,7 +2026,7 @@ tailor_readable_ob (object *book, int msg_type)
  *****************************************************************************/
 
 void 
-free_all_readable ()
+free_all_readable (void)
 {
     titlelist *tlist, *tnext;
     title  *title1, *titlenext;
@@ -2134,7 +2119,7 @@ write_book_archive (void)
 }
 readable_message_type* get_readable_message_type(object* readable){
 	uint8 subtype = readable->subtype;
-	if ((subtype<0) || (subtype>last_readable_subtype))
+	if (subtype>last_readable_subtype)
 	    return &(readable_message_types[0]);
 	return &(readable_message_types[subtype]);
 }
