@@ -24,9 +24,11 @@
 #include <object.h>
 #include <ctype.h>
 
+static void build_stringlist(const char *str, char ***result_list, size_t *result_size);
+
 static recipelist *formulalist;
 
-static recipelist *init_recipelist() {
+static recipelist *init_recipelist(void) {
   recipelist *tl = (recipelist *) malloc(sizeof(recipelist));
   if(tl==NULL)
     fatal(OUT_OF_MEMORY);
@@ -37,7 +39,7 @@ static recipelist *init_recipelist() {
   return tl;
 }
 
-static recipe *get_empty_formula() {
+static recipe *get_empty_formula(void) {
   recipe *t = (recipe *) malloc(sizeof(recipe));
   if(t==NULL)
     fatal(OUT_OF_MEMORY);
@@ -49,6 +51,7 @@ static recipe *get_empty_formula() {
   t->exp=0;
   t->keycode = 0;
   t->title = NULL;
+  t->arch_names = 0;
   t->arch_name = NULL;
   t->skill = NULL;
   t->cauldron = NULL;
@@ -57,7 +60,9 @@ static recipe *get_empty_formula() {
   return t;
 }
  
-/* get_formulalist() - returns pointer to the formula list */
+/* get_formulalist() - returns pointer to the formula list
+ * i is the number of ingredients.
+ */
 
 recipelist * get_formulalist ( int i ) {
   recipelist *fl=formulalist;
@@ -73,22 +78,25 @@ recipelist * get_formulalist ( int i ) {
 /* check_recipe() - makes sure we actually have the requested artifact 
  * and archetype. */
 
-static int check_recipe(recipe *rp) {
+static int check_recipe(const recipe *rp) {
+    size_t i;
+    int result;
 
-    if(find_archetype(rp->arch_name)!=NULL) { 
-          artifact *art=locate_recipe_artifact(rp);
-          if (!art && strcmp(rp->title,"NONE")) {
-		LOG(llevError,"\n WARNING: Formula %s of %s has no artifact.\n",
-                rp->arch_name,rp->title); 
-		return 0;
-	  }
-    } else { 
-           LOG(llevError,"\n WARNING: Can't find archetype:%s for formula:%s\n", 
-                rp->arch_name,rp->title); 
-	   return 0;
+    result = 1;
+    for (i = 0; i < rp->arch_names; i++) {
+        if (find_archetype(rp->arch_name[i]) != NULL) { 
+            artifact *art = locate_recipe_artifact(rp, i);
+            if (!art && strcmp(rp->title, "NONE") != 0) {
+                LOG(llevError,"\nWARNING: Formula %s of %s has no artifact.\n", rp->arch_name[i], rp->title); 
+                result = 0;
+            }
+        } else { 
+            LOG(llevError,"\nWARNING: Can't find archetype %s for formula %s\n", rp->arch_name[i], rp->title); 
+            result = 0;
+        }
     }
    
-    return 1;
+    return result;
 }
 
 
@@ -97,7 +105,7 @@ static int check_recipe(recipe *rp) {
  * the libdir. -b.t. 
  */
  
-void init_formulae() {
+void init_formulae(void) {
   static int has_been_done=0;
   FILE *fp;
   char filename[MAX_BUF], buf[MAX_BUF], *cp, *next;
@@ -170,7 +178,7 @@ void init_formulae() {
       formula->next = fl->items; 
       fl->items = formula;
     } else if (!strncmp(cp, "arch",4)) { 
-        formula->arch_name = add_string(strchr(cp,' ')+1);
+        build_stringlist(strchr(cp, ' ')+1, &formula->arch_name, &formula->arch_names);
         (void) check_recipe(formula);
     } else if (!strncmp(cp, "skill", 5)) {
 	formula->skill = add_string(strchr(cp, ' ')+1);
@@ -205,7 +213,7 @@ void check_formulae( void ) {
         if(check->index==formula->index) { 
           LOG(llevError," ERROR: On %d ingred list: ", numb);
           LOG(llevError, "Formulae [%s] of %s and [%s] of %s have matching index id (%d)\n",
-            formula->arch_name,formula->title,check->arch_name,check->title,formula->index);
+            formula->arch_name[0],formula->title,check->arch_name[0],check->title,formula->index);
         }
     numb++;
   }
@@ -228,12 +236,13 @@ void dump_alchemy( void ) {
 	num_ingred, num_ingred>1?"s.":".",fl->number,fl->total_chance);
     for (formula=fl->items; formula!=NULL; formula=formula->next) {
       artifact *art=NULL;
-      char buf[MAX_BUF], *string;
-      
-      string=strtok(formula->arch_name,",");
-      while(string) { 
+      char buf[MAX_BUF];
+      size_t i;
+
+      for (i = 0; i < formula->arch_names; i++) {
+        const char *string = formula->arch_name[i];
 	if(find_archetype(string)!=NULL) { 
-          art = locate_recipe_artifact(formula);
+          art = locate_recipe_artifact(formula, i);
           if (!art && strcmp(formula->title,"NONE")) 
 		LOG(llevError,"Formula %s has no artifact\n",formula->title);
 	  else {
@@ -266,7 +275,6 @@ void dump_alchemy( void ) {
 	} else 
 	   LOG(llevError,"Can't find archetype:%s for formula %s\n", string,
 		formula->title); 
-	string = strtok(NULL,",");
       }
     }  
   fprintf(logfile,"\n");
@@ -278,7 +286,7 @@ void dump_alchemy( void ) {
 /* Find a treasure with a matching name.  The 'depth' parameter is
  * only there to prevent infinite loops in treasure lists (a list
  * referencing another list pointing back to the first one). */
-archetype *find_treasure_by_name (treasure *t, const char *name, int depth)
+archetype *find_treasure_by_name (const treasure *t, const char *name, int depth)
 {
   treasurelist *tl;
   archetype    *at;
@@ -324,7 +332,7 @@ archetype *find_treasure_by_name (treasure *t, const char *name, int depth)
  * (e.g. sage & c_sage) or if only one of the monsters generates the
  * body parts that we are looking for (e.g. big_dragon and
  * big_dragon_worthless). */
-long find_ingred_cost (char *name)
+long find_ingred_cost (const char *name)
 {
   archetype    *at;
   archetype    *at2;
@@ -424,12 +432,13 @@ void dump_alchemy_costs (void)
     for (formula = fl->items; formula != NULL; formula = formula->next) {
       artifact *art=NULL;
       archetype *at=NULL;
-      char buf[MAX_BUF], *string;
-      
-      string = strtok (formula->arch_name, ",");
-      while (string) {
+      char buf[MAX_BUF];
+      size_t i;
+
+      for (i = 0; i < formula->arch_names; i++) {
+        const char *string = formula->arch_name[i];
 	if ((at = find_archetype (string)) != NULL) {
-          art = locate_recipe_artifact (formula);
+          art = locate_recipe_artifact (formula, i);
           if (!art && strcmp (formula->title,"NONE")) 
 	    LOG (llevError, "Formula %s has no artifact\n", formula->title);
 	  else
@@ -484,7 +493,6 @@ void dump_alchemy_costs (void)
 	else 
 	  LOG(llevError, "Can't find archetype:%s for formula %s\n", string,
 	      formula->title); 
-	string = strtok (NULL, ",");
       }
     }
   fprintf (logfile,"\n");
@@ -496,8 +504,8 @@ void dump_alchemy_costs (void)
 	     num_errors);
 }
 
-char * ingred_name (char *name) {
-  char *cp=name;
+const char * ingred_name (const char *name) {
+  const char *cp=name;
  
   if(atoi(cp)) cp = strchr(cp,' ') + 1;
   return cp;
@@ -508,8 +516,8 @@ char * ingred_name (char *name) {
  * ASCII values in buf (times prepended integers).
  */
 
-int strtoint (char *buf) {
-  char *cp = ingred_name(buf);
+int strtoint (const char *buf) {
+  const char *cp = ingred_name(buf);
   int val=0, len=strlen(cp), mult=numb_ingred(buf);
 
   while (len) { 
@@ -519,8 +527,8 @@ int strtoint (char *buf) {
   return val*mult;
 }
 
-artifact * locate_recipe_artifact(recipe *rp) {
-   object *item=get_archetype(rp->arch_name);
+artifact * locate_recipe_artifact(const recipe *rp, size_t idx) {
+   object *item=create_archetype(rp->arch_name[idx]);
    artifactlist *at=NULL;
    artifact *art=NULL;
 
@@ -535,7 +543,7 @@ artifact * locate_recipe_artifact(recipe *rp) {
    return art;
 }
 
-int numb_ingred (char *buf) {
+int numb_ingred (const char *buf) {
   int numb;
 
   if((numb=atoi(buf))) return numb;
@@ -583,7 +591,7 @@ recipe * get_random_recipe ( recipelist *rpl ) {
   return rp;
 }
 
-void free_all_recipes()
+void free_all_recipes(void)
 {
     recipelist *fl=formulalist,*flnext;
     recipe *formula=NULL,*next;
@@ -596,8 +604,8 @@ void free_all_recipes()
 	for (formula=fl->items; formula!=NULL; formula=next) {
 	    next=formula->next;
       
-	    if (formula->arch_name)
-		free_string(formula->arch_name);
+	    free(formula->arch_name[0]);
+	    free(formula->arch_name);
 	    if (formula->title)
 		free_string(formula->title);
 	    if (formula->skill)
@@ -614,3 +622,40 @@ void free_all_recipes()
 	free(fl);
     }
 }
+
+/**
+ * Split a comma separated string list into words.
+ *
+ * @param str the string to split
+ *
+ * @param result_list pointer to return value for the newly created list; the
+ * caller is responsible for freeing both *result_list and **result_list.
+ *
+ * @param result_size pointer to return value for the size of the newly
+ * created list
+ */
+static void build_stringlist(const char *str, char ***result_list, size_t *result_size)
+{
+    char *dup;
+    char *p;
+    size_t size;
+    size_t i;
+
+    dup = strdup_local(str);
+    if (dup == NULL)
+        fatal(OUT_OF_MEMORY);
+
+    size = 0;
+    for (p = strtok(dup, ","); p != NULL; p = strtok(NULL, ","))
+        size++;
+
+    *result_list = malloc(size*sizeof(*result_list));
+    if (*result_list == NULL)
+        fatal(OUT_OF_MEMORY);
+    *result_size = size;
+
+    for (i = 0; i < size; i++) {
+        (*result_list)[i] = dup;
+        dup = dup+strlen(dup)+1;
+    }
+}
