@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_shstr_c =
- *   "$Id: shstr.c,v 1.4 2001/06/11 03:47:53 michtoen Exp $";
+ *   "$Id: shstr.c,v 1.10 2006/03/18 17:15:24 ryo_saeba Exp $";
  *
  * shstr.c
  *
@@ -44,7 +44,7 @@ static shared_string *hash_table[TABLESIZE];
  */
 
 void
-init_hash_table() {
+init_hash_table(void) {
     /* A static object should be zeroed out always */
 #if !defined(__STDC__)
     (void) memset((void *)hash_table, 0, TABLESIZE * sizeof(shared_string *));
@@ -58,7 +58,8 @@ init_hash_table() {
 static int
 hashstr(const char *str) {
     unsigned long hash = 0;
-    int i = 0, rot = 0;
+    int i = 0;
+    unsigned rot = 0;
     const char *p;
 
     GATHER(hash_stats.calls);
@@ -103,7 +104,7 @@ new_shared_string(const char *str) {
  *      - pointer to string identical to str
  */
 
-char *
+const char *
 add_string(const char *str) {
     shared_string *ss;
     int ind;
@@ -197,14 +198,10 @@ add_string(const char *str) {
  *      - str
  */
 
-char *
-add_refcount(char *str) {
+const char *
+add_refcount(const char *str) {
     GATHER(add_ref_stats.calls);
     ++(SS(str)->refcount);
-    /* This function should be declared 
-     *    const char *add_refcount(const char *)
-     * Unfortunately, that would require changing a lot of structs
-     */
     return str;
 }
 
@@ -218,7 +215,7 @@ add_refcount(char *str) {
 
 int
 query_refcount(const char *str) {
-    return SS(str)->refcount;
+    return (SS(str)->refcount) & ~TOPBIT;
 }
 
 /*
@@ -229,7 +226,7 @@ query_refcount(const char *str) {
  *      - pointer to identical string or NULL
  */
 
-char *
+const char *
 find_string(const char *str) {
     shared_string *ss;
     int ind;
@@ -276,7 +273,7 @@ find_string(const char *str) {
  */
 
 void
-free_string(char *str) {
+free_string(const char *str) {
     shared_string *ss;
 
     GATHER(free_stats.calls);
@@ -322,7 +319,7 @@ extern char errmsg[];
  */
 
 void
-ss_dump_statistics() {
+ss_dump_statistics(void) {
     static char line[80];
 
     sprintf(errmsg, "%-13s %6s %6s %6s %6s %6s\n", 
@@ -356,7 +353,7 @@ ss_dump_statistics() {
  *      - a string or NULL
  */
 
-char *
+const char *
 ss_dump_table(int what) {
     static char totals[80];
     int entries = 0, refs = 0, links = 0;
@@ -402,7 +399,7 @@ ss_dump_table(int what) {
  */
 
 int 
-buf_overflow (char *buf1, char *buf2, int bufsize)
+buf_overflow (const char *buf1, const char *buf2, int bufsize)
 {
     int     len1 = 0, len2 = 0;
 
