@@ -1,7 +1,7 @@
 
 /*
  * static char *rcs_treasure_c =
- *   "$Id: treasure.c,v 1.56 2005/07/16 21:10:37 akirschbaum Exp $";
+ *   "$Id: treasure.c,v 1.66 2006/05/21 12:50:17 akirschbaum Exp $";
  */
 
 /*
@@ -70,7 +70,7 @@ void init_archetype_pointers() {
  * Allocate and return the pointer to an empty treasurelist structure.
  */
 
-static treasurelist *get_empty_treasurelist() {
+static treasurelist *get_empty_treasurelist(void) {
   treasurelist *tl = (treasurelist *) malloc(sizeof(treasurelist));
   if(tl==NULL)
     fatal(OUT_OF_MEMORY);
@@ -82,7 +82,7 @@ static treasurelist *get_empty_treasurelist() {
  * Allocate and return the pointer to an empty treasure structure.
  */
 
-static treasure *get_empty_treasure() {
+static treasure *get_empty_treasure(void) {
   treasure *t = (treasure *) calloc(1,sizeof(treasure));
   if(t==NULL)
     fatal(OUT_OF_MEMORY);
@@ -157,7 +157,7 @@ static treasure *load_treasure(FILE *fp, int *line) {
 /* recursived checks the linked list.  Treasurelist is passed only
  * so that the treasure name can be printed out
  */
-static void check_treasurelist(treasure *t, treasurelist *tl)
+static void check_treasurelist(const treasure *t, const treasurelist *tl)
 {
     if (t->item==NULL && t->name==NULL)
 	LOG(llevError,"Treasurelist %s has element with no name or archetype\n", tl->name);
@@ -178,7 +178,7 @@ static void check_treasurelist(treasure *t, treasurelist *tl)
  * Each treasure is parsed with the help of load_treasure().
  */
 
-void load_treasures() {
+void load_treasures(void) {
     FILE *fp;
     char filename[MAX_BUF], buf[MAX_BUF], name[MAX_BUF];
     treasurelist *previous=NULL;
@@ -243,7 +243,7 @@ void load_treasures() {
  */
 
 treasurelist *find_treasurelist(const char *name) {
-  char *tmp=find_string(name);
+  const char *tmp=find_string(name);
   treasurelist *tl;
 
   /* Special cases - randomitems of none is to override default.  If
@@ -382,8 +382,9 @@ void create_one_treasure(treasurelist *tl, object *op, int flag, int difficulty,
 	    create_one_treasure(tl, op, flag, difficulty, tries);
 	return;
     }
-    if(t->item->clone.invisible != 0 || flag != GT_INVISIBLE) {
+    if((t->item && t->item->clone.invisible != 0) || flag != GT_INVISIBLE) {
 	object *tmp=arch_to_object(t->item);
+	if (!tmp) return;
         if(t->nrof && tmp->nrof<=1)
           tmp->nrof = RANDOM()%((int) t->nrof) + 1;
         fix_generated_item (tmp, op, difficulty, t->magic, flag);
@@ -484,7 +485,7 @@ static int difftomagic_list[DIFFLEVELS][MAXMAGIC+1] =
  * This code presumes that op has had its spell object created (in op->inv)
  */
 
-int level_for_item(object *op, int difficulty, int retmult)
+int level_for_item(const object *op, int difficulty, int retmult)
 {
     int level, mult, olevel;
 
@@ -545,7 +546,7 @@ int magic_from_difficulty(int difficulty)
     LOG(llevError,"Warning, table for difficulty %d bad.\n",difficulty);
     loop=0;
   }
-/*  printf("Chose magic %d for difficulty %d\n",loop,difficulty);*/
+/*  LOG(llevDebug, "Chose magic %d for difficulty %d\n", loop, difficulty);*/
   return (RANDOM()%3)?loop:-loop;
 }
 
@@ -826,7 +827,7 @@ void fix_generated_item (object *op, object *creator, int difficulty,
 	    if (op->stats.sp && !op->randomitems) {
 		object *tmp;
 
-		tmp = get_archetype(spell_mapping[op->stats.sp]);
+		tmp = create_archetype(spell_mapping[op->stats.sp]);
 		insert_ob_in_ob(tmp, op);
 		op->stats.sp=0;
 	    }
@@ -857,7 +858,7 @@ void fix_generated_item (object *op, object *creator, int difficulty,
 		if (op->stats.sp && !op->randomitems) {
 		    object *tmp;
 
-		    tmp = get_archetype(spell_mapping[op->stats.sp]);
+		    tmp = create_archetype(spell_mapping[op->stats.sp]);
 		    insert_ob_in_ob(tmp, op);
 		    op->stats.sp=0;
 		}
@@ -1040,7 +1041,7 @@ void fix_generated_item (object *op, object *creator, int difficulty,
  * Allocate and return the pointer to an empty artifactlist structure.
  */
 
-static artifactlist *get_empty_artifactlist() {
+static artifactlist *get_empty_artifactlist(void) {
   artifactlist *tl = (artifactlist *) malloc(sizeof(artifactlist));
   if(tl==NULL)
     fatal(OUT_OF_MEMORY);
@@ -1054,7 +1055,7 @@ static artifactlist *get_empty_artifactlist() {
  * Allocate and return the pointer to an empty artifact structure.
  */
 
-static artifact *get_empty_artifact() {
+static artifact *get_empty_artifact(void) {
   artifact *t = (artifact *) malloc(sizeof(artifact));
   if(t==NULL)
     fatal(OUT_OF_MEMORY);
@@ -1083,7 +1084,7 @@ artifactlist *find_artifactlist(int type) {
  * For debugging purposes.  Dumps all tables.
  */
 
-void dump_artifacts() {
+void dump_artifacts(void) {
   artifactlist *al;
   artifact *art;
   linked_char *next;
@@ -1108,7 +1109,7 @@ void dump_artifacts() {
 /*
  * For debugging purposes.  Dumps all treasures recursively (see below).
  */
-void dump_monster_treasure_rec (char *name, treasure *t, int depth)
+void dump_monster_treasure_rec (const char *name, treasure *t, int depth)
 {
   treasurelist *tl;
   int           i;
@@ -1160,7 +1161,7 @@ void dump_monster_treasure_rec (char *name, treasure *t, int depth)
  * Created originally by Raphael Quinet for debugging the alchemy code.
  */
 
-void dump_monster_treasure (char *name)
+void dump_monster_treasure (const char *name)
 {
   archetype *at;
   int        found;
@@ -1188,7 +1189,7 @@ void dump_monster_treasure (char *name)
  * Builds up the lists of artifacts from the file in the libdir.
  */
 
-void init_artifacts() {
+void init_artifacts(void) {
     static int has_been_inited=0;
     FILE *fp;
     char filename[MAX_BUF], buf[HUGE_BUF], *cp, *next;
@@ -1292,13 +1293,14 @@ void add_abilities(object *op, object *change) {
 #endif
 	op->face = change->face;
     }
-    for (i = 0; i < 7; i++)
+    for (i = 0; i < NUM_STATS; i++)
 	change_attr_value(&(op->stats), i, get_attr_value(&(change->stats), i));
 
     op->attacktype |= change->attacktype;
     op->path_attuned |= change->path_attuned;
     op->path_repelled |= change->path_repelled;
     op->path_denied |= change->path_denied;
+    op->move_type |= change->move_type;
     op->stats.luck += change->stats.luck;
 
     if (QUERY_FLAG(change,FLAG_CURSED))	    SET_FLAG(op, FLAG_CURSED);
@@ -1310,7 +1312,6 @@ void add_abilities(object *op, object *change) {
     if (QUERY_FLAG(change,FLAG_LIFESAVE))	SET_FLAG(op,FLAG_LIFESAVE);
     if (QUERY_FLAG(change,FLAG_REFL_SPELL))	SET_FLAG(op,FLAG_REFL_SPELL);
     if (QUERY_FLAG(change,FLAG_STEALTH))	SET_FLAG(op,FLAG_STEALTH);
-    if (QUERY_FLAG(change,FLAG_FLYING))		SET_FLAG(op,FLAG_FLYING);
     if (QUERY_FLAG(change,FLAG_XRAYS))		SET_FLAG(op,FLAG_XRAYS);
     if (QUERY_FLAG(change,FLAG_BLIND))		SET_FLAG(op,FLAG_BLIND);
     if (QUERY_FLAG(change,FLAG_SEE_IN_DARK))    SET_FLAG(op,FLAG_SEE_IN_DARK);
@@ -1502,7 +1503,7 @@ void add_abilities(object *op, object *change) {
 static int legal_artifact_combination(object *op, artifact *art) {
   int neg, success = 0;
   linked_char *tmp;
-  char *name;
+  const char *name;
 
   if (art->allowed == (linked_char *) NULL)
     return 1; /* Ie, "all" */
@@ -1533,14 +1534,14 @@ static int legal_artifact_combination(object *op, artifact *art) {
  * it should have due to the second artifact-template.
  */
 
-void give_artifact_abilities(object *op, object *artifct) {
+void give_artifact_abilities(object *op, object *artifact) {
   char new_name[MAX_BUF];
 
-  sprintf(new_name, "of %s", artifct->name);
+  sprintf(new_name, "of %s", artifact->name);
   if (op->title)
     free_string(op->title);
   op->title = add_string(new_name);
-  add_abilities(op, artifct); /* Give out the bonuses */
+  add_abilities(op, artifact); /* Give out the bonuses */
 
 #if 0 /* Bit verbose, but keep it here until next time I need it... */
   {
@@ -1717,7 +1718,7 @@ void free_artifactlist(artifactlist *al)
     }
 }
 
-void free_all_treasures() {
+void free_all_treasures(void) {
 treasurelist *tl, *next;
 
 
