@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_utils_c =
- *   "$Id: utils.c,v 1.18 2005/07/17 11:00:03 ryo_saeba Exp $";
+ *   "$Id: utils.c,v 1.26 2006/05/21 12:23:40 akirschbaum Exp $";
  */
 
 /*
@@ -49,7 +49,7 @@
  * not the recipient (ie, the poor slob getting hit). [garbled 20010916]
  */
 
-int random_roll(int min, int max, object *op, int goodbad) {
+int random_roll(int min, int max, const object *op, int goodbad) {
     int omin, diff, luck, base, ran;
 
     omin = min;
@@ -85,7 +85,7 @@ int random_roll(int min, int max, object *op, int goodbad) {
  * for exp loss calculations for players changing religions.
  */
 
-sint64 random_roll64(sint64 min, sint64 max, object *op, int goodbad) {
+sint64 random_roll64(sint64 min, sint64 max, const object *op, int goodbad) {
     sint64 omin, diff, luck, ran;
     int base;
 
@@ -134,7 +134,7 @@ sint64 random_roll64(sint64 min, sint64 max, object *op, int goodbad) {
  * The args are num D size (ie 4d6)  [garbled 20010916]
  */
 
-int die_roll(int num, int size, object *op, int goodbad) {
+int die_roll(int num, int size, const object *op, int goodbad) {
     int min, diff, luck, total, i, gotlucky, base, ran;
 
     diff = size;
@@ -268,7 +268,7 @@ materialtype_t *name_to_material(const char *name)
  * as some that did not apply previously, may apply now.
  */
 
-void transmute_materialname(object *op, object *change)
+void transmute_materialname(object *op, const object *change)
 {
     materialtype_t *mt;
     int j;
@@ -284,6 +284,10 @@ void transmute_materialname(object *op, object *change)
 	return;
 
     mt = name_to_material(op->materialname);
+    if (!mt) {
+        LOG(llevError, "archetype '%s>%s' uses nonexistent material '%s'\n", op->arch->name, op->name, op->materialname);
+        return;
+    }
 
     for (j=0; j < NROFATTACKS; j++)
 	if (op->resist[j] == 0 && change->resist[j] != 0) {
@@ -299,7 +303,6 @@ void transmute_materialname(object *op, object *change)
 void set_materialname(object *op, int difficulty, materialtype_t *nmt)
 {
     materialtype_t *mt, *lmt;
-    int j;
 
     if (op->materialname != NULL)
 	return;
@@ -335,7 +338,7 @@ void set_materialname(object *op, int difficulty, materialtype_t *nmt)
 #ifndef NEW_MATERIAL_CODE
 	op->materialname = add_string(lmt->name);
 	return;
-#endif
+#else
 
 	if (op->stats.dam && IS_WEAPON(op)) {
 	    op->stats.dam += lmt->damage;
@@ -347,6 +350,7 @@ void set_materialname(object *op, int difficulty, materialtype_t *nmt)
 	if (op->stats.wc && IS_WEAPON(op))
 	    op->stats.wc += lmt->wc;
 	if (IS_ARMOR(op)) {
+	    int j;
 	    if (op->stats.ac)
 		op->stats.ac += lmt->ac;
 	    for (j=0; j < NROFATTACKS; j++)
@@ -364,6 +368,7 @@ void set_materialname(object *op, int difficulty, materialtype_t *nmt)
 	    op->weight = (op->weight * lmt->weight)/100;
 	    op->value = (op->value * lmt->value)/100;
 	}
+#endif
     }
 }
 
@@ -393,7 +398,7 @@ void strip_media_tag(char *message){
 const char* strrstr(const char* haystack, const char* needle){
     const char* lastneedle;
     lastneedle=NULL;
-    while(haystack=strstr(haystack,needle)){
+    while((haystack=strstr(haystack,needle))!=NULL){
         lastneedle=haystack;
         haystack++;
     }
@@ -408,3 +413,71 @@ void strip_endline(char* buf){
     if (!strcmp(buf+strlen(buf)-EOL_SIZE,"\n"))
         buf[strlen(buf)-EOL_SIZE]='\0';    
 }
+
+/**
+ * Replace in string src all occurrences of key by replacement. The resulting
+ * string is put into result; at most resultsize characters (including the
+ * terminating null character) will be written to result.
+ */
+void replace(const char *src, const char *key, const char *replacement, char *result, size_t resultsize)
+{
+	size_t resultlen;
+	size_t keylen;
+
+	/* special case to prevent infinite loop if key==replacement=="" */
+	if(strcmp(key, replacement) == 0)
+	{
+		snprintf(result, resultsize, "%s", src);
+		return;
+	}
+
+	keylen = strlen(key);
+
+	resultlen = 0;
+	while(*src != '\0' && resultlen+1 < resultsize)
+	{
+		if(strncmp(src, key, keylen) == 0)
+		{
+			snprintf(result+resultlen, resultsize-resultlen, "%s", replacement);
+			resultlen += strlen(result+resultlen);
+			src += keylen;
+		}
+		else
+		{
+			result[resultlen++] = *src++;
+		}
+	}
+	result[resultlen] = '\0';
+}
+
+/**
+ * Taking a string as an argument, mutate it into a string that looks like a list.
+ * a 'list' for the purposes here, is a string of items, seperated by commas, except
+ * for the last entry, which has an 'and' before it, and a full stop (period) after it.
+ * This function will also strip all trailing non alphanumeric characters.
+ * It does not insert an oxford comma.
+ */
+
+void make_list_like(char *input) {
+    char *p, tmp[MAX_BUF];
+    int i;
+    if (!input || strlen(input) > MAX_BUF-5) return; 
+    /* bad stuff would happen if we continued here, the -5 is to make space for ' and ' */
+
+    strncpy(tmp, input, MAX_BUF-5);
+    /*trim all trailing commas, spaces etc.*/
+    for (i=strlen(tmp); !isalnum(tmp[i]) && i >= 0; i--) 
+	tmp[i]='\0';
+    strcat(tmp, ".");
+
+    p=strrchr(tmp, ',');
+    if (p) { 
+	*p='\0';
+	strcpy(input, tmp);
+	p++;
+	strcat(input, " and");
+	strcat(input, p);
+    }
+    else strcpy(input, tmp);
+    return;
+}
