@@ -381,10 +381,10 @@ AttrDef AttrDescription[] = {
     {"WeightL",	        TypeString, getWeightL,		putWeightL},
     {"Brand",	        TypeString, getBrand,		putBrand},
 	 {"Makes",  TypeString, getMakes, putMakes},  /* other_arch */
-    {NULL,		0,		0}
+    {NULL,		0,		NULL,		NULL}
 };
 
-char *allowed_variables[] = {
+static char *allowed_variables[] = {
 
     "name", "race", "slaying", "other_arch", "last_heal", "last_sp",
     "last_eat", "speed", "speed_left", "slow_move", "face", "Str", "Dex",
@@ -572,7 +572,7 @@ static void InsertCb(Widget w,XtPointer client,XtPointer call)
 
     if((obj = AppItemGetObject(self->app)) && AttrGetObject(self)) {
 	debug1("Attr-InsertCb() %s\n",obj->name);
-	(void) insert_ob_in_ob(ObjectCreateClone(obj),AttrGetObject(self));
+	(void) insert_ob_in_ob(object_create_clone(obj),AttrGetObject(self));
     }
 }
 
@@ -691,7 +691,7 @@ static void AttrVarGetCb(Widget w,XtPointer client,XtPointer call)
  * members
  **********************************************************************/
 
-void AppLayout(Attr self,Widget parent, char *name)
+static void AppLayout(Attr self,Widget parent, char *name)
 {
     Widget form,view1,pane;
     Widget ok,apply,cancel;
@@ -944,7 +944,6 @@ static void AttrReset(Attr self)
  */
 void AttrDestroy(Attr self)
 {
-    Edit tmp;
     Attr tmp2;
 
     if (self->attr)
@@ -959,29 +958,21 @@ void AttrDestroy(Attr self)
      * here we find out to what part this window belongs to.
      * it may be:
      *  - Attr of Look in App
-     *  - MapAttr in Edit
      *  - Attr of other Attr (inventory)
      */
     
     if (self == self->app->attr)
 	self->app->attr = NULL;
     else {
-	for (tmp = self->app->edit; tmp; tmp = tmp->next)
-	    if (self == tmp->mapattr) {
-		tmp->mapattr = NULL;
+	for (tmp2 = self->app->attr; tmp2; tmp2 = tmp2->attr)
+	    if (self == tmp2->attr) {
+		tmp2->attr = NULL;
 		break;
 	    }
-	if (!tmp) {
-	    for (tmp2 = self->app->attr; tmp2; tmp2 = tmp2->attr)
-		if (self == tmp2->attr) {
-		    tmp2->attr = NULL;
-		    break;
-		}
 #ifdef DEBUG
-	    if (!tmp2)
-		debug0 ("Cannot find origin of Attr!!\n");
+	if (!tmp2)
+	    debug0 ("Cannot find origin of Attr!!\n");
 #endif
-	} 
     }
     XtFree((char*)self);
 }
@@ -995,7 +986,8 @@ void AttrApply(Attr self)
     String str,var;
     object *ob;
     char buf[BUFSIZ];
-    int len, i, mask,set_all=1;
+    int len, mask,set_all=1;
+    size_t i;
 
     /* check out, that object exist */
     if(!self->op) {
