@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_map_c =
- *   "$Id: CrEdit.c,v 1.6 2001/12/03 07:51:40 mwedel Exp $";
+ *   "$Id: CrEdit.c,v 1.11 2006/05/18 05:27:43 mwedel Exp $";
  */
 
 /*
@@ -136,7 +136,7 @@ static XtResource resources[] = {
  **********************************************************************/
 
 static void UpdatePosition (Widget w, int x, int y,Boolean inv);
-static void ClassInitialize();
+static void ClassInitialize(void);
 static void Initialize(Widget req,Widget new,Arg args[],Cardinal *num);
 static void Destroy(Widget w);
 static void Resize(Widget w);
@@ -171,7 +171,7 @@ static void SelectEndAc ( Widget w, XEvent * e,
 static void FeedAc ( Widget w, XEvent * e, 
 		    String * argv, Cardinal * argc );
 
-XtActionsRec actions[] = {
+static XtActionsRec actions[] = {
   {"Insert", InsertAc},
   {"SelectBegin", SelectBeginAc},
   {"SelectExpand", SelectExpandAc},
@@ -380,12 +380,13 @@ static void UpdatePosition (Widget w, int x, int y,Boolean inv)
 	    yb = diff + y * self->crEdit.fontSize - 1;
 
 	    while (op) {
-		New_Face f;
-		if (QUERY_FLAG(op, FLAG_TEAR_DOWN) && self->crEdit.show_weak_walls)
-		    f=new_faces[GET_ANIMATION(op,
-			NUM_ANIMATIONS(op)/2)];
-		else f = *op->face;
-		FaceDraw (w, self->crEdit.gc, &f, xb, yb);
+		if (QUERY_FLAG(op, FLAG_TEAR_DOWN) && self->crEdit.show_weak_walls) {
+		    New_Face * f;
+		    f = &new_faces[GET_ANIMATION(op,NUM_ANIMATIONS(op)/2)];
+		    DrawFacePart(w, self->crEdit.gc, f, xb, yb, 0, 0);
+		} else {
+		    DrawPartObject(w, self->crEdit.gc, op, xb, yb);
+		}
 
 		if (HAS_COLOUR(w)) {
 		    XSetForeground(XtDisplay(w), self->crEdit.gc, 
@@ -401,34 +402,40 @@ static void UpdatePosition (Widget w, int x, int y,Boolean inv)
 	}
     } else { /* Normal map drawing routine */
         New_Face *f;
+
+        if (GET_MAP_FLAGS(self->crEdit.map, x, y) & P_NEED_UPDATE) {
+            update_position(self->crEdit.map, x, y);
+        }
+
         if (displaymode==Dm_Png) {
-          f = GET_MAP_FACE(self->crEdit.map, x, y,2);
-          if (f) FaceDraw (w, self->crEdit.gc, f,
-                    x * self->crEdit.fontSize,
-                    y * self->crEdit.fontSize);
-          f = GET_MAP_FACE(self->crEdit.map, x, y,1);
-	  if ((f) && f->number != blank_face->number)
-	    FaceDraw (w, self->crEdit.gc, f,
-                    x * self->crEdit.fontSize,
-                    y * self->crEdit.fontSize);
-          f = GET_MAP_FACE(self->crEdit.map, x, y,0);
-	  if ((f) && f->number != blank_face->number)
-	    FaceDraw (w, self->crEdit.gc, f,
-                    x * self->crEdit.fontSize,
-                    y * self->crEdit.fontSize);
+	    int l;
+
+	    for (l = 0; l < MAP_LAYERS; l++) {
+		op = GET_MAP_FACE_OBJ(self->crEdit.map, x, y, l);
+		if (op != NULL)
+		    DrawPartObject(w, self->crEdit.gc, op,
+				   x * self->crEdit.fontSize,
+				   y * self->crEdit.fontSize);
+	    }
 	}
 	op=get_map_ob(self->crEdit.map, x, y);
 	while (op && op->above) op=op->above;
-	if (!op) f=blank_face;
-	else if (QUERY_FLAG(op, FLAG_TEAR_DOWN) && self->crEdit.show_weak_walls)
-		f=&new_faces[GET_ANIMATION(op,NUM_ANIMATIONS(op)/2)/2];
+	if (op == NULL) {
+	    ; /* Paint nothing */
+	} else if (QUERY_FLAG(op, FLAG_TEAR_DOWN) && self->crEdit.show_weak_walls) {
+            f = &new_faces[GET_ANIMATION(op, NUM_ANIMATIONS(op)/2)];
 
-	else f = op->face;
-
-	if (displaymode==Dm_Png || f->number != blank_face->number)
-	    FaceDraw (w, self->crEdit.gc, f, 
-		  x * self->crEdit.fontSize, 
-		  y * self->crEdit.fontSize);
+            if (displaymode == Dm_Png || f->number != blank_face->number)
+                DrawFacePart (w, self->crEdit.gc, f,
+                    x * self->crEdit.fontSize,
+                    y * self->crEdit.fontSize,
+                    0, 0);
+        } else {
+            if (displaymode == Dm_Png || op->face->number != blank_face->number)
+                DrawPartObject (w, self->crEdit.gc, op,
+                    x * self->crEdit.fontSize,
+                    y * self->crEdit.fontSize);
+        }
     }
     return;
 }
@@ -926,7 +933,7 @@ static void MoveAc (Widget w, XEvent * e, String * argv, Cardinal * argc)
  * members
  **********************************************************************/
 
-static void ClassInitialize() 
+static void ClassInitialize(void)
 {
     debug0("CrEdit-ClassInitialize()\n");
 
