@@ -32,11 +32,16 @@
 #include "CrEdit.h"
 
 #include "Cnv.h"
+/* HACK Make sure we know how big CnvPromptMax is. */
+#include "Cnv/config.h"
 #include "App.h"
 #include "Attr.h"
+#include "MapAttr.h"
 #include "Str.h"
 #include "Bitmaps.h"
 
+#include "libproto.h"
+#include "proto.h"
 
 /*
  * This function relinks all _pointers_ to the objects from
@@ -47,17 +52,27 @@
  * moved to Edit.c from common/map.c since Edit.c is the only file that uses it.
  */
 
-void copy_map(mapstruct *m1, mapstruct *m2) {
+static void copy_map(mapstruct *m1, mapstruct *m2) {
     int x,y;
 
     memcpy(m2, m1, sizeof(mapstruct));
 
+#if 0
+    /* I'm really not sure how this is used - in any
+     * case, we need to copy the objects now, and
+     * I think we'd really need to make a copy of the objects,
+     * but I'm not sure of the callback structure of this and
+     * where this is used.  Once that is figured out,
+     * this can probably be fixed to do the right thing.
+     * MSW 2006-03-14
+     */
     for(x=0;x<MAP_WIDTH(m1)&&x<MAP_WIDTH(m2);x++)
 	for(y=0;y<MAP_HEIGHT(m1)&&y<MAP_HEIGHT(m2);y++) {
 	    SET_MAP_FACE(m2,x,y,GET_MAP_FACE(m1,x,y,0),0);
 	    SET_MAP_FACE(m2,x,y,GET_MAP_FACE(m1,x,y,1),1);
 	    SET_MAP_FACE(m2,x,y,GET_MAP_FACE(m1,x,y,2),2);
 	}
+#endif
 }
 
 /*
@@ -68,27 +83,70 @@ void copy_map(mapstruct *m1, mapstruct *m2) {
  * dx    : positive translate to right
  * dy    : positive translate to down
  */
-mapstruct *MapMoveScrollResize(mapstruct *source, 
+static mapstruct *MapMoveScrollResize(mapstruct *source, 
 				int width, int height, int dx, int dy) 
 {
     mapstruct *target;
     object *obj,*prt; /* PaRT of obj */
     int x,y,sx = MAP_WIDTH(source), sy = MAP_HEIGHT(source);
     int linked = 0, link=0;
+    int i;
 
     if (!width) width = sx;
     if (!height) height = sy;
     target = get_empty_map (width, height);
 
-    strncpy (target->path, source->path, sizeof(target->path)-1);
-    target->path[sizeof(target->path)-1] = '\0';
-
-    MAP_WIDTH(target) = width;
-    MAP_HEIGHT(target) = height;  
-
     if(dx < 0) dx += MAP_WIDTH(target);
     if(dy < 0) dy += MAP_HEIGHT(target);
 
+    /*
+     * Copy the map-headers from source to target.
+     *
+     * The name, msg and tiling paths are allocated for
+     * each map (see map.c::load_map_header()). Copy
+     * the pointer, then NULL the pointer in source so
+     * the strings aren't freed.
+     *
+     * Statements below in the order the fields appear
+     * in map.h.
+     */
+
+#define MOVE_AND_CLEAR(to, from) to = from; from = NULL;
+    strncpy (target->path, source->path, HUGE_BUF - 1);
+    target->path[HUGE_BUF - 1] = '\0';    
+    MOVE_AND_CLEAR(target->tmpname, source->tmpname);    
+    MOVE_AND_CLEAR(target->name, source->name);    
+    target->region = source->region;
+    MAP_WHEN_RESET(target) = MAP_WHEN_RESET(source);
+    MAP_RESET_TIMEOUT(target) = MAP_RESET_TIMEOUT(source);
+    target->fixed_resettime = source->fixed_resettime;
+    target->unique = source->unique;
+    MAP_NOSMOOTH(target) = MAP_NOSMOOTH(source);
+    MAP_TIMEOUT(target) = MAP_TIMEOUT(source);
+    MAP_SWAP_TIME(target) = MAP_SWAP_TIME(source);
+    /* fields players/in_memory not copied */
+    target->compressed = source->compressed;
+    MAP_DIFFICULTY(target) = MAP_DIFFICULTY(source);    
+    MAP_DARKNESS(target) = MAP_DARKNESS(source);
+    MAP_WIDTH(target) = width;
+    MAP_HEIGHT(target) = height;  
+    MAP_ENTER_X(target) = (MAP_ENTER_X(source) + dx) % width;
+    MAP_ENTER_Y(target) = (MAP_ENTER_Y(source) + dy) % height;
+    target->outdoor = MAP_OUTDOORS(source);     
+    MAP_TEMP(target) = MAP_TEMP(source);
+    MAP_PRESSURE(target) = MAP_PRESSURE(source);
+    MAP_HUMID(target) = MAP_HUMID(source);
+    MAP_WINDSPEED(target) = MAP_WINDSPEED(source);
+    MAP_WINDDIRECTION(target) = MAP_WINDDIRECTION(source);
+    MAP_SKYCOND(target) = MAP_SKYCOND(source);
+    /* fields wpartx/wparty not copied */
+    MOVE_AND_CLEAR(target->msg, source->msg)
+    /* Tiling paths. */
+    for (i = 0; i < 4; i++) { 
+        MOVE_AND_CLEAR(target->tile_path[i], source->tile_path[i]);
+    }    
+#undef MOVE_AND_CLEAR    
+
     for(y=0; y < sy && y < MAP_HEIGHT(target); y++)
 	for(x=0; x < sx && x < MAP_WIDTH(target); x++)
 	    while((obj = get_map_ob(source,x,y)) && !obj->head) {
@@ -114,13 +172,13 @@ mapstruct *MapMoveScrollResize(mapstruct *source,
 }
 
 
-object * MapGetRealObject (mapstruct * emap, int x, int y, int z)
+static object * MapGetRealObject (mapstruct * emap, int x, int y, int z)
 {
     object *tmp = MapGetObjectZ (emap, x, y, z);
     return tmp ? (tmp->head ? tmp->head : tmp) : tmp;
 }
 
-int MapInsertObjectZ(mapstruct *emap,object *o,int x, int y, int z)
+static int MapInsertObjectZ(mapstruct *emap,object *o,int x, int y, int z)
 {
     object *op, *above, *below;
 
@@ -153,7 +211,7 @@ int MapInsertObjectZ(mapstruct *emap,object *o,int x, int y, int z)
     if (above)
         above->below = o;
     else {
-	SET_MAP_FACE (emap, o->x, o->y, o->face,0);
+	SET_MAP_FACE_OBJ (emap, o->x, o->y, o, 0);
     }
     if (below)
         below->above = o;
@@ -163,7 +221,7 @@ int MapInsertObjectZ(mapstruct *emap,object *o,int x, int y, int z)
     return (0);
 }
 
-int MapObjectOut (mapstruct *target, object *obj, int x, int y) {
+static int MapObjectOut (mapstruct *target, object *obj, int x, int y) {
     object *tmp;
     for(tmp = obj; tmp; tmp = tmp->more)
         if(OUT_OF_REAL_MAP(target,x + tmp->x,y + tmp->y)) return 1;
@@ -214,210 +272,6 @@ static void EditInsertArch (Edit self, int x, int y, int i, archetype * at);
 object *EditCloneInsert (Edit self,object *obj,int x, int y, int z);
 Boolean EditObjectDelete (Edit self, int x, int y, int z);
 
-/*
- * getValue functions
- *
- */
-
-static void getX (object *ob, char *str, XtPointer c) {
-    sprintf(str,"%d",EXIT_X(ob));
-}
-
-static void getY (object *ob, char *str, XtPointer c) {
-    sprintf(str,"%d",EXIT_Y(ob));
-}
-
-static void getWidth (object *ob, char *str, XtPointer c) {
-    sprintf(str,"%d",ob->x);
-}
-
-static void getHeight (object *ob, char *str, XtPointer c) {
-    sprintf(str,"%d",ob->y);
-}
-
-static void getPath (object *ob, char *str, XtPointer c) {
-    Edit edit = (Edit)c;
-    strcpy (str, edit->emap->path);
-}
-
-static void getStacking (object *ob, char *str, XtPointer c) {
-    Cardinal stacking;
-    Edit edit = (Edit)c;
-
-    XtVaGetValues (edit->w, XtNstacking, &stacking, NULL);
-    sprintf (str, "%d", stacking);
-}
-
-static void getTimeout (object *ob, char *str, XtPointer c) {
-    sprintf(str,"%d", ob->value);
-}
-
-static void getReset (object *ob, char *str, XtPointer c) {
-    sprintf(str,"%d", ob->weight);
-}
-
-static void getDarkness (object *ob, char *str, XtPointer c) {
-    sprintf(str,"%d", ob->invisible);
-}
-
-static void getDifficulty (object *ob, char *str, XtPointer c) {
-    sprintf(str,"%d", ob->level);
-}
-
-static void getOverwrite (object *ob, char *str, XtPointer c) {
-    Edit edit = (Edit)c;
-    *str = edit->overwrite ? ~0 : 0;
-}
-
-static void getChoose (object *ob, char *str, XtPointer c) {
-    Edit edit = (Edit)c;
-    *str = edit->auto_choose ? ~0 : 0;
-}
-
-static void getReadOnly (object *ob, char *str, XtPointer c) {
-    Edit edit = (Edit)c;
-    *str = edit->read_only ? ~0 : 0;
-}
-
-static void getNoReset (object *ob, char *str, XtPointer c) {
-    *str = (QUERY_FLAG(ob, FLAG_STAND_STILL))? ~0 : 0;
-}
-
-static void getShowWeakWalls(object *ob, char *str, XtPointer c) {
-    Cardinal weak_walls;
-    Edit edit = (Edit)c;
-
-    XtVaGetValues (edit->w, XtNshow_weak_walls, &weak_walls, NULL);
-    *str= weak_walls ? ~0 : 0;
-}
-
-/*
- * putValue functions
- *
- */
-
-static int new_x, new_y;
-
-
-/*** coord ***/
-static void putX (object *ob, char *str, XtPointer c) {
-    EXIT_X(ob) = atoi(str);
-}
-
-static void putY (object *ob, char *str, XtPointer c) {
-    EXIT_Y(ob) = atoi(str);
-}
-
-/*** path ***/
-static void putPath (object *ob, char *str, XtPointer c) {
-    Edit edit = (Edit)c;
-    strcpy (edit->emap->path, str);
-}
-
-/*** size ***/
-static void putWidth (object *ob, char *str, XtPointer c) {
-    new_x = atoi(str);
-}
-static void putHeight (object *ob, char *str, XtPointer c) {
-    new_y = atoi(str);
-}
-
-static void putStacking (object *ob, char *str, XtPointer c) {
-    Edit edit = (Edit)c;
-    int stacking = atoi(str);
-
-    if (stacking > 48) {
-	CnvNotify ("Illegal space","Ok",NULL);
-	return;
-    }
-    XtVaSetValues(edit->w,XtNstacking,stacking,NULL);
-}
-
-static void putOverwrite (object *ob, char *str, XtPointer c) {
-    Edit edit = (Edit)c;
-    edit->overwrite = *str;
-}
-
-static void putReadOnly (object *ob, char *str, XtPointer c) {
-    Edit edit = (Edit)c;
-    edit->read_only = *str;
-
-
-    /* kludge */
-    /* this function will be called last in this list */
-    if (new_y != ob->y || new_x != ob->x) {
-	/* update map window */
-	EditResizeScroll (edit, new_x, new_y, 0, 0);
-    } else {
-	EditUpdate (edit);
-    }
-}
-
-static void putChoose (object *ob, char *str, XtPointer c) {
-    Edit edit = (Edit)c;
-    edit->auto_choose = *str;
-}
-
-static void putTimeout (object *ob, char *str, XtPointer c) {
-    ob->value = atoi(str);
-}
-
-static void putReset (object *ob, char *str, XtPointer c) {
-    ob->weight = atoi(str);
-}
-
-static void putDarkness (object *ob, char *str, XtPointer c) {
-    ob->invisible = atoi(str);
-}
-
-static void putDifficulty (object *ob, char *str, XtPointer c) {
-    ob->level = atoi(str);
-}
-
-static void putNoReset(object *ob, char *str, XtPointer c) {
-	if (*str) SET_FLAG(ob,FLAG_STAND_STILL);
-	else CLEAR_FLAG(ob, FLAG_STAND_STILL);
-}
-
-static void putShowWeakWalls (object *ob, char *str, XtPointer c) {
-    Edit edit = (Edit)c;
-    Cardinal weak_walls;
-
-    if (*str) weak_walls=1;
-    else weak_walls=0;
-
-    XtVaSetValues(edit->w,XtNshow_weak_walls,weak_walls,NULL);
-}
-
-
-/**********************************************************************
- * tags
- **********************************************************************/
-
-static AttrDef MapDescription[] = {
-    {"Path",		TypeString, getPath,		putPath},
-    {"Start X",		TypeString, getX,		putX},
-    {"Start Y",		TypeString, getY,		putY},
-    {"Width",		TypeString, getWidth,		putWidth},
-    {"Height",		TypeString, getHeight,		putHeight},
-    {"Stacking",	TypeString, getStacking,	putStacking},
-    {"Timeout",		TypeString, getTimeout,		putTimeout},
-    {"Reset",		TypeString, getReset,		putReset},
-    {"Difficulty",	TypeString, getDifficulty,	putDifficulty},
-    {"Darkness",	TypeString, getDarkness,	putDarkness},
-    {"FixedReset",	TypeToggle, getNoReset,		putNoReset},
-    {"Show Weak Walls",	TypeToggle, getShowWeakWalls,	putShowWeakWalls},
-    /* Make sure the ReadOnly is the last thing on the list, otherwise
-     * things won't work properly.
-     */
-    {"ReadOnly",	TypeToggle, getReadOnly,	putReadOnly},
-    {NULL,		TypeString, 0,			0},
-    {"Overwrite",	TypeToggle, getOverwrite,	putOverwrite},
-    {"Choose",		TypeToggle, getChoose,		putChoose},
-    {NULL,		0, 0,				0}
-};
-
-
 /*
  * to all refresh  
  */
@@ -550,11 +404,11 @@ static void EditInsertArch (Edit self, int x, int y, int i, archetype * at)
     if (op && op->arch == at)
         return;
 
-    if (arch_blocked(at, self->emap, x, y) & P_OUT_OF_MAP) {
+    if (ob_blocked(&at->clone, self->emap, x, y) & P_OUT_OF_MAP) {
         debug("Out of Map\n");
         return;
     }
-    op = ObjectCreateArch (at);
+    op = object_create_arch (at);
     if (op) {
         MapInsertObjectZ(emap,op,x,y,i);
         /*debug3 ("Inserting %s %d %d\n", op->name, op->x, op->y);*/
@@ -795,7 +649,8 @@ static void EnterCb (Widget w, XtPointer client, XtPointer call)
     Edit self = (Edit) client;
     object *obj = NULL, *tmp;
     char buf[BUFSIZ];
-    char *path,loadPath[PATH_MAX+1];
+    char loadPath[PATH_MAX+1];
+    const char *path;
     XRectangle rect;
     
     /*** find object ***/
@@ -1059,24 +914,11 @@ static void AttributesCb (Widget w, XtPointer client, XtPointer call)
 	
     debug0("Edit::AttributesCb()\n");
 
-    if(!self->mapattr) {
-	self->mapattr = (Attr)XtMalloc (sizeof(struct _Attr));
-	self->mapattr->op = NULL;
-	self->mapattr->app = self->app;
-	self->mapattr->client = self;
-	self->mapattr->attr = NULL;
-	self->mapattr->desc = MapDescription;
-
-	AppLayout (self->mapattr, self->mapattr->app->shell, "mapattr");
-
-	AttrChange(self->mapattr,self->mapattr->op, -1, self->mapattr->client);
-	self->mapattr->dump = CnvBrowseCreate("dump", self->mapattr->app->shell, NULL);
-	XtPopup(self->mapattr->shell,XtGrabNone);
-	self->mapattr->isup = True;
+    if (self->mapattr == NULL) {
+        self->mapattr = MapAttrCreate(self->emap, self->app, self);
     } else {
-	AttrDestroy(self->mapattr);
+	MapAttrDestroy(self->mapattr);
     }
-    return;
 }
 
 /* 
@@ -1133,6 +975,55 @@ static void ToggleAutoCb (Widget w, XtPointer client, XtPointer call)
   debug ("CbEditToggleAuto()\n");
 }
 
+/* 
+ * callback: Toggle showing weak walls
+ */
+static void ToggleWeakCb (Widget w, XtPointer client, XtPointer call)
+{
+    Edit self = (Edit)client;
+    Cardinal weak_walls;
+  
+    XtVaGetValues(self->w, XtNshow_weak_walls, &weak_walls, NULL);
+    XtVaSetValues(self->w, XtNshow_weak_walls, !weak_walls, NULL);
+
+    EditUpdate(self);
+    debug("CbEditToggleWeak()\n");
+}
+
+/*
+ * callback: Adjust stacking size
+ */
+static void SetStackingCb(Widget w, XtPointer client, XtPointer call)
+{
+    char buf[MAX_BUF];
+    char retbuf[CnvPromptMax+1];
+    Edit self = (Edit)client;
+    Cardinal stacking;
+    int ret;
+    
+    XtVaGetValues(self->w, XtNstacking, &stacking, NULL);
+    
+    snprintf(buf, sizeof(buf), "%d", stacking);
+    
+    ret = CnvPrompt("Set stacking (0 = off)", buf, retbuf, "OK", "Cancel", NULL);
+        
+    if (ret != 1)
+        return;
+    
+    stacking = atoi(retbuf);
+    
+    debug2("SetStackingCb %s %d\n", retbuf, stacking);
+
+    /* TODO How did they come up with this magic number? */
+    if (stacking > 48) {
+        CnvNotify("Illegal space", "Ok", NULL);
+        return;
+    }
+    XtVaSetValues(self->w, XtNstacking, stacking, NULL);
+
+    EditUpdate(self);
+} 
+
 static Widget OptionMenu(String name,Edit self,Widget parent)
 {
     Widget shell,refresh, use;
@@ -1162,6 +1053,14 @@ static Widget OptionMenu(String name,Edit self,Widget parent)
 	("autoChoose", smeBSBObjectClass, shell, NULL);
     XtAddCallback (self->iw.auto_choose,
 		   XtNcallback,ToggleAutoCb, (XtPointer) self);
+    self->iw.weak_walls = XtVaCreateManagedWidget 
+	("weakWalls", smeBSBObjectClass, shell, NULL);
+    XtAddCallback (self->iw.weak_walls,
+		   XtNcallback,ToggleWeakCb, (XtPointer) self);
+    self->iw.stacking = XtVaCreateManagedWidget 
+	("sparse", smeBSBObjectClass, shell, NULL);
+    XtAddCallback (self->iw.stacking,
+		   XtNcallback,SetStackingCb, (XtPointer) self);
 
     return shell;
 }
@@ -1362,7 +1261,8 @@ Edit EditCreate(App app,EditType type,String path)
     self->emap = NULL;
     self->type = type;
     self->overwrite = 0;
-    self->mapattr = 0;
+    self->mapattr = NULL;
+    self->modified = False;
 
     /*** load or create map ***/
     if (type == ClipBoard) {
@@ -1370,20 +1270,23 @@ Edit EditCreate(App app,EditType type,String path)
 	    self->emap = get_empty_map(MapMinWidth,MapMinHeight);
 	strcpy (self->emap->path, "/ClipBoard");
 
+	if (!self->emap->msg) {
+	    self->emap->msg = MapMessageCreate (app);
+	}
     } else if(path && *path != '\0') {
 	if (!Load(self,path)) return 0;
     } else {
 	self->emap = get_empty_map(16,16);
 	strcpy (self->emap->path, "/Noname");
+
+	if (!self->emap->msg) {
+	    self->emap->msg = MapMessageCreate (app);
+	}
     }
     
     if (!self->emap)
 	return 0;
     
-    if (!self->emap->msg) {
-	self->emap->msg = MapMessageCreate (app);
-    }
-
     /*** creating ***/
     Layout(self,self->app->shell,0);
     EditUnmodified(self);
@@ -1442,7 +1345,7 @@ void EditDestroy(Edit self)
     }
     /*** outer coonections ***/
     if (self->mapattr) {
-	AttrDestroy (self->mapattr);
+	MapAttrDestroy(self->mapattr);
     }
     if (self->app->attr && self->app->attr->op &&
 	self->app->attr->op->map == self->emap) {
@@ -1470,7 +1373,7 @@ void EditUpdate(Edit self)
     if(!(self && self->shell && self->emap)) return;
 
     /*** map title ***/
-    sprintf(buf,
+    snprintf(buf, sizeof(buf),
 	    "Edit %s %s",
 	    self->emap->path,
 	    self->modified ? "*" : "");
@@ -1480,11 +1383,14 @@ void EditUpdate(Edit self)
 		  NULL);
     
     /*** info ***/
-    if (self->mapattr)
-	AttrChange (self->mapattr, NULL, -1, self);
+    if (self->mapattr != NULL)
+	MapAttrChange(self->mapattr, self->emap);
 
     /*** toggles ***/
     if(self->type != ClipBoard) {
+        Cardinal weak_walls;
+        Cardinal stacking;
+    
 #if 0
 	XtVaSetValues 
 	    (self->iw.read_only,
@@ -1497,6 +1403,18 @@ void EditUpdate(Edit self)
 	    (self->iw.auto_choose, 
 	     XtNleftBitmap, self->auto_choose ? bitmaps.mark : None, 
 	     NULL);
+        
+	XtVaGetValues(self->w, XtNshow_weak_walls, &weak_walls, NULL);
+	XtVaSetValues 
+	    (self->iw.weak_walls, 
+	     XtNleftBitmap, weak_walls ? bitmaps.mark : None, 
+	     NULL);
+
+	XtVaGetValues(self->w, XtNstacking, &stacking, NULL);
+	XtVaSetValues 
+	    (self->iw.stacking, 
+	     XtNleftBitmap, (stacking != 0) ? bitmaps.mark : None, 
+	     NULL);
     }
     /*** map area ***/
     if(self->emap && self->w)
@@ -1861,7 +1779,7 @@ object *EditCloneInsert (Edit self,object *obj,int x, int y, int z)
     int button_value;
 
     /*** create & insert ***/
-    if ((op = ObjectCreateClone(obj))) {
+    if ((op = object_create_clone(obj))) {
 	/*** object do not fit ***/
 	if(!MapObjectOut(self->emap,op,x,y)) {
 	    MapInsertObjectZ (self->emap, op, x, y, z);
