@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_xutil_c =
- *   "$Id: xutil.c,v 1.12 2005/01/24 07:29:25 mwedel Exp $";
+ *   "$Id: xutil.c,v 1.15 2006/03/18 15:05:31 ryo_saeba Exp $";
  */
 
 /*
@@ -42,23 +42,6 @@
 #include <funcpoint.h>
 #include <loader.h>
 
-/*
- * Converts between Fontindex and XChar2b types.
- * Used in global.h, for draw_face (in face, FontindextoXChars 
- * is defined to this function name
- */
-
-XChar2b fontindex_to_XChar2b(Fontindex s)
-{
-  XChar2b c;
-
-  c.byte1 = s/256;
-  c.byte2 = s%256;
-
-  return c;
-}
-
-
 #define IMAGE_BUF 65536
 
 /*
@@ -99,7 +82,7 @@ int ReadImages(Display *gdisp, Pixmap **pixmaps, Pixmap **masks,
     GC	gc= XCreateGC(gdisp, root, 0, NULL);
 
     if (!nrofpixmaps)
-	nrofpixmaps = ReadBmapNames ();
+        read_bmap_names();
 
     /* The processes that create a pixmap already allocates the space for
      * the pixmap data.  Therefor, only space for the pointers to that data
