@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_arch_h =
- *   "$Id: arch.h,v 1.2 1999/07/13 06:02:52 cvs Exp $";
+ *   "$Id: arch.h,v 1.3 2005/08/12 13:46:34 ryo_saeba Exp $";
  */
 
 /*
@@ -24,13 +24,3 @@
 
     The author can be reached via e-mail to mark@pyramid.com.
 */
-
-#define ARCH_CACHE_SIZE 20
-
-typedef struct arch_recent_struct {
-  char *name; /* Don't use add_ref or similar functions on this */
-  archetype *arch;
-} arch_recent;
-
-arch_recent arch_cache[ARCH_CACHE_SIZE];
-int last_arch_cache=0;
