@@ -6,9 +6,13 @@
  
 /* Message buf size. If this is changed, keep in mind that big strings
  * may be unreadable by the player as the tail of the message
- * can scroll over the beginning (as of v0.92.2).  */
- 
-#define BOOK_BUF        800
+ * can scroll over the beginning (as of v0.92.2).
+ * Note that the book messages are stored in the msg buf,
+ * which is limited by 'HUGE_BUF' in the loader.
+ */
+#ifndef BOOK_H
+#define BOOK_H
+#define BOOK_BUF        HUGE_BUF-10
  
 /* if little books arent getting enough text generated, enlarge this */
  
@@ -22,3 +26,8 @@
 #define BOOKSIZE(xyz)   BASE_BOOK_BUF+((xyz)->weight/10)>BOOK_BUF? \
                                 BOOK_BUF:BASE_BOOK_BUF+((xyz)->weight/10);
 
+typedef struct {
+	uint8 message_type;
+	uint8 message_subtype;
+} readable_message_type;
+#endif
