@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_define_h =
- *   "$Id: define.h,v 1.86 2005/05/29 17:35:53 tchize Exp $";
+ *   "$Id: define.h,v 1.109 2006/05/21 13:44:42 akirschbaum Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2003 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2003-2005 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -100,7 +100,7 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define BIG_NAME 32
 #define MAX_EXT_TITLE 98
 
-/* Fatal variables: */
+/* Fatal variables; used as arguments to fatal() */
 #define OUT_OF_MEMORY		0
 #define MAP_ERROR		1
 #define ARCHTABLE_TOO_SMALL	2
@@ -114,12 +114,13 @@ error - Your ANSI C compiler should be defining __STDC__;
  * the monster type, that can not be handled by any of the numerous
  * flags
  * Also, if you add new entries, try and fill up the holes in this list.
+ * Additionally, when you add a new entry, include it in the table in item.c
  */
 
 /* type 0 will be undefined and shows a non valid type information */
 
 #define PLAYER		            1
-/*#define BULLET		    2 */ 
+#define TRANSPORT		    2	/* see doc/Developers/objects */
 #define ROD		            3
 #define TREASURE	            4
 #define POTION		            5
@@ -136,7 +137,7 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define ARMOUR		            16
 #define PEDESTAL	            17
 #define ALTAR		            18
-#define CONFUSION	            19
+/*#define CONFUSION	            19 */
 #define LOCKED_DOOR	            20
 #define SPECIAL_KEY	            21
 #define MAP		            22
@@ -259,7 +260,7 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define GIRDLE		            113
 #define FORCE		            114
 #define POTION_EFFECT		    115    /* a force, holding the effect of a potion */
-
+#define EVENT_CONNECTOR             116    /* Lauwenmark: an invisible object holding a plugin event hook */
 #define CLOSE_CON	            121    /* Eneq(@csd.uu.se): Id for close_container archetype. */
 #define CONTAINER	            122
 #define ARMOUR_IMPROVER		    123
@@ -286,8 +287,11 @@ error - Your ANSI C compiler should be defining __STDC__;
 
 #define BUILDER            160 /* Generic item builder, see subtypes */
 #define MATERIAL            161 /* Material for building */
-#define GPS                 162 /* Ground positionning system */
+/* #define GPS                 162 Ground positionning system, moved to Python plugin */
 #define ITEM_TRANSFORMER    163 /* Transforming one item with another */
+#define QUEST               164 /* See below for subtypes */
+
+#define OBJECT_TYPE_MAX		    164 /* update if you add new types */
 /* END TYPE DEFINE */
 
 /* Subtypes for BUILDER */
@@ -299,6 +303,17 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define ST_MAT_WALL     2 /* Wall */
 #define ST_MAT_ITEM     3 /* All other items, including doors & such */
 
+/* Subtypes for QUEST  */
+#define QUEST_IN_PROGRESS   1
+#define QUEST_DONE_QUEST    2
+#define QUEST_DONE_TASK     3
+#define QUEST_START_QUEST   4
+#define QUEST_END_QUEST     5
+#define QUEST_START_TASK    6
+#define QUEST_END_TASK      7
+#define QUEST_OVERRIDE      8
+#define QUEST_ON_ACTIVATE   9
+
 /* definitions for weapontypes */
 
 #define WEAP_HIT	0  /* the basic */
@@ -311,6 +326,16 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define WEAP_CRUSH	7  /* big hammers, flails */
 #define WEAP_BLUD	8  /* bludgeoning, club, stick */
 
+typedef struct typedata {
+    int number;
+    const char *name; 
+    const char *name_pl;
+    int identifyskill;
+    int identifyskill2;
+} typedata;
+
+extern const int ItemTypesSize;
+extern typedata ItemTypes[];
 
 /* definitions for detailed pickup descriptions.
  *   The objective is to define intelligent groups of items that the
@@ -351,11 +376,14 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define PU_READABLES		0x00400000
 #define PU_MAGIC_DEVICE		0x00800000
 
+#define PU_NOT_CURSED		0x01000000
+#define PU_JEWELS		0x02000000
+
 
 /* Instead of using arbitrary constants for indexing the
  * freearr, add these values.  <= SIZEOFFREE1 will get you
  * within 1 space.  <= SIZEOFFREE2 wll get you withing 
- * 2 spaces, and the entire array (<= SIZEOFFREE) is 
+ * 2 spaces, and the entire array (< SIZEOFFREE) is 
  * three spaces
  */
 #define SIZEOFFREE1 8
@@ -399,7 +427,13 @@ error - Your ANSI C compiler should be defining __STDC__;
 	((xyz)->flags[p/32] &= ~(1U << (p % 32)))
 #define QUERY_FLAG(xyz, p) \
 	((xyz)->flags[p/32] & (1U << (p % 32)))
-
+#define COMPARE_FLAGS(p,q) \
+	(                                            \
+		((p)->flags[0] == (q)->flags[0]) &&  \
+		((p)->flags[1] == (q)->flags[1]) &&  \
+		((p)->flags[2] == (q)->flags[2]) &&  \
+		((p)->flags[3] == (q)->flags[3])     \
+	)
 /* convenience macros to determine what kind of things we are dealing with */
 
 #define IS_WEAPON(op) \
@@ -437,11 +471,11 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define FLAG_USE_SHIELD		7 /* Can this creature use a shield? */
 
 #define FLAG_NO_PICK	 	8 /* Object can't be picked up */
-#define FLAG_WALK_ON	 	9 /* Applied when it's walked upon */
-#define FLAG_NO_PASS		10 /* Nothing can pass (wall() is true) */
+#define FLAG_CLIENT_ANIM_SYNC 	9 /* Let client animate this, synchronized */
+#define FLAG_CLIENT_ANIM_RANDOM	10/* Client animate this, randomized */
 #define FLAG_ANIMATE		11 /* The object looks at archetype for faces */
-#define FLAG_SLOW_MOVE		12 /* Uses the stats.exp/1000 to slow down */
-#define FLAG_FLYING		13 /* Not affected by WALK_ON or SLOW_MOVE) */
+/*#define FLAG_SLOW_MOVE	12*//* Uses the stats.exp/1000 to slow down */
+/*#define FLAG_FLYING		13*//* Not affected by WALK_ON or SLOW_MOVE) */
 #define FLAG_MONSTER		14 /* Will attack players */
 #define FLAG_FRIENDLY		15 /* Will help players */
 
@@ -449,14 +483,14 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define FLAG_IS_THROWN		17 /* Object is designed to be thrown. */
 #define FLAG_AUTO_APPLY		18 /* Will be applied when created */
 #define FLAG_TREASURE		19 /* Will generate treasure when applied */
-/*#define FLAG_INVIS_UNDEAD	20 *//* Player is invisible to undead */
+#define FLAG_PLAYER_SOLD	20 /* Object was sold to a shop by a player. */
 #define FLAG_SEE_INVISIBLE 	21 /* Will see invisible player */
 #define FLAG_CAN_ROLL		22 /* Object can be rolled */
 #define FLAG_OVERLAY_FLOOR	23 /* Object is an overlay floor */
 #define FLAG_IS_TURNABLE 	24 /* Object can change face with direction */
-#define FLAG_WALK_OFF		25 /* Object is applied when left */
-#define FLAG_FLY_ON		26 /* As WALK_ON, but only with FLAG_FLYING */
-#define FLAG_FLY_OFF		27 /* As WALK_OFF, but only with FLAG_FLYING */
+/*#define FLAG_WALK_OFF		25*//* Object is applied when left */
+/*#define FLAG_FLY_ON		26*//* As WALK_ON, but only with FLAG_FLYING */
+/*#define FLAG_FLY_OFF		27*//* As WALK_OFF, but only with FLAG_FLYING */
 #define FLAG_IS_USED_UP		28 /* When (--food<0) the object will exit */
 #define FLAG_IDENTIFIED		29 /* Player knows full info about item */
 #define FLAG_REFLECTING		30 /* Object reflects from walls (lightning) */
@@ -475,18 +509,18 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define FLAG_REFL_SPELL		40 /* Spells (some) will reflect from object */
 #define FLAG_NO_MAGIC		41 /* Spells (some) can't pass this object */
 #define FLAG_NO_FIX_PLAYER	42 /* fix_player() won't be called */
-/*#define FLAG_NEED_IE		43 Not used - take care of by the name_pl now 1 ruby -> 2 rubies */
+#define FLAG_IS_LIGHTABLE	43 /* object can be lit */
 #define FLAG_TEAR_DOWN		44 /* at->faces[hp*animations/maxhp] at hit */
 #define FLAG_RUN_AWAY		45 /* Object runs away from nearest player \
 				      but can still attack at a distance */
-#define FLAG_PASS_THRU		46 /* Objects with can_pass_thru can pass \
+/*#define FLAG_PASS_THRU	46*/ /* Objects with can_pass_thru can pass \
 				      thru this object as if it wasn't there */
-#define FLAG_CAN_PASS_THRU	47 /* Can pass thru... */
+/*#define FLAG_CAN_PASS_THRU	47*/ /* Can pass thru... */
 
 #define FLAG_PICK_UP		48 /* Can pick up */
 #define FLAG_UNIQUE		49 /* Item is really unique (UNIQUE_ITEMS) */
 #define FLAG_NO_DROP		50 /* Object can't be dropped */
-/* #define FLAG_NO_PRETEXT	51 Not used No text is added before name. */
+#define FLAG_WIZCAST		51 /* The wizard can cast spells in no-magic area */
 #define FLAG_CAST_SPELL		52 /* (Monster) can learn and cast spells */
 #define FLAG_USE_SCROLL		53 /* (Monster) can read scroll */
 #define FLAG_USE_RANGE		54 /* (Monster) can apply and use range items */
@@ -580,8 +614,59 @@ error - Your ANSI C compiler should be defining __STDC__;
 
 #define NROFNEWOBJS(xyz)	((xyz)->stats.food)
 
+#if 0
+/* These should no longer be needed - access move_slow_penalty
+ * directly.
+ */
 #define SLOW_PENALTY(xyz)   ((xyz)->stats.exp)/1000.0
 #define SET_SLOW_PENALTY(xyz,fl)	(xyz)->stats.exp=(fl)*1000
+#endif
+
+/* If you add new movement types, you may need to update
+ * describe_item() so properly describe those types.
+ * change_abil() probably should be updated also.
+ */
+#define MOVE_WALK	0x1	/* Object walks */
+#define MOVE_FLY_LOW	0x2	/* Low flying object */
+#define MOVE_FLY_HIGH	0x4	/* High flying object */
+#define	MOVE_FLYING	0x6	/* Combo of fly_low and fly_high */
+#define MOVE_SWIM	0x8	/* Swimming object */
+#define MOVE_BOAT	0x10	/* Boats/sailing */
+#define MOVE_ALL	0x1f	/* Mask of all movement types */
+
+/* the normal assumption is that objects are walking/flying.
+ * So often we don't want to block movement, but still don't want
+ * to allow all types (swimming is rather specialized) - I also
+ * expect as more movement types show up, this is likely to get
+ * updated.  Basically, this is the default for spaces that allow
+ * movement - anything but swimming right now.  If you really
+ * want nothing at all, then can always set move_block to 0
+ */
+#define MOVE_BLOCK_DEFAULT  MOVE_SWIM
+
+/* typdef here to define type large enough to hold bitmask of
+ * all movement types.  Make one declaration so easy to update.
+ * uint8 is defined yet, so just use what that would define it
+ * at anyways.
+ */
+typedef unsigned char	MoveType;
+
+/* Basic macro to see if ob2 blocks ob1 from moving onto this space.
+ * Basically, ob2 has to block all of ob1 movement types.
+ */
+#define OB_MOVE_BLOCK(ob1, ob2) \
+    ((ob1->move_type & ob2->move_block) == ob1->move_type)
+
+/* Basic macro to see if if ob1 can not move onto a space based
+ * on the 'type' move_block parameter
+ * Add check - if type is 0, don't stop anything from moving
+ * onto it.
+ * 
+ */
+#define OB_TYPE_MOVE_BLOCK(ob1, type) \
+    ( (type != 0) && (ob1->move_type & type) == ob1->move_type)
+
+
 #define SET_GENERATE_TYPE(xyz,va)	(xyz)->stats.sp=(va)
 #define GENERATE_TYPE(xyz)	((xyz)->stats.sp)
 #define GENERATE_SPEED(xyz)	((xyz)->stats.maxsp) /* if(!RANDOM()%<speed>) */
@@ -630,6 +715,8 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define F_NO_BARGAIN	4   /* combine with F_BUY or F_SELL to disable bargaining calc */
 #define F_IDENTIFIED	8   /* flag to calculate value of identified item */
 #define F_NOT_CURSED	16  /* flag to calculate value of uncursed item */
+#define F_APPROX	32  /* flag to give a guess of item value */
+#define F_SHOP		64  /* consider the effect that the shop that the player is in has */
 
 #define DIRX(xyz)	freearr_x[(xyz)->direction]
 #define DIRY(xyz)	freearr_y[(xyz)->direction]
@@ -678,7 +765,7 @@ error - Your ANSI C compiler should be defining __STDC__;
 #define DISTHIT  7 /* attack from a distance if hit as recommended by Frank */
 #define WAIT2    8 /* monster does not try to move towards player if far    */
                    /* maintains comfortable distance                        */
-#define PETMOVE 16 /* if the upper four bits of move_type / attack_movement */
+#define PETMOVE 16 /* if the upper four bits of attack_movement             */
                    /* are set to this number, the monster follows a player  */
                    /* until the owner calls it back or off                  */
                    /* player followed denoted by 0b->owner                  */
@@ -727,7 +814,6 @@ error - Your ANSI C compiler should be defining __STDC__;
 
 #define BLANK_FACE_NAME "blank.111"
 #define EMPTY_FACE_NAME "empty.111"
-#define POTION_FACE_NAME	"potiongen.111" 
 #define DARK_FACE1_NAME		"dark1.111"
 #define DARK_FACE2_NAME		"dark2.111"
 #define DARK_FACE3_NAME		"dark3.111"
@@ -743,7 +829,7 @@ error - Your ANSI C compiler should be defining __STDC__;
 /* Simple function we use below to keep adding to the same string
  * but also make sure we don't overwrite that string.
  */
-static inline void safe_strcat(char *dest, char *orig, int *curlen, int maxlen)
+static inline void safe_strcat(char *dest, const char *orig, int *curlen, int maxlen)
 {
     if (*curlen == (maxlen-1)) return;
     strncpy(dest+*curlen, orig, maxlen-*curlen-1);
@@ -835,7 +921,7 @@ enum apply_flag {
   /* Optional flags, for bitwise or with a basic flag */
     AP_NO_MERGE			= 16,
     AP_IGNORE_CURSE		= 32,
-    AP_PRINT			= 64,	/* Print what to do, don't actually do it */
+    AP_PRINT			= 64	/* Print what to do, don't actually do it */
 					/* Note this is supported in all the functions */
 
 };
@@ -882,7 +968,7 @@ enum apply_flag {
  *
  * 0.93.3: It looks like linux has random (previously, it was set below
  * to use rand).  Perhaps old version of linux lack rand?  IF you run into
- * problems, add || defined(linux) the #if immediately below.
+ * problems, add || defined(__linux__) the #if immediately below.
  *
  * 0.94.2 - you probably shouldn't need to change any of the rand stuff
  * here.
@@ -931,7 +1017,7 @@ enum apply_flag {
   * But use as debug only (loses all speed gained by those macros)
   */
 /*#define FAST_STRNCAT(buf__,buf2__,size__) {memcpy (buf__,buf2__,size__);buf__+=size__;\
- if (size__!=strlen(buf2__)) printf ("Error, bad length for %s\n",buf2__);}*/
+ if (size__!=strlen(buf2__)) LOG(llevError, "Error, bad length for %s\n",buf2__);}*/
 
  
  
