@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_face_h =
- *   "$Id: face.h,v 1.3 2001/12/03 07:51:40 mwedel Exp $";
+ *   "$Id: face.h,v 1.5 2006/05/18 05:27:44 mwedel Exp $";
  */
 
 /*
@@ -34,19 +34,13 @@
 typedef struct new_face_struct {
     uint16	number;		/* This is the image id.  It should be the */
 				/* same value as its position in the array */
-    char	*name;
+    const char	*name;
     uint8	visibility;
     uint8	magicmap;	/* Color to show this in magic map */
 } New_Face;
 
-typedef struct map_look_struct {
-    New_Face *face;
-    uint8	flags;
-} MapLook;
-
-
 typedef struct {
-    char *name;             /* Name of the animation sequence */
+    const char *name;             /* Name of the animation sequence */
     uint8 num_animations;   /* How many different faces to animate */
     uint8 facings;          /* How many facings (1,2,4,8) */
     Fontindex *faces;       /* The different animations */
