@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_funcpoint_h =
- *   "$Id: funcpoint.h,v 1.13 2005/07/17 11:00:04 ryo_saeba Exp $";
+ *   "$Id: funcpoint.h,v 1.17 2006/05/05 09:25:18 ryo_saeba Exp $";
  */
 
 /*
@@ -30,31 +30,6 @@
 #ifndef FUNCPOINT_H
 #define FUNCPOINT_H
 
-/*
- * Some function types
- */
-
-typedef void (*type_move_apply_func) (object *, object *, object *);
-typedef void (*type_func_int)(int);
-typedef void (*type_func_int_int)(int,int);
-typedef void (*type_func_void)(void);
-typedef void (*type_func_map)(mapstruct *);
-typedef void (*type_func_map_char)(mapstruct *, char *);
-typedef void (*type_func_int_map_char)(int, mapstruct *, char *);
-typedef void (*type_func_ob)(object *);
-typedef void (*type_func_ob_char)(object *, char *);
-typedef void (*type_func_ob_cchar)(object *, const char *);
-typedef void (*type_func_int_int_ob_cchar)(int, int, object *, const char *);
-typedef void (*type_func_ob_ob)(object *, object *);
-typedef void (*type_func_ob_int)(object *, int);
-typedef int (*type_int_func_ob_ob)(object *, object *);
-typedef void (*type_func_char_int)(char *, int);
-typedef void (*type_func_int_ob_ob)(int, object *, object *);
-typedef void (*type_func_player_int)(player *, int);
-typedef void (*type_func_dragon_gain)(object *who, int atnr, int level);
-typedef void (*type_func_char)(char *);
-typedef object* (*type_ob_func_ob_int)(object *, int);
-
 /*
  * These function-pointers are defined in common/glue.c
  * The functions used to set and initialise them are also there.
@@ -67,14 +42,12 @@ typedef object* (*type_ob_func_ob_int)(object *, int);
 extern void	move_apply(object *, object *, object *);
 extern void	draw_info(int, int, object *, const char *);
 extern void	emergency_save(int);
-extern void	clean_tmp_files();
+extern void	clean_tmp_files(void);
 extern void	fix_auto_apply(mapstruct *);
-extern void	init_blocksview_players();
+extern void	init_blocksview_players(void);
 extern void	monster_check_apply(object *, object *);
-extern void	process_active_maps();
 extern void	remove_friendly_object(object *);
 extern void	update_buttons(mapstruct *);
-extern void	info_map(int, mapstruct *, char *);
 extern void	move_teleporter(object *);
 extern void	move_firewall(object *);
 extern void	move_creator(object *);
@@ -84,6 +57,7 @@ extern void trap_adjust(object *, int);
 extern void	esrv_send_item(object *, object *);
 extern void	esrv_del_item(player *, int);
 extern void	esrv_update_item(int, object *, object *);
+extern void	esrv_update_spells(player *);
 extern void	set_darkness_map(mapstruct *m);
 extern void dragon_ability_gain(object *, int, int);
 extern void	weather_effect(const char *);
