@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_object_h =
- *   "$Id: loader.h,v 1.15 2002/07/15 04:57:12 mwedel Exp $";
+ *   "$Id: loader.h,v 1.16 2005/10/24 20:48:15 akirschbaum Exp $";
  */
 
 /*
@@ -38,4 +38,4 @@
 #define LO_NEWFILE  2
 #define	LO_NOREAD   3
 
-extern int nroffiles, nrofpixmaps;
+extern int nrofpixmaps;
