@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_define_h =
- *   "$Id: map.h,v 1.28 2005/04/16 13:19:30 cavesomething Exp $";
+ *   "$Id: map.h,v 1.39 2006/05/30 07:16:23 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2002-2005 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -48,7 +48,26 @@
 #define MAGIC_MAP_HALF  MAGIC_MAP_SIZE/2
 
 
-#define MAP_LAYERS		4
+/* This correspondes to the map layers in the map2 protocol.
+ * The MAP_LAYER_... correspond to what goes on what layer -
+ * this removes the need for hardcoding, and also makes sure
+ * we don't overstep the number of layers.
+ */
+#define MAP_LAYERS		10
+
+extern const char *map_layer_name[MAP_LAYERS];
+
+
+#define MAP_LAYER_FLOOR		0
+#define MAP_LAYER_NO_PICK1	1   /* Non pickable ground objects */
+#define MAP_LAYER_NO_PICK2	2   /* Non pickable ground objects */
+#define MAP_LAYER_ITEM1		3   /* Items that can be picked up */
+#define MAP_LAYER_ITEM2		4
+#define MAP_LAYER_ITEM3		5
+#define MAP_LAYER_LIVING1	6   /* Living creatures */
+#define MAP_LAYER_LIVING2	7
+#define MAP_LAYER_FLY1		8   /* Flying objects - creatures, spells */
+#define MAP_LAYER_FLY2		9   /* Arrows, etc */
 
 /* This is when the map will reset */
 #define MAP_WHEN_RESET(m)	((m)->reset_time)
@@ -109,17 +128,32 @@
 #define SET_MAP_LIGHT(M,X,Y,L)	( (M)->spaces[(X) + (M)->width * (Y)].light = L )
 
 #define GET_MAP_OB(M,X,Y)	( (M)->spaces[(X) + (M)->width * (Y)].bottom )
+#define GET_MAP_TOP(M,X,Y)	( (M)->spaces[(X) + (M)->width * (Y)].top )
 /* legacy */
 #define get_map_ob		GET_MAP_OB
 
 #define SET_MAP_OB(M,X,Y,tmp)	( (M)->spaces[(X) + (M)->width * (Y)].bottom = (tmp) )
+#define SET_MAP_TOP(M,X,Y,tmp)	( (M)->spaces[(X) + (M)->width * (Y)].top = (tmp) )
 #define set_map_ob SET_MAP_OB
 
-#define SET_MAP_FACE(M,X,Y,C,L) ( (M)->spaces[(X) + (M)->width * (Y)].faces[L] = C )
-#define GET_MAP_FACE(M,X,Y,L) ( (M)->spaces[(X) + (M)->width * (Y)].faces[L]  )
-
 #define SET_MAP_FACE_OBJ(M,X,Y,C,L) ( (M)->spaces[(X) + (M)->width * (Y)].faces_obj[L] = C )
-#define GET_MAP_FACE_OBJ(M,X,Y,L) ( (M)->spaces[(X) + (M)->width * (Y)].faces_obj[L]  )
+#define GET_MAP_FACE_OBJ(M,X,Y,L)   ( (M)->spaces[(X) + (M)->width * (Y)].faces_obj[L]  )
+/* Returns the layers array so update_position can just copy
+ * the entire array over.
+ */
+#define GET_MAP_FACE_OBJS(M,X,Y)   ( (M)->spaces[(X) + (M)->width * (Y)].faces_obj  )
+
+#define GET_MAP_MOVE_BLOCK(M,X,Y)   ( (M)->spaces[(X) + (M)->width * (Y)].move_block )
+#define SET_MAP_MOVE_BLOCK(M,X,Y,C) ( (M)->spaces[(X) + (M)->width * (Y)].move_block = C )
+
+#define GET_MAP_MOVE_SLOW(M,X,Y)    ( (M)->spaces[(X) + (M)->width * (Y)].move_slow )
+#define SET_MAP_MOVE_SLOW(M,X,Y,C)  ( (M)->spaces[(X) + (M)->width * (Y)].move_slow = C )
+
+#define GET_MAP_MOVE_ON(M,X,Y)      ( (M)->spaces[(X) + (M)->width * (Y)].move_on )
+#define SET_MAP_MOVE_ON(M,X,Y,C)    ( (M)->spaces[(X) + (M)->width * (Y)].move_on = C )
+
+#define GET_MAP_MOVE_OFF(M,X,Y)      ( (M)->spaces[(X) + (M)->width * (Y)].move_off )
+#define SET_MAP_MOVE_OFF(M,X,Y,C)    ( (M)->spaces[(X) + (M)->width * (Y)].move_off = C )
 
 /* You should really know what you are doing before using this - you
  * should almost always be using out_of_map instead, which takes into account
@@ -133,8 +167,16 @@
 
 #define P_BLOCKSVIEW	0x01
 #define P_NO_MAGIC      0x02	/* Spells (some) can't pass this object */
-#define P_NO_PASS       0x04	/* Nothing can pass (wall() is true) */
-/*#define P_PASS_THRU     0x08	*//* */
+
+/* AB_NO_PASS is used for arch_blocked() return value.  It needs
+ * to be here to make sure the bits don't match with anything.
+ * Changed name to have AB_ prefix just to make sure no one
+ * is using the P_NO_PASS.  AB_.. should only be used for
+ * arch_blocked and functions that examine the return value.
+ */
+
+#define AB_NO_PASS       0x04
+/*#define P_PASS_THRU	0x08	*//* */
 #define P_IS_ALIVE      0x10	/* something alive is on this space */
 #define P_NO_CLERIC     0x20	/* no clerical spells cast here */
 #define P_NEED_UPDATE	0x40	/* this space is out of date */
@@ -149,10 +191,16 @@
  */
 #define P_OUT_OF_MAP	0x100	/* This space is outside the map */
 #define	P_NEW_MAP	0x200	/* Coordinates passed result in a new tiled map  */
+
+#if 0
+/* These go away with new movement code - can't do such simplistic
+ * checks anymore
+ */
 #define P_BLOCKED	(P_NO_PASS | P_IS_ALIVE)    /* convenience macro */
 #define P_WALL		P_NO_PASS   /* Just to match naming of wall function */
+#endif
 
-/* Can't use MapCell as that is used in newserver.h
+/* Can't use map_cell_struct as that is used in newserver.h
  * Instead of having numerous arrays that have information on a
  * particular space (was map, floor, floor2, map_ob),
  * have this structure take care of that information.
@@ -162,10 +210,14 @@
 
 typedef struct MapSpace {
     object	*bottom;	/* lowest object on this space */
-    New_Face	*faces[MAP_LAYERS];	/* faces for the 3 layers */
-    object	*faces_obj[MAP_LAYERS];	/* face objects for the 3 layers */
+    object	*top;		/* Highest object on this space */
+    object	*faces_obj[MAP_LAYERS];	/* face objects for the layers */
     uint8	flags;		/* flags about this space (see the P_ values above) */
     sint8	light;		/* How much light this space provides */
+    MoveType	move_block;	/* What movement types this space blocks */
+    MoveType	move_slow;	/* What movement types this space slows */
+    MoveType	move_on;	/* What movement types are activated */
+    MoveType	move_off;	/* What movement types are activated */
 } MapSpace;
 
 /*
@@ -198,8 +250,8 @@ typedef struct wmapdef {
  */
 typedef struct regiondef {
     struct	regiondef *next; /* pointer to next region, NULL for the last one */
-    char	*name;		 /* Shortend name of the region as maps refer to it */
-    char	*parent_name;	 /* 
+    const char	*name;		 /* Shortend name of the region as maps refer to it */
+    const char	*parent_name;	 /* 
     				  * So that parent and child regions can be defined in
     				  * any order, we keep hold of the parent_name during
 				  * initialisation, and the children get assigned to their
@@ -211,14 +263,26 @@ typedef struct regiondef {
     				  * region, if a value isn't defined in the current region
 				  * we traverse this series of pointers until it is.
 				  */
-    char	*longname;	 /* Official title of the region, this might be defined
+    const char	*longname;	 /* Official title of the region, this might be defined
     				  * to be the same as name*/
-    char	*msg;		 /* the description of the region */
+    const char	*msg;		 /* the description of the region */
     uint32	counter;	 /* A generic counter for holding temporary data. */
     sint8	fallback;	 /* whether, in the event of a region not existing,
     				  * this should be the one we fall back on as the default */
+    char	*jailmap;        /*where a player that is arrested in this region should be imprisoned.*/
+    sint16	jailx, jaily;	 /* The coodinates in jailmap to which the player should be sent. */
 } region;
 
+
+typedef struct shopitem {
+    const char *name;     /* name of the item in question, null if it is the default item */
+    const char *name_pl;  /* plural name */
+    int typenum;    /* itemtype number we need to match 0 if it is the default price*/
+    sint8 strength; /* the degree of specialisation the shop has in this item, 
+		     * as a percentage from -100 to 100 */
+    int index;      /* being the size of the shopitems array.*/
+} shopitems;
+
 /* In general, code should always use the macros 
  * above (or functions in map.c) to access many of the 
  * values in the map structure.  Failure to do this will
@@ -229,7 +293,6 @@ typedef struct regiondef {
  */
 typedef struct mapdef {
     struct mapdef *next;	/* Next map, linked list */
-    char path[HUGE_BUF];	/* Filename of the map */
     char *tmpname;	/* Name of temporary file */
     char *name;		/* Name of map as given by its creator */
     region *region;	/* What jurisdiction in the game world this map is ruled by 
@@ -243,6 +306,7 @@ typedef struct mapdef {
 				 * players entering/exiting map
 				 */
     uint32 unique:1;	/* if set, this is a per player unique map */
+    uint32 template:1;	/* if set, this is a template map */
     uint32 nosmooth:1;  /* if set the content of this map has smoothlevel=0 forced*/
     sint32 timeout;	/* swapout is set to this */
     sint32 swap_time;	/* When it reaches 0, the map will be swapped out */
@@ -267,9 +331,16 @@ typedef struct mapdef {
     sint8     winddir;   /* direction of wind */
     sint8     sky;	/* sky conditions */
     int     wpartx,wparty; /*Highly fasten conversion between worldmap and weathermap*/
+    shopitems   *shopitems; /* a semi-colon seperated list of item-types the map's shop will trade in */
+    char    *shoprace;  /* the preffered race of the local shopkeeper */
+    double  shopgreed;  /* how much our shopkeeper overcharges */
+    uint64  shopmin;    /* minimum price a shop will trade for */
+    uint64  shopmax;    /* maximum price a shop will offer */
     char    *msg;	/* Message map creator may have left */
+    char    *maplore;	/* Map lore information */
     char    *tile_path[4];  /* path to adjoining maps */
     struct mapdef *tile_map[4];	/* Next map, linked list */
+    char path[HUGE_BUF];	/* Filename of the map */
 } mapstruct;
 
 /* This is used by get_rangevector to determine where the other
