@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_material_h =
- *   "$Id: material.h,v 1.9 2003/02/14 05:50:33 garbled Exp $";
+ *   "$Id: material.h,v 1.10 2005/08/12 13:46:34 ryo_saeba Exp $";
  */
 
 /*
@@ -49,8 +49,8 @@
 								   materialname */
 
 typedef struct _materialtype {
-    char	*name;
-	char 	*description;
+    const char	*name;
+	const char 	*description;
     int		material;
     sint8	save[NROFATTACKS];
     sint8	mod[NROFATTACKS];
