@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_newclient_h =
- *   "$Id: newclient.h,v 1.11 2003/09/13 05:01:33 mwedel Exp $";
+ *   "$Id: newclient.h,v 1.14 2006/05/21 10:35:10 akirschbaum Exp $";
  */
 
 /*
@@ -48,11 +48,17 @@
 #ifndef NEWCLIENT_H
 #define NEWCLIENT_H
 
-/* Maximum size of any packet we expect.  Using this makes it so we don't need to
- * allocated and deallocated teh same buffer over and over again and the price
- * of using a bit of extra memory.  IT also makes the code simpler.
+/* Maximum size of any packet we expect. This number includes both the length
+ * bytes (2 bytes) at the start of each packet and the trailing '\0' (1 byte)
+ * at the end of each packet.
  */
-#define MAXSOCKBUF 10240
+#define MAXSOCKRECVBUF (2+65535+1)
+
+/* Maximum size of any packet we send. This number does not include the length
+ * bytes at the start of each packet. The value is chosen to not overflow the
+ * input buffer of old clients (2006-05-21).
+ */
+#define MAXSOCKSENDBUF 10239
 
 
 #define CS_QUERY_YESNO	0x1	/* Yes/no question */
@@ -99,6 +105,9 @@
 #define CS_STAT_FLAGS	25
 #define CS_STAT_WEIGHT_LIM	26
 #define CS_STAT_EXP64	28
+#define CS_STAT_SPELL_ATTUNE	29
+#define CS_STAT_SPELL_REPEL	30
+#define CS_STAT_SPELL_DENY	31
 
 /* Start & end of resistances, inclusive. */
 #define CS_STAT_RESIST_START	100
@@ -225,9 +234,18 @@ enum {a_none, a_readied, a_wielded, a_worn, a_active, a_applied};
 #define UPD_ANIMSPEED	0x40
 #define UPD_NROF	0x80
 
+/* The flag values sent by the updspell command */
+#define UPD_SP_MANA	0x01
+#define UPD_SP_GRACE	0x02
+#define UPD_SP_DAMAGE	0x04
+
 #define SOUND_NORMAL	0
 #define SOUND_SPELL	1
 
+#define FACE_IS_ANIM    1<<15
+#define ANIM_RANDOM	1<<13
+#define ANIM_SYNC	2<<13
+
 /* Contains the base information we use to make up a packet we want to send. */
 typedef struct SockList {
     int len;
