@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_object_h =
- *   "$Id: object.h,v 1.36 2005/08/10 13:38:31 ryo_saeba Exp $";
+ *   "$Id: object.h,v 1.48 2006/05/30 07:16:23 mwedel Exp $";
  */
 
 /*
@@ -35,23 +35,40 @@ typedef uint32 tag_t;
 
 /* See common/item.c */
 
-typedef struct Body_Locations {
-    char    *save_name;		/* Name used to load/save it to disk */
-    char    *use_name;		/* Name used when describing an item we can use */
-    char    *nonuse_name;	/* Name to describe objects we can't use */
-} Body_Locations;
+typedef struct body_locations_struct {
+    const char *save_name;	/* Name used to load/save it to disk */
+    const char *use_name;	/* Name used when describing an item we can use */
+    const char *nonuse_name;	/* Name to describe objects we can't use */
+} body_locations_struct;
 
-extern Body_Locations body_locations[NUM_BODY_LOCATIONS];
+extern body_locations_struct body_locations[NUM_BODY_LOCATIONS];
 
 typedef struct _event
 {
     int type;
-    char *hook;
-    char *plugin;
-    char *options;
+    const char *hook;
+    const char *plugin;
+    const char *options;
     struct _event *next;
 } event;
 
+/*
+ * Each object (this also means archetypes!) could have a few of these
+ * "dangling" from it; this could also end up containing 'parse errors'.
+ *
+ * key and value are shared-strings.
+ *
+ * Please use get_ob_key_value(), set_ob_key_value() from object.c rather than
+ * accessing the list directly.
+ * Exception is if you want to walk this list for some reason.
+ */
+typedef struct _key_value {
+    const char * key;
+    const char * value;
+    struct _key_value * next;
+} key_value;
+
+
 /* Definition for WILL_APPLY values.  Replaces having harcoded values
  * sprinkled in the code.  Note that some of these also replace fields
  * that were in the can_apply area.  What is the point of having both
@@ -69,6 +86,10 @@ typedef struct _event
  * Thus, values that need to be copied need to be located beyond that
  * point.
  *
+ * However, if you're keeping a pointer of some sort, you probably
+ * don't just want it copied, so you'll need to add to common/object.c,
+ * e.g. copy-object
+ *
  * I've tried to clean up this structure a bit (in terms of formatting)
  * by making it more consistent.  I've also tried to locate some of the fields
  * more logically together (put the item related ones together, the monster
@@ -108,15 +129,15 @@ typedef struct obj {
      * copy_object to do so.  Everything below here also gets cleared
      * by clear_object()
      */
-    char	*name;		/* The name of the object, obviously... */
-    char	*name_pl;	/* The plural name of the object */
-    char	*title;		/* Of foo, etc */
-    char	*race;		/* human, goblin, dragon, etc */
-    char	*slaying;	/* Which race to do double damage to */
+    const char	*name;		/* The name of the object, obviously... */
+    const char	*name_pl;	/* The plural name of the object */
+    const char	*title;		/* Of foo, etc */
+    const char	*race;		/* human, goblin, dragon, etc */
+    const char	*slaying;	/* Which race to do double damage to */
 				/* If this is an exit, this is the filename */
-    char	*skill;		/* Name of the skill this object uses/grants */
-    char	*msg;		/* If this is a book/sign/magic mouth/etc */
-    char	*lore;		/* Obscure information about this object, */
+    const char	*skill;		/* Name of the skill this object uses/grants */
+    const char	*msg;		/* If this is a book/sign/magic mouth/etc */
+    const char	*lore;		/* Obscure information about this object, */
 				/* To get put into books and the like. */
 
     sint16	x,y;		/* Position in the map for this object */
@@ -138,7 +159,7 @@ typedef struct obj {
     uint32	path_repelled;	/* Paths the object is repelled from */
     uint32	path_denied; 	/* Paths the object is denied access to */
     uint16	material;      	/* What materials this object consist of */
-    char 	*materialname;  /* specific material name */
+    const char 	*materialname;  /* specific material name */
     sint8	magic;		/* Any magical bonuses to this item */
     uint8	state;          /* How the object was last drawn (animation) */
     sint32	value;		/* How much money it is worth (or contains) */
@@ -160,7 +181,7 @@ typedef struct obj {
     sint8	glow_radius;	/* indicates the glow radius of the object */
     living	stats;		/* Str, Con, Dex, etc */
     sint64	perm_exp;	/* Permanent exp */
-    char	*current_weapon_script;  /* The script of the currently used weapon. Executed */
+    const char	*current_weapon_script;  /* The script of the currently used weapon. Executed */
 				/* each time the object attacks something */
     struct obj	*current_weapon;   /* Pointer to the weapon currently used */
     uint32	weapontype;	/* type of weapon */
@@ -185,7 +206,7 @@ typedef struct obj {
     /* changes made by kholland@sunlab.cit.cornell.edu */
     /* allows different movement patterns for attackers */
     sint32	move_status;	/* What stage in attack mode */
-    uint16	move_type;	/* What kind of attack movement */
+    uint16	attack_movement;/* What kind of attack movement */
     uint8	will_apply;     /* See crossfire.doc */
     struct obj	*spellitem;	/* Spell ability monster is choosing to use */
     double	expmul;		/* needed experience = (calc_exp*expmul) - means some */
@@ -215,16 +236,27 @@ typedef struct obj {
     uint8	last_anim;	/* last sequence used to draw face */
     sint32	elevation;	/* elevation of this terrain - not currently used */
     uint8	smoothlevel;    /* how to smooth this square around*/
+    uint8	map_layer;	/* What level to draw this on the map */
+
+    MoveType   move_type;	/* Type of movement this object uses */
+    MoveType   move_block;	/* What movement types this blocks */
+    MoveType   move_allow;	/* What movement types explicitly allowd */
+    MoveType   move_on;		/* Move types affected moving on to this space */
+    MoveType   move_off;	/* Move types affected moving off this space */
+    MoveType   move_slow;	/* Movement types this slows down */
+    float   move_slow_penalty;	/* How much this slows down the object */
+
     event   *events;
 
-    char  *custom_name; /* Custom name assigned by player */
+    const char  *custom_name; /* Custom name assigned by player */
+    key_value *key_values; /* Fields not explictly known by the loader. */
 
 } object;
 
 typedef struct oblnk { /* Used to link together several objects */
   object *ob;
   struct oblnk *next;
-  int id;
+  tag_t id;
 } objectlink;
 
 typedef struct oblinkpt { /* Used to link together several object links */
@@ -243,7 +275,7 @@ typedef struct oblinkpt { /* Used to link together several object links */
  */
 
 typedef struct archt {
-    char *name;		    /* More definite name, like "generate_kobold" */
+    const char *name;		    /* More definite name, like "generate_kobold" */
     struct archt *next;	    /* Next archetype in a linked list */
     struct archt *head;	    /* The main part of a linked object */
     struct archt *more;	    /* Next part of a linked object */
@@ -265,7 +297,7 @@ extern int nroffreeobjects;
 /* This returns TRUE if the object is somethign that
  * should be displayed in the look window
  */
-#define LOOK_OBJ(ob) (!ob->invisible && ob->type!=PLAYER)
+#define LOOK_OBJ(ob) (!ob->invisible && ob->type!=PLAYER && ob->type!=EVENT_CONNECTOR)
 
 /* Used by update_object to know if the object being passed is
  * being added or removed.
@@ -292,6 +324,9 @@ extern int nroffreeobjects;
  *     beneath the player, but not actually above it.  Note - the
  *     map and x,y coordinates for the object to be inserted must
  *     match the originator.
+ * INS_MAP_LOAD: disable lots of checkings done at insertion to
+ *     speed up map loading process, as we assume the ordering in
+ *     loaded map is correct.
  *
  * Note that INS_BELOW_ORIGINATOR, INS_ON_TOP, INS_ABOVE_FLOOR_ONLY
  * are mutually exclusive.  The behaviour for passing more than one
@@ -304,10 +339,12 @@ extern int nroffreeobjects;
 #define INS_NO_WALK_ON		0x0004
 #define INS_ON_TOP		0x0008
 #define INS_BELOW_ORIGINATOR	0x0010
+#define INS_MAP_LOAD		0x0020
 
 #define ARCH_SINGULARITY        "singularity"
 #define ARCH_SINGULARITY_LEN    11
 #define ARCH_DETECT_MAGIC       "detect_magic"
 #define ARCH_DEPLETION          "depletion"
+#define ARCH_SYMPTOM            "symptom"
 
 #endif
