@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_player_h =
- *   "$Id: player.h,v 1.38 2005/07/16 19:03:02 qal21 Exp $";
+ *   "$Id: player.h,v 1.47 2006/03/18 15:05:32 ryo_saeba Exp $";
  */
 
 /*
@@ -28,7 +28,7 @@
 
 #define NUM_OUTPUT_BUFS	5
 typedef struct {
-    char *buf;			/* Actual string pointer */
+    const char *buf;		/* Actual string pointer */
     uint32 first_update;	/* First time this message was stored  */
     uint16 count;		/* How many times we got this message */
 } Output_Buf;
@@ -74,7 +74,7 @@ typedef enum _petmode {
 typedef enum usekeytype {
     key_inventory=0,
     keyrings=1,
-    containers=2,
+    containers=2
 } usekeytype;
 
 /* This is used to control what to do when we need to unapply
@@ -83,20 +83,38 @@ typedef enum usekeytype {
 typedef enum unapplymode {
     unapply_nochoice=0,	    /* Will unapply objects when there no choice to unapply */
     unapply_never=1,	    /* will not unapply objects automatically */
-    unapply_always=2,	    /* Will unapply whatever is necessary - this goes beyond */
+    unapply_always=2	    /* Will unapply whatever is necessary - this goes beyond */
 			    /* no choice - if there are multiple ojbect of the same type */
 			    /* that need to be unapplied, there is no way for the player */
 			    /* to control which of these will be unapplied. */
 } unapplymode;
 
+/* not really the player, but tied pretty closely */  
+typedef struct party_struct {
+    char * partyleader;
+    char passwd[9];
+    struct party_struct *next;
+    char *partyname;
+
+#ifdef PARTY_KILL_LOG
+    struct party_kill {
+	char killer[MAX_NAME+1],dead[MAX_NAME+1];
+	sint64 exp;
+    } party_kills[PARTY_KILL_LOG];
+#endif
+    sint64 total_exp;
+    uint32  kills;
+} partylist;
+
 typedef struct pl {
     struct pl	*next;		    /* Pointer to next player, NULL if this is last */
-    NewSocket	socket;		    /* Socket information for this player */
+    socket_struct	socket;		    /* Socket information for this player */
     char	maplevel[MAX_BUF];  /* On which level is the player? */
     struct mapdef *loading;	    /* When entering a map in progress of loading, not really used */
     char	savebed_map[MAX_BUF];  /* map where player will respawn after death */
     sint16	bed_x, bed_y;	    /* x,y - coordinates of respawn (savebed) */
     rangetype	shoottype;	    /* Which range-attack is being used by player */
+    char	spellparam[MAX_BUF];	/* What param to add to spells */
     bowtype_t   bowtype;	    /* which firemode? */
     petmode_t   petmode;	    /* which petmode? */
     object	*ranges[range_size];/* object for each range.  Set up in fix player.  Note */
@@ -121,7 +139,7 @@ typedef struct pl {
     /* Try to put all the bitfields together - saves some small amount of memory */
     uint32	braced:1;	    /* Will not move if braced, only attack */
     uint32	tmp_invis:1;	    /* Will invis go away when we attack ? */
-    char	*invis_race;	    /* What race invisible to? */
+    const char	*invis_race;	    /* What race invisible to? */
     uint32	do_los:1;	    /* If true, need to call update_los() in draw(), and clear */
     uint32	fire_on:1;	    /* Player should fire object, not move */
     uint32	run_on:1;	    /* Player should keep moving in dir until run is off */
@@ -130,7 +148,7 @@ typedef struct pl {
     uint32	peaceful:1;	    /* If set, won't attack friendly creatures */
     uint32	hidden:1;	    /* If True, player (DM) is hidden from view */
     uint32	explore:1;	    /* if True, player is in explore mode */
-	uint32  no_shout:1;		/* if True, player is *not* able to use shout command */
+    uint32	no_shout:1;	    /* if True, player is *not* able to use shout command */
 
     object	*last_skill_ob[NUM_SKILLS];	/* the exp object */
     sint64	last_skill_exp[NUM_SKILLS];	/* shadow register. if != exp. obj update client */
@@ -138,8 +156,11 @@ typedef struct pl {
     float	weapon_sp;	    /* Penalties to speed when fighting w speed >ws/10*/
     float	last_weapon_sp;	    /* if diff than weapon_sp, update client */
     uint16	last_flags;	    /* fire/run on flags for last tick */
-    uint32	last_weight;	    /* Last weight as sent to client; (uint32)-1 means do not send weight */
-    uint32	last_weight_limit;  /* Last weight limit transmitted to client */
+    sint32	last_weight;	    /* Last weight as sent to client; -1 means do not send weight */
+    sint32	last_weight_limit;  /* Last weight limit transmitted to client */
+    uint32	last_path_attuned;  /* Last spell attunment sent to client */
+    uint32	last_path_repelled; /* Last spell repelled sent to client */
+    uint32	last_path_denied;   /* Last spell denied sent to client */
     living	orig_stats;	    /* Permanent real stats of player */
     living	last_stats;	    /* Last stats as sent to client */
     float	last_speed;	    /* Last speed as sent to client */
@@ -173,8 +194,8 @@ typedef struct pl {
 #ifdef AUTOSAVE
     uint32	last_save_tick;
 #endif
-    sint16    party_number;	    /* Party number this player is part of */
-    sint16    party_number_to_join; /* used when player wants to join a party */
+    partylist    *party;	    /* Party this player is part of */
+    partylist    *party_to_join; /* used when player wants to join a party */
 				    /* but we will have to get password first */
 				    /* so we have to remember which party to */
 				    /* join */
@@ -185,26 +206,8 @@ typedef struct pl {
     uint16	outputs_count;	    /* Print if this count is exceeded */
     object	*mark;		    /* marked object */
     uint32	mark_count;	    /* count of mark object */
+    object	*transport;	    /* transport the player is in */
     /* Special DM fields */
     tag_t*  stack_items;    /* Item stack for patch/dump/... commands */
     int     stack_position; /* Current stack position, 0 for no item */
 } player;
-
-
-/* not really the player, but tied pretty closely */  
-typedef struct party_struct {
-    sint16 partyid;
-    char * partyleader;
-    char passwd[9];
-    struct party_struct *next;
-    char *partyname;
-
-#ifdef PARTY_KILL_LOG
-    struct party_kill {
-	char killer[MAX_NAME+1],dead[MAX_NAME+1];
-	sint64 exp;
-    } party_kills[PARTY_KILL_LOG];
-#endif
-    sint64 total_exp;
-    uint32  kills;
-} partylist;
