@@ -1,7 +1,8 @@
 /* 'recipe' and 'recipelist' are used by the alchemy code */
 typedef struct recipestruct {
-    char *title;       /* distinguishing name of product */
-    char *arch_name;   /* the archetype of the final product made */
+    const char *title; /* distinguishing name of product */
+    size_t arch_names; /* the size of the arch_name[] array */
+    char **arch_name;  /* the possible archetypes of the final product made */
     int chance;        /* chance that recipe for this item will appear
 		        * in an alchemical grimore */
     int diff;	       /* alchemical dfficulty level */
@@ -10,11 +11,11 @@ typedef struct recipestruct {
     int transmute;     /* if defined, one of the formula ingredients is
 			* used as the basis for the product object */
     int yield;         /* The maximum number of items produced by the recipe */
-    linked_char *ingred; /* comma delimited list of ingredients */
+    linked_char *ingred; /* list of ingredients */
     struct recipestruct *next;
-    char *keycode;     /* keycode needed to use the recipe */
-    char *skill;       /* skill name used to make this recipe */
-    char *cauldron;    /* the arch of the cauldron/workbench used to house the
+    const char *keycode;     /* keycode needed to use the recipe */
+    const char *skill;       /* skill name used to make this recipe */
+    const char *cauldron;    /* the arch of the cauldron/workbench used to house the
 			* formulae. */
 } recipe;
 
