@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_skills_h =
- *   "$Id: skills.h,v 1.10 2003/09/13 05:01:34 mwedel Exp $";
+ *   "$Id: skills.h,v 1.11 2005/08/12 13:46:34 ryo_saeba Exp $";
  */
 
 /*
@@ -114,7 +114,7 @@
 
 
 
-extern char *skill_names[NUM_SKILLS];
+extern const char *skill_names[NUM_SKILLS];
 
 #ifdef WANT_UNARMED_SKILLS
 /* Table of unarmed attack skills.  Terminated by -1.  This
