@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_spells_h =
- *   "$Id: spells.h,v 1.20 2005/07/10 21:12:14 ryo_saeba Exp $";
+ *   "$Id: spells.h,v 1.21 2006/02/10 23:59:26 akirschbaum Exp $";
  */
 
 /*
@@ -55,7 +55,7 @@
 				((op->path_repelled & spell->path_attuned) ? 1.25 : 1))
 
 #define NRSPELLPATHS	20
-extern char *spellpathnames[NRSPELLPATHS];
+extern const char* const spellpathnames[NRSPELLPATHS];
 
 /* The only place this is really used is to allocate an array
  * when printing out the spells the player knows. 
