@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_door_c =
- *   "$Id: door.c,v 1.13 2003/09/25 03:39:02 temitchell Exp $";
+ *   "$Id: door.c,v 1.14 2006/02/10 23:59:27 akirschbaum Exp $";
  */
 
 /*
@@ -44,7 +44,7 @@ int surround_check2(char **layout,int i,int j,int Xsize, int Ysize){
   return surround_index;
 }
 
-void put_doors(mapstruct *the_map, char **maze, char *doorstyle, RMParms *RP) {
+void put_doors(mapstruct *the_map, char **maze, const char *doorstyle, RMParms *RP) {
 	int i,j;
 	mapstruct *vdoors;
 	mapstruct *hdoors;
