@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_monster_c =
- *   "$Id: monster.c,v 1.14 2005/07/08 23:48:59 mwedel Exp $";
+ *   "$Id: monster.c,v 1.15 2005/11/16 08:16:08 mwedel Exp $";
  */
 
 /*
@@ -87,7 +87,7 @@ void place_monsters(mapstruct *map, char *monsterstyle, int difficulty,RMParms *
     if(this_monster == NULL) return;  /* no monster?? */
     x = RANDOM() % RP->Xsize;
     y = RANDOM() % RP->Ysize;
-    freeindex = find_first_free_spot(this_monster->arch,map,x,y);
+    freeindex = find_first_free_spot(this_monster,map,x,y);
     if(freeindex!=-1) {
 	 object *new_monster = arch_to_object(this_monster->arch);
 	 x += freearr_x[freeindex];
