@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_random_map_c =
- *   "$Id: random_map.c,v 1.38 2005/04/14 06:57:01 mwedel Exp $";
+ *   "$Id: random_map.c,v 1.42 2006/06/03 21:56:27 ryo_saeba Exp $";
  */
 
 /*
@@ -48,7 +48,7 @@ void dump_layout(char **layout,RMParms *RP) {
     printf("\n");
 }
 EXTERN FILE *logfile;
-mapstruct *generate_random_map(char *OutFileName, RMParms *RP) {
+mapstruct *generate_random_map(const char *OutFileName, RMParms *RP) {
     char **layout, buf[HUGE_BUF];
     mapstruct *theMap;
     int i;
@@ -102,22 +102,39 @@ mapstruct *generate_random_map(char *OutFileName, RMParms *RP) {
     /* set the name of the map. */
     strcpy(theMap->path,OutFileName);
 
-    make_map_walls(theMap,layout,RP->wallstyle,RP);
+    /* set region */
+    theMap->region=RP->region;
+    
+    /* create walls unless the wallstyle is "none" */
+    if (strcmp (RP->wallstyle, "none")) {
+        make_map_walls(theMap,layout,RP->wallstyle,RP);
+        
+        /* place doors unless doorstyle or wallstyle is "none"*/
+        if (strcmp (RP->doorstyle, "none"))
+        put_doors(theMap,layout,RP->doorstyle,RP);
+        
+    }
 
-    put_doors(theMap,layout,RP->doorstyle,RP);
-
-    place_exits(theMap,layout,RP->exitstyle,RP->orientation,RP);
+    /* create exits unless the exitstyle is "none" */
+    if (strcmp (RP->exitstyle, "none"))
+        place_exits(theMap,layout,RP->exitstyle,RP->orientation,RP);
 
     place_specials_in_map(theMap,layout,RP);
 
-    place_monsters(theMap,RP->monsterstyle,RP->difficulty,RP);
+    /* create monsters unless the monsterstyle is "none" */
+    if (strcmp (RP->monsterstyle, "none"))
+        place_monsters(theMap,RP->monsterstyle,RP->difficulty,RP);
 
     /* treasures needs to have a proper difficulty set for the map. */
     theMap->difficulty=calculate_difficulty(theMap);
 
-    place_treasure(theMap,layout,RP->treasurestyle,RP->treasureoptions,RP);
+    /* create treasure unless the treasurestyle is "none" */
+    if (strcmp (RP->treasurestyle, "none"))
+        place_treasure(theMap,layout,RP->treasurestyle,RP->treasureoptions,RP);
 
-    put_decor(theMap,layout,RP->decorstyle,RP->decoroptions,RP);
+    /* create decor unless the decorstyle is "none" */
+    if (strcmp (RP->decorstyle, "none"))
+        put_decor(theMap,layout,RP->decorstyle,RP->decoroptions,RP);
 
     /* generate treasures, etc. */
     fix_auto_apply(theMap);
@@ -625,6 +642,11 @@ void write_map_parameters_to_string(char *buf,RMParms *RP) {
     strcat(buf,small_buf);
   }
 
+  if(RP->dungeon_name[0]) {
+      sprintf(small_buf,"dungeon_name %s\n",RP->dungeon_name);
+      strcat(buf,small_buf);
+  }
+
   if(RP->decoroptions) {
     sprintf(small_buf,"decoroptions %d\n",RP->decoroptions);
     strcat(buf,small_buf);
