@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_special_c =
- *   "$Id: special.c,v 1.21 2005/04/14 06:57:02 mwedel Exp $";
+ *   "$Id: special.c,v 1.24 2006/04/06 21:18:35 tchize Exp $";
  */
 
 /*
@@ -140,13 +140,13 @@ int find_spot_for_submap(mapstruct *map,char **layout,int *ix, int *iy,int xsize
 void place_fountain_with_specials(mapstruct *map) {
   int ix,iy,i=-1,tries=0;
   mapstruct *fountain_style=find_style("/styles/misc","fountains",-1);
-  object *fountain=get_archetype("fountain");
+  object *fountain=create_archetype("fountain");
   object *potion=get_object();
   copy_object(pick_random_object(fountain_style),potion);
   while(i<0 && tries<10) {
     ix = RANDOM() % (MAP_WIDTH(map) -2) +1;
     iy = RANDOM() % (MAP_HEIGHT(map) -2) +1;
-    i = find_first_free_spot(fountain->arch,map,ix,iy);
+    i = find_first_free_spot(fountain,map,ix,iy);
     tries++;
   };
   if(i==-1) {  /* can't place fountain */
@@ -185,7 +185,7 @@ void place_special_exit(mapstruct * map, int hole_type,RMParms *RP) {
     while(i<0) {
 	ix = RANDOM() % (MAP_WIDTH(map) -2) +1;
 	iy = RANDOM() % (MAP_HEIGHT(map) -2) +1;
-	i = find_first_free_spot(the_exit->arch,map,ix,iy);
+	i = find_first_free_spot(the_exit,map,ix,iy);
     }
   
     ix += freearr_x[i];
@@ -225,6 +225,11 @@ void place_special_exit(mapstruct * map, int hole_type,RMParms *RP) {
 		mon = "none";
 		break;
 	    }
+
+	default:  /* undefined */
+	    LOG(llevError, "place_special_exit: undefined hole type %d\n", hole_type);
+	    return;
+	    break;
     }
 
     /* Need to be at least this size, otherwise the load
