@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_standalone_c =
- *   "$Id: standalone.c,v 1.14 2005/07/17 11:00:05 ryo_saeba Exp $";
+ *   "$Id: standalone.c,v 1.18 2006/05/05 09:41:15 ryo_saeba Exp $";
  */
 
 /*
@@ -73,7 +73,7 @@ int main(int argc, char *argv[]) {
   exit(0);
 }
 
-void set_map_timeout() {}   /* doesn't need to do anything */
+void set_map_timeout(void) {}   /* doesn't need to do anything */
 
 #include <global.h>
 
@@ -171,11 +171,11 @@ void fix_auto_apply(mapstruct *m) {
  * Ryo 2005-07-15
  **/
  
-void new_draw_info(int a, int b, object *ob, const char *txt)
+void new_draw_info(int a, int b, const object *ob, const char *txt)
     {
     fprintf(logfile, "%s\n", txt);
     }
-void new_info_map(int color, mapstruct *map, const char *str)
+void new_info_map(int color, const mapstruct *map, const char *str)
     {
     fprintf(logfile, "new_info_map: %s\n", str);
     }
@@ -204,7 +204,7 @@ void emergency_save( int x )
     {
     }
  
-void clean_tmp_files( )
+void clean_tmp_files( void )
     {
     }
  
@@ -237,6 +237,10 @@ void esrv_del_item(player *pl, int tag)
     {
     }
  
+void esrv_update_spells(player *pl)
+    {
+    }
+
 void monster_check_apply( object* ob, object* obt )
     {
     }
