@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_style_c =
- *   "$Id: style.c,v 1.21 2005/04/14 06:57:02 mwedel Exp $";
+ *   "$Id: style.c,v 1.26 2006/02/19 17:27:32 akirschbaum Exp $";
  */
 
 /*
@@ -37,6 +37,14 @@
 #endif /* win32 */
 
 
+static int pointer_strcmp(const void *p1, const void *p2)
+{
+    const char *s1 = *(const char **)p1;
+    const char *s2 = *(const char **)p2;
+
+    return(strcmp(s1, s2));
+}
+
 /* This is our own version of scandir/select_regular_files/sort.
  * To support having subdirectories in styles, we need to know
  * if in fact the directory we read is a subdirectory.  However,
@@ -82,7 +90,7 @@ int load_dir (const char *dir, char ***namelist, int skip_dirs)
     }
     (void) closedir (dp);
 
-    qsort(rn, entries, sizeof(char*), (int(*)())strcmp);
+    qsort(rn, entries, sizeof(char*), pointer_strcmp);
 
     *namelist = rn;
     return entries;
@@ -131,7 +139,7 @@ mapstruct *load_style_map(char *style_name)
     return style_map;
 }
 
-mapstruct *find_style(char *dirname,char *stylename,int difficulty) {
+mapstruct *find_style(const char *dirname,const char *stylename,int difficulty) {
     char style_file_path[256];
     char style_file_full_path[256];
     mapstruct *style_map = NULL;
@@ -146,9 +154,8 @@ mapstruct *find_style(char *dirname,char *stylename,int difficulty) {
 
     /* is what we were given a directory, or a file? */
     sprintf(style_file_full_path,"%s/maps%s",settings.datadir,style_file_path);
-    stat(style_file_full_path,&file_stat);
-
-    if(! (S_ISDIR(file_stat.st_mode))) {
+    if (stat(style_file_full_path, &file_stat) == 0
+    && !S_ISDIR(file_stat.st_mode)) {
 	style_map=load_style_map(style_file_path);
     }
     if(style_map == NULL)  /* maybe we were given a directory! */
@@ -250,7 +257,7 @@ object *pick_random_object(mapstruct *style) {
 }
 				
 
-void free_style_maps()			 
+void free_style_maps(void)
 {
     mapstruct *next;
     int  style_maps=0;
