@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_treasure_c =
- *   "$Id: treasure.c,v 1.20 2004/04/14 07:24:31 mwedel Exp $";
+ *   "$Id: treasure.c,v 1.28 2006/06/03 21:56:28 ryo_saeba Exp $";
  */
 
 /*
@@ -49,6 +49,7 @@
 #define NO_PASS_DOORS 0
 #define PASS_DOORS 1
 
+static object ** surround_by_doors(mapstruct *map,char **layout,int x,int y,int opts);
 
 /* returns true if square x,y has P_NO_PASS set, which is true for walls
  * and doors but not monsters.
@@ -56,11 +57,12 @@
  */
 
 int wall_blocked(mapstruct *m, int x, int y) {
-  int r;
-  if(OUT_OF_REAL_MAP(m,x,y))
-    return 1;
-  r = GET_MAP_FLAGS(m,x,y) & (P_NO_PASS );
-  return r;
+    int r;
+
+    if(OUT_OF_REAL_MAP(m,x,y))
+	return 1;
+    r = GET_MAP_MOVE_BLOCK(m,x,y) & ~MOVE_BLOCK_DEFAULT;
+    return r;
 }
 
 /* place treasures in the map, given the 
@@ -174,37 +176,39 @@ object * place_chest(int treasureoptions,int x, int y,mapstruct *map, mapstruct
   object *the_chest;
   int i,xl,yl;
 
+  the_chest = create_archetype("chest");  /* was "chest_2" */
+
   /* first, find a place to put the chest. */
-  i = find_first_free_spot(find_archetype("chest"),map,x,y);
+  i = find_first_free_spot(the_chest,map,x,y);
+  if (i == -1) {
+    free_object(the_chest);
+    return NULL;
+  }
   xl = x + freearr_x[i]; yl = y +  freearr_y[i];
 
   /* if the placement is blocked, return a fail. */
-  if(wall_blocked(map,xl,yl)) return 0;
+  if(wall_blocked(map,xl,yl)) {
+      free_object(the_chest);
+      return 0;
+  }
   
   
-  /* use the nicer container-chests for multiple treasures...  Allows locking. */
-  if(n_treasures > 1) 
-    the_chest = get_archetype("chest");  /* was "chest_2" */
-  else
-    the_chest = get_archetype("chest");
-
-
   /* put the treasures in the chest. */
   /*  if(style_map) { */
-  if(0) {  /* don't use treasure style maps for now!  */
-    int ti;
-    /* if treasurestyle lists a treasure list, use it. */
-    treasurelist *tlist=find_treasurelist(RP->treasurestyle);
-    if(tlist!=NULL)
-      for(ti=0;ti<n_treasures;ti++) {  /* use the treasure list */
-        object *new_treasure=pick_random_object(style_map);
-        insert_ob_in_ob(arch_to_object(new_treasure->arch),the_chest);
-      }
-    else { /* use the style map */
-      the_chest->randomitems=tlist;
-      the_chest->stats.hp = n_treasures;
+#if 0 /* don't use treasure style maps for now!  */
+  int ti;
+  /* if treasurestyle lists a treasure list, use it. */
+  treasurelist *tlist=find_treasurelist(RP->treasurestyle);
+  if(tlist!=NULL)
+    for(ti=0;ti<n_treasures;ti++) {  /* use the treasure list */
+      object *new_treasure=pick_random_object(style_map);
+      insert_ob_in_ob(arch_to_object(new_treasure->arch),the_chest);
     }
+  else { /* use the style map */
+    the_chest->randomitems=tlist;
+    the_chest->stats.hp = n_treasures;
   }
+#endif
   else { /* neither style_map no treasure list given */
     treasurelist *tlist=find_treasurelist("chest");
     the_chest->randomitems=tlist;
@@ -257,7 +261,7 @@ object *find_closest_monster(mapstruct *map,int x,int y,RMParms *RP) {
     lx=x+freearr_x[i];
     ly=y+freearr_y[i];
     /* boundscheck */
-    if(lx > 0 && ly > 0 && lx < RP->Xsize && ly < RP->Ysize)
+    if(lx >= 0 && ly >= 0 && lx < RP->Xsize && ly < RP->Ysize)
       /* don't bother searching this square unless the map says life exists.*/
       if(GET_MAP_FLAGS(map,lx,ly) & P_IS_ALIVE) {
         object *the_monster=get_map_ob(map,lx,ly);
@@ -287,10 +291,14 @@ int keyplace(mapstruct *map,int x,int y,char *keycode,int door_flag,int n_keys,R
   int kx,ky;
   object *the_keymaster; /* the monster that gets the key. */
   object *the_key;
+  char keybuf[256];
 
   /* get a key and set its keycode */
-  the_key = get_archetype("key2");
+  the_key = create_archetype("key2");
   the_key->slaying = add_string(keycode); 
+  free_string(the_key->name);
+  snprintf( keybuf,256, "key from level %d of %s", RP->dungeon_level, RP->dungeon_name[0] != '\0' ? RP->dungeon_name : "a random map" );
+  the_key->name = add_string(keybuf);
 
 
   if(door_flag==PASS_DOORS) {
@@ -310,7 +318,7 @@ int keyplace(mapstruct *map,int x,int y,char *keycode,int door_flag,int n_keys,R
       for(tries = 0; tries < 15 && freeindex == -1; tries++) {
 	kx = (RANDOM()%(RP->Xsize-2))+1;
 	ky = (RANDOM()%(RP->Ysize-2))+1;
-	freeindex = find_first_free_spot(the_key->arch,map,kx,ky);
+	freeindex = find_first_free_spot(the_key,map,kx,ky);
       }
       if(freeindex != -1) {
 	kx += freearr_x[freeindex];
@@ -504,7 +512,7 @@ void find_enclosed_spot(mapstruct *map, int *cx, int *cy,RMParms *RP) {
   int i;
   x = *cx;y=*cy;
 
-  for(i=0;i<SIZEOFFREE;i++) {
+  for(i=0;i<=SIZEOFFREE1;i++) {
     int lx,ly,sindex;
     lx = x +freearr_x[i];
     ly = y +freearr_y[i];
@@ -518,7 +526,7 @@ void find_enclosed_spot(mapstruct *map, int *cx, int *cy,RMParms *RP) {
 
   /* OK, if we got here, we're obviously someplace where there's no enclosed
      spots--try to find someplace which is 2x enclosed.  */
-  for(i=0;i<SIZEOFFREE;i++) {
+  for(i=0;i<=SIZEOFFREE1;i++) {
     int lx,ly,sindex;
     lx = x +freearr_x[i];
     ly = y +freearr_y[i];
@@ -531,7 +539,7 @@ void find_enclosed_spot(mapstruct *map, int *cx, int *cy,RMParms *RP) {
   }
 
   /* settle for one surround point */
-  for(i=0;i<SIZEOFFREE;i++) {
+  for(i=0;i<=SIZEOFFREE1;i++) {
     int lx,ly,sindex;
     lx = x +freearr_x[i];
     ly = y +freearr_y[i];
@@ -543,10 +551,11 @@ void find_enclosed_spot(mapstruct *map, int *cx, int *cy,RMParms *RP) {
     }
   }
   /* give up and return the closest free spot. */
-  i = find_first_free_spot(find_archetype("chest"),map,x,y);
-  if(i!=-1&&i<SIZEOFFREE) {
+  i = find_first_free_spot(&find_archetype("chest")->clone,map,x,y);
+  if(i!=-1&&i<=SIZEOFFREE1) {
     *cx = x +freearr_x[i];
     *cy = y +freearr_y[i];
+    return;
   }
   /* indicate failure */
   *cx=*cy=-1;
@@ -571,7 +580,7 @@ void remove_monsters(int x,int y,mapstruct *map) {
 	 a chest.  It only goes as far as the 8 squares surrounding, and
 	 it'll remove any monsters it finds.*/
 
-object ** surround_by_doors(mapstruct *map,char **layout,int x,int y,int opts) {
+static object ** surround_by_doors(mapstruct *map,char **layout,int x,int y,int opts) {
   int i;
   char *doors[2];
   object **doorlist;
@@ -594,7 +603,7 @@ object ** surround_by_doors(mapstruct *map,char **layout,int x,int y,int opts) {
     
     if(!wall_blocked(map,x1,y1)
        || layout[x1][y1]=='>') {/* place a door */
-      object * new_door=get_archetype( (freearr_x[i]==0)?doors[1]:doors[0]);
+      object * new_door=create_archetype( (freearr_x[i]==0)?doors[1]:doors[0]);
       new_door->x = x + freearr_x[i];
       new_door->y = y + freearr_y[i];
       remove_monsters(new_door->x,new_door->y,map);
@@ -608,7 +617,7 @@ object ** surround_by_doors(mapstruct *map,char **layout,int x,int y,int opts) {
 
 
 /* returns the first door in this square, or NULL if there isn't a door. */
-object *door_in_square(mapstruct *map,int x,int y) {
+static object *door_in_square(mapstruct *map,int x,int y) {
   object *tmp;
   for(tmp=get_map_ob(map,x,y);tmp!=NULL;tmp=tmp->above) 
     if(tmp->type == DOOR || tmp->type== LOCKED_DOOR) return tmp;
@@ -635,7 +644,7 @@ void find_doors_in_room_recursive(char **layout,mapstruct *map,int x,int y,objec
       doorlist[*ndoors]=door;
       if(*ndoors>254) /* eek!  out of memory */
         {
-          printf("find_doors_in_room_recursive:Too many doors for memory allocated!\n");
+          LOG(llevError, "find_doors_in_room_recursive:Too many doors for memory allocated!\n");
           return;
         }
       *ndoors=*ndoors+1;
@@ -692,7 +701,7 @@ void lock_and_hide_doors(object **doorlist,mapstruct *map,int opts,RMParms *RP)
   
   if(opts & DOORED) {
     for(i=0,door=doorlist[0];doorlist[i]!=NULL;i++) {
-      object *new_door=get_archetype("locked_door1");
+      object *new_door=create_archetype("locked_door1");
       char keybuf[256];
       door=doorlist[i];
       new_door->face = door->face;
@@ -702,7 +711,7 @@ void lock_and_hide_doors(object **doorlist,mapstruct *map,int opts,RMParms *RP)
       free_object(door);
       doorlist[i]=new_door;
       insert_ob_in_map(new_door,map,NULL,0);
-      sprintf(keybuf,"%d",(int)RANDOM());
+      snprintf(keybuf,256,"%d",(int)RANDOM());
       new_door->slaying = add_string(keybuf);
       keyplace(map,new_door->x,new_door->y,keybuf,NO_PASS_DOORS,2,RP);
     }
