@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_attack_c =
- *   "$Id: attack.c,v 1.108 2005/07/26 00:23:00 akirschbaum Exp $";
+ *   "$Id: attack.c,v 1.125 2006/05/05 09:26:34 ryo_saeba Exp $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -24,6 +24,7 @@
 
     The authors can be reached via e-mail to crossfire-devel@real-time.com
 */
+#include <assert.h>
 #include <global.h>
 #include <living.h>
 #include <material.h>
@@ -42,10 +43,18 @@ typedef struct att_msg_str {
 
 /*#define ATTACK_DEBUG*/
 
-/* cancels object *op.  Cancellation basically means an object loses
+static void slow_player(object *op, object *hitter, int dam);
+static void deathstrike_player(object *op, object *hitter, int *dam);
+static int adj_attackroll(object *hitter, object *target);
+static int is_aimed_missile(object *op);
+static int did_make_save_item(object *op, int type, object *originator);
+static void poison_player(object *op, object *hitter, int dam);
+
+/**
+ * Cancels object *op.  Cancellation basically means an object loses
  * its magical benefits.
  */
-void cancellation(object *op)
+static void cancellation(object *op)
 {
     object *tmp;
 
@@ -74,12 +83,13 @@ void cancellation(object *op)
 
 
 
-/* did_make_save_item just checks to make sure the item actually
+/**
+ * did_make_save_item just checks to make sure the item actually
  * made its saving throw based on the tables.  It does not take
  * any further action (like destroying the item).
  */
 
-int did_make_save_item(object *op, int type, object *originator) {
+static int did_make_save_item(object *op, int type, object *originator) {
     int i, roll, saves=0, attacks=0, number;
     materialtype_t *mt;
 
@@ -135,7 +145,8 @@ int did_make_save_item(object *op, int type, object *originator) {
     return TRUE;
 }
 
-/* This function calls did_make_save_item.  It then performs the
+/**
+ * This function calls did_make_save_item.  It then performs the
  * appropriate actions to the item (such as burning the item up,
  * calling cancellation, etc.)
  */
@@ -161,13 +172,13 @@ void save_throw_object (object *op, int type, object *originator)
 	 * So forget lighting magical swords on fire with this!) -b.t.
 	 */ 
 	if(type&(AT_FIRE|AT_ELECTRICITY)
-           &&op->other_arch&&op->glow_radius) { 
-		char *arch=op->other_arch->name;
+           &&op->other_arch&&QUERY_FLAG(op, FLAG_IS_LIGHTABLE)) { 
+		const char *arch=op->other_arch->name;
 
 		op = decrease_ob_nr (op, 1);
                 if (op)
                     fix_stopped_item (op, m, originator);
-		if((op = get_archetype(arch))!=NULL) {
+		if((op = create_archetype(arch))!=NULL) {
                    if(env) {  
 			op->x=env->x,op->y=env->y;
 			insert_ob_in_ob(op,env);
@@ -203,7 +214,7 @@ void save_throw_object (object *op, int type, object *originator)
 	}
 	if(type&(AT_FIRE|AT_ELECTRICITY)) {
 	      if(env) {
-                op=get_archetype("burnout");
+                op=create_archetype("burnout");
 		op->x=env->x,op->y=env->y;
                 insert_ob_in_ob(op,env);
 	      } else { 
@@ -225,7 +236,12 @@ void save_throw_object (object *op, int type, object *originator)
         if ((tmp = present_arch(at,op->map,op->x,op->y)) == NULL) {
 	    tmp = arch_to_object(at);
 	    tmp->x=op->x,tmp->y=op->y;
-	    SET_SLOW_PENALTY(tmp, 0);
+	    /* This was in the old (pre new movement code) -
+	     * icecubes have slow_move set to 1 - don't want
+	     * that for ones we create.
+	     */
+	    tmp->move_slow_penalty=0;
+	    tmp->move_slow=0;
 	    insert_ob_in_map(tmp,op->map,originator,0);
         }
         if ( ! QUERY_FLAG (op, FLAG_REMOVED))
@@ -235,13 +251,15 @@ void save_throw_object (object *op, int type, object *originator)
     }
 }
 
-/* Object op is hitting the map.
+/**
+ * Object op is hitting the map.
  * op is going in direction 'dir'
  * type is the attacktype of the object.
+ * full_hit is set if monster area does not matter.
  * returns 1 if it hits something, 0 otherwise.
  */
 
-int hit_map(object *op,int dir,int type) {
+int hit_map(object *op, int dir, int type, int full_hit) {
     object *tmp, *next;
     mapstruct *map;
     sint16 x, y;
@@ -332,8 +350,18 @@ int hit_map(object *op,int dir,int type) {
 	if (tmp->map != map || tmp->x != x || tmp->y != y)
 	    continue;
 
+	/* Need to hit everyone in the transport with this spell */
+	if (tmp->type == TRANSPORT) {
+	    object *pl;
+
+	    for (pl=tmp->inv; pl; pl=pl->below) {
+		if (pl->type == PLAYER)
+		    hit_player(pl,op->stats.dam,op,type,full_hit);
+	    }
+	}
+
 	if (QUERY_FLAG (tmp, FLAG_ALIVE)) {
-	    hit_player(tmp,op->stats.dam,op,type);
+	    hit_player(tmp,op->stats.dam,op,type,full_hit);
 	    retflag |=1;
 	    if (was_destroyed (op, op_tag))
 		break;
@@ -341,10 +369,12 @@ int hit_map(object *op,int dir,int type) {
 	/* Here we are potentially destroying an object.  If the object has
 	 * NO_PASS set, it is also immune - you can't destroy walls.  Note 
 	 * that weak walls have is_alive set, which prevent objects from
-	 * passing over/through them.
+	 * passing over/through them.  We don't care what type of movement
+	 * the wall blocks - if it blocks any type of movement, can't be
+	 * destroyed right now.
 	 */
 	else if ((tmp->material || tmp->materialname) && op->stats.dam > 0 &&
-		   !QUERY_FLAG(tmp, FLAG_NO_PASS)) {
+		   !tmp->move_block) {
 	    save_throw_object(tmp,type,op);
 	    if (was_destroyed (op, op_tag))
 		break;
@@ -353,7 +383,7 @@ int hit_map(object *op,int dir,int type) {
     return 0;
 }
 
-void attack_message(int dam, int type, object *op, object *hitter) {
+static void attack_message(int dam, int type, object *op, object *hitter) {
   char buf[MAX_BUF], buf1[MAX_BUF], buf2[MAX_BUF];
   int i, found=0;
   mapstruct *map;
@@ -382,7 +412,8 @@ void attack_message(int dam, int type, object *op, object *hitter) {
 	(type & AT_POISON && IS_LIVE(op))) && !found) {
         for (i=0; i < MAXATTACKMESS && attack_mess[ATM_SUFFER][i].level != -1;
 	     i++)
-	    if (dam < attack_mess[ATM_SUFFER][i].level) {
+	    if (dam < attack_mess[ATM_SUFFER][i].level
+	    || attack_mess[ATM_SUFFER][i+1].level == -1) {
 	        sprintf(buf1, "%s %s%s", attack_mess[ATM_SUFFER][i].buf1,
 			op->name, attack_mess[ATM_SUFFER][i].buf2);
 		sprintf(buf2, "%s", attack_mess[ATM_SUFFER][i].buf3);
@@ -392,7 +423,8 @@ void attack_message(int dam, int type, object *op, object *hitter) {
     } else if (op->type == DOOR && !found) {
         for (i=0; i < MAXATTACKMESS && attack_mess[ATM_DOOR][i].level != -1;
 	     i++)
-	    if (dam < attack_mess[ATM_DOOR][i].level) {
+	    if (dam < attack_mess[ATM_DOOR][i].level
+	    || attack_mess[ATM_DOOR][i+1].level == -1) {
 	        sprintf(buf1, "%s %s%s", attack_mess[ATM_DOOR][i].buf1,
 			op->name, attack_mess[ATM_DOOR][i].buf2);
 		sprintf(buf2, "%s", attack_mess[ATM_DOOR][i].buf3);
@@ -403,7 +435,8 @@ void attack_message(int dam, int type, object *op, object *hitter) {
         if (USING_SKILL(hitter, SK_KARATE)) {
           for (i=0; i < MAXATTACKMESS && attack_mess[ATM_KARATE][i].level != -1;
 	       i++)
-	      if (dam < attack_mess[ATM_KARATE][i].level) {
+	      if (dam < attack_mess[ATM_KARATE][i].level
+	      || attack_mess[ATM_KARATE][i+1].level == -1) {
 	          sprintf(buf1, "%s %s%s", attack_mess[ATM_KARATE][i].buf1,
 			  op->name, attack_mess[ATM_KARATE][i].buf2);
 		  sprintf(buf2, "%s", attack_mess[ATM_KARATE][i].buf3);
@@ -413,7 +446,8 @@ void attack_message(int dam, int type, object *op, object *hitter) {
 	} else if (USING_SKILL(hitter, SK_CLAWING)) {
           for (i=0; i < MAXATTACKMESS && attack_mess[ATM_CLAW][i].level != -1;
 	       i++)
-	      if (dam < attack_mess[ATM_CLAW][i].level) {
+	      if (dam < attack_mess[ATM_CLAW][i].level
+	      || attack_mess[ATM_CLAW][i+1].level == -1) {
 	          sprintf(buf1, "%s %s%s", attack_mess[ATM_CLAW][i].buf1,
 			  op->name, attack_mess[ATM_CLAW][i].buf2);
 		  sprintf(buf2, "%s", attack_mess[ATM_CLAW][i].buf3);
@@ -423,7 +457,8 @@ void attack_message(int dam, int type, object *op, object *hitter) {
 	} else if (USING_SKILL(hitter, SK_PUNCHING)) {
           for (i=0; i < MAXATTACKMESS && attack_mess[ATM_PUNCH][i].level != -1;
 	       i++)
-	      if (dam < attack_mess[ATM_PUNCH][i].level) {
+	      if (dam < attack_mess[ATM_PUNCH][i].level
+	      || attack_mess[ATM_PUNCH][i+1].level == -1) {
 	          sprintf(buf1, "%s %s%s", attack_mess[ATM_PUNCH][i].buf1,
 			  op->name, attack_mess[ATM_PUNCH][i].buf2);
 		  sprintf(buf2, "%s", attack_mess[ATM_PUNCH][i].buf3);
@@ -432,52 +467,63 @@ void attack_message(int dam, int type, object *op, object *hitter) {
 	      }
 	}
     }
-    if (IS_ARROW(hitter) && (type == AT_PHYSICAL || type == AT_MAGIC) &&
-	!found) {
+    if (found) {
+	/* done */
+    } else if (IS_ARROW(hitter) && (type == AT_PHYSICAL || type == AT_MAGIC)) {
         sprintf(buf1, "hit"); /* just in case */
         for (i=0; i < MAXATTACKMESS; i++)
-	    if (dam < attack_mess[ATM_ARROW][i].level) {
+	    if (dam < attack_mess[ATM_ARROW][i].level
+	    || attack_mess[ATM_ARROW][i+1].level == -1) {
 	        sprintf(buf2, "%s", attack_mess[ATM_ARROW][i].buf3);
+		found++;
 		break;
 	    }
-    } else if (type & AT_DRAIN && IS_LIVE(op) && !found) {
+    } else if (type & AT_DRAIN && IS_LIVE(op)) {
       /* drain is first, because some items have multiple attypes */
         for (i=0; i < MAXATTACKMESS && attack_mess[ATM_DRAIN][i].level != -1;
 	     i++)
-	    if (dam < attack_mess[ATM_DRAIN][i].level) {
+	    if (dam < attack_mess[ATM_DRAIN][i].level
+	    || attack_mess[ATM_DRAIN][i+1].level == -1) {
 	        sprintf(buf1, "%s %s%s", attack_mess[ATM_DRAIN][i].buf1,
 			op->name, attack_mess[ATM_DRAIN][i].buf2);
 		sprintf(buf2, "%s", attack_mess[ATM_DRAIN][i].buf3);
+		found++;
 		break;
 	    }
-    } else if (type & AT_ELECTRICITY && IS_LIVE(op) && !found) {
+    } else if (type & AT_ELECTRICITY && IS_LIVE(op)) {
         for (i=0; i < MAXATTACKMESS && attack_mess[ATM_ELEC][i].level != -1;
 	     i++)
-	    if (dam < attack_mess[ATM_ELEC][i].level) {
+	    if (dam < attack_mess[ATM_ELEC][i].level
+	    || attack_mess[ATM_ELEC][i+1].level == -1) {
 	        sprintf(buf1, "%s %s%s", attack_mess[ATM_ELEC][i].buf1,
 			op->name, attack_mess[ATM_ELEC][i].buf2);
 		sprintf(buf2, "%s", attack_mess[ATM_ELEC][i].buf3);
+		found++;
 		break;
 	    }
-    } else if (type & AT_COLD && IS_LIVE(op) && !found) {
+    } else if (type & AT_COLD && IS_LIVE(op)) {
         for (i=0; i < MAXATTACKMESS && attack_mess[ATM_COLD][i].level != -1;
 	     i++)
-	    if (dam < attack_mess[ATM_COLD][i].level) {
+	    if (dam < attack_mess[ATM_COLD][i].level
+	    || attack_mess[ATM_COLD][i+1].level == -1) {
 	        sprintf(buf1, "%s %s%s", attack_mess[ATM_COLD][i].buf1,
 			op->name, attack_mess[ATM_COLD][i].buf2);
 		sprintf(buf2, "%s", attack_mess[ATM_COLD][i].buf3);
+		found++;
 		break;
 	    }
-    } else if (type & AT_FIRE && !found) {
+    } else if (type & AT_FIRE) {
         for (i=0; i < MAXATTACKMESS && attack_mess[ATM_FIRE][i].level != -1;
 	     i++)
-	    if (dam < attack_mess[ATM_FIRE][i].level) {
+	    if (dam < attack_mess[ATM_FIRE][i].level
+	    || attack_mess[ATM_FIRE][i+1].level == -1) {
 	        sprintf(buf1, "%s %s%s", attack_mess[ATM_FIRE][i].buf1,
 			op->name, attack_mess[ATM_FIRE][i].buf2);
 		sprintf(buf2, "%s", attack_mess[ATM_FIRE][i].buf3);
+		found++;
 		break;
 	    }
-    } else if (hitter->current_weapon != NULL && !found) {
+    } else if (hitter->current_weapon != NULL) {
 	int mtype;
 	switch (hitter->current_weapon->weapontype) {
 	    case WEAP_HIT: mtype = ATM_BASIC; break;
@@ -493,23 +539,32 @@ void attack_message(int dam, int type, object *op, object *hitter) {
 	}
         for (i=0; i < MAXATTACKMESS && attack_mess[mtype][i].level != -1;
 	     i++)
-	    if (dam < attack_mess[mtype][i].level) {
+	    if (dam < attack_mess[mtype][i].level
+	    || attack_mess[mtype][i+1].level == -1) {
 	        sprintf(buf1, "%s %s%s", attack_mess[mtype][i].buf1,
 			op->name, attack_mess[mtype][i].buf2);
 		sprintf(buf2, "%s", attack_mess[mtype][i].buf3);
+		found++;
 		break;
 	    }
-    } else if (!found){
+    } else {
         for (i=0; i < MAXATTACKMESS && attack_mess[ATM_BASIC][i].level != -1;
 	     i++)
-	    if (dam < attack_mess[ATM_BASIC][i].level) {
+	    if (dam < attack_mess[ATM_BASIC][i].level
+	    || attack_mess[ATM_BASIC][i+1].level == -1) {
 	        sprintf(buf1, "%s %s%s", attack_mess[ATM_BASIC][i].buf1,
 			op->name, attack_mess[ATM_BASIC][i].buf2);
 		sprintf(buf2, "%s", attack_mess[ATM_BASIC][i].buf3);
+		found++;
 		break;
 	    }
     }
 
+    if (!found) {
+	sprintf(buf1, "hit");
+	sprintf(buf2, "hits");
+    }
+
     /* bail out if a monster is casting spells */
     if (!(hitter->type == PLAYER ||
 	(get_owner(hitter) != NULL && hitter->owner->type == PLAYER)))
@@ -637,66 +692,27 @@ static int attack_ob_simple (object *op, object *hitter, int base_dam,
 {
     int simple_attack, roll, dam=0;
     uint32 type;
-    char *op_name = NULL;
+    const char *op_name = NULL;
     tag_t op_tag, hitter_tag;
-    event *evt;
 
     if (get_attack_mode (&op, &hitter, &simple_attack))
         goto error;
 
-    /* GROS: Handle for plugin attack event */
-    if ((evt = find_event(op, EVENT_ATTACK)) != NULL)
-    {
-        CFParm CFP;
-        int k, l;
-        uint32 n;
-        k = EVENT_ATTACK;
-        l = SCRIPT_FIX_ALL;
-        n = 0;
-        CFP.Value[0] = &k;
-        CFP.Value[1] = hitter;
-        CFP.Value[2] = hitter;
-        CFP.Value[3] = op;
-        CFP.Value[4] = NULL;
-        CFP.Value[5] = &n;
-        CFP.Value[6] = &base_dam;
-        CFP.Value[7] = &base_wc;
-        CFP.Value[8] = &l;
-        CFP.Value[9] = evt->hook;
-        CFP.Value[10]= evt->options;
-        if (findPlugin(evt->plugin)>=0)
-            ((PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP));
-    }
-    /* GROS: This is used to handle script_weapons with weapons. Only used for players */
+    /* Lauwenmark: Handle for plugin attack event */
+    execute_event(op, EVENT_ATTACK,hitter,hitter,NULL,SCRIPT_FIX_ALL);
+
+    /* Lauwenmark: This is used to handle script_weapons with weapons.
+     * Only used for players.
+     */
     if (hitter->type==PLAYER)
     {
         if (hitter->current_weapon != NULL)
         {
-            /* GROS: Handle for plugin attack event */
-            if ((evt = find_event(hitter->current_weapon, EVENT_ATTACK)) != NULL)
-            {
-                CFParm CFP;
-                int k, l;
-                uint32 n;
-                n = 0;
-                k = EVENT_ATTACK;
-                l = SCRIPT_FIX_ALL;
-                CFP.Value[0] = &k;
-                CFP.Value[1] = hitter;
-                CFP.Value[2] = hitter->current_weapon;
-                CFP.Value[3] = op;
-                CFP.Value[4] = NULL;
-                CFP.Value[5] = &n;
-                CFP.Value[6] = &base_dam;
-                CFP.Value[7] = &base_wc;
-                CFP.Value[8] = &l;
-                CFP.Value[9] = evt->hook;
-                CFP.Value[10]= evt->options;
-		if (findPlugin(evt->plugin)>=0)
-		    (PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP);
+            /* Lauwenmark: Handle for plugin attack event */
+            execute_event(hitter, EVENT_ATTACK,hitter->current_weapon,
+                          op,NULL,SCRIPT_FIX_ALL);
+        }
             }
-        };
-    };
     op_tag = op->count;
     hitter_tag = hitter->count;
 
@@ -796,7 +812,7 @@ static int attack_ob_simple (object *op, object *hitter, int base_dam,
 	    if (op->attacktype & AT_ACID && hitter->type==PLAYER)
 		new_draw_info(NDI_UNIQUE, 0,hitter,"You are splashed by acid!\n");
 	    hit_player(hitter, random_roll(0, (op->stats.dam), hitter,
-				      PREFER_LOW),op, op->attacktype);
+				      PREFER_LOW),op, op->attacktype, 1);
 	    if (was_destroyed (op, op_tag)
                 || was_destroyed (hitter, hitter_tag)
                 || abort_attack (op, hitter, simple_attack))
@@ -807,7 +823,7 @@ static int attack_ob_simple (object *op, object *hitter, int base_dam,
 	 * types in its area, so remove it from here.
 	 */
 	dam=hit_player(op, random_roll(1, hitdam, hitter, PREFER_HIGH),
-		       hitter, type);
+		       hitter, type, 1);
 	if (was_destroyed (op, op_tag) || was_destroyed (hitter, hitter_tag)
             || abort_attack (op, hitter, simple_attack))
 	    goto leave;
@@ -837,7 +853,8 @@ int attack_ob (object *op, object *hitter)
     return attack_ob_simple (op, hitter, hitter->stats.dam, hitter->stats.wc);
 }
 
-/* op is the arrow, tmp is what is stopping the arrow.
+/**
+ * op is the arrow, tmp is what is stopping the arrow.
  *
  * Returns 1 if op was inserted into tmp's inventory, 0 otherwise.
  */
@@ -861,7 +878,8 @@ static int stick_arrow (object *op, object *tmp)
 	return 0;
 }
 
-/* hit_with_arrow() disassembles the missile, attacks the victim and
+/**
+ * hit_with_arrow() disassembles the missile, attacks the victim and
  * reassembles the missile.
  *
  * It returns a pointer to the reassembled missile, or NULL if the missile
@@ -870,11 +888,9 @@ static int stick_arrow (object *op, object *tmp)
 object *hit_with_arrow (object *op, object *victim)
 {
     object *container, *hitter;
-    int hit_something;
-    int sretval = 0; /* GROS - Needed for script return value */
+    int hit_something=0;
     tag_t victim_tag, hitter_tag;
     sint16 victim_x, victim_y;
-    event *evt;
 
     /* Disassemble missile */
     if (op->inv) {
@@ -896,34 +912,8 @@ object *hit_with_arrow (object *op, object *victim)
     victim_y = victim->y;
     victim_tag = victim->count;
     hitter_tag = hitter->count;
-    /* GROS: Handling plugin attack event for thrown items */
-    if ((evt = find_event(op, EVENT_ATTACK)) != NULL)
-    {
-        CFParm CFP;
-        CFParm* CFR;
-        int k, l;
-        uint32 n;
-        k = EVENT_ATTACK;
-        l = SCRIPT_FIX_ALL;
-        n = 0;
-        CFP.Value[0] = &k;
-        CFP.Value[1] = hitter;
-        CFP.Value[2] = op;
-        CFP.Value[3] = victim;
-        CFP.Value[4] = NULL;
-        CFP.Value[5] = &n;
-        CFP.Value[6] = &(op->stats.dam);
-        CFP.Value[7] = &(op->stats.wc);
-        CFP.Value[8] = &l;
-        CFP.Value[9] = evt->hook;
-        CFP.Value[10]= evt->options;
-        if (findPlugin(evt->plugin)>=0)
-        {
-            CFR = (PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP);
-            sretval = *(int *)(CFR->Value[0]);
-        }
-    }
-    else
+    /* Lauwenmark: Handling plugin attack event for thrown items */
+    if (execute_event(op, EVENT_ATTACK,hitter,victim,NULL,SCRIPT_FIX_ALL) == 0)
         hit_something = attack_ob_simple (victim, hitter, op->stats.dam,
                                         op->stats.wc);
 
@@ -962,10 +952,14 @@ object *hit_with_arrow (object *op, object *victim)
              && stick_arrow (hitter, victim))
             return NULL;
 
-        /* Else try to put arrow on victim's map square */
-        if ((victim_x != hitter->x || victim_y != hitter->y)
-            && !(get_map_flags(hitter->map, NULL, victim_x, victim_y, NULL, NULL) & P_WALL))
-	{
+        /* Else try to put arrow on victim's map square
+	 * remove check for P_WALL here.  If the arrow got to this
+	 * space, that is good enough - with the new movement code,
+	 * there is now the potential for lots of spaces where something
+	 * can fly over but not otherwise move over.  What is the correct
+	 * way to handle those otherwise?
+	 */
+        if (victim_x != hitter->x || victim_y != hitter->y) {
             remove_ob (hitter);
             hitter->x = victim_x;
             hitter->y = victim_y;
@@ -989,7 +983,7 @@ object *hit_with_arrow (object *op, object *victim)
 }
 
 
-void tear_down_wall(object *op)
+static void tear_down_wall(object *op)
 {
     int perc=0;
 
@@ -1026,13 +1020,13 @@ void tear_down_wall(object *op)
 	} else { /* The last face was not blank, leave an image */
 	    CLEAR_FLAG(op, FLAG_BLOCKSVIEW);
 	    update_all_los(op->map, op->x, op->y);
-	    CLEAR_FLAG(op, FLAG_NO_PASS);
+	    op->move_block = 0;
 	    CLEAR_FLAG(op, FLAG_ALIVE);
 	}
     }
 }
 
-void scare_creature(object *target, object *hitter)
+static void scare_creature(object *target, object *hitter)
 {
     object *owner = get_owner(hitter);
 
@@ -1043,21 +1037,22 @@ void scare_creature(object *target, object *hitter)
 }
 
 
-/* This returns the amount of damage hitter does to op with the
+/**
+ * This returns the amount of damage hitter does to op with the
  * appropriate attacktype.  Only 1 attacktype should be set at a time.
  * This doesn't damage the player, but returns how much it should
  * take.  However, it will do other effects (paralyzation, slow, etc.)
  * Note - changed for PR code - we now pass the attack number and not
  * the attacktype.  Makes it easier for the PR code.  */
 
-int hit_player_attacktype(object *op, object *hitter, int dam, 
+static int hit_player_attacktype(object *op, object *hitter, int dam, 
 	uint32 attacknum, int magic) {
   
     int doesnt_slay = 1;
 
     /* Catch anyone that may be trying to send us a bitmask instead of the number */
-    if ((attacknum >= NROFATTACKS) || (attacknum < 0)) {
-	LOG(llevError, "hit_player_attacktype: Invalid attacknumber passed: %x\n", attacknum);
+    if (attacknum >= NROFATTACKS) {
+	LOG(llevError, "hit_player_attacktype: Invalid attacknumber passed: %u\n", attacknum);
 	return 0;
     }
     
@@ -1211,7 +1206,7 @@ int hit_player_attacktype(object *op, object *hitter, int dam,
 	
 	if(op->resist[ATNR_DRAIN] >= 0) 
 	  rate = 50 + op->resist[ATNR_DRAIN] / 2;
-	else if(op->resist[ATNR_DRAIN] < 0)
+	else
 	  rate = 5000 / (100 - op->resist[ATNR_DRAIN]);
 	
 	/* full protection has no effect.  Nothing else in this
@@ -1343,7 +1338,8 @@ int hit_player_attacktype(object *op, object *hitter, int dam,
 }
 
 
-/* GROS: This code comes from hit_player. It has been made external to
+/**
+ * GROS: This code comes from hit_player. It has been made external to
  * allow script procedures to "kill" objects in a combat-like fashion.
  * It was initially used by (kill-object) developed for the Collector's
  * Sword. Note that nothing has been changed from the original version
@@ -1358,57 +1354,25 @@ int hit_player_attacktype(object *op, object *hitter, int dam,
  * a bit - I think it should be functionally equivalant.
  * MSW 2002-07-17
  */
-int kill_object(object *op,int dam, object *hitter, int type)
+static int kill_object(object *op,int dam, object *hitter, int type)
 {
-    char buf[MAX_BUF], *skill;
+    char buf[MAX_BUF];
+    const char *skill;
     int maxdam=0;
     int battleg=0;    /* true if op standing on battleground */
     int pk=0;         /* true if op and what controls hitter are both players*/
-    int killed_script_rtn = 0;
     object *owner=NULL;
-    int evtid;
-    CFParm CFP;
     object *skop=NULL;
-    event *evt;
 
     if (op->stats.hp>=0)
 	return -1;
 
-    /* GROS: Handle for plugin death event */
-    if ((evt = find_event(op, EVENT_DEATH)) != NULL)
-    {
-        CFParm* CFR;
-        int k, l, m;
-        uint32 n;
-        k = EVENT_DEATH;
-        l = SCRIPT_FIX_ALL;
-        m = 0;
-        n = type;
-        CFP.Value[0] = &k;
-        CFP.Value[1] = hitter;
-        CFP.Value[2] = op;
-        CFP.Value[3] = NULL;
-        CFP.Value[4] = NULL;
-        CFP.Value[5] = &n;
-        CFP.Value[6] = &m;
-        CFP.Value[7] = &m;
-        CFP.Value[8] = &l;
-        CFP.Value[9] = evt->hook;
-        CFP.Value[10]= evt->options;
-        if (findPlugin(evt->plugin)>=0)
-        {
-            CFR =(PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP);
-            killed_script_rtn = *(int *)(CFR->Value[0]);
-            if (killed_script_rtn)
+    /* Lauwenmark: Handle for plugin death event */
+    if (execute_event(op, EVENT_DEATH,hitter,NULL,NULL,SCRIPT_FIX_ALL)!=0)
                 return 0;
-        }
-    }
-    /* GROS: Handle for the global kill event */
-    evtid = EVENT_GKILL;
-    CFP.Value[0] = (void *)(&evtid);
-    CFP.Value[1] = (void *)(hitter);
-    CFP.Value[2] = (void *)(op);
-    GlobalEvent(&CFP);
+    /* Lauwenmark: Handle for the global kill event */
+    execute_global_event(EVENT_GKILL, op, hitter);
+
     /* maxdam needs to be the amount of damage it took to kill
      * this creature.  The function(s) that call us have already
      * adjusted the creatures HP total, so that is negative.
@@ -1453,7 +1417,7 @@ int kill_object(object *op,int dam, object *hitter, int type)
 
     /* Player killed something */
     if(owner->type==PLAYER) {
-	Log_Kill(owner->name,
+	log_kill(owner->name,
 		 query_name(op),op->type,
                 (owner!=hitter) ? query_name(hitter) : NULL,
                 (owner!=hitter) ? hitter->type : 0);
@@ -1532,7 +1496,9 @@ int kill_object(object *op,int dam, object *hitter, int type)
 		}
 	}
     } /* Was it a player that hit somethign */
-
+    else {
+	skill = NULL;
+    }
 
     /* Pet (or spell) killed something. */
     if(owner != hitter ) {
@@ -1540,8 +1506,9 @@ int kill_object(object *op,int dam, object *hitter, int type)
 	       query_name(op),query_name(hitter), battleg? " (duel)":"", pk? " (pk)":"");
     }
     else {
-	(void) sprintf(buf,"%s killed %s in hand to hand combat %s%s.",hitter->name,op->name,
-	       battleg? " (duel)":"", pk? " (pk)":"");
+	(void) sprintf(buf,"%s killed %s%s%s%s.",hitter->name,op->name,
+	       (QUERY_FLAG(hitter, FLAG_MONSTER)) || hitter->type == PLAYER ?
+	       " in hand to hand combat":"", battleg? " (duel)":"", pk? " (pk)":"");
     }
     /* These may have been set in the player code section above */
     if (!skop) skop = hitter->chosen_skill;
@@ -1578,7 +1545,7 @@ int kill_object(object *op,int dam, object *hitter, int type)
 	 */
 	if (battleg) exp = 0;
 
-	if(owner->type!=PLAYER || owner->contr->party_number<=0) {
+	if(owner->type!=PLAYER || owner->contr->party==NULL) {
 	    change_exp(owner,exp, skill, 0);
 	}
 	else {
@@ -1586,12 +1553,12 @@ int kill_object(object *op,int dam, object *hitter, int type)
 
 	    player *pl;
 
-	    int no=owner->contr->party_number;
+	    partylist *party=owner->contr->party;
 #ifdef PARTY_KILL_LOG
-	    add_kill_to_party(no,query_name(owner),query_name(op),exp);
+	    add_kill_to_party(party, query_name(owner), query_name(op), exp);
 #endif
 	    for(pl=first_player;pl!=NULL;pl=pl->next) {
-		if(pl->ob->contr->party_number==no && on_same_map(pl->ob, owner)) {
+		if(party && pl->ob->contr->party==party && on_same_map(pl->ob, owner)) {
 		    count++;
 		    shares+=(pl->ob->level+4);
 		}
@@ -1601,7 +1568,7 @@ int kill_object(object *op,int dam, object *hitter, int type)
 	    else {
 		int share=exp/shares,given=0,nexp;
 		for(pl=first_player;pl!=NULL;pl=pl->next) {
-		    if(pl->ob->contr->party_number==no && on_same_map(pl->ob, owner)) {
+		    if(party && pl->ob->contr->party==party && on_same_map(pl->ob, owner)) {
 			nexp=(pl->ob->level+4)*share;
 			change_exp(pl->ob,nexp, skill, SK_EXP_TOTAL);
 			given+=nexp;
@@ -1646,7 +1613,8 @@ int kill_object(object *op,int dam, object *hitter, int type)
     return maxdam;
 }
 
-/* Find out if this is friendly fire (PVP and attacker is peaceful) or not 
+/**
+ * Find out if this is friendly fire (PVP and attacker is peaceful) or not 
  *  Returns 0 this is not friendly fire
  */
 
@@ -1676,17 +1644,19 @@ int friendly_fire(object *op, object *hitter){
 }
 
 
-/* This isn't used just for players, but in fact most objects.
+/**
+ * This isn't used just for players, but in fact most objects.
  * op is the object to be hit, dam is the amount of damage, hitter
- * is what is hitting the object, and type is the attacktype.
+ * is what is hitting the object, type is the attacktype, and
+ * full_hit is set if monster area does not matter.
  * dam is base damage - protections/vulnerabilities/slaying matches can
  * modify it.
+ *
+ * Oct 95 - altered the following slightly for MULTIPLE_GODS hack
+ * which needs new attacktype AT_HOLYWORD to work . b.t.
  */
 
-  /* Oct 95 - altered the following slightly for MULTIPLE_GODS hack
-   * which needs new attacktype AT_HOLYWORD to work . b.t. */
-
-int hit_player(object *op,int dam, object *hitter, int type) {
+int hit_player(object *op,int dam, object *hitter, int type, int full_hit) {
     int maxdam=0, ndam=0, attacktype=1, magic=(type & AT_MAGIC);
     int maxattacktype, attacknum;
     int body_attack = op && op->head;   /* Did we hit op's head? */
@@ -1823,6 +1793,23 @@ int hit_player(object *op,int dam, object *hitter, int type) {
 #endif
     }
 
+    if (!full_hit) {
+	archetype *at;
+	int area;
+	int remainder;
+
+	area = 0;
+	for(at = op->arch; at != NULL; at = at->more)
+	    area++;
+	assert(area > 0);
+
+	/* basically: maxdam /= area; we try to "simulate" a float
+	   value-effect */
+	remainder = 100*(maxdam%area)/area;
+	maxdam /= area;
+	if (RANDOM()%100 < remainder)
+	    maxdam++;
+    }
 
 #ifdef ATTACK_DEBUG
     LOG(llevDebug,"Attacktype %d did %d damage\n", type, maxdam);
@@ -1900,13 +1887,13 @@ int hit_player(object *op,int dam, object *hitter, int type) {
 	    if (friendly) {
 		SET_FLAG(tmp, FLAG_FRIENDLY);
 		add_friendly_object(tmp);
-		tmp->move_type = PETMOVE;
+		tmp->attack_movement = PETMOVE;
 		if (owner!=NULL)
 		    set_owner(tmp,owner);
 	    }
 	    if (unaggressive)
 		SET_FLAG(tmp, FLAG_UNAGGRESSIVE);
-	    j=find_first_free_spot(tmp->arch,op->map,op->x,op->y);
+	    j=find_first_free_spot(tmp,op->map,op->x,op->y);
 	    if (j==-1) /* No spot to put this monster */
 		free_object(tmp);
 	    else {
@@ -1926,7 +1913,7 @@ int hit_player(object *op,int dam, object *hitter, int type) {
 }
 
 
-void poison_player(object *op, object *hitter, int dam)
+static void poison_player(object *op, object *hitter, int dam)
 {
     archetype *at = find_archetype("poisoning");
     object *tmp=present_arch_in_ob(at,op);
@@ -1980,7 +1967,7 @@ void poison_player(object *op, object *hitter, int dam)
 	tmp->stats.food++;
 }
 
-void slow_player(object *op,object *hitter,int dam)
+static void slow_player(object *op,object *hitter,int dam)
 {    archetype *at = find_archetype("slowness");
     object *tmp;
     if(at == NULL) {
@@ -2004,7 +1991,7 @@ void confuse_player(object *op, object *hitter, int dam)
     
     tmp = present_in_ob_by_name(FORCE,"confusion", op);
     if(!tmp) {
-	tmp = get_archetype(FORCE_NAME);
+	tmp = create_archetype(FORCE_NAME);
 	tmp = insert_ob_in_ob(tmp,op);
     }
     
@@ -2034,7 +2021,7 @@ void blind_player(object *op, object *hitter, int dam)
 
     tmp = present_in_ob(BLINDNESS,op);
     if(!tmp) { 
-      tmp = get_archetype("blindness");
+      tmp = create_archetype("blindness");
       SET_FLAG(tmp, FLAG_BLIND);
       SET_FLAG(tmp, FLAG_APPLIED);
       /* use floats so we don't lose too much precision due to rounding errors.
@@ -2090,10 +2077,11 @@ void paralyze_player(object *op, object *hitter, int dam)
 }
 
 
-/* Attempts to kill 'op'.  hitter is the attack object, dam is
+/**
+ * Attempts to kill 'op'.  hitter is the attack object, dam is
  * the computed damaged.
  */
-void deathstrike_player(object *op, object *hitter, int *dam) 
+static void deathstrike_player(object *op, object *hitter, int *dam) 
 {
     /*  The intention of a death attack is to kill outright things
     **  that are a lot weaker than the attacker, have a chance of killing
@@ -2140,7 +2128,8 @@ void deathstrike_player(object *op, object *hitter, int *dam)
     }
 }
 
-/* thrown_item_effect() - handles any special effects of thrown
+/**
+ * handles any special effects of thrown
  * items (like attacking living creatures--a potion thrown at a
  * monster).
  */
@@ -2172,9 +2161,9 @@ static void thrown_item_effect (object *hitter, object *victim)
     }
 }
 
-/* adj_attackroll() - adjustments to attacks by various conditions */
+/** adj_attackroll() - adjustments to attacks by various conditions */
 
-int adj_attackroll (object *hitter, object *target) {
+static int adj_attackroll (object *hitter, object *target) {
   object *attacker = hitter;
   int adjust=0;
 
@@ -2221,7 +2210,7 @@ int adj_attackroll (object *hitter, object *target) {
     adjust -= 3;
 
   /* if we attack at a different 'altitude' its harder */
-  if(QUERY_FLAG(attacker,FLAG_FLYING)!=QUERY_FLAG(target,FLAG_FLYING))
+  if((attacker->move_type & target->move_type)==0)
     adjust -= 2;
 
 #if 0
@@ -2243,13 +2232,13 @@ int adj_attackroll (object *hitter, object *target) {
 } 
 
 
-/* determine if the object is an 'aimed' missile */
-int is_aimed_missile ( object *op) {
+/** determine if the object is an 'aimed' missile */
+static int is_aimed_missile ( object *op) {
 
     /* I broke what used to be one big if into a few nested
      * ones so that figuring out the logic is at least possible.
      */
-    if (op && QUERY_FLAG(op,FLAG_FLYING)) {
+    if (op && (op->move_type & MOVE_FLYING)) {
 	if (op->type==ARROW || op->type==THROWN_OBJ)
 	    return 1;
 	else if (op->type==SPELL_EFFECT && (op->subtype == SP_BULLET ||
