@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_build_map =
- *   "$Id: build_map.c,v 1.4 2005/07/19 20:59:34 ryo_saeba Exp $";
+ *   "$Id: build_map.c,v 1.11 2006/04/06 21:18:35 tchize Exp $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -32,6 +32,44 @@
 #include <tod.h>
 #include <sproto.h>
 
+/**
+ *  Check if objects on a square interfere with building
+ */
+int can_build_over( struct mapdef* map, object* tmp, short x, short y)
+    {
+    object* ob;
+    
+    ob = GET_MAP_OB( map, x, y );
+    while ( ob )
+        {
+	/* if ob is not a marking rune or floor, then check special cases */
+	if ( strcmp( ob->arch->name, "rune_mark" ) && ob->type != FLOOR )
+	    {
+            switch ( tmp->type )
+                {
+	        case SIGN:
+	        case MAGIC_EAR:
+		    /* Allow signs and magic ears to be built on books */
+	            if ( ob->type != BOOK ) {
+		        return 0; } 
+                    break;
+		case BUTTON:
+	        case DETECTOR:
+                case PEDESTAL:
+                case CF_HANDLE:
+		    /* Allow buttons and levers to be built under gates */
+	            if ( ob->type != GATE && ob->type != DOOR ) {
+		        return 0; } 
+                    break;
+		default:
+                    return 0;
+                }
+            }
+        ob = ob->above;
+	}
+    return 1;
+    }
+
 /**
  *  Erases marking runes at specified location
  */
@@ -89,15 +127,13 @@ int find_unused_connected_value( struct mapdef* map )
  * If found, returns the connection value associated
  * else searches a new connection value, and adds the force to the player.
  */
-int find_or_create_connection_for_map( object* pl, short x, short y )
+int find_or_create_connection_for_map( object* pl, short x, short y, object* rune )
     {
-    object* rune;
     object* force;
     int connected;
 
-    rune = GET_MAP_OB( pl->map, x, y );
-    while ( rune && ( ( rune->type != SIGN ) || ( strcmp( rune->arch->name, "rune_mark" ) ) ) )
-        rune = rune->above;
+    if ( !rune )
+        rune = get_connection_rune( pl, x, y );
 
     if ( !rune )
         {
@@ -121,7 +157,7 @@ int find_or_create_connection_for_map( object* pl, short x, short y )
             return -1;
             }
 
-        force = get_archetype( FORCE_NAME );
+        force = create_archetype( FORCE_NAME );
         force->speed = 0;
         update_ob_speed( force );
         force->slaying = add_string( pl->map->path );
@@ -135,6 +171,32 @@ int find_or_create_connection_for_map( object* pl, short x, short y )
     /* Found the force, everything's easy. */
     return force->path_attuned;
     }
+    
+/**
+ * Returns the marking rune on the square, for purposes of building connections
+ */
+object* get_connection_rune( object* pl, short x, short y )
+    {
+    object* rune;
+    
+    rune = GET_MAP_OB( pl->map, x, y );
+    while ( rune && ( ( rune->type != SIGN ) || ( strcmp( rune->arch->name, "rune_mark" ) ) ) )
+        rune = rune->above;
+    return rune;
+    }
+    
+/**
+ * Returns the book/scroll on the current square, for purposes of building
+ */
+object* get_msg_book( object* pl, short x, short y )
+    {
+    object* book;
+    
+    book = GET_MAP_OB( pl->map, x, y );
+    while ( book && ( book->type != BOOK ) )
+        book = book->above;
+    return book;
+    }
 
 /**
  * Returns first item of type WALL.
@@ -289,7 +351,6 @@ void apply_builder_floor(object* pl, object* material, short x, short y )
     {
     object* tmp, *above;
     object* above_floor; /* Item above floor, if any */
-    char has_floor = 0;
     struct archt* new_floor;
     struct archt* new_wall;
     int i, xt, yt, floor_removed;
@@ -318,7 +379,7 @@ void apply_builder_floor(object* pl, object* material, short x, short y )
                 free_object( tmp );
                 sprintf( message, "You destroy the wall and redo the floor." );
                 }
-            else if ( FLOOR == tmp->type )
+            else if ( ( FLOOR == tmp->type ) || ( QUERY_FLAG(tmp, FLAG_IS_FLOOR ) ) )
                 {
                 remove_ob( tmp );
                 free_object( tmp );
@@ -346,6 +407,7 @@ void apply_builder_floor(object* pl, object* material, short x, short y )
     tmp = arch_to_object( new_floor );
     SET_FLAG( tmp, FLAG_IS_BUILDABLE );
     SET_FLAG( tmp, FLAG_UNIQUE );
+	SET_FLAG( tmp, FLAG_IS_FLOOR );
     tmp->type = FLOOR;
     insert_ob_in_map_at( tmp, pl->map, above_floor, above_floor ? INS_BELOW_ORIGINATOR : INS_ON_TOP, x, y );
 
@@ -482,6 +544,7 @@ void apply_builder_item( object* pl, object* item, short x, short y )
     struct archt* arch;
     int insert_flag;
     object* floor;
+    object* con_rune;
     int connected;
 
     /* Find floor */
@@ -492,7 +555,7 @@ void apply_builder_item( object* pl, object* item, short x, short y )
         return;
         }
 
-    while ( floor && ( floor->type != FLOOR ) )
+    while ( floor && ( floor->type != FLOOR ) && ( !QUERY_FLAG( floor, FLAG_IS_FLOOR ) ) )
         floor = floor->above;
 
     if ( !floor )
@@ -500,20 +563,20 @@ void apply_builder_item( object* pl, object* item, short x, short y )
         new_draw_info( NDI_UNIQUE, 0, pl, "This square has no floor, you can't build here." );
         return;
         }
-
-    if ( ( floor->above ) && ( strcmp( floor->above->arch->name, "rune_mark" ) ) )
-        /* Floor has something on top, and it isn't a marking rune */
-        {
-        new_draw_info( NDI_UNIQUE, 0, pl, "You can't build here." );
-        return;
-        }
-
     /* Create item, set flag, insert in map */
     arch = find_archetype( item->slaying );
     if ( !arch )
         return;
 
     tmp = arch_to_object( arch );
+    
+    if ( ( floor->above ) && ( !can_build_over(pl->map, tmp, x, y) ) )
+        /* Floor has something on top that interferes with building */
+        {
+        new_draw_info( NDI_UNIQUE, 0, pl, "You can't build here." );
+        return;
+        }
+    
     SET_FLAG( tmp, FLAG_IS_BUILDABLE );
     SET_FLAG( tmp, FLAG_NO_PICK );
 
@@ -524,7 +587,6 @@ void apply_builder_item( object* pl, object* item, short x, short y )
     insert_flag = INS_ABOVE_FLOOR_ONLY;
 
     connected = 0;
-
     switch( tmp->type )
         {
         case DOOR:
@@ -534,8 +596,13 @@ void apply_builder_item( object* pl, object* item, short x, short y )
         case TIMED_GATE:
         case PEDESTAL:
         case CF_HANDLE:
-            /* Those items require a valid connection */
-            connected = find_or_create_connection_for_map( pl, x, y );
+	case MAGIC_EAR:
+	case SIGN:
+	    /* Signs don't need a connection, but but magic mouths do. */
+	    if (tmp->type == SIGN && strcmp( tmp->arch->name, "magic_mouth" ))
+	        break;
+	    con_rune = get_connection_rune( pl, x, y );
+	    connected = find_or_create_connection_for_map( pl, x, y, con_rune );
             if ( connected == -1 )
                 {
                 /* Player already informed of failure by the previous function */
@@ -543,8 +610,19 @@ void apply_builder_item( object* pl, object* item, short x, short y )
                 return;
                 }
             /* Remove marking rune */
-            remove_marking_runes( pl->map, x, y );
+            remove_ob( con_rune );
+            free_object( con_rune );
         }
+	
+    /* For magic mouths/ears, and signs, take the msg from a book of scroll */	
+    if ((tmp->type == SIGN) || (tmp->type == MAGIC_EAR))
+        {
+	if (adjust_sign_msg( pl, x, y, tmp ) == -1)
+	    {
+            free_object( tmp );
+            return;
+	    }
+	}
 
     insert_ob_in_map_at( tmp, pl->map, floor, insert_flag, x, y );
     if ( connected != 0 )
@@ -577,7 +655,7 @@ void apply_builder_remove( object* pl, int dir )
         return;
         }
 
-    if ( item->type == FLOOR )
+    if ( item->type == FLOOR || QUERY_FLAG(item,FLAG_IS_FLOOR) )
         item = item->above;
 
     if ( !item )
@@ -600,8 +678,11 @@ void apply_builder_remove( object* pl, int dir )
         case DETECTOR:
         case PEDESTAL:
         case CF_HANDLE:
+        case MAGIC_EAR:
+        case SIGN:
             /* Special case: must unconnect */
-            remove_button_link( item );
+	    if (QUERY_FLAG(item,FLAG_IS_LINKED))
+                remove_button_link( item );
 
             /* Fall through */
 
@@ -623,6 +704,7 @@ void apply_map_builder( object* pl, int dir )
     {
     object* builder;
     object* tmp;
+    object* tmp2;
     short x, y;
 
     if ( !pl->type == PLAYER )
@@ -664,13 +746,20 @@ void apply_map_builder( object* pl, int dir )
         new_draw_info( NDI_UNIQUE, 0, pl, "You'd better not build here, it looks weird." );
         return;
         }
-
+    tmp2 = find_marked_object( pl );
     while ( tmp )
         {
-        if ( !QUERY_FLAG( tmp, FLAG_IS_BUILDABLE ) && ( ( tmp->type != SIGN ) || ( strcmp( tmp->arch->name, "rune_mark" ) ) ) )
+        if ( !QUERY_FLAG( tmp, FLAG_IS_BUILDABLE ) && ( ( tmp->type != SIGN )
+	|| ( strcmp( tmp->arch->name, "rune_mark" ) ) ) )
             {
-	        new_draw_info( NDI_UNIQUE, 0, pl, "You can't build here." );
-	        return;
+	        /* The item building function already has it's own special
+		 * checks for this
+		 */
+	        if ((!tmp2) || (tmp2->subtype != ST_MAT_ITEM ))
+		    {
+	            new_draw_info( NDI_UNIQUE, 0, pl, "You can't build here." );
+	            return;
+		    }
             }
         tmp = tmp->above;
         }
@@ -691,7 +780,7 @@ void apply_map_builder( object* pl, int dir )
          * Find marked item to build, call specific function
          */
         {
-        tmp = find_marked_object( pl );
+        tmp = tmp2;
         if ( !tmp )
             {
 	        new_draw_info( NDI_UNIQUE, 0, pl, "You need to mark raw materials to use." );
@@ -729,3 +818,52 @@ void apply_map_builder( object* pl, int dir )
     new_draw_info( NDI_UNIQUE, 0, pl, "Don't know how to apply this tool, sorry." );
     LOG( llevError, "apply_map_builder: invalid builder subtype %d\n", builder->subtype );
     }
+    
+/**
+ * Make the built object inherit the msg of books that are used with it.
+ * For objects already invisible (i.e. magic mouths & ears), also make it
+ * it inherit the face and the name with "talking " prepended.
+ */
+int adjust_sign_msg( object* pl, short x, short y, object* tmp )
+    {
+    object* book;
+    char buf[MAX_BUF];
+    char buf2[MAX_BUF];
+
+    book = get_msg_book( pl, x, y );
+    if ( !book )
+        {
+	new_draw_info( NDI_UNIQUE, 0, pl, "You need to put a book or scroll with the message." );
+	return -1;
+	}
+	
+    tmp->msg = book->msg;
+    add_refcount( tmp->msg );
+    
+    if (tmp->invisible)
+        {
+	if(book->custom_name != NULL)
+	    {
+	    snprintf(buf, sizeof(buf), "talking %s", book->custom_name);
+	    } else {
+	    snprintf(buf, sizeof(buf), "talking %s", book->name);
+	    }
+	if ( tmp->name )
+	    free_string( tmp->name );
+	tmp->name = add_string( buf );
+	
+	if(book->name_pl != NULL)
+	    {
+	    snprintf(buf2, sizeof(buf2), "talking %s", book->name_pl);
+	    if ( tmp->name_pl )
+                free_string( tmp->name_pl );
+            tmp->name_pl = add_string( buf2 );
+	    }
+	
+	tmp->face = book->face;
+	tmp->invisible = 0;
+        }
+    remove_ob( book );
+    free_object( book );
+    return 0;
+    }
