@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_chat_c =
- *   "$Id: c_chat.c,v 1.19 2004/12/25 18:09:29 ryo_saeba Exp $";
+ *   "$Id: c_chat.c,v 1.23 2006/05/05 09:47:40 ryo_saeba Exp $";
  */
 
 /*
@@ -74,7 +74,7 @@ int command_cointoss(object *op, char *params)
     return 0;
 }
 
-char *orcknuckle[7] = {"none", "beholder", "ghost", "knight",
+static const char* const orcknuckle[7] = {"none", "beholder", "ghost", "knight",
 		       "princess", "dragon", "orc"};
 
 int command_orcknuckle(object *op, char *params)
@@ -93,15 +93,12 @@ int command_orcknuckle(object *op, char *params)
     snprintf(buf, MAX_BUF-1, "You roll %s, %s, %s, %s!",
 	orcknuckle[i], orcknuckle[j], orcknuckle[k], orcknuckle[l]);
     new_draw_info(NDI_UNIQUE, 0, op, buf);
-    new_info_map_except(NDI_WHITE, op->map, op, buf2);
+    new_info_map_except(NDI_UNIQUE, op->map, op, buf2);
     return 0;
 }
 
-static int command_tell_all(object *op, char *params, int pri, int color, char *desc)
+static int command_tell_all(object *op, char *params, int pri, int color, const char *desc)
 {
-    int evtid;
-    CFParm CFP;
-
     if (op->contr->no_shout == 1){
 	new_draw_info(NDI_UNIQUE, 0,op,"You are no longer allowed to shout or chat.");
 	return 1;
@@ -113,14 +110,8 @@ static int command_tell_all(object *op, char *params, int pri, int color, char *
 	new_draw_info_format(NDI_UNIQUE | NDI_ALL | color, pri, NULL, 
 		 "%s %s: %s", op->name, desc, params);
  
-	/* GROS : Here we handle the SHOUT global event */
-	evtid = EVENT_SHOUT;
-	CFP.Value[0] = (void *)(&evtid);
-	CFP.Value[1] = (void *)(op);
-	CFP.Value[2] = (void *)(params);
-	/* Pass in priority - not sure if the plugin can use it or not */
-	CFP.Value[3] = (void *)(&pri);
-	GlobalEvent(&CFP);
+        /* Lauwenmark : Here we handle the SHOUT global event */
+        execute_global_event(EVENT_SHOUT,op,params,pri);
 	return 1;
     }
 }
