@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_misc_c =
- *   "$Id: c_misc.c,v 1.63 2005/07/16 21:10:39 akirschbaum Exp $";
+ *   "$Id: c_misc.c,v 1.79 2006/06/06 21:38:41 akirschbaum Exp $";
  */
 
 /*
@@ -34,7 +34,8 @@
 
 extern weathermap_t **weathermap;
 
-/* Handles misc. input request - things like hash table, malloc, maps,
+/**
+ * Handles misc. input request - things like hash table, malloc, maps,
  * who, etc.
  */
 
@@ -61,7 +62,8 @@ void map_info(object *op, char *search) {
   }
 }
 
-/* This command dumps the body information for object *op.
+/**
+ * This command dumps the body information for object *op.
  * it doesn't care what the params are.
  * This is mostly meant as a debug command.
  */
@@ -99,6 +101,18 @@ int command_motd(object *op, char *params)
 	return 1;
 }
 
+int command_rules(object *op, char *params)
+{
+	send_rules(op);
+	return 1;
+}
+
+int command_news(object *op, char *params)
+{
+	send_news(op);
+	return 1;
+}
+
 int command_bug(object *op, char *params)
 {
     char buf[MAX_BUF];
@@ -198,7 +212,7 @@ void malloc_info(object *op) {
   new_draw_info(NDI_UNIQUE, 0,op,errmsg);
 }
 
-/* 
+/**
  * Pretty much identical to current map_info, but on a bigger scale 
  * This function returns the name of the players current region, and
  * a description of it. It is there merely for flavour text.
@@ -296,7 +310,7 @@ typedef struct
       int login_order;
     } chars_names;
 
-/*local functon for qsort comparison*/
+/** local functon for qsort comparison*/
 static int name_cmp (const chars_names *c1, const chars_names *c2)
     {
       return strcasecmp (c1->namebuf, c2->namebuf);
@@ -310,6 +324,7 @@ int command_who (object *op, char *params) {
     int num_players = 0;
     int num_wiz = 0;
     int num_afk = 0;
+    int num_bot = 0;
     chars_names *chars = NULL;
     
     /* 
@@ -319,9 +334,9 @@ int command_who (object *op, char *params) {
      * new formats have been specified, and if not we will use the old defaults.
      */
     if (!strcmp(settings.who_format,"")) 
-    	strcpy(settings.who_format, "%N_%T%t%h%d%n[%m]");
+    	strcpy(settings.who_format, "%N_%T%t%h%d%b%n[%m]");
     if (!strcmp(settings.who_wiz_format,"")) 
-    	strcpy(settings.who_wiz_format, "%N_%T%t%h%d%nLevel %l [%m](@%i)(%c)");
+    	strcpy(settings.who_wiz_format, "%N_%T%t%h%d%b%nLevel %l [%m](@%i)(%c)");
     if (op == NULL || QUERY_FLAG(op, FLAG_WIZ))
     	format=settings.who_wiz_format;
     else
@@ -351,31 +366,33 @@ int command_who (object *op, char *params) {
 	      num_wiz++;
 	    if (QUERY_FLAG(pl->ob,FLAG_AFK))
 	      num_afk++;
+		if (pl->socket.is_bot)
+			num_bot++;
 	}
     }
     if (first_player != (player *) NULL) {
     	if (reg == NULL) 
-            new_draw_info_format(NDI_UNIQUE, 0, op, "Total Players (%d) -- WIZ(%d) AFK(%d)", 
-      	    	num_players, num_wiz, num_afk);
-	else if (reg->longname == NULL)
-	    new_draw_info_format(NDI_UNIQUE, 0, op, "Total Players in %s (%d) -- WIZ(%d) AFK(%d)", 
-      	    	reg->name, num_players, num_wiz, num_afk);
-	else
-	    new_draw_info_format(NDI_UNIQUE, 0, op, "Total Players in %s (%d) -- WIZ(%d) AFK(%d)", 
-      	    	reg->longname, num_players, num_wiz, num_afk);
+			new_draw_info_format(NDI_UNIQUE, 0, op, "Total Players (%d) -- WIZ(%d) AFK(%d) BOT(%d)", 
+      	    	num_players, num_wiz, num_afk, num_bot);
+		else if (reg->longname == NULL)
+     		new_draw_info_format(NDI_UNIQUE, 0, op, "Total Players in %s (%d) -- WIZ(%d) AFK(%d) BOT(%d)", 
+				reg->name, num_players, num_wiz, num_afk, num_bot);
+		else
+			new_draw_info_format(NDI_UNIQUE, 0, op, "Total Players in %s (%d) -- WIZ(%d) AFK(%d) BOT(%d)", 
+      	    		reg->longname, num_players, num_wiz, num_afk, num_bot);
     }
-    qsort (chars, num_players, sizeof(chars_names), name_cmp);
+    qsort (chars, num_players, sizeof(chars_names), (int (*)(const void *, const void *))name_cmp);
     for (i=0;i<num_players;i++)
 	display_who_entry(op, find_player(chars[i].namebuf), format);
     free(chars);    
     return 1;
 }
 
-/* Display a line of 'who' to op, about pl, using the formatting specified by format */
-void display_who_entry(object *op, player *pl, char *format) {
+/** Display a line of 'who' to op, about pl, using the formatting specified by format */
+void display_who_entry(object *op, player *pl, const char *format) {
     char tmpbuf[MAX_BUF];
     char outbuf[MAX_BUF];
-    sint16 i;
+    size_t i;
     outbuf[0]='\0'; /* we strcat to this, so reset it here. */
     if (pl==NULL) {
     	LOG(llevError,"display_who_entry(): I was passed a null player");
@@ -397,7 +414,8 @@ void display_who_entry(object *op, player *pl, char *format) {
     new_draw_info(NDI_UNIQUE, 0, op, outbuf);
 }
 
-/* Returns the value of the escape code used in the who format specifier
+/**
+ * Returns the value of the escape code used in the who format specifier
  * the values are:
  * N	Name of character
  * t	title of character
@@ -407,12 +425,13 @@ void display_who_entry(object *op, player *pl, char *format) {
  * h	[Hostile] if character is hostile, nothing otherwise
  * d	[WIZ] if character is a dm, nothing otherwise
  * a	[AFK] if character is afk, nothing otherwise
+ * b	[BOT] if character is a bot, nothing otherwise
  * l	the level of the character
  * m	the map path the character is currently on
  * M	the map name of the map the character is currently on
  * r	the region name (eg scorn, wolfsburg)
  * R	the regional title (eg The Kingdom of Scorn, The Port of Wolfsburg)
- * i	player's ip adress
+ * i	player's ip address
  * %	a literal %
  * _	a literal underscore
  */
@@ -441,9 +460,11 @@ void get_who_escape_code_value(char *return_val, const char letter, player *pl)
 			  break;
 	case 'a' :    strcpy(return_val,(QUERY_FLAG(pl->ob,FLAG_AFK)?" [AFK]":""));
 			  break;
-	case 'm' :    strcpy(return_val,pl->ob->map->path);
+	case 'b' :    strcpy(return_val,(pl->socket.is_bot == 1)?" [BOT]":"");
+			  break;
+     case 'm' :    strcpy(return_val,pl->ob->map->path);
 		 	  break;
-	case 'M' :    strcpy(return_val,pl->ob->map->name);
+	case 'M' :    strcpy(return_val,pl->ob->map->name?pl->ob->map->name:"Untitled");
 			  break;		
 	case 'r' :    strcpy(return_val,get_name_of_region_for_map(pl->ob->map));
 			  break;
@@ -651,7 +672,7 @@ int command_debug (object *op, char *params)
   }
 
 
-/*
+/**
  * Those dumps should be just one dump with good parser
  */
 
@@ -688,48 +709,70 @@ int command_wizpass (object *op, char *params)
   return 0;
 }
 
+int command_wizcast (object *op, char *params)
+{
+  int i;
+
+  if (!op)
+    return 0;
+
+  if (!params)
+    i = (QUERY_FLAG(op, FLAG_WIZCAST)) ? 0 : 1;
+  else
+    i = onoff_value(params);
+
+  if (i) {
+    new_draw_info(NDI_UNIQUE, 0, op, "You can now cast spells anywhere.");
+    SET_FLAG(op, FLAG_WIZCAST);
+  } else {
+    new_draw_info(NDI_UNIQUE, 0, op, "You now cannot cast spells in no-magic areas.");
+    CLEAR_FLAG(op, FLAG_WIZCAST);
+  }
+  return 0;
+}
+
 int command_dumpallobjects (object *op, char *params)
 {
-        dump_all_objects();
-  return 0;
+    dump_all_objects();
+    return 0;
 }
 
 int command_dumpfriendlyobjects (object *op, char *params)
 {
-        dump_friendly_objects();
-  return 0;
+    dump_friendly_objects();
+    return 0;
 }
 
 int command_dumpallarchetypes (object *op, char *params)
 {
-        dump_all_archetypes();
-  return 0;
-      }
+    dump_all_archetypes();
+    return 0;
+}
 
 int command_ssdumptable (object *op, char *params)
 {
-      (void) ss_dump_table(1);
-  return 0;
+    ss_dump_table(1);
+    return 0;
 }
 
 int command_dumpmap (object *op, char *params)
 {
-  if(op)
+    if(op)
         dump_map(op->map);
-  return 0;
+    return 0;
 }
 
 int command_dumpallmaps (object *op, char *params)
 {
-        dump_all_maps();
-  return 0;
+    dump_all_maps();
+    return 0;
 }
 
 int command_printlos (object *op, char *params)
 {
-  if (op)
+    if (op)
         print_los(op);
-  return 0;
+    return 0;
 }
 
 
@@ -743,13 +786,13 @@ int command_version (object *op, char *params)
 #ifndef BUG_LOG
 #define BUG_LOG "bug_log"
 #endif
-void bug_report(char * reportstring){
+void bug_report(const char * reportstring){
   FILE * fp;
   if((fp = fopen( BUG_LOG , "a")) != NULL){
       fprintf(fp,"%s\n", reportstring);
       fclose(fp);
   } else {
-      perror(BUG_LOG);
+      LOG(llevError, "Cannot write bugs file %s: %s\n", BUG_LOG, strerror_local(errno));
   }
 }
 
@@ -810,7 +853,8 @@ int command_listen (object *op, char *params)
     return 1;
 }
 
-/* Prints out some useful information for the character.  Everything we print
+/**
+ * Prints out some useful information for the character.  Everything we print
  * out can be determined by the docs, so we aren't revealing anything extra -
  * rather, we are making it convenient to find the values.  params have
  * no meaning here.
@@ -843,28 +887,6 @@ int command_statistics(object *pl, char *params)
 	pl->contr->orig_stats.Cha, pl->stats.Cha, 20+pl->arch->clone.stats.Cha);
     new_draw_info_format(NDI_UNIQUE, 0, pl, "\nAttack Mode: %s",pl->contr->peaceful? "Peaceful":"Hostile");
 
-    /* If dragon player, let's display natural resistances */
-    if ( is_dragon_pl( pl ) )
-        {
-        int attack;
-        object* tmp;
-        for ( tmp = pl->inv; tmp != NULL; tmp = tmp->below )
-            {
-            if ( ( tmp->type == FORCE ) && ( strcmp( tmp->arch->name, "dragon_skin_force" )== 0 ) )
-                {
-                new_draw_info( NDI_UNIQUE, 0, pl, "\nNatural skin resistances:" );
-                for ( attack = 0; attack < NROFATTACKS; attack++ )
-                    {
-                    if ( atnr_is_dragon_enabled( attack ) )
-                        {
-                        new_draw_info_format( NDI_UNIQUE, 0, pl, "%s: %d", change_resist_msg[ attack ], tmp->resist[ attack ] );
-                        }
-                    }
-                break;
-                }
-            }
-        }
-
    /* Can't think of anything else to print right now */
    return 0;
 }
@@ -947,7 +969,7 @@ int command_logs (object *op, char *params)
 int command_applymode(object *op, char *params)
 {
     unapplymode unapply = op->contr->unapply;
-    static char *types[]={"nochoice", "never", "always"};
+    static const char* const types[]={"nochoice", "never", "always"};
 
     if (!params) {
 	new_draw_info_format(NDI_UNIQUE, 0, op, "applymode is set to %s",
@@ -976,7 +998,7 @@ int command_applymode(object *op, char *params)
 int command_bowmode(object *op, char *params)
 {
     bowtype_t oldtype=op->contr->bowtype;
-    static char *types[] =
+    static const char* const types[] =
 	{"normal", "threewide", "spreadshot", "firenorth",
 	 "firene", "fireeast", "firese", "firesouth",
 	 "firesw", "firewest", "firenw", "bestarrow"};
@@ -1018,7 +1040,7 @@ int command_bowmode(object *op, char *params)
 int command_petmode(object *op, char *params)
 {
     petmode_t oldtype=op->contr->petmode;
-    static char *types[]={"normal", "sad", "defend", "arena"};
+    static const char* const types[]={"normal", "sad", "defend", "arena"};
 
     if (!params) {
 	new_draw_info_format(NDI_UNIQUE, 0, op, "petmode is set to %s",
@@ -1046,10 +1068,50 @@ int command_petmode(object *op, char *params)
     return 1;
 }
 
+int command_showpets(object *op, char *params)
+{
+    objectlink *obl, *next;
+    int counter=0, target=0;
+    int have_shown_pet=0;
+    if (params !=NULL) target= atoi(params);
+    for (obl = first_friendly_object; obl != NULL; obl = next) {
+    	object *ob = obl->ob;
+    	next = obl->next;
+    	if (get_owner(ob) == op) {
+	    if (target ==0) {
+	    	if (counter==0)
+		    new_draw_info(NDI_UNIQUE, 0, op, "Pets:");	
+		new_draw_info_format(NDI_UNIQUE, 0, op, "%d  %s - level %d", ++counter, ob->name, ob->level );
+	    }	
+	    else if (!have_shown_pet && ++counter==target) {
+	    	new_draw_info_format(NDI_UNIQUE, 0, op, "level %d %s", ob->level, ob->name);
+		new_draw_info_format(NDI_UNIQUE, 0, op, "%d/%d HP, %d/%d SP",
+		    ob->stats.hp, ob->stats.maxhp, ob->stats.sp, ob->stats.maxsp);
+		/* this is not a nice way to do this, it should be made to be more like the statistics command */
+		new_draw_info_format(NDI_UNIQUE, 0, op, "Str %d", ob->stats.Str);
+		new_draw_info_format(NDI_UNIQUE, 0, op, "Dex %d", ob->stats.Dex);
+		new_draw_info_format(NDI_UNIQUE, 0, op, "Con %d", ob->stats.Con);
+		new_draw_info_format(NDI_UNIQUE, 0, op, "Int %d", ob->stats.Int);
+		new_draw_info_format(NDI_UNIQUE, 0, op, "Wis %d", ob->stats.Wis);
+		new_draw_info_format(NDI_UNIQUE, 0, op, "Cha %d", ob->stats.Cha);
+		new_draw_info_format(NDI_UNIQUE, 0, op, "Pow %d", ob->stats.Pow);
+		new_draw_info_format(NDI_UNIQUE, 0, op, "wc %d  damage %d ac %d ", 
+		    ob->stats.wc, ob->stats.dam, ob->stats.ac);
+		have_shown_pet=1;
+	    }
+	}
+    }
+    if (counter == 0) 
+    	new_draw_info(NDI_UNIQUE, 0, op, "you have no pets.");
+    else if (target !=0 && have_shown_pet==0)
+    	new_draw_info(NDI_UNIQUE, 0, op, "no such pet.");
+    return 0;
+}
+
 int command_usekeys(object *op, char *params)
 {
     usekeytype oldtype=op->contr->usekeys;
-    static char *types[]={"inventory", "keyrings", "containers"};
+    static const char* const types[]={"inventory", "keyrings", "containers"};
 
     if (!params) {
 	new_draw_info_format(NDI_UNIQUE, 0, op, "usekeys is set to %s",
@@ -1087,14 +1149,36 @@ int command_resistances(object *op, char *params)
 	new_draw_info_format(NDI_UNIQUE, 0, op, "%-20s %+5d", 
 		attacktype_desc[i], op->resist[i]);
     }
-  return 0;
+
+    /* If dragon player, let's display natural resistances */
+    if ( is_dragon_pl( op ) )
+        {
+        int attack;
+        object* tmp;
+        for ( tmp = op->inv; tmp != NULL; tmp = tmp->below )
+            {
+            if ( ( tmp->type == FORCE ) && ( strcmp( tmp->arch->name, "dragon_skin_force" )== 0 ) )
+                {
+                new_draw_info( NDI_UNIQUE, 0, op, "\nNatural skin resistances:" );
+                for ( attack = 0; attack < NROFATTACKS; attack++ )
+                    {
+                    if ( atnr_is_dragon_enabled( attack ) )
+                        {
+                        new_draw_info_format( NDI_UNIQUE, 0, op, "%s: %d", change_resist_msg[ attack ], tmp->resist[ attack ] );
+                        }
+                    }
+                break;
+                }
+            }
+        }
+
+return 0;
 }
-/*
+
+/**
  * Actual commands.
  * Those should be in small separate files (c_object.c, c_wiz.c, cmove.c,...)
  */
-
-
 static void help_topics(object *op, int what)
 {
     DIR *dirp;
@@ -1143,8 +1227,8 @@ static void show_commands(object *op, int what)
 {
   char line[80];
   int i, size, namelen, linelen=0;
-  CommArray_s *ap;
-  extern CommArray_s Commands[], WizCommands[];
+  command_array_struct *ap;
+  extern command_array_struct Commands[], WizCommands[];
   extern const int CommandsSize, WizCommandsSize;
   
   switch (what) {
@@ -1198,8 +1282,7 @@ int command_help (object *op, char *params)
   if (!params) {
     sprintf(filename, "%s/def_help", settings.datadir);
     if ((fp=fopen(filename, "r")) == NULL) {
-      LOG(llevError, "Can't open %s\n", filename);
-      perror("Can't read default help");
+      LOG(llevError, "Cannot open help file %s: %s\n", filename, strerror_local(errno));
       return 0;
     }
     while (fgets(line, MAX_BUF, fp)) {
@@ -1263,8 +1346,7 @@ int command_help (object *op, char *params)
    * Found that. Just cat it to screen.
    */
   if ((fp=fopen(filename, "r")) == NULL) {
-    LOG(llevError, "Can't open %s\n", filename);
-    perror("Can't read helpfile");
+    LOG(llevError, "Cannot open help file %s: %s\n", filename, strerror_local(errno));
     return 0;
       }
   sprintf(line, "Help about '%s'", params);
@@ -1289,7 +1371,7 @@ int command_help (object *op, char *params)
 }
 
 
-int onoff_value(char *line)
+int onoff_value(const char *line)
 {
   int i;
 
@@ -1323,7 +1405,7 @@ int command_quit (object *op, char *params)
     return 1;
   }
 
-/*
+/**
  * don't allow people to exit explore mode.  It otherwise becomes
  * really easy to abuse this.
  */
@@ -1359,7 +1441,8 @@ int command_sound (object *op, char *params)
     return 1;
 }
 
-/* Perhaps these should be in player.c, but that file is
+/**
+ * Perhaps these should be in player.c, but that file is
  * already a bit big.
  */
 
@@ -1384,6 +1467,14 @@ void receive_player_password(object *op,char k) {
     return;
   }
   new_draw_info(NDI_UNIQUE, 0,op,"          "); /* To hide the password better */
+
+  if (checkbanned(op->name, op->contr->socket.host)) {
+    LOG(llevInfo, "Banned player tried to add: [%s@%s]\n", op->name, op->contr->socket.host);
+    new_draw_info(NDI_UNIQUE|NDI_RED, 0, op, "You are not allowed to play.");
+    get_name(op);
+    return;
+  }
+
   if(op->contr->state==ST_CONFIRM_PASSWORD) {
     if(!check_password(op->contr->write_buf+1,op->contr->password)) {
       new_draw_info(NDI_UNIQUE, 0,op,"The passwords did not match.");
@@ -1395,7 +1486,7 @@ void receive_player_password(object *op,char k) {
     new_draw_info(NDI_UNIQUE, 0,op," ");
     new_draw_info(NDI_UNIQUE, 0,op,"Welcome, Brave New Warrior!");
     new_draw_info(NDI_UNIQUE, 0,op," ");
-    Roll_Again(op);
+    roll_again(op);
     op->contr->state=ST_ROLL_STAT;
     return;
   }
@@ -1406,7 +1497,7 @@ void receive_player_password(object *op,char k) {
 }
 
 
-int explore_mode() {
+int explore_mode(void) {
     player *pl;
 
     if (settings.explore_mode == TRUE) {
@@ -1545,8 +1636,31 @@ int command_style_map_info(object *op, char *params)
 
 int command_kill_pets(object *op, char *params)
 {
-    terminate_all_pets(op);
-    new_draw_info(NDI_UNIQUE, 0, op, "Your pets have been killed.");
+    objectlink *obl, *next;
+    int counter=0, removecount=0;
+    if (params == NULL) { 
+    	terminate_all_pets(op);
+	new_draw_info(NDI_UNIQUE, 0, op, "Your pets have been killed.");
+    }
+    else {
+	int target = atoi(params);
+	for(obl = first_friendly_object; obl != NULL; obl = next) {
+	    object *ob = obl->ob;
+	    next = obl->next;
+	    if (get_owner(ob) == op)
+	    	if (++counter==target || (target==0 && !strcasecmp(ob->name, params)))  {
+		if (!QUERY_FLAG(ob, FLAG_REMOVED))
+		    remove_ob(ob);
+		remove_friendly_object(ob);
+		free_object(ob);
+		removecount++;
+            }
+	}
+	if (removecount!=0) 
+	    new_draw_info_format(NDI_UNIQUE, 0, op, "killed %d pets.\n", removecount);
+	else
+	    new_draw_info(NDI_UNIQUE, 0, op, "Couldn't find any suitable pets to kill.\n");
+    }
     return 0;
 }
 
@@ -1555,12 +1669,12 @@ int command_kill_pets(object *op, char *params)
  **/
 static void display_quest_details( object* pl, object* quest )
     {
-    object* item;
+/*    object* item; */
 
     new_draw_info_format( NDI_WHITE, 0, pl, "Quest: %s\n%s\n",
         quest_get_name( quest ),
         quest->lore ? quest->lore : "(no description available)" );
-
+/*
     for ( item = pl->inv; item; item = item->below )
         {
         if ( ( item != quest )
@@ -1572,6 +1686,7 @@ static void display_quest_details( object* pl, object* quest )
             new_draw_info( NDI_WHITE, 0, pl, "------\n" );
             }
         }
+     */
     }
 
 /**
@@ -1593,7 +1708,7 @@ int command_quests( object *pl, char *params )
         new_draw_info( NDI_WHITE, 0, pl, "Completed quests:\n" );
         for ( item = pl->inv; item; item = item->below )
             {
-            if ( quest_is_quest_marker( item ) && quest_is_end( item->slaying ) )
+            if ( quest_is_quest_marker( item, 0 ) )
                 {
                 new_draw_info( NDI_WHITE, 0, pl, quest_get_name( item ) );
                 new_draw_info( NDI_WHITE, 0, pl, "\n" );
@@ -1606,8 +1721,8 @@ int command_quests( object *pl, char *params )
         {
         for ( item = pl->inv; item; item = item->below )
             {
-            if ( quest_is_quest_marker( item )
-                && strstr( quest_get_name( item ), params ) )
+            if ( quest_is_quest_marker( item, 0 )
+                && !strcmp( quest_get_name( item ), params ) )
                 {
                 display_quest_details( pl, item );
                 }
@@ -1615,17 +1730,16 @@ int command_quests( object *pl, char *params )
         return 1;
         }
 
-    /* Display current quests */
+    /*Display current quests */
     new_draw_info( NDI_WHITE, 0, pl, "Current quests:\n" );
     for ( item = pl->inv; item; item = item->below )
         {
-        if ( quest_is_quest_marker( item )
-            && quest_is_start( item->slaying ) )
+        if ( quest_is_quest_marker( item, 0 )
+            && quest_is_in_progress( item, 0 ) )
             {
             new_draw_info( NDI_WHITE, 0, pl, quest_get_name( item ) );
             new_draw_info( NDI_WHITE, 0, pl, "\n" );
             }
         }
-
     return 1;
     }
