@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_object_c =
- *   "$Id: c_object.c,v 1.62 2005/07/16 10:53:10 ryo_saeba Exp $";
+ *   "$Id: c_object.c,v 1.82 2006/06/02 08:19:35 akirschbaum Exp $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -35,6 +35,9 @@
 #endif
 #include <living.h>
 #include <math.h>
+
+static void set_pickup_mode(const object *op, int i);
+
 /*
  * Object id parsing functions
  */
@@ -42,55 +45,36 @@
 #define OBLINKMALLOC(p) if(!((p)=(objectlink *)malloc(sizeof(objectlink))))\
                           fatal(OUT_OF_MEMORY);
 
-#define ADD_ITEM(NEW,COUNT)\
-	  if(!first) {\
-	    OBLINKMALLOC(first);\
-	    last=first;\
-	  } else {\
-	    OBLINKMALLOC(last->next);\
-	    last=last->next;\
-	  }\
-	  last->next=NULL;\
-	  last->ob=(NEW);\
-          last->id=(COUNT);
-
-/* Search the inventory of 'pl' for what matches best with params.
+/**
+ * Search the inventory of 'pl' for what matches best with params.
  * we use item_matched_string above - this gives us consistent behaviour
  * between many commands.  Return the best match, or NULL if no match.
- */
-object *find_best_object_match(object *pl, char *params)
+ * aflag is used with apply -u , and apply -a to
+ * only unapply applied, or apply unapplied objects
+ **/
+static object *find_best_apply_object_match(object *pl, const char *params, enum apply_flag aflag)
 {
     object *tmp, *best=NULL;
     int match_val=0,tmpmatch;
 
     for (tmp=pl->inv; tmp; tmp=tmp->below) {
-	if (tmp->invisible) continue;
-	if ((tmpmatch=item_matched_string(pl, tmp, params))>match_val) {
-	    match_val=tmpmatch;
-	    best=tmp;
-	}
+        if (tmp->invisible) continue;
+        if ((tmpmatch=item_matched_string(pl, tmp, params))>match_val) {
+            if ((aflag==AP_APPLY) && (QUERY_FLAG(tmp,FLAG_APPLIED))) continue;
+            if ((aflag==AP_UNAPPLY) && (!QUERY_FLAG(tmp,FLAG_APPLIED))) continue;
+            match_val=tmpmatch;
+            best=tmp;
+        }
     }
     return best;
 }
-/* Simlilar to find_best_object_match , but accepts an
- * additional parameter for apply -u , and apply -a to
- * only unapply applied , or apply unapplied objects
- */
-static object *find_best_apply_object_match(object *pl, char *params, enum apply_flag aflag)
-{
-    object *tmp, *best=NULL;
-    int match_val=0,tmpmatch;
 
-    for (tmp=pl->inv; tmp; tmp=tmp->below) {
-	if (tmp->invisible) continue;
-	if ((tmpmatch=item_matched_string(pl, tmp, params))>match_val) {
-	  if ((aflag==AP_APPLY) && (QUERY_FLAG(tmp,FLAG_APPLIED))) continue;
-	  if ((aflag==AP_UNAPPLY) && (!QUERY_FLAG(tmp,FLAG_APPLIED))) continue;
-	    match_val=tmpmatch;
-	    best=tmp;
-	}
-    }
-    return best;
+/**
+ * Shortcut to find_best_apply_object_match(pl, params, AF_NULL);
+ **/
+static object *find_best_object_match(object *pl, const char *params)
+{
+    return find_best_apply_object_match(pl, params, AP_NULL);
 }
 
 /*
@@ -438,11 +422,11 @@ int command_rskill ( object *pl, char *params) {
  * and replaced with those.
  */
 int command_search (object *op, char *params) {
-    return command_uskill(op, skill_names[SK_FIND_TRAPS]);
+    return use_skill(op, skill_names[SK_FIND_TRAPS]);
 }
 
 int command_disarm (object *op, char *params) {
-    return command_uskill(op, skill_names[SK_DISARM_TRAPS]);
+    return use_skill(op, skill_names[SK_DISARM_TRAPS]);
 }
 
 
@@ -502,7 +486,7 @@ int command_apply (object *op, char *params)
  * certain number of objects to drop, so we can pass that number, and
  * not need to use split_ob and stuff.
  */
-int sack_can_hold (object *pl, object *sack, object *op, int nrof) {
+int sack_can_hold(const object *pl, const object *sack, const object *op, uint32 nrof) {
 
     if (! QUERY_FLAG (sack, FLAG_APPLIED)) {
 	new_draw_info_format(NDI_UNIQUE, 0, pl, 
@@ -557,7 +541,7 @@ static void pick_up_object (object *pl, object *op, object *tmp, int nrof)
      * (sack, luggage, etc), tmp->env->env then points to the player (nested 
      * containers not allowed as of now)
      */
-    if(QUERY_FLAG(pl, FLAG_FLYING) && !QUERY_FLAG(pl, FLAG_WIZ) && 
+    if((pl->move_type & MOVE_FLYING) && !QUERY_FLAG(pl, FLAG_WIZ) && 
        is_player_inv(tmp)!=pl) {
 	new_draw_info(NDI_UNIQUE, 0,pl, "You are levitating, you can't reach the ground!");
 	return;
@@ -617,7 +601,7 @@ static void pick_up_object (object *pl, object *op, object *tmp, int nrof)
     }
     if(QUERY_FLAG(tmp, FLAG_UNPAID))
 	(void) sprintf(buf,"%s will cost you %s.", query_name(tmp),
-		query_cost_string(tmp,pl,F_BUY));
+		query_cost_string(tmp,pl,F_BUY | F_SHOP));
     else
 	(void) sprintf(buf,"You pick up the %s.", query_name(tmp));
     new_draw_info(NDI_UNIQUE, 0,pl,buf);
@@ -715,7 +699,7 @@ void pick_up(object *op,object *alt)
 	alt = op;
     }
 #ifdef PICKUP_DEBUG
-    printf ("Pick_up(): %s picks %s (%d) and inserts it %s.\n",op->name, tmp->name,  op->contr->count, alt->name);
+    LOG(llevDebug, "Pick_up(): %s picks %s (%d) and inserts it %s.\n", op->name, tmp->name,  op->contr->count, alt->name);
 #endif
 
     /* startequip items are not allowed to be put into containers: */
@@ -812,53 +796,60 @@ int command_take (object *op, char *params)
  *  This function was part of drop, now is own function. 
  *  Player 'op' tries to put object 'tmp' into sack 'sack', 
  *  if nrof is non zero, then nrof objects is tried to put into sack. 
+ * Note that the 'sack' in question can now be a transport,
+ * so this function isn't named very good anymore.
  */
-void put_object_in_sack (object *op, object *sack, object *tmp, long nrof) 
+void put_object_in_sack (object *op, object *sack, object *tmp, uint32 nrof) 
 {
     tag_t tmp_tag, tmp2_tag;
     object *tmp2, *sack2;
     char buf[MAX_BUF];
 
     if (sack==tmp) return;	/* Can't put an object in itself */
-    if (sack->type != CONTAINER) {
-      new_draw_info_format(NDI_UNIQUE, 0,op,
-	"The %s is not a container.", query_name(sack));
-      return;
+    if (sack->type != CONTAINER && sack->type != TRANSPORT) {
+	new_draw_info_format(NDI_UNIQUE, 0,op,
+		     "The %s is not a container.", query_name(sack));
+	return;
     }
     if (QUERY_FLAG(tmp,FLAG_STARTEQUIP)) {
-      new_draw_info_format(NDI_UNIQUE, 0,op,
-	"You cannot put the %s in the container.", query_name(tmp));
-      return;
+	new_draw_info_format(NDI_UNIQUE, 0,op,
+	     "You cannot put the %s in the %s.", query_name(tmp),
+			   query_name(sack));
+	return;
     }
     if (tmp->type == CONTAINER && tmp->inv) {
 
-      /* Eneq(@csd.uu.se): If the object to be dropped is a container
-       * we instead move the contents of that container into the active
-       * container, this is only done if the object has something in it.
-       */
-      sack2 = tmp;
-      new_draw_info_format(NDI_UNIQUE, 0,op, "You move the items from %s into %s.",
-		    query_name(tmp), query_name(op->container));
-      for (tmp2 = tmp->inv; tmp2; tmp2 = tmp) {
-	  tmp = tmp2->below;
-	if (sack_can_hold(op, op->container, tmp2,tmp2->nrof))
-	  put_object_in_sack (op, sack, tmp2, 0);
-	else {
-	  sprintf(buf,"Your %s fills up.", query_name(op->container));
-	  new_draw_info(NDI_UNIQUE, 0,op, buf);
-	  break;
+	/* Eneq(@csd.uu.se): If the object to be dropped is a container
+	 * we instead move the contents of that container into the active
+	 * container, this is only done if the object has something in it.
+	 */
+	sack2 = tmp;
+	new_draw_info_format(NDI_UNIQUE, 0,op, "You move the items from %s into %s.",
+		    query_name(tmp), query_name(sack));
+	for (tmp2 = tmp->inv; tmp2; tmp2 = tmp) {
+	    tmp = tmp2->below;
+	    if ((sack->type == CONTAINER && sack_can_hold(op, op->container, tmp2,tmp2->nrof)) ||
+		(sack->type == TRANSPORT && transport_can_hold(sack, tmp2, tmp2->nrof))) {
+		    put_object_in_sack (op, sack, tmp2, 0);
+	    } else {
+		sprintf(buf,"Your %s fills up.", query_name(sack));
+		new_draw_info(NDI_UNIQUE, 0,op, buf);
+		break;
+	    }
 	}
-      }
-      esrv_update_item (UPD_WEIGHT, op, sack2);
-      return;
+	esrv_update_item (UPD_WEIGHT, op, sack2);
+	return;
     }
 
-    if (! sack_can_hold (op, sack, tmp,(nrof?nrof:tmp->nrof)))
-      return;
+    /* Don't worry about this for containers - our caller should have
+     * already checked this.
+     */
+    if ((sack->type == CONTAINER) && !sack_can_hold (op, sack, tmp,(nrof?nrof:tmp->nrof)))
+	return;
 
     if(QUERY_FLAG(tmp, FLAG_APPLIED)) {
-      if (apply_special (op, tmp, AP_UNAPPLY | AP_NO_MERGE))
-          return;
+	if (apply_special (op, tmp, AP_UNAPPLY | AP_NO_MERGE))
+	    return;
     }
 
     /* we want to put some portion of the item into the container */
@@ -879,12 +870,10 @@ void put_object_in_sack (object *op, object *sack, object *tmp, long nrof)
     } else
 	remove_ob(tmp);
 
-    sprintf(buf, "You put the %s in ", query_name(tmp));
-    strcat (buf, query_name(sack));
-    strcat (buf, ".");
+    new_draw_info_format(NDI_UNIQUE, 0,op, "You put the %s in %s.",
+		  query_name(tmp), query_name(sack));
     tmp_tag = tmp->count;
     tmp2 = insert_ob_in_ob(tmp, sack);
-    new_draw_info(NDI_UNIQUE, 0,op,buf);
     fix_player(op); /* This is overkill, fix_player() is called somewhere */
 		  /* in object.c */
 
@@ -895,13 +884,23 @@ void put_object_in_sack (object *op, object *sack, object *tmp, long nrof)
 	esrv_del_item (op->contr, tmp_tag);
 
     esrv_send_item (op, tmp2);
-    /* update the sacks weight */
-    esrv_update_item (UPD_WEIGHT, op, sack);
+
+    /* If a transport, need to update all the players in the transport
+     * the view of what is in it.
+     */
+    if (sack->type == TRANSPORT) {
+	for (tmp=sack->inv; tmp; tmp=tmp->below) {
+	    if (tmp->type == PLAYER) tmp->contr->socket.update_look=1;
+	}
+    } else {
+	/* update the sacks weight */
+	esrv_update_item (UPD_WEIGHT, op, sack);
+    }
 }
 
 /*
  *  This function was part of drop, now is own function.
- *  Player 'op' tries to drop object 'tmp', if tmp is non zero, then
+ *  Player 'op' tries to drop object 'tmp', if nrof is non zero, then
  *  nrof objects is tried to dropped.
  * This is used when dropping objects onto the floor.
  */
@@ -909,7 +908,6 @@ void drop_object (object *op, object *tmp, uint32 nrof)
 {
     char buf[MAX_BUF];
     object *floor;
-    event *evt;
 
     if (QUERY_FLAG(tmp, FLAG_NO_DROP)) {
 #if 0
@@ -944,36 +942,14 @@ void drop_object (object *op, object *tmp, uint32 nrof)
                         esrv_del_item (op->contr, tmp2_tag);
                 else
                         esrv_send_item (op, tmp2);
-	};
+        }
     } else
       remove_ob (tmp);
-      /* GROS: Handle for plugin drop event */
-      if ((evt = find_event(tmp, EVENT_DROP)) != NULL)
-      {
-        CFParm CFP;
-        CFParm *CFR;
-        int k, l, m, rtn_script;
-        m = 0;
-        k = EVENT_DROP;
-        l = SCRIPT_FIX_ALL;
-        CFP.Value[0] = &k;
-        CFP.Value[1] = op;
-        CFP.Value[2] = tmp;
-        CFP.Value[3] = NULL;
-        CFP.Value[4] = NULL;
-        CFP.Value[5] = &nrof;
-        CFP.Value[6] = &m;
-        CFP.Value[7] = &m;
-        CFP.Value[8] = &l;
-        CFP.Value[9] = evt->hook;
-        CFP.Value[10]= evt->options;
-        if (findPlugin(evt->plugin)>=0)
-        {
-          CFR = ((PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP));
-          rtn_script = *(int *)(CFR->Value[0]);
-          if (rtn_script!=0) return;
-        }
-      }
+
+    /* Lauwenmark: Handle for plugin drop event */
+    if (execute_event(tmp, EVENT_DROP,op,NULL,NULL,SCRIPT_FIX_ALL)!= 0)
+        return;
+
     if (QUERY_FLAG (tmp, FLAG_STARTEQUIP)) {
       sprintf(buf,"You drop the %s.", query_name(tmp));
       new_draw_info(NDI_UNIQUE, 0,op,buf);
@@ -1028,10 +1004,9 @@ void drop_object (object *op, object *tmp, uint32 nrof)
 
     if (op->type == PLAYER)
     {
-    op->contr->socket.update_look = 1;
-/*    esrv_send_item (op, tmp);*/
-    /* Need to update the weight for the player */
-    esrv_send_item (op, op);
+	op->contr->socket.update_look = 1;
+	/* Need to update the weight for the player */
+	esrv_send_item (op, op);
     }
 }
 
@@ -1371,7 +1346,7 @@ void examine_monster(object *op,object *tmp) {
 
 
 /* tmp is the object being described, pl is who is examing it. */
-char *long_desc(object *tmp, object *pl) {
+const char *long_desc(const object *tmp, const object *pl) {
     static char buf[VERY_BIG_BUF];
     char *cp;
 
@@ -1398,6 +1373,7 @@ char *long_desc(object *tmp, object *pl) {
 	case DRINK:
 	case FLESH:
 	case SKILL_TOOL:
+	case POWER_CRYSTAL:
 	    if(*(cp=describe_item(tmp, pl))!='\0') {
 		int len;
 
@@ -1505,18 +1481,25 @@ void examine(object *op, object *tmp) {
 
     if(tmp->weight) {
 	sprintf(buf,tmp->nrof>1?"They weigh %3.3f kg.":"It weighs %3.3f kg.",
-            (tmp->nrof?tmp->weight*tmp->nrof:tmp->weight)/1000.0);
+            tmp->weight*(tmp->nrof?tmp->nrof:1)/1000.0);
 	new_draw_info(NDI_UNIQUE, 0,op,buf);
     }
 
-    if(tmp->value && !QUERY_FLAG(tmp, FLAG_STARTEQUIP) && !QUERY_FLAG(tmp, FLAG_NO_PICK)) {
-	if(QUERY_FLAG(tmp, FLAG_UNPAID))
-	    sprintf(buf,"%s would cost you %s.",
-		    tmp->nrof>1?"They":"It",query_cost_string(tmp,op,F_BUY));
-	else
-	    sprintf(buf,"You would get %s for %s.",
-		    query_cost_string(tmp,op,F_SELL), tmp->nrof>1?"them":"it");
+    if (tmp->value && !QUERY_FLAG(tmp, FLAG_STARTEQUIP) && !QUERY_FLAG(tmp, FLAG_NO_PICK)) {
+    	object *floor;
+    	sprintf(buf,"You reckon %s worth %s.",
+		    tmp->nrof>1?"they are":"it is",query_cost_string(tmp,op,F_SELL | F_APPROX));
 	new_draw_info(NDI_UNIQUE, 0,op,buf);
+	floor = get_map_ob (op->map, op->x, op->y);
+	if (floor && floor->type == SHOP_FLOOR) {
+	    if(QUERY_FLAG(tmp, FLAG_UNPAID))
+	    	sprintf(buf,"%s would cost you %s.",
+		    tmp->nrof>1?"They":"It",query_cost_string(tmp,op,F_BUY | F_SHOP));
+	    else
+	    	sprintf(buf,"You are offered %s for %s.",
+		    query_cost_string(tmp,op,F_SELL+F_SHOP), tmp->nrof>1?"them":"it");
+	    new_draw_info(NDI_UNIQUE, 0,op,buf);
+	}
     }
 
     if(QUERY_FLAG(tmp, FLAG_MONSTER))
@@ -1530,7 +1513,7 @@ void examine(object *op, object *tmp) {
      * types - especially if the first entry is a match
      */
     if(tmp->msg && tmp->type != EXIT && tmp->type != BOOK && 
-       tmp->type != CORPSE && !QUERY_FLAG(tmp, FLAG_WALK_ON) && 
+       tmp->type != CORPSE && !tmp->move_on && 
        strncasecmp(tmp->msg, "@match",7)) {
 
 	/* This is just a hack so when identifying hte items, we print
@@ -1551,7 +1534,7 @@ void examine(object *op, object *tmp) {
  */
 void inventory(object *op,object *inv) {
   object *tmp;
-  char *in;
+  const char *in;
   int items = 0, length;
 
   if (inv==NULL && op==NULL) {
@@ -1604,7 +1587,7 @@ void inventory(object *op,object *inv) {
   }
 }
 
-static void display_new_pickup( object* op )
+static void display_new_pickup(const object* op)
     {
     int i = op->contr->mode;
 
@@ -1643,6 +1626,10 @@ static void display_new_pickup( object* op )
     new_draw_info_format(NDI_UNIQUE, 0,op,"%d READABLES",i & PU_READABLES?1:0);
     new_draw_info_format(NDI_UNIQUE, 0,op,"%d MAGICDEVICE", i & PU_MAGIC_DEVICE?1:0);
 
+    new_draw_info_format(NDI_UNIQUE, 0,op,"%d NOT CURSED", i & PU_NOT_CURSED?1:0);
+
+    new_draw_info_format(NDI_UNIQUE, 0,op,"%d JEWELS", i & PU_JEWELS?1:0);
+
     new_draw_info_format(NDI_UNIQUE, 0,op,"");
     }
 
@@ -1652,11 +1639,11 @@ int command_pickup (object *op, char *params)
   static const char* names[ ] = {
       "debug", "inhibit", "stop", "food", "drink", "valuables", "bow", "arrow", "helmet",
       "shield", "armour", "boots", "gloves", "cloak", "key", "missile", "allweapon",
-      "magical", "potion", "spellbook", "skillscroll", "readables", "magicdevice", NULL };
+      "magical", "potion", "spellbook", "skillscroll", "readables", "magicdevice", "notcursed", "jewels", NULL };
   static uint32 modes[ ] = {
       PU_DEBUG, PU_INHIBIT, PU_STOP, PU_FOOD, PU_DRINK, PU_VALUABLES, PU_BOW, PU_ARROW, PU_HELMET,
       PU_SHIELD, PU_ARMOUR, PU_BOOTS, PU_GLOVES, PU_CLOAK, PU_KEY, PU_MISSILEWEAPON, PU_ALLWEAPON,
-      PU_MAGICAL, PU_POTION, PU_SPELLBOOK, PU_SKILLSCROLL, PU_READABLES, PU_MAGIC_DEVICE, 0 };
+      PU_MAGICAL, PU_POTION, PU_SPELLBOOK, PU_SKILLSCROLL, PU_READABLES, PU_MAGIC_DEVICE, PU_NOT_CURSED, PU_JEWELS, 0 };
 
   if(!params) {
     /* if the new mode is used, just print the settings */
@@ -1665,12 +1652,12 @@ int command_pickup (object *op, char *params)
         display_new_pickup( op );
         return 1;
     }
-    if(1)fprintf(stderr,"command_pickup: !params\n");
+    if(1) LOG(llevDebug, "command_pickup: !params\n");
     set_pickup_mode(op, (op->contr->mode > 6)? 0: op->contr->mode+1);
     return 0;
   }
 
-  while ( *params == ' ' && *params )
+  while ( *params == ' ' )
       params++;
 
   if ( *params == '+' || *params == '-' )
@@ -1696,8 +1683,8 @@ int command_pickup (object *op, char *params)
       return 1;
       }
 
-  if(!sscanf(params, "%ud", &i) || i<0 ) {
-    if(1)fprintf(stderr,"command_pickup: params==NULL\n");
+  if(sscanf(params, "%u", &i) != 1) {
+    if(1) LOG(llevDebug, "command_pickup: params==NULL\n");
     new_draw_info(NDI_UNIQUE, 0,op,"Usage: pickup <0-7> or <value_density> .");
     return 1;
   }
@@ -1707,7 +1694,7 @@ int command_pickup (object *op, char *params)
   return 1;
 }
 
-void set_pickup_mode(object *op,int i) {
+static void set_pickup_mode(const object *op, int i) {
   switch(op->contr->mode=i) {
     case 0:
       new_draw_info(NDI_UNIQUE, 0,op,"Mode: Don't pick up.");
@@ -1779,17 +1766,27 @@ int command_search_items (object *op, char *params)
 int command_rename_item(object *op, char *params)
 {
   char buf[VERY_BIG_BUF];
-
+  int itemnumber;
   object *item=NULL;
+  object *tmp;
   char *closebrace;
-  int counter;
+  size_t counter;
+  tag_t tag;
 
-	if (params) {
+  if (params) {
     /* Let's skip white spaces */
     while(' '==*params) params++;
 
     /* Checking the first part */
-    if ('<'==*params) {
+    if ((itemnumber = atoi(params))!=0) {
+	for (item=op->inv; item && ((item->count != itemnumber) || item->invisible); item=item->below);
+	if (!item) {
+	    new_draw_info(NDI_UNIQUE,0,op,"Tried to rename an invalid item.");
+	    return 1;
+	}
+	while(isdigit(*params) || ' '==*params) params++;
+    }
+    else if ('<'==*params) {
       /* Got old name, let's get it & find appropriate matching item */
       closebrace=strchr(params,'>');
       if(!closebrace) {
@@ -1885,21 +1882,36 @@ int command_rename_item(object *op, char *params)
 
   /* Coming here, everything is fine... */
   if(!strlen(buf)) {
-     /* Clear custom name */
-    if(item->custom_name) {
-      FREE_AND_CLEAR_STR(item->custom_name);
-
-      new_draw_info_format(NDI_UNIQUE, 0, op,"You stop calling your %s with weird names.",query_base_name(item,item->nrof>1?1:0));
-      esrv_update_item(UPD_NAME,op,item);
-    } else {
+    /* Clear custom name */
+    if(item->custom_name == NULL) {
       new_draw_info(NDI_UNIQUE,0,op,"This item has no custom name.");
+      return 1;
     }
+
+    FREE_AND_CLEAR_STR(item->custom_name);
+
+    new_draw_info_format(NDI_UNIQUE, 0, op,"You stop calling your %s with weird names.",query_base_name(item,item->nrof>1?1:0));
   } else {
+    if(item->custom_name != NULL && strcmp(item->custom_name, buf) == 0) {
+      new_draw_info_format(NDI_UNIQUE, 0, op,"You keep calling your %s %s.",query_base_name(item,item->nrof>1?1:0),buf);
+      return 1;
+    }
+
     /* Set custom name */
     FREE_AND_COPY(item->custom_name,buf);
 
     new_draw_info_format(NDI_UNIQUE, 0, op,"Your %s will now be called %s.",query_base_name(item,item->nrof>1?1:0),buf);
-    esrv_update_item(UPD_NAME,op,item);
+  }
+
+  tag = item->count;
+  tmp = merge_ob(item, NULL);
+  if (tmp == NULL) {
+    /* object was not merged */
+    esrv_update_item(UPD_NAME, op, item);
+  } else {
+    /* object was merged into tmp */
+    esrv_del_item(op->contr, tag);
+    esrv_send_item(op, tmp);
   }
 
   return 1;
