@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_party_c =
- *   "$Id: c_party.c,v 1.11 2005/07/15 23:23:07 akirschbaum Exp $";
+ *   "$Id: c_party.c,v 1.24 2005/12/17 19:45:00 ryo_saeba Exp $";
  */
 
 /*
@@ -35,66 +35,104 @@
 static partylist * firstparty=NULL; /* Keeps track of first party in list */
 static partylist * lastparty=NULL; /*Keeps track of last party in list */
 
-partylist * form_party(object *op, char *params, partylist * firstparty, partylist * lastparty) {
+partylist* get_firstparty(void)
+{
+	return firstparty;
+}
+
+void remove_party(partylist *target_party);
+
+/* Forms the party struct for a party called 'params'. it is the responsibility
+ * of the caller to ensure that the name is unique, and that it is placed in the 
+ * main party list correctly */
+static partylist *form_party(object *op, const char *params) {
 
     partylist * newparty;
-    int nextpartyid;
-
-    if(firstparty == NULL) {
-        nextpartyid = 1;
-    } else {
-        nextpartyid = lastparty->partyid;
-        nextpartyid++;
-    }
 
     newparty = (partylist *)malloc(sizeof(partylist));
-    newparty->partyid = nextpartyid;
     newparty->partyname = strdup_local(params);
     newparty->total_exp=0;
     newparty->kills=0;
-
     newparty->passwd[0] = '\0';
     newparty->next = NULL;
     newparty->partyleader = strdup_local(op->name);
-    if(firstparty != NULL)
-        lastparty->next = newparty;
     new_draw_info_format(NDI_UNIQUE, 0, op,
-	"You have formed party: %s",newparty->partyname);
+	"You have formed party: %s",newparty->partyname);	
+    op->contr->party=newparty;
     return newparty;
 }
 
-char * find_party(int partynumber, partylist * party) {
-
-    while(party != NULL) {
-        if(party->partyid == partynumber)
-            return party->partyname;
-        else
-            party = party->next;
+void remove_party(partylist *target_party) {
+    partylist *tmpparty;
+    partylist *previousparty;
+    partylist *nextparty;
+    player *pl;
+    
+    if (firstparty==NULL) {
+	LOG(llevError, "remove_party(): I was asked to remove party %s, but no parties are defined",
+	    target_party->partyname);
+	return;
     }
-    return NULL;
+    for (pl=first_player;pl!=NULL;pl=pl->next)
+	if (pl->party==target_party) pl->party=NULL;
+    
+    /* special case-ism for parties at the beginning and end of the list */
+    if (target_party==firstparty) {
+	firstparty=firstparty->next;
+	if (target_party->partyleader) free(target_party->partyleader);
+	if (target_party->partyname) free(target_party->partyname);
+	free(target_party);
+	return; 
+    }
+    else if (target_party == lastparty) {
+	for (tmpparty=firstparty;tmpparty->next!=NULL;tmpparty=tmpparty->next) {
+	    if (tmpparty->next==target_party) {
+		lastparty=tmpparty;
+		if (target_party->partyleader) free(target_party->partyleader);
+		if (target_party->partyname) free(target_party->partyname);
+		free(target_party); 
+		lastparty->next=NULL;
+		return;
+	    }
+	}
+    }
+    for (tmpparty=firstparty;tmpparty->next!=NULL;tmpparty=tmpparty->next)
+	if (tmpparty->next == target_party) {
+	    previousparty=tmpparty;
+	    nextparty=tmpparty->next->next; 
+	    /* this should be safe, because we already dealt with the lastparty case */
+	
+	    previousparty->next=nextparty;
+	    if (target_party->partyleader) free(target_party->partyleader);
+	    if (target_party->partyname) free(target_party->partyname);
+	    free(target_party); 
+	    return;
+	}
 }
 
-partylist *find_party_struct(int partynumber)
-{
-  partylist *party;
+/* Remove unused parties, this could be made to scale a lot better. */
+void obsolete_parties(void) {
+    int player_count;
+    player *pl;
+    partylist *party;
+	partylist* next = NULL;
 
-  party=firstparty;
-  while(party!=NULL)
-    {
-      if(party->partyid==partynumber) return party;
-      else
-	party=party->next;
+    if (!firstparty) return; /* we can't obsolete parties if there aren't any */
+    for (party=firstparty; party!=NULL; party=next) {
+        next = party->next;
+        player_count=0; 
+        for (pl=first_player;pl!=NULL;pl=pl->next)
+            if (pl->party==party) player_count++;
+        if (player_count == 0)
+            remove_party(party);
     }
-  return NULL;
 }
 
 #ifdef PARTY_KILL_LOG
-void add_kill_to_party(int numb,char *killer,char *dead,long exp)
+void add_kill_to_party(partylist *party, char *killer, char *dead, long exp)
 {
-  partylist *party;
   int i,pos;
 
-  party=find_party_struct(numb);
   if(party==NULL) return;
   if(party->kills>=PARTY_KILL_LOG)
     {
@@ -116,28 +154,25 @@ void add_kill_to_party(int numb,char *killer,char *dead,long exp)
 #endif
 
 int confirm_party_password(object *op) {
-  partylist * tmppartylist;
-
-  tmppartylist = firstparty;
-  while(tmppartylist != NULL) {
-    if(op->contr->party_number_to_join == tmppartylist->partyid) {
-        if(strcmp(op->contr->write_buf+1,tmppartylist->passwd) == 0)
-          return 0;
-        else
-          return 1;
+    partylist *tmppartylist;
+    for(tmppartylist = firstparty; tmppartylist != NULL;tmppartylist = tmppartylist->next) {
+	if(!strcmp(op->contr->party_to_join->partyname, tmppartylist->partyname)) {
+            if(strcmp(op->contr->write_buf+1,tmppartylist->passwd) == 0)
+          	return 0;
+	    else
+		return 1;
+	}
     }
-    tmppartylist = tmppartylist->next;
-  }
-  return 1;
+    return 1;
 }
 
 void receive_party_password(object *op, char k) {
  
   if(confirm_party_password(op) == 0) {
-    partylist* joined_party = find_party_struct( op->contr->party_number_to_join );
+    partylist* joined_party = op->contr->party_to_join;
     char buf[ MAX_BUF ];
-    op->contr->party_number = op->contr->party_number_to_join;
-    op->contr->party_number_to_join = (-1);
+    op->contr->party = op->contr->party_to_join;
+    op->contr->party_to_join = NULL;
     new_draw_info_format(NDI_UNIQUE, 0,op,
 	"You have joined party: %s\n",joined_party->partyname);
     snprintf( buf, MAX_BUF, "%s joins party %s", op->name, joined_party->partyname );
@@ -147,7 +182,7 @@ void receive_party_password(object *op, char k) {
   }
   else {
     new_draw_info(NDI_UNIQUE, 0,op,"You entered the wrong password");
-    op->contr->party_number_to_join = (-1);
+    op->contr->party_to_join = NULL;
     op->contr->state = ST_PLAYING;
     return;
   }
@@ -156,9 +191,8 @@ void receive_party_password(object *op, char k) {
 void send_party_message(object *op,char *msg)
 {
   player *pl;
-  int no=op->contr->party_number;
   for(pl=first_player;pl!=NULL;pl=pl->next)
-    if(pl->ob->contr->party_number==no && pl->ob!=op)
+    if(pl->ob->contr->party==op->contr->party && pl->ob!=op)
 	new_draw_info(NDI_WHITE, 0, pl->ob, msg);
 }
 
@@ -177,16 +211,16 @@ int command_gsay(object *op, char *params)
 int command_party (object *op, char *params)
 {
   char buf[MAX_BUF];
-  partylist * tmpparty; /* For iterating over linked list */
-  char * currentparty; /* For iterating over linked list */
+  partylist *tmpparty, *oldparty;  /* For iterating over linked list */
+  char * currentparty; 	 /* For iterating over linked list */
 
   if(params == NULL) {
-        if(op->contr->party_number<=0) {
+        if(op->contr->party==NULL) {
           new_draw_info(NDI_UNIQUE, 0,op,"You are not a member of any party.");
           new_draw_info(NDI_UNIQUE, 0,op,"For help try: party help");
         }
         else {
-          currentparty = find_party(op->contr->party_number,firstparty);
+          currentparty = op->contr->party->partyname;
 	  new_draw_info_format(NDI_UNIQUE, 0, op,
 		"You are a member of party %s.", currentparty);
         }
@@ -215,12 +249,12 @@ int command_party (object *op, char *params)
       char buffer[80];
       float exp;
 
-      if(op->contr->party_number<=0)
+      if(op->contr->party==NULL)
 	{
 	  new_draw_info(NDI_UNIQUE, 0,op,"You are not a member of any party.");
 	  return 1;
 	}
-      tmpparty = find_party_struct(op->contr->party_number);
+      tmpparty = op->contr->party;
       if(!tmpparty->kills)
 	{
 	  new_draw_info(NDI_UNIQUE,0,op,"You haven't killed anything yet.");
@@ -259,13 +293,13 @@ int command_party (object *op, char *params)
 #endif /* PARTY_KILL_LOG */
   if(strncmp(params, "say ", 4)==0)
     {
-         if(op->contr->party_number<=0)
+         if(op->contr->party==NULL)
             {
               new_draw_info(NDI_UNIQUE, 0,op,"You are not a member of any party.");
               return 1;
             }
          params += 4;
-         currentparty = find_party(op->contr->party_number,firstparty);
+         currentparty = op->contr->party->partyname;
          snprintf(buf,MAX_BUF-1, "[%s] %s says: %s", currentparty, op->name, params);
          send_party_message(op,buf);
          new_draw_info_format(NDI_WHITE, 0,op,"[%s] You say: %s", currentparty, params);
@@ -273,67 +307,68 @@ int command_party (object *op, char *params)
     }
 
   if(strncmp(params, "form ",5) == 0) {
+    int player_count;
+    player *pl;
 
     params += 5;
-
-    if(firstparty == NULL) {
-      firstparty = form_party(op, params, firstparty, lastparty);
-      lastparty = firstparty;
-      return 0;
-    }
-    else
-      tmpparty = firstparty->next;
-
-    if(tmpparty == NULL) {
-      if(strcmp(firstparty->partyname, params) != 0) {
-        lastparty = form_party(op, params, firstparty, lastparty);
-        return 0;
-      }
-      else {
-	new_draw_info_format(NDI_UNIQUE, 0,op,
+    if (op->contr->party) oldparty = op->contr->party;
+    else oldparty = NULL;
+ 
+    if (firstparty) {
+	for (tmpparty = firstparty; tmpparty != NULL;tmpparty = tmpparty->next) {
+	    if (!strcmp(tmpparty->partyname, params)) {
+		new_draw_info_format(NDI_UNIQUE, 0,op,
 		"The party %s already exists, pick another name",params);
-        return 1;
-      }
+        	return 1;
+	    }
+	}
+    lastparty->next=form_party(op, params);
+    lastparty = lastparty->next;
     }
-    tmpparty=firstparty;
-    while(tmpparty != NULL) {
-      if(strcmp(tmpparty->partyname,params) == 0) {
-        new_draw_info_format(NDI_UNIQUE, 0, op,
-		"The party %s already exists, pick a new name",params);
-        return 1;
-      }
-      tmpparty = tmpparty->next;
+    else {
+	firstparty=form_party(op, params);
+	lastparty=firstparty;
+    }
+    /* 
+     * The player might have previously been a member of a party, if so, he will be leaving
+     * it, so check if there are any other members and if not, delete the party
+     */
+    player_count=0; 
+    if (oldparty) {
+    	for (pl=first_player;pl->next!=NULL;pl=pl->next) {
+	    if (pl->party==oldparty) player_count++;
+	}
+	if (player_count == 0)
+	    remove_party(oldparty);
     }
-
-    lastparty = form_party(op, params, firstparty, lastparty);
     return 0;
   } /* form */
 
   if(strcmp(params, "leave")==0) {
-    if(op->contr->party_number<=0)
+    if(op->contr->party==NULL)
       {
         new_draw_info(NDI_UNIQUE, 0,op,"You are not a member of any party.");
         return 1;
       }
-    currentparty = find_party(op->contr->party_number,firstparty);
+    currentparty = op->contr->party->partyname;
     new_draw_info_format(NDI_UNIQUE, 0, op,
 	"You leave party %s.",currentparty);
     sprintf(buf,"%s leaves party %s.",op->name,currentparty);
     send_party_message(op,buf);
-    op->contr->party_number=-1;
+    op->contr->party=NULL;
     return 1;
   }
   if(strcmp(params, "who")==0) {
     player *pl;
-    int no=op->contr->party_number;
-    if(no<=0) {
+    tmpparty = op->contr->party;
+    if(op->contr->party==NULL) {
       new_draw_info(NDI_UNIQUE, 0,op,"You are not a member of any party.");
       return 1;
     }
     new_draw_info_format(NDI_UNIQUE, 0, op,
-	"Members of party: %s.",find_party(no,firstparty));
+	"Members of party: %s.", op->contr->party->partyname);
     for(pl=first_player;pl!=NULL;pl=pl->next)
-      if(pl->ob->contr->party_number==no) {
+      if(pl->ob->contr->party==op->contr->party) {
 	  if (settings.set_title == TRUE) {
 	      if(pl->ob->contr->own_title[0]!='\0')
 		  sprintf(buf,"%3d %s the %s",
@@ -354,7 +389,7 @@ int command_party (object *op, char *params)
 
     params += 7;
 
-    if(op->contr->party_number <= 0) {
+    if(op->contr->party == NULL) {
       new_draw_info(NDI_UNIQUE, 0,op,"You are not a member of a party");
       return 1;
     }
@@ -366,7 +401,7 @@ int command_party (object *op, char *params)
 
     tmplist = firstparty;
     while(tmplist != NULL) {
-      if(tmplist->partyid == op->contr->party_number) {
+      if(tmplist == op->contr->party) {
         strcpy(tmplist->passwd,params);
 	    new_draw_info_format(NDI_UNIQUE, 0, op,
 		    "The password for party %s is %s", tmplist->partyname,tmplist->passwd);
@@ -421,7 +456,7 @@ int command_party (object *op, char *params)
         return 1;
       }
       else {
-        if(op->contr->party_number == firstparty->partyid) {
+        if(op->contr->party == firstparty) {
 	  new_draw_info_format(NDI_UNIQUE, 0, op,
 		"You are already in party: %s"
                   ,firstparty->partyname);
@@ -429,15 +464,15 @@ int command_party (object *op, char *params)
         }
         /* found party player wants to join */
         if(firstparty->passwd[0] == '\0') {
-          op->contr->party_number = firstparty->partyid;
-    	  new_draw_info_format(NDI_UNIQUE, 0, op,
+          op->contr->party = firstparty;
+	  new_draw_info_format(NDI_UNIQUE, 0, op,
 	    	"You have joined party: %s",firstparty->partyname);
           snprintf( buf, MAX_BUF, "%s joins party %s", op->name, firstparty->partyname );
           send_party_message( op, buf );
           return 0;
         }
         else {
-          get_party_password(op,firstparty->partyid);
+          get_party_password(op,firstparty);
           return 0;
         }
       }
@@ -446,7 +481,7 @@ int command_party (object *op, char *params)
     tmpparty = firstparty;
     while(tmpparty != NULL) {
       if(strcmp(tmpparty->partyname,params) == 0) {
-        if(op->contr->party_number == tmpparty->partyid) {
+        if(op->contr->party == tmpparty) {
 	  new_draw_info_format(NDI_UNIQUE, 0, op,
 		"You are already a member of party: %s"
                   ,tmpparty->partyname);
@@ -456,13 +491,13 @@ int command_party (object *op, char *params)
           if(tmpparty->passwd[0] == '\0') {
             new_draw_info_format(NDI_UNIQUE, 0, op,
 		        "You have joined party: %s",tmpparty->partyname);
-            op->contr->party_number = tmpparty->partyid;
+            op->contr->party = tmpparty;
             snprintf( buf, MAX_BUF, "%s joins party %s", op->name, tmpparty->partyname );
             send_party_message( op, buf );
             return 0;
           }
           else {
-            get_party_password(op, tmpparty->partyid);
+            get_party_password(op, tmpparty);
             return 0;
           }
         }
@@ -490,5 +525,3 @@ int command_party (object *op, char *params)
 #endif
   return 1;
 }
-
-
