@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_c_range_c =
- *   "$Id: c_range.c,v 1.10 2003/10/27 07:24:14 mwedel Exp $";
+ *   "$Id: c_range.c,v 1.16 2006/06/05 06:28:56 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -104,7 +104,7 @@ static void show_matching_spells(object *op, char *params)
 	 * string.  given the code above, this is always
 	 * the case.
 	 */
-	qsort(spell_sort, num_found, MAX_BUF, (int (*)())strcmp);
+	qsort(spell_sort, num_found, MAX_BUF, (int (*)(const void*, const void*))strcmp);
 	strcpy(tmp,"asdfg");	/* Dummy string so initial compare fails */
 	for (i=0; i<num_found; i++) {
 	    /* Different skill name, so print banner */
@@ -149,13 +149,24 @@ int command_cast_spell (object *op, char *params, char command)
     }
 
     if(params!=NULL) {
-	spob = lookup_spell_by_name(op, params);
+	int spellnumber = 0;
+	if ((spellnumber = atoi(params))!=0)
+	    for (spob = op->inv; spob && spob->count != spellnumber; spob=spob->below);
+	else spob = lookup_spell_by_name(op, params);
 
-	if (spob) {
+	if (spob && spob->type == SPELL) {
 	    /* Now grab any extra data, if there is any.  Forward pass
 	     * any 'of' delimiter
 	     */
-	    if (strlen(params) > strlen(spob->name)) {
+	    if (spellnumber) {
+		/* if we passed a number, the options start at the second word */
+		cp = strchr(params, ' ');
+		if (cp) {
+		    cp++;
+		    if (!strncmp(cp, "of ", 3)) cp+=3;
+		}
+	    }
+	    else if (strlen(params) > strlen(spob->name)) {
 		cp = params + strlen(spob->name);
 		*cp = 0;
 		cp++;
@@ -175,6 +186,12 @@ int command_cast_spell (object *op, char *params, char command)
 	    } else {
 		op->contr->ranges[range_magic] = spob;
 		op->contr->shoottype = range_magic;
+		if(cp != NULL) {
+		  strncpy(op->contr->spellparam, cp, MAX_BUF);
+		  op->contr->spellparam[MAX_BUF-1] = '\0';
+		} else {
+		  op->contr->spellparam[0] = '\0';
+		}
 		new_draw_info_format(NDI_UNIQUE, 0, op,
 			"You ready the spell %s", spob->name);
 	    }
@@ -183,11 +200,10 @@ int command_cast_spell (object *op, char *params, char command)
     } /* params supplied */
 
     /* We get here if cast was given without options or we could not find
-     * the requested spell.  List all the spells the player knows (if
-     * spnum = -1) or spells matching params if spnum=-2
+     * the requested spell.  List all the spells the player knows.
      */
     new_draw_info(NDI_UNIQUE, 0,op,"Cast what spell?  Choose one of:");
-    show_matching_spells(op, params?params:NULL);
+    show_matching_spells(op, params);
     return 1;
 }
 
