@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_commands_c =
- *   "$Id: commands.c,v 1.51 2005/07/14 16:21:34 ryo_saeba Exp $";
+ *   "$Id: commands.c,v 1.63 2006/06/06 21:38:41 akirschbaum Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -45,7 +45,7 @@
 /*
  * Normal game commands
  */
-CommArray_s Commands[] = {
+command_array_struct Commands[] = {
   {"save", command_save,	0.0},
 
   {"sound", command_sound,	0.0},
@@ -86,6 +86,8 @@ CommArray_s Commands[] = {
   {"mapinfo", command_mapinfo,	0.0},
   {"mark", command_mark,	0.0},
   {"motd", command_motd,	0.0},
+  {"rules", command_rules,	0.0},
+  {"news", command_news,	0.0},
   {"output-sync", command_output_sync,	0.0},
   {"output-count", command_output_count,0.0},
   {"peaceful", command_peaceful,0.0},
@@ -103,6 +105,7 @@ CommArray_s Commands[] = {
   {"ready_skill", command_rskill, 1.0},
   {"search",command_search,	1.0},
   {"search-items", command_search_items,	0.0},
+  {"showpets", command_showpets,	1.0},
   {"statistics", command_statistics,	0.0},
   {"strings", command_strings,	0.0},
   {"take", command_take,	1.0},
@@ -132,9 +135,9 @@ CommArray_s Commands[] = {
   {"northwest", command_northwest,	1.0},
 };
 
-const int CommandsSize =sizeof(Commands) / sizeof(CommArray_s);
+const int CommandsSize =sizeof(Commands) / sizeof(command_array_struct);
 
-CommArray_s CommunicationCommands [] = {
+command_array_struct CommunicationCommands [] = {
   /* begin emotions */
   {"tell", command_tell,		0.1},
   {"reply", command_reply,		0.0},
@@ -201,23 +204,24 @@ CommArray_s CommunicationCommands [] = {
   {"printlos", command_printlos,0.0},
 };
 
-const int CommunicationCommandSize = sizeof(CommunicationCommands)/ sizeof(CommArray_s);
+const int CommunicationCommandSize = sizeof(CommunicationCommands)/ sizeof(command_array_struct);
 
-CommArray_s NewServerCommands [] = {
+command_array_struct NewServerCommands [] = {
   {"run", command_run, 1.0},
   {"run_stop", command_run_stop, 0.0},
   {"fire", command_fire, 1.0},
   {"fire_stop", command_fire_stop, 0.0}
 };
 
-const int NewServerCommandSize = sizeof(NewServerCommands)/ sizeof(CommArray_s);
+const int NewServerCommandSize = sizeof(NewServerCommands)/ sizeof(command_array_struct);
 
 /*
  * Wizard commands (for both)
  */
-CommArray_s WizCommands [] = {
+command_array_struct WizCommands [] = {
   {"abil", command_abil,0.0},
   {"addexp", command_addexp,0.0},
+  {"arrest", command_arrest,0.0},
   {"banish", command_banish,0.0},
   {"create", command_create,0.0},
   {"debug", command_debug,0.0},
@@ -234,8 +238,9 @@ CommArray_s WizCommands [] = {
   {"freeze", command_freeze,0.0},
   {"goto", command_goto,0.0},
   {"hide", command_hide,0.0},
+  {"insert_into", command_insert_into,0.0},
   {"invisible", command_invisible,0.0},
-  {"kick", command_kick, 0.0},
+  {"kick", (command_function)command_kick, 0.0},
   {"learn_special_prayer", command_learn_special_prayer, 0.0},
   {"learn_spell", command_learn_spell, 0.0},
   {"plugin",command_loadplugin,0.0},
@@ -259,53 +264,58 @@ CommArray_s WizCommands [] = {
   {"teleport", command_teleport,0.0},
   {"toggle_shout", command_toggle_shout,0.0},
   {"wizpass", command_wizpass,0.0},
+  {"wizcast", command_wizcast,0.0},
   {"overlay_save", command_save_overlay, 0.0},
 /*  {"possess", command_possess, 0.0}, */
   {"mon_aggr", command_mon_aggr, 0.0},
   {"loadtest", command_loadtest, 0.0},
 };
-const int WizCommandsSize =sizeof(WizCommands) / sizeof(CommArray_s);
+const int WizCommandsSize =sizeof(WizCommands) / sizeof(command_array_struct);
 
 /* Socket commands - these should really do nothing more than output things
  * to the various players/sockets.
  */
-CommArray_s Socket_Commands[] = {
+command_array_struct Socket_Commands[] = {
   {"hiscore", command_hiscore,	0.0},
   {"logs", command_logs,	0.0},
   {"maps", command_maps,	0.0},
   {"motd", command_motd,	0.0},
+  {"rules", command_rules,	0.0},
+  {"news", command_news,	0.0},
   {"players", command_players,	0.0},
   {"version", command_version,	0.0},
   {"who", command_who,		0.0},
 };
 
-const int Socket_CommandsSize =sizeof(Socket_Commands) / sizeof(CommArray_s);
+const int Socket_CommandsSize =sizeof(Socket_Commands) / sizeof(command_array_struct);
 
 
 /* Socket commands - these should really do nothing more than output things
  * to the various players/sockets.
  */
-CommArray_s Socket2_Commands[] = {
+command_array_struct Socket2_Commands[] = {
   {"shout", command_shout,	0.1},
   {"chat", command_chat,	0.1},
   {"tell", command_tell,	0.1},
 };
 
-const int Socket2_CommandsSize =sizeof(Socket2_Commands) / sizeof(CommArray_s);
+const int Socket2_CommandsSize =sizeof(Socket2_Commands) / sizeof(command_array_struct);
 
 
 
 static int compare_A(const void *a, const void *b)
 {
-  return strcmp(((CommArray_s *)a)->name, ((CommArray_s *)b)->name);
+    return strcmp(((const command_array_struct *)a)->name, ((const command_array_struct *)b)->name);
 }
 
-void init_commands()
+void init_commands(void)
 {
-  qsort((char *)Commands, CommandsSize, sizeof(CommArray_s), compare_A);
-  qsort((char *)CommunicationCommands, CommunicationCommandSize, sizeof(CommArray_s), compare_A);
-  qsort((char *)WizCommands, WizCommandsSize, sizeof(CommArray_s), compare_A);
-  qsort((char *)NewServerCommands, NewServerCommandSize, sizeof(CommArray_s), compare_A);
+    qsort(Commands, CommandsSize, sizeof(command_array_struct), compare_A);
+    qsort(CommunicationCommands, CommunicationCommandSize, sizeof(command_array_struct), compare_A);
+    qsort(NewServerCommands, NewServerCommandSize, sizeof(command_array_struct), compare_A);
+    qsort(WizCommands, WizCommandsSize, sizeof(command_array_struct), compare_A);
+    qsort(Socket_Commands, Socket_CommandsSize, sizeof(command_array_struct), compare_A);
+    qsort(Socket2_Commands, Socket2_CommandsSize, sizeof(command_array_struct), compare_A);
 }
 
 #ifndef tolower
@@ -313,9 +323,9 @@ void init_commands()
 #endif
 
 
-CommFunc find_oldsocket_command(char *cmd)
+command_function find_oldsocket_command(char *cmd)
 {
-  CommArray_s *asp, dummy;
+  command_array_struct *asp, dummy;
   char *cp;
 
   for (cp=cmd; *cp; cp++) {
@@ -323,17 +333,17 @@ CommFunc find_oldsocket_command(char *cmd)
   }
 
   dummy.name =cmd;
-  asp =(CommArray_s *)bsearch((void *)&dummy,
+  asp =(command_array_struct *)bsearch((void *)&dummy,
 			      (void *)Socket_Commands, Socket_CommandsSize,
-			      sizeof(CommArray_s), compare_A);
+			      sizeof(command_array_struct), compare_A);
   if (asp)
     return asp->func;
   return NULL;
 }
 
-CommFunc find_oldsocket_command2(char *cmd)
+command_function find_oldsocket_command2(char *cmd)
 {
-  CommArray_s *asp, dummy;
+  command_array_struct *asp, dummy;
   char *cp;
 
   for (cp=cmd; *cp; cp++) {
@@ -341,35 +351,42 @@ CommFunc find_oldsocket_command2(char *cmd)
   }
 
   dummy.name =cmd;
-  asp =(CommArray_s *)bsearch((void *)&dummy,
+  asp =(command_array_struct *)bsearch((void *)&dummy,
 			      (void *)Socket2_Commands, Socket2_CommandsSize,
-			      sizeof(CommArray_s), compare_A);
+			      sizeof(command_array_struct), compare_A);
   if (asp)
     return asp->func;
   return NULL;
 }
 
-static CommFunc find_command(char *cmd)
+#if 0
+/* find_command is only used by parse_string, so if parse_string
+ * isn't needed, neither is find_command.  But parse_string
+ * is only used by parse_command.
+ * MSW 2006-06-02
+ */
+
+static command_function find_command(char *cmd)
 {
-  CommArray_s *asp, dummy;
+  command_array_struct *asp, dummy;
   char *cp;
 
   for (cp=cmd; *cp; cp++)
     *cp =tolower(*cp);
 
   dummy.name =cmd;
-  asp =(CommArray_s *)bsearch((void *)&dummy,
+  asp =(command_array_struct *)bsearch((void *)&dummy,
 			      (void *)Commands, CommandsSize,
-			      sizeof(CommArray_s), compare_A);
-  printf("Getting asp for command string %s\n", cmd);
+			      sizeof(command_array_struct), compare_A);
+  LOG(llevDebug, "Getting asp for command string %s\n", cmd);
   if (asp)
     return asp->func;
   else
   {
-    printf("Now we are here\n");
-    asp =(CommArray_s *)bsearch((void *)&dummy,
+    LOG(llevDebug, "Now we are here\n");
+    asp =(command_array_struct *)bsearch((void *)&dummy,
       (void *)CommunicationCommands, CommunicationCommandSize,
-      sizeof(CommArray_s), compare_A);
+      sizeof(command_array_struct), compare_A);
     if (asp)
       return asp->func;
     else
@@ -377,36 +394,40 @@ static CommFunc find_command(char *cmd)
   };
 }
 
-static CommFunc find_wizcommand(char *cmd)
+static command_function find_wizcommand(char *cmd)
 {
-  CommArray_s *asp, dummy;
+  command_array_struct *asp, dummy;
   char *cp;
 
   for (cp=cmd; *cp; cp++)
     *cp =tolower(*cp);
 
   dummy.name =cmd;
-  asp =(CommArray_s *)bsearch((void *)&dummy,
+  asp =(command_array_struct *)bsearch((void *)&dummy,
 			      (void *)WizCommands, WizCommandsSize,
-			      sizeof(CommArray_s), compare_A);
+			      sizeof(command_array_struct), compare_A);
   if (asp)
     return asp->func;
   return NULL;
 }
 
+/* parse_string is only used by parse_command, so if parse_command
+ * isn't needed, neither is parse_string
+ * MSW 2006-06-02
+ */
 
-/*
+/**
  * parse_string may be called from a player in the game or from a socket
  * (op is NULL if it's a socket).
  * It returnes 1 if it recognized the command, otherwise 0.
  * Actually return value is used as can-repeat -flag
  */
 
-int parse_string(object *op, char *str)
+static int parse_string(object *op, char *str)
 {
-    CommFunc f;
+    command_function f;
     char *cp;
-    CommArray_s *asp;
+    command_array_struct *asp;
 
 #ifdef INPUT_DEBUG
     LOG(llevDebug, "Command: '%s'\n", str);
@@ -462,15 +483,19 @@ int parse_string(object *op, char *str)
 }
 
 
-/*  this function handles splitting up a ; separated
+/* Parse command is no longer used - should probably be removed.
+ * MSW 2006-06-02
+ */
+
+/**  this function handles splitting up a ; separated
  *  compound command into sub-commands:  it is recursive.
  */
-int parse_command(object *op, char *str) {
+static int parse_command(object *op, char *str) {
   char *tmp,*tmp2;
   int i;
   /* if it's a keybinding command, ignore semicolons */
   if(strstr(str,"bind")) return parse_string(op,str);
-printf("parsin command '%s'\n",str);
+  LOG(llevDebug, "parsin command '%s'\n", str);
   /* If on a socket, you can not do complex commands. */
   if(op && (tmp=strchr(str,';'))!=NULL)  /* we've found a ';' do the 1st and recurse */
 	 {
@@ -498,3 +523,4 @@ printf("parsin command '%s'\n",str);
   }
   return 0;
 }
+#endif
