@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_daemon_c =
- *   "$Id: daemon.c,v 1.5 2002/03/20 06:27:46 mwedel Exp $";
+ *   "$Id: daemon.c,v 1.8 2006/05/05 09:47:40 ryo_saeba Exp $";
  */
 
 /*
@@ -65,14 +65,14 @@
 #include <stdio.h>
 #include <sys/file.h>
 
-FILE *BecomeDaemon (char *filename)
+FILE *become_daemon (const char *filename)
 {
   FILE *logfile;
   register int i;
   int forkresult;
  
   if((logfile=fopen(filename,"a"))==NULL){
-    printf("Couldn't create logfile %s.\n",filename);
+    fprintf(stderr, "Couldn't create logfile %s: %s\n", filename, strerror_local(errno));
     exit(0);
   } 
   fputs("\n========================\n",logfile);    
