@@ -1,11 +1,11 @@
 /*
  * static char *rcsid_disease_c =
- *   "$Id: disease.c,v 1.28 2004/08/20 20:02:10 akirschbaum Exp $";
+ *   "$Id: disease.c,v 1.37 2006/06/05 06:28:56 mwedel Exp $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -139,6 +139,14 @@ speed            speed of movement, from DISEASE
 #include <sounds.h>
 #include <skills.h>
 
+static int remove_symptoms(object *disease);
+static object *find_symptom(object *disease);
+static int check_infection(object *disease);
+static int do_symptoms(object *disease);
+static int grant_immunity(object *disease);
+
+
+
 /*  IMPLEMENTATION NOTES  
 	 
 	 Diseases may be contageous.  They are objects which exist in a player's
@@ -154,7 +162,7 @@ static int is_susceptible_to_disease(object *victim, object *disease)
     if(strstr(disease->race, "*") && !QUERY_FLAG(victim, FLAG_UNDEAD))
 	return 1;
 
-    if(strstr(disease->race, "undead") && QUERY_FLAG(victim, FLAG_UNDEAD))
+    if((disease->race == undead_name) && QUERY_FLAG(victim, FLAG_UNDEAD))
 	return 1;
 
     if((victim->race && strstr(disease->race, victim->race)) || 
@@ -166,11 +174,11 @@ static int is_susceptible_to_disease(object *victim, object *disease)
 
 int move_disease(object *disease) {
     /*  first task is to determine if the disease is inside or outside of someone.
-     * If outside, we decrement 'hp' until we're gone. 
+     * If outside, we decrement 'value' until we're gone. 
      */
 
     if(disease->env==NULL) { /* we're outside of someone */
-	disease->value--;
+	if ( disease->stats.maxhp > 0 ) disease->value--;
 	if(disease->value==0) {
 	    remove_ob(disease);
 	    free_object(disease);
@@ -207,7 +215,7 @@ int move_disease(object *disease) {
  * more than one symtpom to a disease.
  */
  
-int remove_symptoms(object *disease) {
+static int remove_symptoms(object *disease) {
     object *symptom, *victim=NULL;
 
     while ((symptom = find_symptom(disease)) != NULL) {
@@ -220,7 +228,7 @@ int remove_symptoms(object *disease) {
 }
 
 /* argument is a disease */
-object * find_symptom(object *disease) {
+static object * find_symptom(object *disease) {
   object *walk;
 
   /* check the inventory for symptoms */
@@ -230,7 +238,7 @@ object * find_symptom(object *disease) {
 }
   
 /*  searches around for more victims to infect */
-int check_infection(object *disease) {
+static int check_infection(object *disease) {
     int x,y,range, mflags;
     mapstruct *map, *map2;
     object *tmp;
@@ -249,8 +257,8 @@ int check_infection(object *disease) {
     }
 
     if(map == NULL) return 0;
-    for(i=x-range;i<x+range;i++) {
-	for(j=y-range;j<y+range;j++) {
+    for(i=x-range;i<=x+range;i++) {
+	for(j=y-range;j<=y+range;j++) {
 	    mflags = get_map_flags(map,&map2, i,j, &i2, &j2);
 	    if (!(mflags & P_OUT_OF_MAP) && (mflags & P_IS_ALIVE)) {
 		for(tmp=get_map_ob(map2,i2,j2);tmp;tmp=tmp->above) {
@@ -336,7 +344,10 @@ int infect_object(object *victim, object *disease, int force) {
     }
 
     insert_ob_in_ob(new_disease,victim);
-    CLEAR_FLAG(new_disease,FLAG_NO_PASS);
+    /* This appears to be a horrible case of overloading 'NO_PASS'
+     * for meaning in the diseases.
+     */
+    new_disease->move_block = 0;
     if(new_disease->owner && new_disease->owner->type==PLAYER) {
 	 char buf[128];
 	 /* if the disease has a title, it has a special infection message
@@ -365,7 +376,7 @@ int infect_object(object *victim, object *disease, int force) {
 causes symptoms, and modifies existing symptoms in the case of
 existing diseases.  */
 
-int do_symptoms(object *disease) {
+static int do_symptoms(object *disease) {
     object *symptom;
     object *victim;
     object *tmp;
@@ -398,7 +409,7 @@ int do_symptoms(object *disease) {
 		    return 0;  /*Immune! */
 	}
 		
-	new_symptom = get_archetype("symptom");
+	new_symptom = create_archetype(ARCH_SYMPTOM);
 
 	/* Something special done with dam.  We want diseases to be more
 	 * random in what they'll kill, so we'll make the damage they
@@ -445,7 +456,7 @@ int do_symptoms(object *disease) {
 	    if (new_symptom->skill) free_string(new_symptom->skill);
 	    if (disease->skill) new_symptom->skill = add_refcount(disease->skill);
 	}
-	CLEAR_FLAG(new_symptom,FLAG_NO_PASS);
+	new_symptom->move_block=0;
 	insert_ob_in_ob(new_symptom,victim);
 	return 1;
     }
@@ -485,7 +496,7 @@ int do_symptoms(object *disease) {
 
 
 /*  grants immunity to plagues we've seen before.  */
-int grant_immunity(object *disease) {
+static int grant_immunity(object *disease) {
   object * immunity;
   object *walk;
   /* Don't give immunity to this disease if last_heal is set. */
@@ -497,10 +508,10 @@ int grant_immunity(object *disease) {
 		return 1; /* just update the existing immunity. */
 	 }
   }
-  immunity = get_archetype("immunity");
+  immunity = create_archetype("immunity");
   immunity->name = add_string(disease->name);
   immunity->level = disease->level;
-  CLEAR_FLAG(immunity,FLAG_NO_PASS);
+  immunity->move_block = 0;
   insert_ob_in_ob(immunity,disease->env);
   return 1;
 
@@ -520,8 +531,8 @@ int move_symptom(object *symptom) {
 	return 0;
     }
 
-    if(symptom->stats.dam > 0)  hit_player(victim,symptom->stats.dam,symptom,symptom->attacktype);
-    else hit_player(victim,MAX(1,-victim->stats.maxhp * symptom->stats.dam / 100.0),symptom,symptom->attacktype);
+    if(symptom->stats.dam > 0)  hit_player(victim,symptom->stats.dam,symptom,symptom->attacktype,1);
+    else hit_player(victim,MAX(1,-victim->stats.maxhp * symptom->stats.dam / 100.0),symptom,symptom->attacktype,1);
 
     if(symptom->stats.maxsp>0) sp_reduce = symptom->stats.maxsp;
     else sp_reduce = MAX(1,victim->stats.maxsp * symptom->stats.maxsp/100.0);
@@ -562,13 +573,20 @@ int check_physically_infect(object *victim, object *hitter) {
   return 1;
 }
 
+#if 0
+/*
+ * find_disease no longer used - perhaps should be removed.
+ * MSW 2006-06-02
+ */
+
 /*  find a disease in someone*/
-object *find_disease(object *victim) {
+static object *find_disease(object *victim) {
   object *walk;
   for(walk=victim->inv;walk;walk=walk->below)
 	 if(walk->type==DISEASE) return walk;
   return NULL;
 }
+#endif
 	
 /* do the cure disease stuff, from the spell "cure disease" */
 
@@ -611,11 +629,18 @@ int cure_disease(object *sufferer,object *caster) {
   return 1;
 }
 
+
+#if 0
+/*
+ * reduce_symptoms is no longer used - should perhaps be removed.
+ * MSW 2006-06-02
+ */
+
 /* reduces disease progression:  reduce_symptoms 
  * return true if we actually reduce a disease.
  */
 
-int reduce_symptoms(object *sufferer, int reduction) {
+static int reduce_symptoms(object *sufferer, int reduction) {
     object *walk;
     int success=0;
 
@@ -635,3 +660,5 @@ int reduce_symptoms(object *sufferer, int reduction) {
     return success;
 }
   
+#endif
+
