@@ -1,7 +1,29 @@
 /*
  * static char *rcsid_egoitem_c =
- *   "$Id: egoitem.c,v 1.5 2004/10/18 06:35:38 akirschbaum Exp $";
+ *   "$Id: egoitem.c,v 1.8 2006/06/05 06:28:56 mwedel Exp $";
  */
+/*
+    CrossFire, A Multiplayer game for X-windows
+
+    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
+    Copyright (C) 1992 Frank Tore Johansen
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+
+    The authors can be reached via e-mail to crossfire-devel@real-time.com
+*/
  
 
 #include <global.h>
@@ -9,10 +31,15 @@
 #include <sproto.h>
 #endif
 
+#if 0
+/* This is no longer use - should be removed if in fact it won't be used.
+ * MSW 2006-06-02
+ */
+
 /* GROS: I put this here, because no other file seemed quite good. Returns 1 if
  * the artifact could be created.
  */
-int create_artifact(object *op, char *artifactname)
+static int create_artifact(object *op, const char *artifactname)
 {
         artifactlist *al;
         artifact *art;
@@ -36,7 +63,7 @@ int create_artifact(object *op, char *artifactname)
         };
         return 0;
 }
-
+#endif
 
 /*  peterm:  do_power_crystal
 
