@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_gods_c =
- *   "$Id: gods.c,v 1.47 2005/08/10 13:38:31 ryo_saeba Exp $";
+ *   "$Id: gods.c,v 1.61 2006/06/05 06:28:56 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -42,11 +42,20 @@
 #include <sproto.h>
 #endif
 
+static int worship_forbids_use(object *op, object *exp_obj, uint32 flag, const char *string);
+static void stop_using_item(object *op, int type, int number);
+static void update_priest_flag (object *god, object *exp_ob, uint32 flag);
+static void god_intervention(object *op, object *god, object *skill);
+static int god_examines_priest (object *op, object *god);
+static int god_examines_item(object *god, object *item);
+static const char *get_god_for_race(const char *race);
+
 /**
  * Returns the id of specified god.
  */
-int lookup_god_by_name(char *name) {
-    int godnr=-1,nmlen = strlen(name);
+static int lookup_god_by_name(const char *name) {
+    int godnr=-1;
+    size_t nmlen = strlen(name);
  
     if(name && strcmp(name,"none")) { 
 	godlink *gl;
@@ -61,7 +70,7 @@ int lookup_god_by_name(char *name) {
 /**
  * Returns pointer to specified god's object through pntr_to_god_obj..
  */
-object *find_god(char *name) {
+object *find_god(const char *name) {
     object *god=NULL;
 
     if(name) { 
@@ -77,11 +86,14 @@ object *find_god(char *name) {
 /**
  * Determines if op worships a god.
  * Returns the godname if they do or "none" if they have no god.
- * In the case of an NPC, if they have no god, we give them a random one. -b.t.
+ * In the case of an NPC, if they have no god, we try and guess 
+ * who they should worship based on their race. If that fails we 
+ * give them a random one.
  */
 
-char *determine_god(object *op) {
+const char *determine_god(object *op) {
     int godnr = -1;
+    const char *godname;
 
     /* spells */
     if ((op->type == SPELL || op->type == SPELL_EFFECT) &&
@@ -91,16 +103,29 @@ char *determine_god(object *op) {
     }
 
     if(op->type!= PLAYER && QUERY_FLAG(op,FLAG_ALIVE)) {
+	
+	/* find a god based on race */
 	if(!op->title) {
-	    godlink *gl=first_god;
+	    if (op->race !=NULL) {
+		godname=get_god_for_race(op->race);
+		if (godname!=NULL) {
+		    op->title = add_string(godname);
+		}
+	    }
+	}
+
+	/* find a random god */
+	if(!op->title) {
+	    godlink *gl = first_god;
 
 	    godnr = rndm(1, gl->id);
 	    while(gl) {
-		if(gl->id==godnr) break;
+		if(gl->id == godnr) break;
 		gl=gl->next;
 	    }
 	    op->title = add_string(gl->name);
 	}
+
 	return op->title;
     }
 
@@ -223,39 +248,10 @@ static int god_gives_present (object *op, object *god, treasure *tr)
  */
 void pray_at_altar(object *pl, object *altar, object *skill) {
     object *pl_god=find_god(determine_god(pl));
-    int return_pray_script; /* GROS : This is for return value of script */
-    event *evt;
 
-    /* GROS: Handle for plugin altar-parying (apply) event */
-    if ((evt = find_event(altar, EVENT_APPLY)) != NULL)
-    {
-        CFParm CFP;
-        CFParm* CFR;
-        int k, l, m;
-        uint32 n;
-        k = EVENT_APPLY;
-        l = SCRIPT_FIX_ALL;
-        m = 0;
-        n = 0;
-        CFP.Value[0] = &k;
-        CFP.Value[1] = pl;
-        CFP.Value[2] = altar;
-        CFP.Value[3] = NULL;
-        CFP.Value[4] = NULL;
-        CFP.Value[5] = &n;
-        CFP.Value[6] = &m;
-        CFP.Value[7] = &m;
-        CFP.Value[8] = &l;
-        CFP.Value[9] = evt->hook;
-        CFP.Value[10]= evt->options;
-        if (findPlugin(evt->plugin)>=0)
-        {
-            CFR = (PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP);
-            return_pray_script = *(int *)(CFR->Value[0]);
-            free(CFR);
-            if (return_pray_script) return;
-        }
-    }
+    /* Lauwenmark: Handle for plugin altar-parying (apply) event */
+    if (execute_event(altar, EVENT_APPLY,pl,NULL,NULL,SCRIPT_FIX_ALL)!=0)
+        return;
 
     /* If non consecrate altar, don't do anything */
     if (!altar->other_arch) return;
@@ -265,7 +261,8 @@ void pray_at_altar(object *pl, object *altar, object *skill) {
 	become_follower(pl,&altar->other_arch->clone);
 	return;
 
-    } else if(!strcmp(pl_god->name,altar->other_arch->clone.name)) { 	/* pray at your gods altar */
+    } else if(!strcmp(pl_god->name,altar->other_arch->clone.name)) {
+        /* pray at your gods altar */
 	int bonus = (pl->stats.Wis+skill->level)/10;
 
 	/* we can get neg grace up faster */
@@ -298,7 +295,8 @@ void pray_at_altar(object *pl, object *altar, object *skill) {
 	 * is the opposing god - we need to verify that exists before
 	 * using its values.
 	 */
-	if(pl_god->other_arch && (altar->other_arch->name==pl_god->other_arch->name)) {
+        if(pl_god->other_arch &&
+           (altar->other_arch->name==pl_god->other_arch->name)) {
 	    angry=2;
 	    if(random_roll(0, skill->level+2, pl, PREFER_LOW)-5 > 0) {
 		object *tmp;
@@ -307,7 +305,7 @@ void pray_at_altar(object *pl, object *altar, object *skill) {
 		angry=3;
 		new_draw_info_format(NDI_UNIQUE|NDI_NAVY,0,pl,
                                 "Foul Priest! %s punishes you!",pl_god->name);
-		tmp=get_archetype(LOOSE_MANA);
+		tmp=create_archetype(LOOSE_MANA);
 		cast_magic_storm(pl,tmp, pl_god->level+20);
 	    } else 
 		new_draw_info_format(NDI_UNIQUE|NDI_NAVY,0,pl,
@@ -392,6 +390,7 @@ static void check_special_prayers (object *op, object *god)
 	     */
 	    new_draw_info_format (NDI_UNIQUE|NDI_NAVY, 0, op,
                           "You lose knowledge of %s.", tmp->name);
+	    player_unready_range_ob(op->contr, tmp);
 	    remove_ob(tmp);
 	    free_object(tmp);
 	}
@@ -426,6 +425,7 @@ void become_follower (object *op, object *new_god) {
 	    if (item->type == SPELL)
 		new_draw_info_format(NDI_UNIQUE|NDI_NAVY, 0, op,
 			"You lose knowledge of %s.", item->name);
+	    player_unready_range_ob(op->contr, item);
 	    remove_ob(item);
 	    free_object(item);
 	}
@@ -445,7 +445,7 @@ void become_follower (object *op, object *new_god) {
 	new_draw_info_format(NDI_UNIQUE|NDI_NAVY,0,op,"Fool! %s detests your kind!",
 			     new_god->name);
         if(random_roll(0, op->level-1, op, PREFER_LOW)-5>0) {
-	    object *tmp = get_archetype(LOOSE_MANA);
+	    object *tmp = create_archetype(LOOSE_MANA);
 	    cast_magic_storm(op,tmp, new_god->level+10);
 	}
 	return;
@@ -523,8 +523,13 @@ void become_follower (object *op, object *new_god) {
 
     /* Weapon/armour use are special...handle flag toggles here as this can
      * only happen when gods are worshipped and if the new priest could
-     * have used armour/weapons in the first place */
-    update_priest_flag(new_god,skop,FLAG_USE_WEAPON);
+     * have used armour/weapons in the first place.
+     *
+     * This also can happen for monks which cannot use weapons. In this case
+     * do not allow to use weapons even if the god otherwise would allow it.
+     */
+    if (!present_in_ob_by_name(FORCE, "no weapon force", op))
+	update_priest_flag(new_god,skop,FLAG_USE_WEAPON);
     update_priest_flag(new_god,skop,FLAG_USE_ARMOUR);
 
     if(worship_forbids_use(op,skop,FLAG_USE_WEAPON,"weapons"))
@@ -555,7 +560,7 @@ void become_follower (object *op, object *new_god) {
  * string is the string to print out.
  */
 
-int worship_forbids_use (object *op, object *exp_obj, uint32 flag, char *string) {
+static int worship_forbids_use (object *op, object *exp_obj, uint32 flag, const char *string) {
 
   if(QUERY_FLAG(&op->arch->clone,flag))
     if(QUERY_FLAG(op,flag)!=QUERY_FLAG(exp_obj,flag)) {
@@ -573,7 +578,7 @@ int worship_forbids_use (object *op, object *exp_obj, uint32 flag, char *string)
 /**
  * Unapplies up to number worth of items of type
  */
-void stop_using_item ( object *op, int type, int number ) {
+static void stop_using_item ( object *op, int type, int number ) {
   object *tmp;
 
   for(tmp=op->inv;tmp&&number;tmp=tmp->below)
@@ -589,7 +594,7 @@ void stop_using_item ( object *op, int type, int number ) {
  * already exist. For players only!
  */
 
-void update_priest_flag (object *god, object *exp_ob, uint32 flag) {
+static void update_priest_flag (object *god, object *exp_ob, uint32 flag) {
       if(QUERY_FLAG(god,flag)&&!QUERY_FLAG(exp_ob,flag))
           SET_FLAG(exp_ob,flag);
       else if(QUERY_FLAG(exp_ob,flag)&&!QUERY_FLAG(god,flag))
@@ -746,7 +751,7 @@ static int god_enchants_weapon (object *op, object *god, object *tr, object *ski
  * called from pray_at_altar() currently. 
  */
 
-void god_intervention (object *op, object *god, object *skill)
+static void god_intervention (object *op, object *god, object *skill)
 {
     treasure *tr;
 
@@ -801,9 +806,9 @@ void god_intervention (object *op, object *god, object *skill)
                 /* Follower lacks the required grace for the following
                  * treasure list items. */
 
-		tmp = get_archetype(HOLY_POSSESSION);
-                (void) cast_change_ability(op, op, tmp, 0);
-		free_object(tmp);
+		        tmp = create_archetype(HOLY_POSSESSION);
+                cast_change_ability(op, op, tmp, 0, 1);
+		        free_object(tmp);
                 return;
             }
             continue;
@@ -854,7 +859,7 @@ void god_intervention (object *op, object *god, object *skill)
 	    object *tmp;
 	    int success;
 
-	    tmp = get_archetype_by_object_name(item->slaying);
+	    tmp = create_archetype_by_object_name(item->slaying);
 
 	    success = cast_heal (op, op, tmp, 0);
 	    free_object(tmp);
@@ -899,7 +904,7 @@ void god_intervention (object *op, object *god, object *skill)
                 continue;
             new_draw_info (NDI_UNIQUE, 0, op,
                     "Shimmering light surrounds and restores you!");
-            for (i = 0; i < 7; i++)
+            for (i = 0; i < NUM_STATS; i++)
                 if (get_attr_value (&depl->stats, i))
                     new_draw_info (NDI_UNIQUE, 0, op, restore_msg[i]);
             remove_ob (depl);
@@ -970,7 +975,7 @@ void god_intervention (object *op, object *god, object *skill)
  * All applied items are examined, if player is using more items of other gods,
  * s/he loses experience in praying or general experience if no praying.
  */
-int god_examines_priest (object *op, object *god) {
+static int god_examines_priest (object *op, object *god) {
     int reaction=1;
     object *item=NULL, *skop;
 
@@ -993,7 +998,7 @@ int god_examines_priest (object *op, object *god) {
 	change_exp(op, -random_roll(0, loss*angry-1, op, PREFER_LOW),
 		   skop?skop->skill:"none", SK_SUBTRACT_SKILL_EXP);
 	if(random_roll(0, angry, op, PREFER_LOW)) {
-	    object *tmp = get_archetype(LOOSE_MANA);
+	    object *tmp = create_archetype(LOOSE_MANA);
 	    cast_magic_storm(op,tmp,op->level+(angry*3));
 	}
 	new_draw_info_format(NDI_UNIQUE|NDI_NAVY,0,op,
@@ -1009,7 +1014,7 @@ int god_examines_priest (object *op, object *god) {
  * god, it can be bad...-b.t.
  */
 
-int god_examines_item(object *god, object *item) {
+static int god_examines_item(object *god, object *item) {
   char buf[MAX_BUF];
 
   if(!god||!item) return 0;
@@ -1034,18 +1039,44 @@ int god_examines_item(object *god, object *item) {
   return 0; /* item is sacred to a non-enemy god/or is otherwise magical */ 
 }
 
+
+#if 0
+/*
+ * get_god no longer used - should perhaps be removed.
+ * MSW 2006-06-02
+ */
+
 /**
  * Returns priest's god's id. 
  * Straight calls lookup_god_by_name
  */
 
-int get_god(object *priest) {
+static int get_god(object *priest) {
   int godnr=lookup_god_by_name(determine_god(priest)); 
 
   return godnr;
 }
+#endif
 
+/**
+ * Returns a string that is the name of the god that should be natively worshipped by a 
+ * creature of who has race *race
+ * if we can't find a god that is appropriate, we return NULL
+ */
+static const char *get_god_for_race(const char *race) {
+    godlink *gl=first_god;
+    const char *godname=NULL;
 
+    if (race == NULL) return NULL;
+    while(gl) {
+	if (!strcasecmp(gl->arch->clone.race, race)) {
+	    godname=gl->name;
+	    break;
+	}
+	gl=gl->next;
+    }
+    return godname;
+}
 /**
  * Changes the attributes of cone, smite, and ball spells as needed by the code.
  * Returns false if there was no race to assign to the slaying field of the spell, but
