@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_hiscore_c =
- *   "$Id: hiscore.c,v 1.11 2005/07/10 13:54:06 akirschbaum Exp $";
+ *   "$Id: hiscore.c,v 1.16 2006/06/05 06:28:56 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -50,7 +50,7 @@ typedef struct scr {
  * log a specified error message if something goes wrong.
  */
 
-char *spool(char *bp,char *error) {
+static char *spool(char *bp, const char *error) {
   static char *prev_pos = NULL;
   char *next_pos;
   if (bp == NULL) {
@@ -78,7 +78,7 @@ char *spool(char *bp,char *error) {
  * to the second one.
  */
 
-static void copy_score(score *sc1,score *sc2) {
+static void copy_score(const score *sc1, score *sc2) {
     strncpy(sc2->name, sc1->name, BIG_NAME);
     sc2->name[BIG_NAME - 1] = '\0';
     strncpy(sc2->title, sc1->title, BIG_NAME);
@@ -97,16 +97,13 @@ static void copy_score(score *sc1,score *sc2) {
  * a pointer to it.
  */
 
-static char *put_score(score *sc) {
-  static char buf[MAX_BUF];
-#ifndef WIN32
-  sprintf(buf,"%s:%s:%lld:%s:%s:%d:%d:%d",sc->name,sc->title,sc->exp,sc->killer,sc->maplevel,
-          sc->maxhp,sc->maxsp,sc->maxgrace);
-#else
-  sprintf(buf,"%s:%s:%I64d:%s:%s:%d:%d:%d",sc->name,sc->title,sc->exp,sc->killer,sc->maplevel,
-          sc->maxhp,sc->maxsp,sc->maxgrace);
-#endif
-  return buf;
+static char *put_score(const score *sc) {
+    static char buf[MAX_BUF];
+
+    snprintf(buf, MAX_BUF,
+	 "%s:%s:%" FMT64 ":%s:%s:%d:%d:%d",sc->name,sc->title,sc->exp,sc->killer,sc->maplevel,
+         sc->maxhp,sc->maxsp,sc->maxgrace);
+    return buf;
 }
 
 /*
@@ -133,11 +130,8 @@ static score *get_score(char *bp) {
 
   if ((cp = spool(NULL, "score")) == NULL)
     return NULL;
-#ifndef WIN32
-  sscanf(cp,"%lld",&sc.exp);
-#else
-  sscanf(cp,"%I64d",&sc.exp);
-#endif
+
+  sscanf(cp,"%" FMT64,&sc.exp);
 
   if ((cp = spool(NULL, "killer")) == NULL)
     return NULL;
@@ -163,19 +157,19 @@ static score *get_score(char *bp) {
   return &sc;
 }
 
-static char * draw_one_high_score(score *sc) {
+static char * draw_one_high_score(const score *sc) {
     static char retbuf[MAX_BUF];
 
     if(!strncmp(sc->killer,"quit",MAX_NAME))
-	sprintf(retbuf,"%3d %10lld %s the %s quit the game on map %s [%d][%d][%d].",
+	sprintf(retbuf,"%3d %10" FMT64 " %s the %s quit the game on map %s [%d][%d][%d].",
             sc->position,sc->exp,sc->name,sc->title,sc->maplevel,sc->maxhp,sc->maxsp,
 		sc->maxgrace);
     else if(!strncmp(sc->killer,"left",MAX_NAME))
-	sprintf(retbuf,"%3d %10lld %s the %s left the game on map %s [%d][%d][%d].",
+	sprintf(retbuf,"%3d %10" FMT64 " %s the %s left the game on map %s [%d][%d][%d].",
             sc->position,sc->exp,sc->name,sc->title,sc->maplevel,sc->maxhp,sc->maxsp,
 		sc->maxgrace);
     else
-	sprintf(retbuf,"%3d %10lld %s the %s was killed by %s on map %s [%d][%d][%d].",
+	sprintf(retbuf,"%3d %10" FMT64 " %s the %s was killed by %s on map %s [%d][%d][%d].",
             sc->position,sc->exp,sc->name,sc->title,sc->killer,sc->maplevel,
             sc->maxhp,sc->maxsp,sc->maxgrace);
     return retbuf;
@@ -220,7 +214,7 @@ static score *add_score(score *new_score) {
   if(!flag&&nrofscores<HIGHSCORE_LENGTH)
     copy_score(new_score,&pscore[nrofscores++]);
   if((fp=fopen(filename,"w"))==NULL) {
-    perror("Can't write to highscore-list.\n");
+    LOG(llevError, "Cannot write to highscore file %s: %s\n", filename, strerror_local(errno));
     return NULL;
   }
   for(i=0;i<nrofscores;i++) {
@@ -323,15 +317,16 @@ void check_score(object *op) {
  * to match to.
  */
 
-void display_high_score(object *op,int max, char *match) {
+void display_high_score(object *op,int max, const char *match) {
+    const size_t maxchar = 80;
     FILE *fp;
     char buf[MAX_BUF],*scorebuf, *bp, *cp;
-    int i=0,j=0,maxchar=80,comp;
+    int i=0,j=0,comp;
     score *sc;
 
     sprintf(buf,"%s/%s",settings.localdir,HIGHSCORE);
     if((fp=open_and_uncompress(buf,0,&comp))==NULL) {
-	perror("Can't open highscore file");
+	LOG(llevError, "Cannot open highscore file %s: %s\n", buf, strerror_local(errno));
 	if(op!=NULL)
 	    new_draw_info(NDI_UNIQUE, 0,op,"There is no highscore file.");
 	return;
@@ -368,7 +363,7 @@ void display_high_score(object *op,int max, char *match) {
 	    while (*bp != ' ' && bp>cp) bp--;
 	    *bp='\0';
 	    if (op == NULL) {
-		printf("%s\n",cp);
+		LOG(llevDebug, "%s\n", cp);
 	    }
 	    else {
 		new_draw_info(NDI_UNIQUE, 0,op,cp);
@@ -378,7 +373,7 @@ void display_high_score(object *op,int max, char *match) {
 	    i++;
 	}
 	if(op == NULL) 
-		printf("%s\n",buf);
+		LOG(llevDebug, "%s\n", buf);
 	else
 		new_draw_info(NDI_UNIQUE, 0,op,buf);
     }
