@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *   "$Id: init.c,v 1.65 2005/07/16 21:10:39 akirschbaum Exp $";
+ *   "$Id: init.c,v 1.78 2006/06/01 18:59:14 tchize Exp $";
  */
 
 /*
@@ -32,59 +32,71 @@
 #include <sproto.h>
 #endif
 
-/* global weathermap */
+static void help(void);
+static void usage(void);
+static void init_beforeplay(void);
+static void init_startup(void);
+static void compile_info(void);
+static void init_signals(void);
+static void init_races(void);
+static void dump_races(void);
+static void add_to_racelist(const char *race_name, object *op);
+static racelink *get_racelist(void);
+
+/** global weathermap */
 weathermap_t **weathermap;
 
-void set_logfile(char *val) { settings.logfilename=val; }
-void call_version() { version(NULL); exit(0); }
-void showscores() { display_high_score(NULL,9999,NULL); exit(0); }
-void set_debug() { settings.debug=llevDebug; }
-void unset_debug() { settings.debug=llevInfo; }
-void set_mondebug() { settings.debug=llevMonster; }
-void set_dumpmon1() {settings.dumpvalues=1; }
-void set_dumpmon2() {settings.dumpvalues=2; }
-void set_dumpmon3() {settings.dumpvalues=3; }
-void set_dumpmon4() {settings.dumpvalues=4; }
-void set_dumpmon5() {settings.dumpvalues=5; }
-void set_dumpmon6() {settings.dumpvalues=6; }
-void set_dumpmon7() {settings.dumpvalues=7; }
-void set_dumpmon8() {settings.dumpvalues=8; }
-void set_dumpmon9() {settings.dumpvalues=9; }
-void set_dumpmont(char *name) {settings.dumpvalues=10; settings.dumparg=name; }
-void set_daemon() {settings.daemonmode=1; }
-void set_datadir(char *path) { settings.datadir=path; }
-void set_confdir(char *path) { settings.confdir=path; }
-void set_localdir(char *path) { settings.localdir=path; }
-void set_mapdir(char *path) { settings.mapdir=path; }
-void set_archetypes(char *path) { settings.archetypes=path; }
-void set_regions(char *path) { settings.regions=path; }
-void set_treasures(char *path) { settings.treasures=path; }
-void set_uniquedir(char *path) { settings.uniquedir=path; }
-void set_playerdir(char *path) { settings.playerdir=path; }
-void set_tmpdir(char *path) { settings.tmpdir=path; }
+static void set_logfile(char *val) { settings.logfilename=val; }
+static void call_version(void) { version(NULL); exit(0); }
+static void showscores(void) { display_high_score(NULL,9999,NULL); exit(0); }
+static void set_debug(void) { settings.debug=llevDebug; }
+static void unset_debug(void) { settings.debug=llevInfo; }
+static void set_mondebug(void) { settings.debug=llevMonster; }
+static void set_dumpmon1(void) {settings.dumpvalues=1; }
+static void set_dumpmon2(void) {settings.dumpvalues=2; }
+static void set_dumpmon3(void) {settings.dumpvalues=3; }
+static void set_dumpmon4(void) {settings.dumpvalues=4; }
+static void set_dumpmon5(void) {settings.dumpvalues=5; }
+static void set_dumpmon6(void) {settings.dumpvalues=6; }
+static void set_dumpmon7(void) {settings.dumpvalues=7; }
+static void set_dumpmon8(void) {settings.dumpvalues=8; }
+static void set_dumpmon9(void) {settings.dumpvalues=9; }
+static void set_dumpmont(char *name) {settings.dumpvalues=10; settings.dumparg=name; }
+static void set_daemon(void) {settings.daemonmode=1; }
+static void set_datadir(char *path) { settings.datadir=path; }
+static void set_confdir(char *path) { settings.confdir=path; }
+static void set_localdir(char *path) { settings.localdir=path; }
+static void set_mapdir(char *path) { settings.mapdir=path; }
+static void set_archetypes(char *path) { settings.archetypes=path; }
+static void set_regions(char *path) { settings.regions=path; }
+static void set_treasures(char *path) { settings.treasures=path; }
+static void set_uniquedir(char *path) { settings.uniquedir=path; }
+static void set_templatedir(char *path) { settings.templatedir=path; }
+static void set_playerdir(char *path) { settings.playerdir=path; }
+static void set_tmpdir(char *path) { settings.tmpdir=path; }
 
-void showscoresparm(char *data) { 
+static void showscoresparm(char *data) { 
     display_high_score(NULL,9999,data); 
     exit(0); 
 }
 
-void set_csport(char *val)
+static void set_csport(char *val)
 { 
     settings.csport=atoi(val);
 #ifndef WIN32 /* ***win32: set_csport: we remove csport error secure check here, do this later */
     if (settings.csport<=0 || settings.csport>32765 ||
 	(settings.csport<1024 && getuid()!=0)) {
-	fprintf(stderr,"%d is an invalid csport number.\n",settings.csport);
+	LOG(llevError, "%d is an invalid csport number.\n", settings.csport);
 	exit(1);
     }
 #endif /* win32 */
 }
 
-/* Most of this is shamelessly stolen from XSysStats.  But since that is
+/** Most of this is shamelessly stolen from XSysStats.  But since that is
  * also my program, no problem.
  */
 struct Command_Line_Options {
-    char    *cmd_option;    /* how it is called on the command line */
+    const char *cmd_option; /* how it is called on the command line */
     uint8   num_args;	    /* Number or args it takes */
     uint8   pass;           /* What pass this should be processed on. */
     void    (*func)();      /* function to call when we match this.
@@ -93,7 +105,7 @@ struct Command_Line_Options {
 			     */
 };
 
-/* The way this system works is pretty simple - parse_args takes
+/** The way this system works is pretty simple - parse_args takes
  * the options passed to the program and a pass number.  If an option
  * matches both in name and in pass (and we have enough options),
  * we call the associated function.  This makes writing a multi
@@ -101,7 +113,7 @@ struct Command_Line_Options {
  */
 struct Command_Line_Options options[] = {
 
-/* Pass 1 functions - STuff that can/should be called before we actually
+/** Pass 1 functions - STuff that can/should be called before we actually
  * initialize any data.
  */
 {"-h", 0, 1, help},
@@ -120,6 +132,7 @@ struct Command_Line_Options options[] = {
 {"-playerdir", 1, 1, set_playerdir},
 {"-treasures", 1, 1, set_treasures},
 {"-uniquedir", 1, 1, set_uniquedir},
+{"-templatedir", 1, 1, set_templatedir},
 {"-tmpdir", 1, 1, set_tmpdir},
 {"-log", 1, 1, set_logfile},
 
@@ -130,13 +143,13 @@ struct Command_Line_Options options[] = {
 {"-srv", 0, 1, service_handle},
 #endif
 
-/* Pass 2 functions.  Most of these could probably be in pass 1, 
+/** Pass 2 functions.  Most of these could probably be in pass 1, 
  * as they don't require much of anything to bet set up.
  */
 {"-csport", 1, 2, set_csport},
 {"-detach", 0, 2, set_daemon},
 
-/* Start of pass 3 information. In theory, by pass 3, all data paths
+/** Start of pass 3 information. In theory, by pass 3, all data paths
  * and defaults should have been set up. 
  */
 {"-o", 0, 3, compile_info},
@@ -156,12 +169,13 @@ struct Command_Line_Options options[] = {
 };
 
 
-/* Note since this may be called before the library has been set up,
+/** Note since this may be called before the library has been set up,
  * we don't use any of crossfires built in logging functions.
  */
 static void parse_args(int argc, char *argv[], int pass)
 {
-    int i, on_arg=1;
+    size_t i;
+    int on_arg=1;
 
     while (on_arg<argc) {
 	for (i=0; i<sizeof(options)/sizeof(struct Command_Line_Options); i++) {
@@ -203,7 +217,7 @@ static void parse_args(int argc, char *argv[], int pass)
 
 materialtype_t *materialt;
 
-static materialtype_t *get_empty_mat() {
+static materialtype_t *get_empty_mat(void) {
     materialtype_t *mt;
     int i;
 
@@ -229,7 +243,7 @@ static materialtype_t *get_empty_mat() {
     return mt;
 }
 
-static void load_materials()
+static void load_materials(void)
 {
     char buf[MAX_BUF], filename[MAX_BUF], *cp, *next;
     FILE *fp;
@@ -314,22 +328,28 @@ static void load_materials()
 	    mt->value = value;
 	}
     }
-    mt->next = NULL;
+    if (mt->next)
+    {
+        free(mt->next);
+    	mt->next = NULL;
+    }
     LOG(llevDebug, "Done.\n");
     fclose(fp);
 }
 
-/* This loads the settings file.  There could be debate whether this should
+/**
+ * This loads the settings file.  There could be debate whether this should
  * be here or in the common directory - but since only the server needs this
  * information, having it here probably makes more sense.
  */
-static void load_settings()
+static void load_settings(void)
 {
     char buf[MAX_BUF],*cp;
     int	has_val,comp;
     FILE    *fp;
 
     sprintf(buf,"%s/settings",settings.confdir);
+
     /* We don't require a settings file at current time, but down the road,
      * there will probably be so many values that not having a settings file
      * will not be a good thing.
@@ -572,15 +592,30 @@ static void load_settings()
 		    "stat_loss_on_death: %s\n", cp);
 	    }
 	} else if (!strcasecmp(buf, "use_permanent_experience")) {
-	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
-		settings.use_permanent_experience=TRUE;
-	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
-		settings.use_permanent_experience=FALSE;
-	    } else {
-		LOG(llevError, "load_settings: Unknown value for "
-		    "use_permanent_experience: %s\n", cp);
-	    }
-	} else if (!strcasecmp(buf, "balanced_stat_loss")) {
+	    LOG(llevError, "use_permanent_experience is deprecated, use"
+		"permenent_experience_percentage instead\n");
+	} else if (!strcasecmp(buf, "permanent_experience_percentage")) {
+	    int val = atoi(cp);
+	      if (val < 0 || val >100)
+		LOG(llevError, "load_settings: permenent_experience_percentage"
+		    "must be between 0 and 100, %d is invalid\n", val);
+	    else
+		settings.permanent_exp_ratio = val;
+	} else if (!strcasecmp(buf, "death_penalty_percentage")) {
+	    int val = atoi(cp);
+	      if (val < 0 || val >100)
+		LOG(llevError, "load_settings: death_penalty_percentage"
+		    "must be between 0 and 100, %d is invalid\n", val);
+	    else
+		settings.death_penalty_ratio = val;
+	} else if (!strcasecmp(buf, "death_penalty_levels")) {
+	    int val = atoi(cp);
+	      if (val < 0 || val > 255)
+		LOG(llevError, "load_settings: death_penalty_levels"
+		    "can not be negative, %d is invalid\n", val);
+	    else
+		settings.death_penalty_level = val;
+	}else if (!strcasecmp(buf, "balanced_stat_loss")) {
 	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
 		settings.balanced_stat_loss=TRUE;
 	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
@@ -657,6 +692,15 @@ static void load_settings()
 		LOG(llevError, "load_settings: unknown value for armor_speed_linear: %s\n", cp);
         }
 
+    } else if ( !strcasecmp( buf, "no_player_stealing" ) ) {
+	    if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+		settings.no_player_stealing = TRUE;
+	    } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+		settings.no_player_stealing = FALSE;
+	    } else {
+		LOG(llevError, "load_settings: unknown value for no_player_stealing: %s\n", cp);
+        }
+
     } else {
 	    LOG(llevError,"Unknown value in settings file: %s\n", buf);
 	}
@@ -665,7 +709,7 @@ static void load_settings()
 }
 
 
-/*
+/**
  * init() is called only once, when starting the program.
  */
 
@@ -700,9 +744,9 @@ void init(int argc, char **argv) {
 
     parse_args(argc, argv, 3);
 
-#ifndef WIN32 /* ***win32: no BecomeDaemon in windows */
+#ifndef WIN32 /* ***win32: no become_daemon in windows */
     if (settings.daemonmode)
-	logfile = BecomeDaemon(settings.logfilename[0]=='\0'?"logfile":settings.logfilename);
+	logfile = become_daemon(settings.logfilename[0]=='\0'?"logfile":settings.logfilename);
 #endif
 
     init_beforeplay();
@@ -712,12 +756,12 @@ void init(int argc, char **argv) {
     init_done=1;
 }
 
-void usage() {
+static void usage(void) {
   (void) fprintf(logfile,
 	"Usage: crossfire [-h] [-<flags>]...\n");
 }
 
-void help() {
+static void help(void) {
 /* The information in usage is redundant with what is given below, so why call it? */
 /*    usage();*/
     printf("Flags:\n");
@@ -741,6 +785,7 @@ void help() {
     printf(" -arch       Sets the archetype file to use.\n");
     printf(" -regions    Sets the regions file to use.\n");
     printf(" -playerdir  Sets the directory for the player files.\n");
+    printf(" -templatedir Sets the directory for template generate maps.\n");
     printf(" -treasures	 Sets the treasures file to use.\n");
     printf(" -uniquedir  Sets the unique items/maps directory.\n");
     printf(" -tmpdir     Sets the directory for temporary files (mostly maps.)\n");
@@ -757,7 +802,7 @@ void help() {
     exit(0);
 }
 
-void init_beforeplay() {
+static void init_beforeplay(void) {
   init_archetypes(); /* If not called before, reads all archetypes from file */
   init_artifacts();  /* If not called before, reads all artifacts from file */
   init_spells();     /* If not called before, links archtypes used by spells */
@@ -801,7 +846,7 @@ void init_beforeplay() {
   }
 }
 
-void init_startup() {
+static void init_startup(void) {
   char buf[MAX_BUF];
   FILE *fp;
   int comp;
@@ -822,13 +867,13 @@ void init_startup() {
   }
 }
 
-/*
+/**
  * compile_info(): activated with the -o flag.
  * It writes out information on how Imakefile and config.h was configured
  * at compile time.
  */
 
-void compile_info() {
+static void compile_info(void) {
   int i=0;
   printf("Non-standard include files:\n");
 #if !defined (__STRICT_ANSI__) || defined (__sun__)
@@ -885,8 +930,7 @@ void compile_info() {
   exit(0);
 #else
   execl("/bin/uname", "uname", "-a", NULL);
-  LOG(llevError, "Oops, shouldn't have gotten here.\n");
-  perror("execl");
+  LOG(llevError, "Oops, shouldn't have gotten here: execl(/bin/uname) failed: %s\n", strerror_local(errno));
   exit(-1);
 #endif
 }
@@ -958,7 +1002,7 @@ void fatal_signal(int make_core, int close_sockets) {
   exit(0);
 }
 
-void init_signals() {
+static void init_signals(void) {
 #ifndef WIN32 /* init_signals() remove signals */
   signal(SIGHUP,rec_sighup);
   signal(SIGINT,rec_sigint);
@@ -975,13 +1019,14 @@ void init_signals() {
 #endif /* win32 */
 }
 
-/* init_races() - reads the races file in the lib/ directory, then
+/** 
+ * Reads the races file in the lib/ directory, then
  * overwrites old 'race' entries. This routine allow us to quickly
  * re-configure the 'alignment' of monsters, objects. Useful for
  * putting together lists of creatures, etc that belong to gods.
  */
  
-void init_races () {
+static void init_races(void) {
   FILE *file;
   char race[MAX_BUF], fname[MAX_BUF], buf[MAX_BUF], *cp, variable[MAX_BUF];
   archetype *mon=NULL;
@@ -994,7 +1039,8 @@ void init_races () {
   sprintf(fname,"%s/races",settings.datadir);
   LOG(llevDebug, "Reading races from %s...",fname);
   if(! (file=fopen(fname,"r"))) {
-        perror(fname); return;
+    LOG(llevError, "Cannot open races file %s: %s\n", fname, strerror_local(errno));
+    return;
   }
 
   while(fgets(buf,MAX_BUF,file)!=NULL) {
@@ -1043,7 +1089,7 @@ void init_races () {
     LOG(llevDebug,"done.\n");
 }
 
-void dump_races()
+static void dump_races(void)
 { 
     racelink *list;
     objectlink *tmp;
@@ -1055,7 +1101,7 @@ void dump_races()
     fprintf(stderr,"\n");
 }
 
-void add_to_racelist (char *race_name, object *op) {
+static void add_to_racelist(const char *race_name, object *op) {
   racelink *race;
  
   if(!op||!race_name) return;
@@ -1077,7 +1123,7 @@ void add_to_racelist (char *race_name, object *op) {
   race->member->ob = op;
 }
 
-racelink * get_racelist ( ) {
+static racelink * get_racelist(void) {
   racelink *list;
  
   list = (racelink *) malloc(sizeof(racelink ));
@@ -1089,7 +1135,7 @@ racelink * get_racelist ( ) {
   return list;
 }
  
-racelink * find_racelink( char *name ) {
+racelink * find_racelink(const char *name) {
   racelink *test=NULL;
  
   if(name&&first_race)
