@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_login_c =
- *   "$Id: login.c,v 1.50 2005/07/14 09:18:55 akirschbaum Exp $";
+ *   "$Id: login.c,v 1.62 2006/06/07 21:44:07 tchize Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -36,7 +36,8 @@
 
 extern void sub_weight (object *, signed long);
 extern void add_weight (object *, signed long);
-extern long pticks;
+
+static void copy_file(const char *filename, FILE *fpout);
 
 /* If flag is non zero, it means that we want to try and save everyone, but
  * keep the game running.  Thus, we don't want to free any information.
@@ -90,12 +91,12 @@ void emergency_save(int flag) {
  * style directory, otherwise, just delete the old style playfile
  * (needed for transition)
  */
-void delete_character(char *name, int new) {
+void delete_character(const char *name, int new) {
     char buf[MAX_BUF];
 
     sprintf(buf,"%s/%s/%s.pl",settings.localdir,settings.playerdir,name);
-    if(unlink(buf)== -1 && settings.debug >= llevDebug)
-	perror("crossfire (delete character)");
+    if(unlink(buf)== -1)
+	LOG(llevDebug, "Cannot delete character file %s: %s\n", buf, strerror_local(errno));
     if (new) {
 	sprintf(buf,"%s/%s/%s",settings.localdir,settings.playerdir,name);
 	/* this effectively does an rm -rf on the directory */
@@ -108,13 +109,23 @@ void delete_character(char *name, int new) {
  * 2 if incorrect password.
  */
 
-int verify_player(char *name, char *password)
+int verify_player(const char *name, char *password)
 {
     char buf[MAX_BUF];
     int comp;
     FILE *fp;
 
-    sprintf(buf,"%s/%s/%s/%s.pl",settings.localdir,settings.playerdir,name,name);
+    if (strpbrk(name, "/.\\") != NULL) {
+	LOG(llevError, "Username contains illegal characters: %s\n", name);
+	return 1;
+    }
+
+    snprintf(buf, sizeof(buf), "%s/%s/%s/%s.pl", settings.localdir, settings.playerdir, name, name);
+    if (strlen(buf) >= sizeof(buf)-1) {
+	LOG(llevError, "Username too long: %s\n", name);
+	return 1;
+    }
+
     if ((fp=open_and_uncompress(buf,0,&comp))==NULL) return 1;
 
     /* Read in the file until we find the password line.  Our logic could
@@ -150,7 +161,7 @@ int verify_player(char *name, char *password)
  * We return 0 if the name is in use/bad, 1 if it is OK to use this name.
  */
 
-int check_name(player *me,char *name) {
+int check_name(player *me,const char *name) {
     player *pl;
 
     for(pl=first_player;pl!=NULL;pl=pl->next)
@@ -176,26 +187,32 @@ int check_name(player *me,char *name) {
     return 1;
 }
 
-int create_savedir_if_needed(char *savedir)
+#if 0
+/*
+ * create_savedir_if_needed no longer needed.  Should perhaps get removed.
+ * MSW 2006-06-02
+ */
+
+static int create_savedir_if_needed(char *savedir)
 {
   struct stat *buf;
 
   if ((buf = (struct stat *) malloc(sizeof(struct stat))) == NULL) {
-    perror("Unable to save playerfile... out of memory.");
+    LOG(llevError, "Unable to save playerfile... out of memory.\n");
     return 0;
   } else {
     stat(savedir, buf);
     if (!S_ISDIR(buf->st_mode))
       if (mkdir(savedir, SAVE_DIR_MODE))
 	{
-	perror("Unable to create player savedir,");
-	perror(savedir);
+	LOG(llevError, "Unable to create player savedir %s: %s\n", savedir, strerror_local(errno));
 	return 0;
       }
     free(buf);
   }
  return 1;
 }
+#endif
 
 void destroy_object (object *op)
 {
@@ -394,7 +411,7 @@ int save_player(object *op, int flag) {
   return 1;
 }
 
-void copy_file(char *filename, FILE *fpout) {
+static void copy_file(const char *filename, FILE *fpout) {
   FILE *fp;
   char buf[MAX_BUF];
   if((fp = fopen(filename,"r")) == NULL)
@@ -415,8 +432,6 @@ void check_login(object *op) {
     int correct = 0;
     time_t    elapsed_save_time=0;
     struct stat	statbuf;
-    CFParm CFP;
-    int evtid;
 
     strcpy (pl->maplevel,first_map_path);
 
@@ -472,14 +487,22 @@ void check_login(object *op) {
 	new_draw_info(NDI_UNIQUE, 0,op," ");
 	FREE_AND_COPY(op->name, "noname");
 	FREE_AND_COPY(op->name_pl, "noname");
-	get_name(op);
+	op->contr->socket.password_fails++;
+	if (op->contr->socket.password_fails >= MAX_PASSWORD_FAILURES) {
+	    new_draw_info(NDI_UNIQUE, 0,op,
+		"You gave an incorrect password too many times, you will now be dropped from the server.");
+	    LOG(llevInfo, "A player connecting from %s has been dropped for password failure\n", 
+		op->contr->socket.host);
+	    op->contr->socket.status = Ns_Dead; /* the socket loop should handle the rest for us */
+	}
+	else get_name(op);
 	return;	    /* Once again, rest of code just loads the char */
     }
 
 #ifdef SAVE_INTERVAL
     pl->last_save_time=time(NULL);
 #endif /* SAVE_INTERVAL */
-    pl->party_number = (-1);
+    pl->party = NULL;
     if (settings.search_items == TRUE)
 	pl->search_str[0]='\0';
     pl->name_changed=1;
@@ -492,6 +515,7 @@ void check_login(object *op) {
     pl->orig_stats.Cha=0;
     strcpy(pl->savebed_map, first_map_path);
     pl->bed_x=0, pl->bed_y=0;
+    pl->spellparam[0] = '\0';
     
     /* Loop through the file, loading the rest of the values */
     while (fgets(bufall,MAX_BUF,fp)!=NULL) {
@@ -584,6 +608,7 @@ void check_login(object *op) {
 	 * Even the load code below will someday be replaced by spells being
 	 * objects.
 	 */
+     /*TODO remove block inside #if 0*/
 	} else if (!strcmp(buf,"known_spell")) {
 #if 0
 	    /* Logic is left here in case someone wants to try
@@ -602,11 +627,11 @@ void check_login(object *op) {
 		LOG(llevDebug, "Error: unknown spell (%s)\n",cp);
 #endif
 	}
-	/* Remove confkeys, pushkey support - very old */
     } /* End of loop loading the character file */
     leave_map(op);
     op->speed=0;
     update_ob_speed(op);
+    /*FIXME dangerous call, reset_object should be used to init freshly allocated obj struct!*/
     reset_object(op);
     op->contr = pl;
     pl->ob = op;
@@ -681,12 +706,9 @@ void check_login(object *op) {
     new_draw_info(NDI_UNIQUE, 0,op,"Welcome Back!");
     new_draw_info_format(NDI_UNIQUE | NDI_ALL | NDI_DK_ORANGE, 5, NULL,
 	     "%s has entered the game.",pl->ob->name);
-    /* GROS : Here we handle the LOGIN global event */
-    evtid = EVENT_LOGIN;
-    CFP.Value[0] = (void *)(&evtid);
-    CFP.Value[1] = (void *)(pl);
-    CFP.Value[2] = (void *)(pl->socket.host);
-    GlobalEvent(&CFP);
+
+    /* Lauwenmark : Here we handle the LOGIN global event */
+    execute_global_event(EVENT_LOGIN, pl, pl->socket.host);
     op->contr->socket.update_look=1;
     /* If the player should be dead, call kill_player for them
      * Only check for hp - if player lacks food, let the normal
@@ -707,6 +729,7 @@ void check_login(object *op) {
      */
     esrv_new_player(op->contr,op->weight+op->carrying);
     esrv_send_inventory(op, op);
+    esrv_add_spells(op->contr, NULL);
 
     CLEAR_FLAG(op, FLAG_FRIENDLY);
 
