@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_monster_c =
- *    "$Id: monster.c,v 1.82 2005/07/28 07:23:19 qal21 Exp $";
+ *    "$Id: monster.c,v 1.101 2006/06/01 18:59:13 tchize Exp $";
  */
 
 /*
@@ -33,6 +33,31 @@
 #include <skills.h>
 #endif
 
+static int can_hit(object *ob1, object *ob2, rv_vector *rv);
+static int monster_cast_spell(object *head, object *part,object *pl,int dir, rv_vector *rv);
+static int monster_use_scroll(object *head, object *part, object *pl, int dir, rv_vector *rv);
+static int monster_use_skill(object *head, object *part, object *pl, int dir);
+static int monster_use_range(object *head, object *part, object *pl, int dir);
+static int monster_use_bow(object *head, object *part, object *pl, int dir);
+static void monster_check_pickup(object *monster);
+static int monster_can_pick(object *monster, object *item);
+static void monster_apply_below(object *monster);
+static int dist_att(int dir, object *ob, object *enemy, object *part, rv_vector *rv);
+static int run_att(int dir, object *ob, object *enemy, object *part, rv_vector *rv);
+static int hitrun_att(int dir, object *ob, object *enemy);
+static int wait_att(int dir, object *ob, object *enemy, object *part, rv_vector *rv);
+static int disthit_att(int dir, object *ob, object *enemy, object *part, rv_vector *rv);
+static int wait_att2(int dir, object *ob, object *enemy, object *part, rv_vector *rv);
+static void circ1_move(object *ob);
+static void circ2_move(object *ob);
+static void pace_movev(object *ob);
+static void pace_moveh(object *ob);
+static void pace2_movev(object *ob);
+static void pace2_moveh(object *ob);
+static void rand_move(object *ob);
+static int talk_to_npc(object *op, object *npc, const char *txt);
+static int talk_to_wall(object *pl, object *npc, const char *txt);
+
 
 #define MIN_MON_RADIUS 3 /* minimum monster detection radius */
 
@@ -48,7 +73,7 @@ object *check_enemy(object *npc, rv_vector *rv) {
     /* if this is pet, let him attack the same enemy as his owner
      * TODO: when there is no ower enemy, try to find a target,
      * which CAN attack the owner. */
-    if ((npc->move_type & HI4) == PETMOVE)
+    if ((npc->attack_movement & HI4) == PETMOVE)
     {
 	if (npc->owner == NULL)
 	    npc->enemy = NULL;
@@ -167,7 +192,7 @@ object *find_nearest_living_creature(object *npc) {
  * many cases.
  */
 
-object *find_enemy(object *npc, rv_vector *rv)
+static object *find_enemy(object *npc, rv_vector *rv)
 {
     object *attacker, *tmp=NULL;
 
@@ -192,7 +217,7 @@ object *find_enemy(object *npc, rv_vector *rv)
      */
 
     /* pet move */
-    if ((npc->move_type & HI4) == PETMOVE) {
+    if ((npc->attack_movement & HI4) == PETMOVE) {
         tmp= get_pet_enemy(npc,rv);
 	if (tmp) get_rangevector(npc, tmp, rv, 0);
 	return tmp;
@@ -243,7 +268,7 @@ object *find_enemy(object *npc, rv_vector *rv)
  * returns 1 if the monster should wake up, 0 otherwise.
  */
 
-int check_wakeup(object *op, object *enemy, rv_vector *rv) {
+static int check_wakeup(object *op, object *enemy, rv_vector *rv) {
     int radius = op->stats.Wis>MIN_MON_RADIUS?op->stats.Wis:MIN_MON_RADIUS;
 
     /* Trim work - if no enemy, no need to do anything below */
@@ -277,7 +302,7 @@ int check_wakeup(object *op, object *enemy, rv_vector *rv) {
     return 0;
 }
 
-int move_randomly(object *op) {
+static int move_randomly(object *op) {
     int i;
 
     /* Give up to 15 chances for a monster to move randomly */
@@ -332,28 +357,21 @@ int move_monster(object *op) {
 	monster_apply_below(op); /* Check for items to apply below */
 
     /*  generate hp, if applicable */
-    if(op->stats.Con&&op->stats.hp<op->stats.maxhp) {
+    if(op->stats.Con > 0 && op->stats.hp < op->stats.maxhp) {
 
 	/* last heal is in funny units.  Dividing by speed puts
 	 * the regeneration rate on a basis of time instead of
 	 * #moves the monster makes.  The scaling by 8 is
 	 * to capture 8th's of a hp fraction regens 
+	 *
+	 * Cast to sint32 before comparing to maxhp since otherwise an (sint16)
+	 * overflow might produce monsters with negative hp.
 	 */
 
 	op->last_heal+= (int)((float)(8*op->stats.Con)/FABS(op->speed));
-	op->stats.hp+=op->last_heal/32; /* causes Con/4 hp/tick */
+	op->stats.hp = MIN((sint32)op->stats.hp+op->last_heal/32, op->stats.maxhp); /* causes Con/4 hp/tick */
 	op->last_heal%=32;
 
-	/* After putting the debug in, turns out quite a few monsters
-	 * end up with hp higher than they should have - in most cases,
-	 * this is just a few (53 instead of 50)  But I also say cases
-	 * where the difference was much greater (450 vs 250, 1057 vs 500, etc)
-	 * so putting in this sanity check is probably a good thing no
-	 * matter what.
-	 */
-	if (op->stats.hp > op->stats.maxhp)
-	    op->stats.hp = op->stats.maxhp;
-
 	/* So if the monster has gained enough HP that they are no longer afraid */
 	if (QUERY_FLAG(op,FLAG_RUN_AWAY) &&
 	    op->stats.hp >= (signed short)(((float)op->run_away/(float)100)*
@@ -365,19 +383,20 @@ int move_monster(object *op) {
     }
 
     /* generate sp, if applicable */
-    if(op->stats.Pow&&op->stats.sp<op->stats.maxsp) {
+    if(op->stats.Pow > 0 && op->stats.sp < op->stats.maxsp) {
 
 	/*  last_sp is in funny units.  Dividing by speed puts
          * the regeneration rate on a basis of time instead of
          * #moves the monster makes.  The scaling by 8 is
          * to capture 8th's of a sp fraction regens 
+	 *
+	 * Cast to sint32 before comparing to maxhp since otherwise an (sint16)
+	 * overflow might produce monsters with negative sp.
 	 */
 
 	op->last_sp+= (int)((float)(8*op->stats.Pow)/FABS(op->speed));
-	op->stats.sp+=op->last_sp/128;  /* causes Pow/16 sp/tick */
+	op->stats.sp = MIN(op->stats.sp+op->last_sp/128, op->stats.maxsp);  /* causes Pow/16 sp/tick */
 	op->last_sp%=128;
-	if(op->stats.sp>op->stats.maxsp)
-	    op->stats.sp=op->stats.maxsp;
     }
 
     /* this should probably get modified by many more values.
@@ -399,9 +418,9 @@ int move_monster(object *op) {
 	 * stand still and a movement type set.
 	 */
         if(!QUERY_FLAG(op, FLAG_STAND_STILL))  {
-            if (op->move_type & HI4)
+            if (op->attack_movement & HI4)
             {
-                switch (op->move_type & HI4) {
+                switch (op->attack_movement & HI4) {
 		    case (PETMOVE):
 			pet_move (op);
 			break;
@@ -448,7 +467,7 @@ int move_monster(object *op) {
     } /* no enemy */
 
     /* We have an enemy.  Block immediately below is for pets */
-    if((op->type&HI4) == PETMOVE && (owner = get_owner(op)) != NULL && !on_same_map(op,owner))
+    if((op->attack_movement&HI4) == PETMOVE && (owner = get_owner(op)) != NULL && !on_same_map(op,owner))
     {
 	follow_owner(op, owner);
 	/* If the pet was unable to follow the owner, free it */
@@ -466,8 +485,10 @@ int move_monster(object *op) {
      */
     if ( (op->race != NULL)&& strcmp(op->race,"doppleganger") == 0)
     {
-	op->face = enemy->face; 
-	strcpy(op->name,enemy->name);
+	op->face = enemy->face;
+    if (op->name)
+        free_string(op->name);
+	add_refcount(op->name = enemy->name);
     }
 
     /* Calculate range information for closest body part - this
@@ -476,6 +497,13 @@ int move_monster(object *op) {
      * into themselves.
      */
     get_rangevector(op, enemy, &rv, 0);
+    if ( op->direction != rv.direction )
+    {
+    	op->direction = rv.direction;
+     	op->facing = op->direction;
+	if (op->animation_id)
+	    animate_object(op, op->direction);
+    }
 
     /* Move the check for scared up here - if the monster was scared,
      * we were not doing any of the logic below, so might as well save
@@ -533,9 +561,9 @@ int move_monster(object *op) {
     if(QUERY_FLAG(op,FLAG_CONFUSED))
 	dir = absdir(dir + RANDOM()%3 + RANDOM()%3 - 2);
 
-    if ((op->move_type & LO4) && !QUERY_FLAG(op, FLAG_SCARED)) 
+    if ((op->attack_movement & LO4) && !QUERY_FLAG(op, FLAG_SCARED)) 
     {        
-	switch (op->move_type & LO4) {
+	switch (op->attack_movement & LO4) {
 	    case DISTATT:
 		dir = dist_att (dir,op,enemy,part,&rv);
 		break;
@@ -565,7 +593,7 @@ int move_monster(object *op) {
 		break;
 
 	    default:
-		LOG(llevDebug,"Illegal low mon-move: %d\n",op->move_type & LO4);
+		LOG(llevDebug,"Illegal low mon-move: %d\n",op->attack_movement & LO4);
 	}
     }
 
@@ -648,7 +676,8 @@ int move_monster(object *op) {
     }
     return 0;
 }
-int can_hit(object *ob1,object *ob2, rv_vector *rv) {
+
+static int can_hit(object *ob1,object *ob2, rv_vector *rv) {
     object *more;
     rv_vector rv1;
 
@@ -707,7 +736,7 @@ static int monster_should_cast_spell(object *monster, object *spell_ob)
  * wizard spells, as the check is against sp, and not grace.
  * can mosnters know cleric spells?
  */
-object *monster_choose_random_spell(object *monster) {
+static object *monster_choose_random_spell(object *monster) {
     object *altern[MAX_KNOWN_SPELLS];
     object *tmp;
     int i=0;
@@ -738,7 +767,7 @@ object *monster_choose_random_spell(object *monster) {
  * rv is the vector which describes where the enemy is.
  */
 
-int monster_cast_spell(object *head, object *part,object *pl,int dir, rv_vector *rv) {
+static int monster_cast_spell(object *head, object *part,object *pl,int dir, rv_vector *rv) {
     object *spell_item;
     object *owner;
     rv_vector	rv1;
@@ -806,7 +835,7 @@ int monster_cast_spell(object *head, object *part,object *pl,int dir, rv_vector
 }
 
 
-int monster_use_scroll(object *head, object *part,object *pl,int dir, rv_vector *rv) {
+static int monster_use_scroll(object *head, object *part,object *pl,int dir, rv_vector *rv) {
     object *scroll;
     object *owner;
     rv_vector	rv1;
@@ -859,7 +888,7 @@ int monster_use_scroll(object *head, object *part,object *pl,int dir, rv_vector
  * stealing. TODO: This should be more integrated in the game. -MT, 25.11.01
  */  
 
-int monster_use_skill(object *head, object *part, object *pl,int dir) {
+static int monster_use_skill(object *head, object *part, object *pl,int dir) {
     object *skill, *owner;
 
     if(!(dir=path_to_player(part,pl,0)))
@@ -897,7 +926,7 @@ int monster_use_skill(object *head, object *part, object *pl,int dir) {
 
 /* Monster will use a ranged spell attack. */
 
-int monster_use_range(object *head,object *part,object *pl,int dir)
+static int monster_use_range(object *head,object *part,object *pl,int dir)
     {
     object *wand, *owner;
     int at_least_one = 0;
@@ -964,7 +993,7 @@ int monster_use_range(object *head,object *part,object *pl,int dir)
 	return 0;
     }
 
-int monster_use_bow(object *head, object *part, object *pl, int dir) {
+static int monster_use_bow(object *head, object *part, object *pl, int dir) {
     object *owner;
 
     if(!(dir=path_to_player(part,pl,0)))
@@ -990,7 +1019,7 @@ int monster_use_bow(object *head, object *part, object *pl, int dir) {
  * return true if item is a better object.
  */
 
-int check_good_weapon(object *who, object *item) {
+static int check_good_weapon(object *who, object *item) {
     object *other_weap;
     int val=0, i;
 
@@ -1020,7 +1049,7 @@ int check_good_weapon(object *who, object *item) {
 
 }
 
-int check_good_armour(object *who, object *item) {
+static int check_good_armour(object *who, object *item) {
     object *other_armour;
     int val=0,i;
 
@@ -1071,13 +1100,13 @@ int check_good_armour(object *who, object *item) {
  * affect stacking on this space.
  */
 
-void monster_check_pickup(object *monster) {
+static void monster_check_pickup(object *monster) {
     object *tmp,*next;
     int next_tag;
 
     for(tmp=monster->below;tmp!=NULL;tmp=next) {
 	next=tmp->below;
-	if (next) next_tag = next->count;
+	next_tag = next ? next->count : 0;
 	if (monster_can_pick(monster,tmp)) {
 	    remove_ob(tmp);
 	    tmp = insert_ob_in_ob(tmp,monster);
@@ -1097,7 +1126,7 @@ void monster_check_pickup(object *monster) {
  * I've already utilized flags for bows, wands, rings, etc, etc. -Frank.
  */
 
-int monster_can_pick(object *monster, object *item) {
+static int monster_can_pick(object *monster, object *item) {
     int flag=0;
     int i;
 
@@ -1153,6 +1182,7 @@ int monster_can_pick(object *monster, object *item) {
 
 	case SCROLL:
 	    flag = QUERY_FLAG(monster,FLAG_USE_SCROLL);
+	    break;
 
 	case BOW:
 	case ARROW:
@@ -1182,7 +1212,7 @@ int monster_can_pick(object *monster, object *item) {
  * then make him apply it
  */
 
-void monster_apply_below(object *monster) {
+static void monster_apply_below(object *monster) {
     object *tmp, *next;
 
     for(tmp=monster->below;tmp!=NULL;tmp=next) {
@@ -1249,10 +1279,12 @@ void monster_check_apply(object *mon, object *item) {
     /* Eating food gets hp back */
     else if (item->type == FOOD && mon->will_apply & WILL_APPLY_FOOD) flag=1;
     else if (item->type == SCROLL && QUERY_FLAG(mon, FLAG_USE_SCROLL)) {
-	if (monster_should_cast_spell(mon, item->inv))
-	    SET_FLAG(mon, FLAG_READY_SCROLL);
-	/* Don't use it right now */
-	return;
+        if (!item->inv)
+            LOG(llevDebug,"Monster %d having scroll %d with empty inventory!", mon->count, item->count);
+        else if (monster_should_cast_spell(mon, item->inv))
+            SET_FLAG(mon, FLAG_READY_SCROLL);
+        /* Don't use it right now */
+        return;
     }
     else if (item->type == WEAPON) flag = check_good_weapon(mon,item);
     else if (IS_ARMOR(item)) flag = check_good_armour(mon,item);
@@ -1282,12 +1314,11 @@ void monster_check_apply(object *mon, object *item) {
     else if ( item->type == SKILL )
         {
         /*
-         * Ryo 2004-05-08
-         * skills are specials: monsters must have the 'FLAG_CAN_USE_SKILL' flag set,
+         * skills are specials: monsters must have the 'FLAG_READY_SKILL' flag set,
          * else they can't use the skill...
          * Skills also don't need to get applied, so return now.
          */
-        SET_FLAG( item, FLAG_CAN_USE_SKILL );
+        SET_FLAG(mon, FLAG_READY_SKILL);
         return;
         }
 
@@ -1337,7 +1368,7 @@ void npc_call_help(object *op) {
 }
 
 
-int dist_att (int dir , object *ob, object *enemy, object *part, rv_vector *rv) {
+static int dist_att (int dir , object *ob, object *enemy, object *part, rv_vector *rv) {
 
     if (can_hit(part,enemy,rv))
     	return dir;
@@ -1348,7 +1379,7 @@ int dist_att (int dir , object *ob, object *enemy, object *part, rv_vector *rv)
     return 0;
 }
 
-int run_att (int dir, object *ob, object *enemy,object *part, rv_vector *rv) {
+static int run_att (int dir, object *ob, object *enemy,object *part, rv_vector *rv) {
 
     if ((can_hit (part,enemy,rv) && ob->move_status <20) || ob->move_status <20) {
 	ob->move_status++;
@@ -1359,7 +1390,7 @@ int run_att (int dir, object *ob, object *enemy,object *part, rv_vector *rv) {
     return absdir (dir+4);
 }
 
-int hitrun_att (int dir, object *ob,object *enemy) {
+static int hitrun_att (int dir, object *ob,object *enemy) {
     if (ob->move_status ++ < 25)  
 	return dir;
     else if (ob->move_status <50) 
@@ -1369,7 +1400,7 @@ int hitrun_att (int dir, object *ob,object *enemy) {
     return absdir(dir+4);
 }
 
-int wait_att (int dir, object *ob,object *enemy,object *part,rv_vector *rv) {
+static int wait_att (int dir, object *ob,object *enemy,object *part,rv_vector *rv) {
 
     int inrange = can_hit (part, enemy,rv);
       
@@ -1386,7 +1417,7 @@ int wait_att (int dir, object *ob,object *enemy,object *part,rv_vector *rv) {
     return 0;
 }
 
-int disthit_att (int dir, object *ob, object *enemy, object *part,rv_vector *rv) {
+static int disthit_att (int dir, object *ob, object *enemy, object *part,rv_vector *rv) {
 
     /* The logic below here looked plain wrong before.  Basically, what should
      * happen is that if the creatures hp percentage falls below run_away,
@@ -1400,13 +1431,13 @@ int disthit_att (int dir, object *ob, object *enemy, object *part,rv_vector *rv)
     return dist_att (dir,ob,enemy,part,rv);
 }
 
-int wait_att2 (int dir, object *ob,object *enemy,object *part, rv_vector *rv) {
+static int wait_att2 (int dir, object *ob,object *enemy,object *part, rv_vector *rv) {
     if (rv->distance < 9)
 	return absdir (dir+4);
     return 0;
 }
 
-void circ1_move (object *ob) {
+static void circ1_move (object *ob) {
   static int circle [12] = {3,3,4,5,5,6,7,7,8,1,1,2};
   if(++ob->move_status > 11)
     ob->move_status = 0;
@@ -1414,7 +1445,7 @@ void circ1_move (object *ob) {
     (void) move_object(ob,RANDOM()%8+1);
 }
 
-void circ2_move (object *ob) {
+static void circ2_move (object *ob) {
   static int circle[20] = {3,3,3,4,4,5,5,5,6,6,7,7,7,8,8,1,1,1,2,2};
   if(++ob->move_status > 19)
     ob->move_status = 0;
@@ -1422,7 +1453,7 @@ void circ2_move (object *ob) {
     (void) move_object(ob,RANDOM()%8+1);
 }
 
-void pace_movev(object *ob) {
+static void pace_movev(object *ob) {
   if (ob->move_status++ > 6)
     ob->move_status = 0;
   if (ob->move_status < 4)
@@ -1431,7 +1462,7 @@ void pace_movev(object *ob) {
     (void) move_object(ob,1);
 }
 
-void pace_moveh (object *ob) {
+static void pace_moveh (object *ob) {
   if (ob->move_status++ > 6)
     ob->move_status = 0;
   if (ob->move_status < 4)
@@ -1440,7 +1471,7 @@ void pace_moveh (object *ob) {
     (void) move_object(ob,7);
 }
 
-void pace2_movev (object *ob) {
+static void pace2_movev (object *ob) {
   if (ob->move_status ++ > 16)
     ob->move_status = 0;
   if (ob->move_status <6)
@@ -1452,7 +1483,7 @@ void pace2_movev (object *ob) {
   else return;
 }       
 
-void pace2_moveh (object *ob) {
+static void pace2_moveh (object *ob) {
   if (ob->move_status ++ > 16)
     ob->move_status = 0;
   if (ob->move_status <6)
@@ -1464,7 +1495,7 @@ void pace2_moveh (object *ob) {
   else return;
 }       
 
-void rand_move (object *ob) {
+static void rand_move (object *ob) {
   int i;
   if (ob->move_status <1 || ob->move_status >8 ||
       !(move_object(ob,ob->move_status|| ! (RANDOM()% 9))))
@@ -1477,7 +1508,7 @@ void check_earthwalls(object *op, mapstruct *m, int x, int y) {
     object *tmp;
     for (tmp = get_map_ob(m, x, y); tmp!=NULL; tmp=tmp->above) {
 	if (tmp->type == EARTHWALL) {
-	    hit_player(tmp,op->stats.dam,op,AT_PHYSICAL);
+	    hit_player(tmp,op->stats.dam,op,AT_PHYSICAL,1);
 	    return;
 	}
     }
@@ -1487,7 +1518,7 @@ void check_doors(object *op, mapstruct *m, int x, int y) {
     object *tmp;
     for (tmp = get_map_ob(m, x, y); tmp!=NULL; tmp=tmp->above) {
 	if (tmp->type == DOOR) {
-	    hit_player(tmp,1000,op,AT_PHYSICAL);
+	    hit_player(tmp,1000,op,AT_PHYSICAL,1);
 	    return;
 	}
     }
@@ -1505,9 +1536,10 @@ void check_doors(object *op, mapstruct *m, int x, int y) {
  * Returns NULL if no match is found.
  * The player is passed too, so that quest-related messages can be checked too.
  */
-static char *find_matching_message(object* pl, char *msg, char *match)
+static char *find_matching_message(object* pl, const char *msg, const char *match)
 {
-    char *cp=msg, *cp1, *cp2, regex[MAX_BUF], *cp3, gotmatch=0;
+    const char *cp=msg, *cp1, *cp2;
+    char *cp3, regex[MAX_BUF], gotmatch=0;
 
     while (1) {
 	if (strncmp(cp, "@match ", 7)) {
@@ -1544,14 +1576,14 @@ static char *find_matching_message(object* pl, char *msg, char *match)
 		    }
 		}
 	    }
-	    if (gotmatch && ( cp2 = quest_message_check( cp2 + 1, pl ) ) ) {
+	    if (gotmatch) {
 		if (cp1) {
-		    cp3 = malloc(cp1 - cp2);
-		    strncpy(cp3, cp2, cp1 - cp2 - 1);
-		    cp3[cp1 - cp2 - 1] = 0;
+		    cp3 = malloc(cp1 - cp2 + 1);
+		    strncpy(cp3, cp2+1, cp1 - cp2);
+		    cp3[cp1 - cp2] = 0;
 		}
 		else {	/* if no next match, just want the rest of the string */
-		    cp3 = strdup_local(cp2);
+		    cp3 = strdup_local(cp2+1);
 		}
         return cp3;
 	    }
@@ -1581,7 +1613,7 @@ static char *find_matching_message(object* pl, char *msg, char *match)
  * the conversation would continue on the new map, which probably isn't
  * what is really wanted either.
  */
-void communicate(object *op, char *txt) {
+void communicate(object *op, const char *txt) {
     object *npc;
     int i, mflags;
     sint16 x, y;
@@ -1621,88 +1653,79 @@ void communicate(object *op, char *txt) {
     }
 }
 
-int talk_to_npc(object *op, object *npc, char *txt) {
-    object *cobj;
-    CFParm CFP;
-    int k, l, m;
-    uint32 n;
-    char *cp, buf[MAX_BUF];
-    event *evt;
+static int do_talk_npc(object* op, object* npc, object* override, const char* txt)
+{
+    char* cp;
+    char buf[MAX_BUF];
 
-    /* Move this commone area up here - shouldn't cost much extra cpu
-     * time, and makes the function more readable */
-    k = EVENT_SAY;
-    l = SCRIPT_FIX_ALL;
-    m = 0;
-    n = 0;
-    CFP.Value[0] = &k;
-    CFP.Value[1] = op;
-    CFP.Value[4] = txt;
-    CFP.Value[5] = &n;
-    CFP.Value[6] = &m;
-    CFP.Value[7] = &m;
-    CFP.Value[8] = &l;
-
-    /* GROS: Handle for plugin say event */
-    if ((evt = find_event(npc, EVENT_SAY))!=NULL)
-    {
-	CFP.Value[2] = npc;
-	CFP.Value[3] = NULL;
-	CFP.Value[9] = evt->hook;
-	CFP.Value[10]= evt->options;
-	if (findPlugin(evt->plugin)>=0)
-	{
-	    ((PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP));
-	    return 0;
-	}
-    }
-    /* GROS - Here we let the objects inside inventories hear and answer, too. */
-    /* This allows the existence of "intelligent" weapons you can discuss with */
-    for(cobj=npc->inv;cobj!=NULL; cobj = cobj->below)
-    {
-        if ((evt = find_event(cobj, EVENT_SAY)) != NULL)
-        {
-            CFP.Value[2] = cobj;
-            CFP.Value[3] = npc;
-            CFP.Value[9] = evt->hook;
-            CFP.Value[10]= evt->options;
-            if (findPlugin(evt->plugin)>=0)
-            {
-                ((PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP));
-                return 0;
-            }
-	}
-    }
-    if(npc->msg == NULL || *npc->msg != '@')
+    if(override->msg == NULL || *override->msg != '@')
 	return 0;
 
-    cp = find_matching_message(op, npc->msg, txt );
+    cp = find_matching_message(op, override->msg, txt);
     if (cp) {
         sprintf(buf,"%s says:",query_name(npc));
-	new_info_map(NDI_NAVY|NDI_UNIQUE, npc->map,buf);
-	new_info_map(NDI_NAVY | NDI_UNIQUE, npc->map, cp);
-	free(cp);
+	    new_info_map(NDI_NAVY|NDI_UNIQUE, npc->map,buf);
+	    new_info_map(NDI_NAVY | NDI_UNIQUE, npc->map, cp);
+        quest_apply_items(override,op->contr);
+	    free(cp);
         return 1;
     }
     return 0;
 }
 
-int talk_to_wall(object* pl, object *npc, char *txt) {
-    char *cp;
+static int talk_to_npc(object *op, object *npc, const char *txt) {
+    object *cobj;
 
-    if(npc->msg == NULL || *npc->msg != '@')
+    /* Move this commone area up here - shouldn't cost much extra cpu
+     * time, and makes the function more readable */
+    /* Lauwenmark: Handle for plugin say event */
+    if (op==npc) return 0;
+    if (execute_event(npc, EVENT_SAY,op,NULL,txt,SCRIPT_FIX_ALL)!=0)
+	    return 0;
+    /* Lauwenmark - Here we let the objects inside inventories hear and answer, too. */
+    /* This allows the existence of "intelligent" weapons you can discuss with */
+    for(cobj=npc->inv;cobj!=NULL; cobj = cobj->below)
+    {
+        if (execute_event(cobj, EVENT_SAY,npc,NULL,txt,SCRIPT_FIX_ALL)!=0)
+                return 0;
+            }
+    for ( cobj = npc->inv; cobj; cobj = cobj->below )
+        if ( quest_is_override_compatible( cobj, op ) )
+            if ( do_talk_npc( op, npc, cobj, txt ) )
+                return 1;
+    return do_talk_npc( op, npc, npc, txt );
+}
+
+static int do_talk_wall(object* pl, object* npc, object* override, const char* txt)
+{
+    char* cp;
+    if(override->msg == NULL || *override->msg != '@')
 	return 0;
 
-    cp = find_matching_message(pl, npc->msg, txt);
+    cp = find_matching_message(pl, override->msg, txt);
     if (!cp)
 	return 0;
 
     new_info_map(NDI_NAVY | NDI_UNIQUE, npc->map,cp);
     use_trigger(npc);
+    quest_apply_items(npc, pl->contr);
     free(cp);
+
     return 1;
 }
 
+static int talk_to_wall(object* pl, object *npc, const char *txt) {
+
+    object* inv;
+
+    for ( inv = npc->inv; inv; inv = inv->below)
+        if ( quest_is_override_compatible(inv, pl ) )
+            if ( do_talk_wall( pl, npc, inv, txt ) )
+                return 1;
+
+    return do_talk_wall( pl, npc, npc, txt );;
+}
+
 /* find_mon_throw_ob() - modeled on find_throw_ob
  * This is probably overly simplistic as it is now - We want
  * monsters to throw things like chairs and other pieces of
@@ -1711,7 +1734,7 @@ int talk_to_wall(object* pl, object *npc, char *txt) {
  * first, then throw any non equipped weapon.
  */
 
-object *find_mon_throw_ob( object *op ) {
+object *find_mon_throw_ob(object *op) {
     object *tmp = NULL;
   
     if(op->head) tmp=op->head; else tmp=op;  
@@ -1896,8 +1919,8 @@ int stand_in_light( object *op) {
 	/* Check the spaces with the max light radius to see if any of them
 	 * have lights, and if any of them light the player enough, then return 1.
 	 */
-	for (x = op->x - MAX_LIGHT_RADII; x< op->x + MAX_LIGHT_RADII; x++) {
-	    for (y = op->y - MAX_LIGHT_RADII; y< op->y + MAX_LIGHT_RADII; y++) {
+	for (x = op->x - MAX_LIGHT_RADII; x <= op->x + MAX_LIGHT_RADII; x++) {
+	    for (y = op->y - MAX_LIGHT_RADII; y <= op->y + MAX_LIGHT_RADII; y++) {
 		m = op->map;
 		nx = x;
 		ny = y;
