@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_move_c =
- *    "$Id: move.c,v 1.34 2004/12/29 08:45:08 mwedel Exp $";
+ *    "$Id: move.c,v 1.41 2006/05/05 09:26:35 ryo_saeba Exp $";
  */
 
 /*
@@ -31,7 +31,9 @@
 #include <sproto.h>
 #endif
 
-/*
+static int roll_ob(object *op, int dir, object *pusher);
+
+/**
  * move_object() tries to move object op in the direction "dir".
  * If it fails (something blocks the passage), it returns 0,
  * otherwise 1.
@@ -44,7 +46,8 @@ int move_object(object *op, int dir) {
 }
 
 
-/* object op is trying to move in direction dir.
+/**
+ * object op is trying to move in direction dir.
  * originator is typically the same as op, but
  * can be different if originator is causing op to
  * move (originator is pushing op)
@@ -61,14 +64,14 @@ int move_ob (object *op, int dir, object *originator)
     mapstruct *m;
     int mflags;
 
-    m = op->map;
-    mflags = get_map_flags(m, &m, newx, newy, &newx, &newy);
-
     if(op==NULL) {
 	LOG(llevError,"Trying to move NULL.\n");
 	return 0;
     }
 
+    m = op->map;
+    mflags = get_map_flags(m, &m, newx, newy, &newx, &newy);
+
     /* If the space the player is trying to is out of the map,
      * bail now - we know it can't work.
      */
@@ -120,11 +123,18 @@ int move_ob (object *op, int dir, object *originator)
 
     remove_ob(op);
 
-    for(tmp = op; tmp != NULL; tmp = tmp->more)
-	tmp->x+=freearr_x[dir], tmp->y+=freearr_y[dir];
+    /* we already have newx, newy, and m, so lets use them.
+     * In addition, this fixes potential crashes, because multipart object was
+     * on edge of map, +=x, +=y doesn't make correct coordinates.
+     */
+    for(tmp = op; tmp != NULL; tmp = tmp->more) {
+	tmp->x += freearr_x[dir];
+	tmp->y += freearr_y[dir];
+	tmp->map =  get_map_from_coord(tmp->map, &tmp->x, &tmp->y);
+    }
 
     /* insert_ob_in_map will deal with any tiling issues */
-    insert_ob_in_map(op, op->map, originator,0);
+    insert_ob_in_map(op, m, originator,0);
 
     /* Hmmm.  Should be possible for multispace players now */
     if (op->type==PLAYER) {
@@ -132,12 +142,27 @@ int move_ob (object *op, int dir, object *originator)
 	op->contr->socket.update_look=1;
 	op->contr->socket.look_position=0;
     }
+    else if (op->type == TRANSPORT) {
+	object *pl;
+
+	for (pl=op->inv; pl; pl=pl->below) {
+	    if (pl->type == PLAYER) {
+		pl->contr->do_los=1;
+		pl->map = op->map;
+		pl->x = op->x;
+		pl->y = op->y;
+		esrv_map_scroll(&pl->contr->socket, freearr_x[dir],freearr_y[dir]);
+		pl->contr->socket.update_look=1;
+		pl->contr->socket.look_position=0;
+	    }
+	}
+    }
 
     return 1;	/* this shouldn't be reached */
 }
 
 
-/*
+/**
  * transfer_ob(): Move an object (even linked objects) to another spot
  * on the same map.
  *
@@ -155,9 +180,9 @@ int transfer_ob (object *op, int x, int y, int randomly, object *originator)
     object *tmp;
 
     if (randomly)
-	i = find_free_spot (op->arch,op->map,x,y,0,SIZEOFFREE);
+	i = find_free_spot (op,op->map,x,y,0,SIZEOFFREE);
     else
-	i = find_first_free_spot(op->arch,op->map,x,y);
+	i = find_first_free_spot(op,op->map,x,y);
 
     if (i==-1)
 	return 0;	/* No free spot */
@@ -170,12 +195,12 @@ int transfer_ob (object *op, int x, int y, int randomly, object *originator)
 	tmp->y=y+freearr_y[i]+(tmp->arch==NULL?0:tmp->arch->clone.y);
 
     tmp = insert_ob_in_map(op,op->map,originator,0);
-    if (op && op->type == PLAYER) MapNewmapCmd(op->contr);
+    if (op && op->type == PLAYER) map_newmap_cmd(op->contr);
     if (tmp) return 0;
     else return 1;
 }
 
-/*
+/**
  * Return value: 1 if object was destroyed, 0 otherwise.
  * Modified so that instead of passing the 'originator' that had no
  * real use, instead we pass the 'user' of the teleporter.  All the
@@ -192,6 +217,8 @@ int teleport (object *teleporter, uint8 tele_type, object *user)
     object *altern[120]; /* Better use c/malloc here in the future */
     int i,j,k,nrofalt=0;
     object *other_teleporter, *tmp;
+    mapstruct *m;
+    sint16  sx, sy;
 
     if(user==NULL) return 0;
     if(user->head!=NULL)
@@ -225,7 +252,7 @@ int teleport (object *teleporter, uint8 tele_type, object *user)
     }
 
     other_teleporter=altern[RANDOM()%nrofalt];
-    k=find_free_spot(user->arch,other_teleporter->map,
+    k=find_free_spot(user,other_teleporter->map,
                         other_teleporter->x,other_teleporter->y,1,9);
 
     /* if k==-1, unable to find a free spot.  If this is shop
@@ -238,10 +265,13 @@ int teleport (object *teleporter, uint8 tele_type, object *user)
     if (k==-1) {
 	if (tele_type == SHOP_MAT && user->type == PLAYER) {
 	    for (k=1; k<9; k++) {
-		if (!(get_map_flags(other_teleporter->map, NULL, 
+		if (get_map_flags(other_teleporter->map, &m, 
 			other_teleporter->x + freearr_x[k],
-			other_teleporter->y + freearr_y[k], NULL,NULL) &
-		      (P_NO_PASS | P_OUT_OF_MAP))) break;
+			other_teleporter->y + freearr_y[k], &sx,&sy) &
+			P_OUT_OF_MAP) continue;
+
+		if (!OB_TYPE_MOVE_BLOCK(user, GET_MAP_MOVE_BLOCK(m, sx, sy))) break;
+
 	    }
 	    if (k==9) {
 		LOG(llevError,"Shop mat %s (%d, %d) is in solid rock?\n",
@@ -262,7 +292,7 @@ int teleport (object *teleporter, uint8 tele_type, object *user)
            (tmp->arch==NULL?0:tmp->arch->clone.y);
     }
     tmp = insert_ob_in_map(user,other_teleporter->map,NULL,0);
-    if (tmp && tmp->type == PLAYER) MapNewmapCmd(tmp->contr);
+    if (tmp && tmp->type == PLAYER) map_newmap_cmd(tmp->contr);
     return (tmp == NULL);
 }
 
@@ -278,15 +308,16 @@ void recursive_roll(object *op,int dir,object *pusher) {
   return;
 }
 
-/*
+/**
  * This is a new version of blocked, this one handles objects
  * that can be passed through by monsters with the CAN_PASS_THRU defined.
  *
  * very new version handles also multipart objects
  * This is currently only used for the boulder roll code.
+ * Returns 1 if object does not fit, 0 if it does.
  */
 
-int try_fit (object *op, mapstruct *m, int x, int y) 
+static int try_fit (object *op, mapstruct *m, int x, int y) 
 {
     object *tmp, *more;
     sint16 tx, ty;
@@ -312,26 +343,25 @@ int try_fit (object *op, mapstruct *m, int x, int y)
 	    if ((QUERY_FLAG(tmp,FLAG_ALIVE) && tmp->type!=DOOR))
 		return 1;
 
-	    if (QUERY_FLAG(tmp,FLAG_NO_PASS) && 
-		(!QUERY_FLAG(tmp,FLAG_PASS_THRU) ||
-		 !QUERY_FLAG(more,FLAG_CAN_PASS_THRU)))
-		return 1;
+	    if (OB_MOVE_BLOCK(op, tmp)) return 1;
+
 	}
     }
     return 0;
 }
 
-/*
+/**
  * this is not perfect yet. 
  * it does not roll objects behind multipart objects properly.
  * Support for rolling multipart objects is questionable.
  */
 
-int roll_ob(object *op,int dir, object *pusher) {
+static int roll_ob(object *op,int dir, object *pusher) {
     object *tmp;
     sint16 x, y;
     int flags;
     mapstruct *m;
+    MoveType	move_block;
 
     if (op->head) 
 	op = op->head;
@@ -350,14 +380,14 @@ int roll_ob(object *op,int dir, object *pusher) {
     if (flags & (P_OUT_OF_MAP | P_IS_ALIVE))
 	return 0;
 
-    
+    move_block = GET_MAP_MOVE_BLOCK(m, x, y);
 
     /* If the target space is not blocked, no need to look at the objects on it */
-    if (flags & P_BLOCKED) {
+    if ((op->move_type & move_block) == op->move_type) {
 	for (tmp=get_map_ob(m, x, y); tmp!=NULL; tmp=tmp->above) {
 	if (tmp->head == op)
 	    continue;
-	if (QUERY_FLAG(tmp,FLAG_NO_PASS) && !roll_ob(tmp,dir,pusher))
+	if (OB_MOVE_BLOCK(op, tmp) && !roll_ob(tmp,dir,pusher))
 	    return 0;
 	}
     }
@@ -371,7 +401,7 @@ int roll_ob(object *op,int dir, object *pusher) {
     return 1;
 }
 
-/* returns 1 if pushing invokes a attack, 0 when not */
+/** returns 1 if pushing invokes a attack, 0 when not */
 int push_ob(object *who, int dir, object *pusher) {
     int str1, str2;
     object *owner;
