@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_pets_c =
- *    "$Id: pets.c,v 1.33 2005/08/01 20:36:25 qal21 Exp $";
+ *    "$Id: pets.c,v 1.44 2006/06/08 04:30:17 qal21 Exp $";
  */
 
 /*
@@ -44,6 +44,7 @@ object *get_pet_enemy(object * pet, rv_vector *rv){
     sint16 x,y;
     mapstruct *nm;
     int search_arr[SIZEOFFREE];
+    int mflags;
 
     attacker = pet->attacked_by; /*pointer to attacking enemy*/
     pet->attacked_by = NULL;     /*clear this, since we are dealing with it*/
@@ -55,7 +56,7 @@ object *get_pet_enemy(object * pet, rv_vector *rv){
 	if ((check_enemy(owner,rv)) == pet) {
 	    CLEAR_FLAG(pet, FLAG_FRIENDLY);
 	    remove_friendly_object(pet);
-	    pet->move_type &=~PETMOVE;
+	    pet->attack_movement &=~PETMOVE;
 	    return owner;
 	}
     } else {
@@ -64,7 +65,7 @@ object *get_pet_enemy(object * pet, rv_vector *rv){
 	 */
 	CLEAR_FLAG(pet, FLAG_FRIENDLY);
 	remove_friendly_object(pet);
-	pet->move_type &=~PETMOVE;
+	pet->attack_movement &=~PETMOVE;
 	return NULL;
     }
     /* If they are not on the same map, the pet won't be agressive */
@@ -111,7 +112,8 @@ object *get_pet_enemy(object * pet, rv_vector *rv){
         y = owner->y + freearr_y[search_arr[i]];
 	nm = owner->map;
 	/* Only look on the space if there is something alive there. */
-	if (get_map_flags(nm, &nm, x, y, &x, &y) & P_IS_ALIVE) {
+	mflags = get_map_flags(nm, &nm, x, y, &x, &y);
+	if (!(mflags & P_OUT_OF_MAP) && mflags & P_IS_ALIVE) {
 	    for (tmp = get_map_ob(nm, x, y); tmp != NULL; tmp = tmp->above) {
 		object *tmp2 = tmp->head == NULL?tmp:tmp->head;
 
@@ -179,7 +181,8 @@ object *get_pet_enemy(object * pet, rv_vector *rv){
 	    y = pet->y + freearr_y[search_arr[i]];
 	    nm = pet->map;
 	    /* Only look on the space if there is something alive there. */
-	    if (get_map_flags(nm, &nm, x,y, &x, &y) & P_IS_ALIVE) {
+	    mflags = get_map_flags(nm, &nm, x,y, &x, &y);
+	    if (!(mflags & P_OUT_OF_MAP) && mflags & P_IS_ALIVE) {
 		for (tmp = get_map_ob(nm, x, y); tmp != NULL; tmp = tmp->above) {
 		    object *tmp2 = tmp->head == NULL?tmp:tmp->head;
 		    if (QUERY_FLAG(tmp2,FLAG_ALIVE) && ((
@@ -250,7 +253,7 @@ void remove_all_pets(mapstruct *map) {
   for(obl = first_friendly_object; obl != NULL; obl = next) {
     next = obl->next;
     if(obl->ob->type != PLAYER && QUERY_FLAG(obl->ob,FLAG_FRIENDLY) &&
-       (owner = get_owner(obl->ob)) != NULL && owner->map != obl->ob->map)
+       (owner = get_owner(obl->ob)) != NULL && !on_same_map(owner, obl->ob))
     {
 	/* follow owner checks map status for us */
 	follow_owner(obl->ob,owner);
@@ -279,7 +282,7 @@ void follow_owner(object *ob, object *owner) {
 	LOG(llevError,"Owner of the pet not on a map in memory!?\n");
 	return;
     }
-    dir = find_free_spot(ob->arch, owner->map,
+    dir = find_free_spot(ob, owner->map,
                        owner->x, owner->y, 1, SIZEOFFREE);
 
     if (dir==-1) {
@@ -317,7 +320,7 @@ void pet_move(object * ob)
     }
 
     /* move monster into the owners map if not in the same map */
-    if (ob->map != owner->map) {
+    if (!on_same_map(ob, owner)) {
 	follow_owner(ob, owner);
 	return;
     }
@@ -328,7 +331,10 @@ void pet_move(object * ob)
 	    dir = rndm(1, 8);
 	    dx = ob->x + freearr_x[dir];
 	    dy = ob->y + freearr_y[dir];
-	    if (get_map_flags(owner->map, NULL, dx, dy, NULL, NULL) & (P_WALL | P_OUT_OF_MAP))
+	    m = ob->map;
+	    if (get_map_flags(ob->map, &m, dx, dy, &dx, &dy) & P_OUT_OF_MAP)
+		continue;
+	    else if (OB_TYPE_MOVE_BLOCK(ob, GET_MAP_MOVE_BLOCK(m, dx, dy)))
 		continue;
 	    else
 		break;
@@ -402,7 +408,7 @@ void pet_move(object * ob)
  * direction the monster should be placed in.
  * is_golem is to note that this is a golem spell.
  */
-object *fix_summon_pet(archetype *at, object *op, int dir, int is_golem ) {
+static object *fix_summon_pet(archetype *at, object *op, int dir, int is_golem) {
     archetype *atmp;
     object *tmp=NULL, *prev=NULL, *head=NULL; 
  
@@ -422,14 +428,14 @@ object *fix_summon_pet(archetype *at, object *op, int dir, int is_golem ) {
 		    object *owner = get_owner(op);
 		    if(owner != NULL) {/* For now, we transfer ownership */
 			set_owner(tmp,owner);
-			tmp->move_type = PETMOVE;
+			tmp->attack_movement = PETMOVE;
 			add_friendly_object(tmp);
 			SET_FLAG(tmp, FLAG_FRIENDLY);
 		    }
 		}
 	    }
 	    if(op->type!=PLAYER || !is_golem) { 
-		tmp->move_type = PETMOVE;
+		tmp->attack_movement = PETMOVE;
 		tmp->speed_left = -1;
 		tmp->type = 0;
 		tmp->enemy = op->enemy; 
@@ -604,9 +610,9 @@ int summon_golem(object *op,object *caster,int dir,object *spob) {
     }
 
     if(!dir)
-	dir=find_free_spot(NULL,op->map,op->x,op->y,1,9);
+	dir=find_free_spot(NULL,op->map,op->x,op->y,1,SIZEOFFREE1+1);
 
-    if ((dir==-1) || arch_blocked(at, op->map, op->x + freearr_x[dir], op->y + freearr_y[dir])) {
+    if ((dir==-1) || ob_blocked(&at->clone, op->map, op->x + freearr_x[dir], op->y + freearr_y[dir])) {
 	new_draw_info(NDI_UNIQUE, 0,op,"There is something in the way.");
 	return 0;
     }
@@ -630,7 +636,7 @@ int summon_golem(object *op,object *caster,int dir,object *spob) {
 	    object *owner = get_owner(op);
 	    if (owner != NULL) { /* For now, we transfer ownership */
 		set_owner (tmp, owner);
-		tmp->move_type = PETMOVE;
+		tmp->attack_movement = PETMOVE;
 		add_friendly_object (tmp);
 		SET_FLAG (tmp, FLAG_FRIENDLY);
 	    }
@@ -706,8 +712,9 @@ int summon_golem(object *op,object *caster,int dir,object *spob) {
  * This returns NULL if no match was found.
  */
 
-object *choose_cult_monster(object *pl, object *god, int summon_level) {
-    char buf[MAX_BUF],*race;
+static object *choose_cult_monster(object *pl, object *god, int summon_level) {
+    char buf[MAX_BUF];
+    const char *race;
     int racenr, mon_nr,i;
     racelink *list;
     objectlink *tobl;
@@ -774,10 +781,11 @@ object *choose_cult_monster(object *pl, object *god, int summon_level) {
 
 
 
-int summon_object(object *op, object *caster, object *spell_ob, int dir)
+int summon_object(object *op, object *caster, object *spell_ob, int dir, const char *stringarg)
 {
     sint16 x,y, nrof=1, i;
     archetype *summon_arch;
+    int ndir;
 
     if (spell_ob->other_arch) {
 	summon_arch = spell_ob->other_arch;
@@ -792,6 +800,7 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir)
 	for (tr=spell_ob->randomitems->items; tr; tr=tr->next) {
 	    if (level < tr->magic) break;
 	    lasttr = tr;
+	    if(stringarg && !strcmp(tr->item->name,stringarg)) break;
 	    if (tr->next == NULL || tr->next->item == NULL) break;
 	}
 	if (!lasttr) {
@@ -805,7 +814,7 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir)
 
     } else if (spell_ob->race && !strcmp(spell_ob->race,"GODCULTMON")) {
 	object *god=find_god(determine_god(op)), *mon, *owner;
-	int summon_level, tries, ndir;
+	int summon_level, tries;
 
 	if (!god && ((owner=get_owner(op))!=NULL)) {
 	    god = find_god(determine_god(owner));
@@ -831,8 +840,8 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir)
 	    }
 	    ndir = dir;
 	    if (!ndir)
-		ndir = find_free_spot(mon->arch, op->map, op->x, op->y, 1, SIZEOFFREE);
-	    if (ndir == -1 || arch_blocked(mon->arch,op->map, op->x + freearr_x[ndir], op->y+freearr_y[ndir])) {
+		ndir = find_free_spot(mon, op->map, op->x, op->y, 1, SIZEOFFREE);
+	    if (ndir == -1 || ob_blocked(mon,op->map, op->x + freearr_x[ndir], op->y+freearr_y[ndir])) {
 		ndir=-1;
 		if (++tries == 5) {
 		    new_draw_info(NDI_UNIQUE, 0,op, "There is something in the way.");
@@ -845,6 +854,8 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir)
 	else
 	nrof =  die_roll(2, 2, op, PREFER_HIGH);
 	summon_arch = mon->arch;
+    } else {
+	summon_arch = NULL;
     }
 
     if (spell_ob->stats.dam)
@@ -855,23 +866,29 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir)
 	return 0;
     }
 
-    if (!dir)
-	dir = find_free_spot(summon_arch, op->map, op->x, op->y, 1, SIZEOFFREE);
-
-    if (dir > 0) {
-	x = op->x + freearr_x[dir];
-	y = op->y + freearr_y[dir];
-    }
-
-    if (dir == -1 || arch_blocked(summon_arch, op->map, x, y)){
-	new_draw_info(NDI_UNIQUE, 0, op, "There is something in the way.");
-	return 0;
-    }
-
     for (i=1; i <= nrof; i++) {
 	archetype *atmp;
 	object *prev=NULL, *head=NULL, *tmp;
 
+        if (dir) {
+            ndir = dir;
+            dir = absdir (dir+1);
+        } else
+            ndir = find_free_spot(&summon_arch->clone, op->map, op->x, op->y, 1, SIZEOFFREE);
+
+        if (ndir > 0) {
+            x = freearr_x[ndir];
+            y = freearr_y[ndir];
+        }
+
+        if (ndir == -1 || ob_blocked(&summon_arch->clone, op->map, op->x + x, op->y + y)){
+            new_draw_info(NDI_UNIQUE, 0, op, "There is something in the way.");
+            if (nrof > 1)
+		new_draw_info(NDI_UNIQUE, 0,op, "No more pets for this casting.");
+
+            return nrof > 1;
+        }
+
 	for (atmp = summon_arch; atmp!=NULL; atmp=atmp->more) {
 	    tmp = arch_to_object(atmp);
 	    if (atmp == summon_arch) {
@@ -887,8 +904,8 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir)
 			    SET_FLAG(tmp, FLAG_FRIENDLY);
 			    add_friendly_object(tmp);
 			    tmp->stats.exp = 0;
-			    if (spell_ob->move_type) 
-				tmp->move_type = spell_ob->move_type;
+			    if (spell_ob->attack_movement) 
+				tmp->attack_movement = spell_ob->attack_movement;
 			    if (get_owner(op))
 				set_owner(tmp, get_owner(op));
 			}
@@ -902,11 +919,11 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir)
 		prev->more = tmp;
 	    }
 	    prev = tmp;
-	    tmp->x = op->x + freearr_x[dir] + tmp->arch->clone.x;
-	    tmp->y = op->y + freearr_y[dir] + tmp->arch->clone.y;
+	    tmp->x = op->x + x + tmp->arch->clone.x;
+	    tmp->y = op->y + y + tmp->arch->clone.y;
 	    tmp->map = op->map;
 	}
-	head->direction = dir;
+	head->direction = freedir[ndir];
 	head->stats.exp = 0;
 	head = insert_ob_in_map(head, head->map, op, 0);
 	if (head && head->randomitems) {
@@ -915,21 +932,13 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir)
 	    for (tmp=head->inv; tmp; tmp=tmp->below)
 		if (!tmp->nrof) SET_FLAG(tmp, FLAG_NO_DROP);
 	}
-	dir = absdir(dir + 1);
-	if (arch_blocked(summon_arch, op->map, op->x + freearr_x[dir], op->y + freearr_y[dir])) {
-	    if (i < nrof) {
-		new_draw_info(NDI_UNIQUE, 0,op, "There is something in the way,");
-		new_draw_info(NDI_UNIQUE, 0,op, "No more pets for this casting.");
-		return 1;
-	    }
-	}
     } /* for i < nrof */
     return 1;
 }
 
 /* recursively look through the owner property of objects until the real owner
 is found */
-object *get_real_owner(object *ob) {
+static object *get_real_owner(object *ob) {
 	object *realowner = ob;
 	
 	if (realowner == NULL) return NULL;
@@ -958,7 +967,7 @@ int should_arena_attack(object *pet,object *owner,object *target) {
 		towner = 0;	
 	}
 	
-	/* if the pet has now owner, exit with error */
+	/* if the pet has no owner, exit with error */
 	if (rowner == NULL) {
 		LOG(llevError,"Pet has no owner.\n");
 		return 0;
@@ -991,14 +1000,14 @@ int should_arena_attack(object *pet,object *owner,object *target) {
 	if (target->type != PLAYER) {
 		/* if the target is owned by a player make sure than make sure
 		it's not in the same party */
-		if ((towner->type == PLAYER) && (rowner->contr->party_number > 0)) {
-			if (rowner->contr->party_number == towner->contr->party_number) return 0;
+		if ((towner->type == PLAYER) && (rowner->contr->party != NULL)) {
+			if (rowner->contr->party == towner->contr->party) return 0;
 		}
 	} else {
 		/* if the target is a player make sure than make sure it's not
 		in the same party */
-		if (rowner->contr->party_number > 0){
-			if (rowner->contr->party_number == target->contr->party_number) return 0;
+		if (rowner->contr->party != NULL){
+			if (rowner->contr->party == target->contr->party) return 0;
 		}
 	}
 	
