@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_resurrection_c =
- *   "$Id: resurrection.c,v 1.13 2005/07/16 21:10:39 akirschbaum Exp $";
+ *   "$Id: resurrection.c,v 1.18 2006/06/05 06:28:56 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -43,6 +43,7 @@ extern int sys_nerr;
 extern char **classname;
 extern object *objects;
 
+static int resurrection_fails(int levelcaster,int leveldead);
 
 
 /*  name of the person to resurrect and which spell was used
@@ -57,7 +58,7 @@ static int resurrect_player(object *op,char *playername,object *spell)
     char path[MAX_BUF];
     char buf[MAX_BUF];
     char buf2[MAX_BUF];
-    char *race=NULL;
+    const char *race=NULL;
     sint64 exp;
     int Con;
 
@@ -118,18 +119,10 @@ static int resurrect_player(object *op,char *playername,object *spell)
 	fgets(buf,255,deadplayer);
 	sscanf(buf,"%s",buf2);
 	if( ! (strcmp(buf2,"exp"))) {
-#ifndef WIN32
-	    sscanf(buf,"%s %lld",buf2,&exp);
-#else
-	    sscanf(buf,"%s %I64d",buf2,&exp);
-#endif
+	    sscanf(buf,"%s %" FMT64, buf2, &exp);
 	    if (spell->stats.exp) {
 		exp-=exp/spell->stats.exp;
-#ifndef WIN32
-		sprintf(buf,"exp %lld\n",exp);
-#else
-		sprintf(buf,"exp %I64d\n",exp);
-#endif
+		sprintf(buf,"exp %" FMT64 "\n",exp);
 	    }
 	}
 	if(! (strcmp(buf2,"Con"))) {
@@ -159,7 +152,7 @@ static int resurrect_player(object *op,char *playername,object *spell)
  * dir  --  direction the spell is cast
  * corpseobj - corpse to raise - can be null, in which case this function will find it
  */
-int cast_raise_dead_spell(object *op, object *caster, object *spell, int dir, char *arg)
+int cast_raise_dead_spell(object *op, object *caster, object *spell, int dir, const char *arg)
 {
     object *temp, *newob;
     char name_to_resurrect[MAX_BUF];
@@ -232,7 +225,7 @@ int cast_raise_dead_spell(object *op, object *caster, object *spell, int dir, ch
 }
 
 
-int resurrection_fails(int levelcaster,int leveldead)
+static int resurrection_fails(int levelcaster,int leveldead)
 {
     int chance=9;
     /*  scheme:  equal in level, 50% success.
@@ -260,15 +253,19 @@ void dead_player(object *op)
     strcpy(newname,filename);
     strcat(newname,".dead");
 
-    if(!rename(filename,newname)) {
-	perror(filename);
-	perror(newname);
+    if(rename(filename,newname) != 0) {
+	LOG(llevError, "Cannot rename dead player's file %s into %s: %s\n", filename, newname, strerror_local(errno));
     }
 }
 
 
 
-void dead_character(char *name) {
+#if 0
+/* dead_character & dead_player_exists are no longer used - should perhaps be
+ * removed.  MSW 2006-06-02
+ */
+
+static void dead_character(const char *name) {
     char buf[MAX_BUF];
     char buf2[MAX_BUF];
 
@@ -277,13 +274,12 @@ void dead_character(char *name) {
     strcpy(buf2,buf);
     strcat(buf,".dead");
     if(rename(buf2,buf)== -1){
-	LOG(llevDebug, "Crossfire character rename to dead: %s --", buf);
-	perror("rename()");
+	LOG(llevError, "Cannot rename dead player's file %s into %s: %s\n", buf2, buf, strerror_local(errno));
     }
 }
 
 
-int dead_player_exists(char *name) {
+static int dead_player_exists(const char *name) {
     char buf[MAX_BUF];
 
     sprintf(buf,"%s/%s/%s/%s",settings.localdir,settings.playerdir,name, name);
@@ -291,4 +287,4 @@ int dead_player_exists(char *name) {
     return !(access(buf,0));
 }
 
-
+#endif 
