@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_rune_c =
- *   "$Id: rune.c,v 1.41 2005/07/16 21:10:39 akirschbaum Exp $";
+ *   "$Id: rune.c,v 1.49 2006/06/29 06:09:23 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2003 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2003,2006 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -51,7 +51,7 @@
  *		a rune of marking 
  */
 
-int write_rune(object *op,object *caster, object *spell, int dir,char *runename) { 
+int write_rune(object *op,object *caster, object *spell, int dir, const char *runename) { 
     object *tmp, *rune_spell, *rune;
     char buf[MAX_BUF];
     mapstruct *m;
@@ -136,7 +136,7 @@ int write_rune(object *op,object *caster, object *spell, int dir,char *runename)
     if (rune_spell->type == RUNE) {
 	rune = rune_spell;
     } else {
-	rune = get_archetype(GENERIC_RUNE);
+	rune = create_archetype(GENERIC_RUNE);
 	sprintf(buf,"You set off a rune of %s\n",rune_spell->name);
 	rune->msg=add_string(buf);
 	tmp = get_object();
@@ -193,11 +193,11 @@ void move_rune(object *op) {
  */
 
 
-void rune_attack(object *op,object *victim)
+static void rune_attack(object *op,object *victim)
 {
     if(victim) {
          tag_t tag = victim->count;
-	 hit_player(victim,op->stats.dam,op,op->attacktype);
+	 hit_player(victim,op->stats.dam,op,op->attacktype,1);
          if (was_destroyed (victim, tag))
                 return;
 	 /*  if there's a disease in the needle, put it in the player */
@@ -210,7 +210,7 @@ void rune_attack(object *op,object *victim)
 		free_object(disease);
 	 }
     }
-    else  hit_map(op,0,op->attacktype);
+    else  hit_map(op,0,op->attacktype,1);
 }
 
 /*  This function generalizes attacks by runes/traps.  This ought to make
@@ -246,7 +246,7 @@ void spring_trap(object *trap,object *victim)
     /*  Flash an image of the trap on the map so the poor sod
      *   knows what hit him.  
      */
-    for (env = trap; env->env != NULL; env = env->env) ;
+    env = object_get_env_recursive(trap);
 
     /* If the victim is not next to this trap, don't set it off.
      * players shouldn't get hit by firing arrows at a door for example.
@@ -261,11 +261,13 @@ void spring_trap(object *trap,object *victim)
     if ((trap->inv && trap->inv->type == SPELL) ||
 	(trap->other_arch && trap->other_arch->clone.type == SPELL)) {
 	object *spell;
+
 	/* This is necessary if the trap is inside something else */
 	remove_ob(trap);
 	trap->x=victim->x;
 	trap->y=victim->y;
 	insert_ob_in_map(trap,victim->map,trap,0);
+
 	if (was_destroyed (trap, trap_tag))
 	    return;
 
@@ -314,6 +316,7 @@ int dispel_rune(object *op,object *caster, object *spell, object *skill, int dir
      */
     if (mflags & P_OUT_OF_MAP) {
 	new_draw_info(NDI_UNIQUE, 0,op,"There's nothing there!");
+	return 0;
     }
 
     for(tmp=get_map_ob(m,x, y); tmp!=NULL;  tmp=tmp->above)  {
@@ -371,7 +374,7 @@ int trap_show(object *trap, object *where) {
     object *tmp2;
 
     if(where==NULL) return 0;
-    tmp2=get_archetype("runedet");
+    tmp2=create_archetype("runedet");
     tmp2->face=&new_faces[GET_ANIMATION(trap, 0)];
     tmp2->x=where->x;tmp2->y=where->y;tmp2->map=where->map;
     insert_ob_in_map(tmp2,where->map,NULL,0);
