@@ -1,11 +1,11 @@
 /*
  * static char *rcsid_skill_util_c =
- *   "$Id: skill_util.c,v 1.53 2005/07/23 10:06:36 ryo_saeba Exp $";
+ *   "$Id: skill_util.c,v 1.67 2006/06/05 06:28:56 mwedel Exp $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copryight (C) 2002 Mark Wedel & Crossfire Development Team
+    Copryight (C) 2006 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -52,13 +52,16 @@
 #include <living.h>     /* for defs of STR,CON,DEX,etc. -b.t.*/
 #include <spells.h>
 
-char *skill_names[NUM_SKILLS];
+static int attack_hth(object *pl, int dir, const char *string, object *skill);
+static int attack_melee_weapon(object *op, int dir, const char *string, object *skill);
+
+const char *skill_names[NUM_SKILLS];
 
 /* init_skills basically just sets up the skill_names table
  * above.  The index into the array is set up by the
  * subtypes.
  */
-void init_skills() {
+void init_skills(void) {
     int i;
     archetype *at;
 
@@ -121,7 +124,7 @@ void link_player_skills(object *op)
  * use the skill, so thus if use of the skill requires a skill
  * tool, this code will equip it.
  */
-object *find_skill_by_name(object *who, char *name)
+object *find_skill_by_name(object *who, const char *name)
 {
     object *skill=NULL, *skill_tool=NULL, *tmp;
 
@@ -284,7 +287,7 @@ void clear_skill(object *who)
  * It returns 0 if no skill was used.
  */
 
-int do_skill (object *op, object *part, object *skill, int dir, char *string) {
+int do_skill (object *op, object *part, object *skill, int dir, const char *string) {
     int success=0, exp=0;
     int did_alc = 0;
     object *tmp, *next;
@@ -306,14 +309,20 @@ int do_skill (object *op, object *part, object *skill, int dir, char *string) {
 
     switch(skill->subtype) {
 	case SK_LEVITATION:
-	    if(QUERY_FLAG(op,FLAG_FLYING)) { 
-		CLEAR_FLAG(op,FLAG_FLYING);
+	    /* Not 100% sure if this will work with new movement code -
+	     * the levitation skill has move_type for flying, so when
+	     * equipped, that should transfer to player, when not,
+	     * shouldn't.
+	     */
+	    if(QUERY_FLAG(skill,FLAG_APPLIED)) { 
+		CLEAR_FLAG(skill,FLAG_APPLIED);
 		new_draw_info(NDI_UNIQUE,0,op,"You come to earth.");
 	    }
 	    else {
-		SET_FLAG(op,FLAG_FLYING);
+		SET_FLAG(skill,FLAG_APPLIED);
 		new_draw_info(NDI_UNIQUE,0,op,"You rise into the air!.");
 	    }
+	    fix_player(op);
 	    success=1;
 	    break;
 
@@ -425,12 +434,15 @@ int do_skill (object *op, object *part, object *skill, int dir, char *string) {
 	    success = pray(op, skill);
 	    break;
 
+	case SK_BARGAINING:
+	    success = describe_shop(op);
+	    break;
+
 	case SK_SORCERY:
 	case SK_EVOCATION:
 	case SK_PYROMANCY:
 	case SK_SUMMONING:
 	case SK_CLIMBING:
-	case SK_BARGAINING:
 	    new_draw_info(NDI_UNIQUE, 0,op,"This skill is already in effect.");
 	    break;
 
@@ -637,47 +649,34 @@ static int clipped_percent(sint64 a, sint64 b)
  * simpler.
  */
  
-void show_skills(object *op) {
+void show_skills(object *op, const char* search) {
     object *tmp=NULL;
-    char buf[MAX_BUF], *cp;
+    char buf[MAX_BUF];
+    const char *cp;
     int i,num_skills_found=0;
-    static char *periods="........................................";
+    static const char *const periods = "........................................";
     /* Need to have a pointer and use strdup for qsort to work properly */
     char skills[NUM_SKILLS][MAX_BUF];
 
 
     for (tmp=op->inv; tmp!=NULL; tmp=tmp->below) {
 	if (tmp->type == SKILL) {
+		if ( search && strstr(tmp->name,search)==NULL ) continue;
 	    /* Basically want to fill this out to 40 spaces with periods */
 	    sprintf(buf,"%s%s", tmp->name, periods);
 	    buf[40] = 0;
 
-	    if (settings.use_permanent_experience) {
-#ifdef WIN32
-		sprintf(skills[num_skills_found++],"%slvl:%3d (xp:%I64d/%I64d/%d%%)",
+	    if (settings.permanent_exp_ratio) {
+		sprintf(skills[num_skills_found++],"%slvl:%3d (xp:%" FMT64 "/%" FMT64 "/%d%%)",
 			 buf,tmp->level,
 			 tmp->stats.exp,
 			 level_exp(tmp->level+1, op->expmul),
 			 clipped_percent(tmp->perm_exp,tmp->stats.exp));
-#else
-		sprintf(skills[num_skills_found++],"%slvl:%3d (xp:%lld/%lld/%d%%)",
-			 buf,tmp->level,
-			 tmp->stats.exp,
-			 level_exp(tmp->level+1, op->expmul),
-			 clipped_percent(tmp->perm_exp,tmp->stats.exp));
-#endif
 	    } else {
-#ifdef WIN32
-		sprintf(skills[num_skills_found++], "%slvl:%3d (xp:%I64d/%I64d)",
+		sprintf(skills[num_skills_found++], "%slvl:%3d (xp:%" FMT64 "/%" FMT64 ")",
 			 buf,tmp->level,
 			 tmp->stats.exp,
 			 level_exp(tmp->level+1, op->expmul));
-#else
-		sprintf(skills[num_skills_found++], "%slvl:%3d (xp:%lld/%lld)",
-			 buf,tmp->level,
-			 tmp->stats.exp,
-			 level_exp(tmp->level+1, op->expmul));
-#endif
 	    }
 	    /* I don't know why some characters get a bunch of skills, but
 	     * it sometimes happens (maybe a leftover from bugier earlier code
@@ -695,7 +694,7 @@ void show_skills(object *op) {
 
     clear_win_info(op);
     new_draw_info(NDI_UNIQUE, 0,op,"Player skills:");
-    if (num_skills_found > 1) qsort(skills, num_skills_found, MAX_BUF, (int (*)())strcmp);
+    if (num_skills_found > 1) qsort(skills, num_skills_found, MAX_BUF, (int (*)(const void*, const void*))strcmp);
 
     for (i=0; i<num_skills_found; i++) {
 	new_draw_info(NDI_UNIQUE, 0, op, skills[i]);
@@ -719,9 +718,9 @@ void show_skills(object *op) {
  * our own find_skill_by_name so we can try to do better string matching.
  */
 
-int use_skill(object *op, char *string) {
+int use_skill(object *op, const char *string) {
     object *skop;
-    int len;
+    size_t len;
 
     if (!string) return 0;
 
@@ -819,7 +818,7 @@ static object *find_best_player_hth_skill(object *op)
  * the damage.
  */
  
-static int do_skill_attack(object *tmp, object *op, char *string, object *skill) {
+static int do_skill_attack(object *tmp, object *op, const char *string, object *skill) {
     int success; 
 
    /* For Players only: if there is no ready weapon, and no "attack" skill
@@ -830,7 +829,7 @@ static int do_skill_attack(object *tmp, object *op, char *string, object *skill)
     */
     if(op->type==PLAYER) {
 	if (!QUERY_FLAG(op,FLAG_READY_WEAPON)) {
-	    int i;
+	    size_t i;
 
 	    if (!skill) {
 		/* See if the players chosen skill is a combat skill, and use
@@ -930,7 +929,7 @@ static int do_skill_attack(object *tmp, object *op, char *string, object *skill)
  * Initial implementation by -bt thomas@astro.psu.edu
  */
  
-int skill_attack (object *tmp, object *pl, int dir, char *string, object *skill) {
+int skill_attack (object *tmp, object *pl, int dir, const char *string, object *skill) {
     sint16 tx,ty;
     mapstruct *m;
     int mflags;
@@ -950,16 +949,20 @@ int skill_attack (object *tmp, object *pl, int dir, char *string, object *skill)
 
 	mflags = get_map_flags(m, &m, tx, ty, &tx, &ty);
 	if (mflags & P_OUT_OF_MAP) return 0;
+
 	/* space must be blocked for there to be anything interesting to do */
-	if (!(mflags & P_BLOCKED)) return 0;
+	if (!(mflags&P_IS_ALIVE)
+	&& !OB_TYPE_MOVE_BLOCK(pl, GET_MAP_MOVE_BLOCK(m, tx, ty))) {
+	    return 0;
+	}
 
 	for(tmp=get_map_ob(m, tx, ty); tmp; tmp=tmp->above)
 	    if((QUERY_FLAG(tmp,FLAG_ALIVE) && tmp->stats.hp>=0)
 	       || QUERY_FLAG(tmp, FLAG_CAN_ROLL)
 	       || tmp->type==LOCKED_DOOR ) {
 		/* Don't attack party members */
-                if((pl->type==PLAYER && tmp->type==PLAYER) && (pl->contr->party_number!=-1
-                       && pl->contr->party_number==tmp->contr->party_number))
+                if((pl->type==PLAYER && tmp->type==PLAYER) && (pl->contr->party!=NULL
+                       && pl->contr->party==tmp->contr->party))
                                 return 0;
                 break;
 	    }
@@ -980,7 +983,7 @@ int skill_attack (object *tmp, object *pl, int dir, char *string, object *skill)
  * function skill_attack() we actually attack.
  */
 
-int attack_hth(object *pl, int dir, char *string, object *skill) {
+static int attack_hth(object *pl, int dir, const char *string, object *skill) {
     object *enemy=NULL,*weapon;
 
     if(QUERY_FLAG(pl, FLAG_READY_WEAPON))
@@ -1010,7 +1013,7 @@ int attack_hth(object *pl, int dir, char *string, object *skill) {
  * weapon type.
  */
 
-int attack_melee_weapon(object *op, int dir, char *string, object *skill) {
+static int attack_melee_weapon(object *op, int dir, const char *string, object *skill) {
 
     if(!QUERY_FLAG(op, FLAG_READY_WEAPON)) {
 	if(op->type==PLAYER)
@@ -1020,4 +1023,3 @@ int attack_melee_weapon(object *op, int dir, char *string, object *skill) {
     return skill_attack(NULL,op,dir,string, skill);
 
 }
-
