@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_skills_c =
- *   "$Id: skills.c,v 1.61 2005/07/27 17:54:54 ryo_saeba Exp $";
+ *   "$Id: skills.c,v 1.75 2006/06/29 06:09:23 mwedel Exp $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -47,7 +47,7 @@ static int adj_stealchance (object *op, object *victim, int roll) {
     if(op->type==PLAYER && op->body_used[BODY_ARMS] <=0 &&
        op->body_info[BODY_ARMS]) {
 	new_draw_info(NDI_UNIQUE, 0,op,"But you have no free hands to steal with!");
-	roll=-1;
+	return -1;
     }
 
     /* ADJUSTMENTS */
@@ -119,6 +119,11 @@ static int attempt_steal(object* op, object* who, object *skill)
 	new_draw_info(NDI_UNIQUE, 0, who, "You can't steal from the dungeon master!\n");
 	return 0;
     }
+    if(op->type == PLAYER && who->type == PLAYER && settings.no_player_stealing) {
+      new_draw_info(NDI_UNIQUE, 0, who, "You can't steal from other players!\n");
+      return 0;
+    }
+
 
     /* Ok then, go thru their inventory, stealing */
     for(tmp = op->inv; tmp != NULL; tmp = next) {
@@ -175,8 +180,8 @@ static int attempt_steal(object* op, object* who, object *skill)
     } /* for loop looking for an item */
 
     if (!tmp) {
-	new_draw_info_format(NDI_UNIQUE, 0, who, "The %s has nothing you can steal!",
-			     query_name(op));
+	new_draw_info_format(NDI_UNIQUE, 0, who, "%s%s has nothing you can steal!",
+			     op->type == PLAYER ? "" : "The ", query_name(op));
 	return 0;
     }
 
@@ -232,6 +237,7 @@ int steal(object* op, int dir, object *skill)
     object *tmp, *next;
     sint16 x, y;
     mapstruct *m;
+    int mflags;
 
     x = op->x + freearr_x[dir];
     y = op->y + freearr_y[dir];
@@ -242,9 +248,16 @@ int steal(object* op, int dir, object *skill)
     }
 
     m = op->map;
-    if(get_map_flags(m, &m ,x,y, &x, &y) & (P_WALL | P_OUT_OF_MAP)) {
+    mflags = get_map_flags(m, &m ,x,y, &x, &y);
+    /* Out of map - can't do it.  If nothing alive on this space,
+     * don't need to look any further.
+     */
+    if ((mflags & P_OUT_OF_MAP) || !(mflags & P_IS_ALIVE))
+	return 0;
+
+    /* If player can't move onto the space, can't steal from it. */
+    if (OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, x, y)))
 	return 0;
-    }
 
     /* Find the topmost object at this spot */
     for(tmp = get_map_ob(m,x,y);
@@ -253,19 +266,29 @@ int steal(object* op, int dir, object *skill)
 
     /* For all the stacked objects at this point, attempt a steal */
     for(; tmp != NULL; tmp = next) {
-      next = tmp->below;
-      /* Minor hack--for multi square beings - make sure we get 
-       * the 'head' coz 'tail' objects have no inventory! - b.t. 
-       */ 
-      if (tmp->head) tmp=tmp->head;
-      if(tmp->type!=PLAYER&&!QUERY_FLAG(tmp, FLAG_MONSTER)) continue;
-      /* do not reveal hidden DMs */
-      if (tmp->type == PLAYER && QUERY_FLAG(tmp, FLAG_WIZ) && tmp->contr->hidden) continue;
-      if (attempt_steal(tmp, op, skill)) {
-	  if(tmp->type==PLAYER) /* no xp for stealing from another player */
-	    return 0;
-	  else return (calc_skill_exp(op,tmp, skill));
-      }
+	next = tmp->below;
+	/* Minor hack--for multi square beings - make sure we get 
+	 * the 'head' coz 'tail' objects have no inventory! - b.t. 
+	 */ 
+	if (tmp->head) tmp=tmp->head;
+
+	if(tmp->type!=PLAYER && !QUERY_FLAG(tmp, FLAG_MONSTER)) continue;
+
+	/* do not reveal hidden DMs */
+	if (tmp->type == PLAYER && QUERY_FLAG(tmp, FLAG_WIZ) && tmp->contr->hidden) continue;
+	if (attempt_steal(tmp, op, skill)) {
+	    if (tmp->type==PLAYER) /* no xp for stealing from another player */
+		return 0;
+
+	    /* no xp for stealing from pets (of players) */
+	    if (QUERY_FLAG(tmp, FLAG_FRIENDLY) && tmp->attack_movement == PETMOVE) {
+		object *owner = get_owner(tmp);
+		if (owner != NULL && owner->type == PLAYER)
+		    return 0;
+	    }
+
+	    return (calc_skill_exp(op,tmp, skill));
+	}
     }
     return 0;
 }
@@ -321,7 +344,7 @@ int pick_lock(object *pl, int dir, object *skill)
 	return 0;
     }
 
-    if (!QUERY_FLAG(tmp, FLAG_NO_PASS)) {
+    if (!tmp->move_block) {
 	new_draw_info(NDI_UNIQUE, 0,pl,"The door has no lock!");
 	return 0;
     }
@@ -402,7 +425,7 @@ int hide(object *op, object *skill) {
  * of jumping.
  */
 static void stop_jump(object *pl, int dist, int spaces) {
-    CLEAR_FLAG(pl,FLAG_FLYING);
+    fix_player(pl);
     insert_ob_in_map(pl,pl->map,pl,0);
 }
 
@@ -421,7 +444,14 @@ static int attempt_jump (object *pl, int dir, int spaces, object *skill) {
      */ 
 
     remove_ob(pl);
-    SET_FLAG(pl,FLAG_FLYING);
+
+    /* 
+     * I don't think this is actually needed - all the movement
+     * code is handled in this function, and I don't see anyplace
+     * that cares about the move_type being flying.
+     */
+    pl->move_type |= MOVE_FLY_LOW;
+
     for(i=0;i<=spaces;i++) { 
 	x = pl->x + dx;
 	y = pl->y + dy;
@@ -433,7 +463,7 @@ static int attempt_jump (object *pl, int dir, int spaces, object *skill) {
 	    (void) stop_jump(pl,i,spaces);
 	    return 0;
 	}
-	if (mflags & P_WALL) {
+	if (OB_TYPE_MOVE_BLOCK(pl, GET_MAP_MOVE_BLOCK(m, x, y))) {
 	    new_draw_info(NDI_UNIQUE, 0,pl,"Your jump is blocked.");
 	    stop_jump(pl,i,spaces);
 	    return 0;
@@ -446,9 +476,9 @@ static int attempt_jump (object *pl, int dir, int spaces, object *skill) {
 		new_draw_info_format(NDI_UNIQUE, 0,pl,"You jump into %s%s.", 
 		    tmp->type == PLAYER ? "" : "the ", tmp->name);
 		if(tmp->type!=PLAYER || 
-		   (pl->type==PLAYER && pl->contr->party_number==-1) ||
+		   (pl->type==PLAYER && pl->contr->party==NULL) ||
 		   (pl->type==PLAYER && tmp->type==PLAYER &&
-		    pl->contr->party_number!=tmp->contr->party_number)) 
+		    pl->contr->party!=tmp->contr->party)) 
 			exp = skill_attack(tmp,pl,pl->facing,"kicked", skill); /* pl makes an attack */ 
 		stop_jump(pl,i,spaces);
 		return exp;  /* note that calc_skill_exp() is already called by skill_attack() */ 
@@ -457,7 +487,7 @@ static int attempt_jump (object *pl, int dir, int spaces, object *skill) {
 	     * we should get the effects - after all, the player is
 	     * effectively flying.
 	     */
-	    if(QUERY_FLAG(tmp, FLAG_FLY_ON)) { 
+	    if (tmp->move_on & MOVE_FLY_LOW) { 
 		pl->x = x;
 		pl->y = y;
 		pl->map = m;
@@ -578,7 +608,7 @@ static int do_skill_detect_magic(object *pl, object *skill) {
 /* Helper function for do_skill_ident, so that we can loop
  * over inventory AND objects on the ground conveniently.  
  */
-int do_skill_ident2(object *tmp,object *pl, int obj_class, object *skill) 
+static int do_skill_ident2(object *tmp,object *pl, int obj_class, object *skill) 
 {
     int success=0,chance, ip;
     int skill_value = skill->level * pl->stats.Int?pl->stats.Int:10;
@@ -756,7 +786,7 @@ int use_oratory(object *pl, int dir, object *skill) {
     }
 
     /* it's already allied! */ 
-    if(QUERY_FLAG(tmp,FLAG_FRIENDLY)&&(tmp->move_type==PETMOVE)){  
+    if(QUERY_FLAG(tmp,FLAG_FRIENDLY)&&(tmp->attack_movement==PETMOVE)){  
 	if(get_owner(tmp)==pl) {
 	    new_draw_info(NDI_UNIQUE, 0,pl, 
 		      "Your follower loves your speech.\n");
@@ -791,7 +821,7 @@ int use_oratory(object *pl, int dir, object *skill) {
 	tmp->stats.exp = 0;
 	add_friendly_object(tmp);
 	SET_FLAG(tmp,FLAG_FRIENDLY);
-	tmp->move_type = PETMOVE;
+	tmp->attack_movement = PETMOVE;
 	return calc_skill_exp(pl,tmp, skill);
     }
     /* Charm failed.  Creature may be angry now */
@@ -801,7 +831,7 @@ int use_oratory(object *pl, int dir, object *skill) {
 	if(QUERY_FLAG(tmp,FLAG_FRIENDLY)) {
 	    CLEAR_FLAG(tmp,FLAG_FRIENDLY);
 	    remove_friendly_object(tmp);
-	    tmp->move_type = 0; 	/* needed? */ 
+	    tmp->attack_movement = 0; 	/* needed? */ 
 	}
 	CLEAR_FLAG(tmp,FLAG_UNAGGRESSIVE);
     }
@@ -967,6 +997,9 @@ int remove_trap (object *op, int dir, object *skill) {
 			if(trap_disarm(op,tmp2,1, skill) && (!tmp2->owner || tmp2->owner->type!=PLAYER)) {
 			    tmp->stats.exp = tmp->stats.Cha * tmp->level; 
 			    success += calc_skill_exp(op,tmp2, skill);
+			} else {
+			    /* Can't continue to disarm after failure */
+			    return success;
 			}
 		    }
 	    }
@@ -975,6 +1008,9 @@ int remove_trap (object *op, int dir, object *skill) {
 		if (trap_disarm(op,tmp,1,skill) && (!tmp->owner || tmp->owner->type!=PLAYER)) {
 		    tmp->stats.exp = tmp->stats.Cha * tmp->level; 
 		    success += calc_skill_exp(op,tmp,skill);
+		} else {
+		    /* Can't continue to disarm after failure */
+		    return success;
 		}
 	    }
 	}
@@ -1076,10 +1112,9 @@ void meditate (object *pl, object *skill) {
  * ordinary 'books' (anything that is type BOOK). b.t.
  */
 
-static int write_note(object *pl, object *item, char *msg, object *skill) {
+static int write_note(object *pl, object *item, const char *msg, object *skill) {
     char buf[BOOK_BUF]; 
     object *newBook = NULL;
-    event *evt;
 
     /* a pair of sanity checks */
     if(!item||item->type!=BOOK) return 0;
@@ -1094,31 +1129,10 @@ static int write_note(object *pl, object *item, char *msg, object *skill) {
 	new_draw_info(NDI_UNIQUE,0,pl,"Trying to cheat now are we?");
 	return 0;
     }
-    /* GROS: Handle for plugin book writing (trigger) event */
-    if ((evt = find_event(item, EVENT_TRIGGER)) != NULL) {
-	CFParm CFP;
-	int k, l, m;
-	uint32 n;
-	k = EVENT_TRIGGER;
-	l = SCRIPT_FIX_ALL;
-	m = 0;
-	n = 0;
-	CFP.Value[0] = &k;
-	CFP.Value[1] = pl;
-	CFP.Value[2] = item;
-	CFP.Value[3] = NULL;
-	CFP.Value[4] = msg;
-	CFP.Value[5] = &n;
-	CFP.Value[6] = &m;
-	CFP.Value[7] = &m;
-	CFP.Value[8] = &l;
-	CFP.Value[9] = evt->hook;
-	CFP.Value[10]= evt->options;
-	if (findPlugin(evt->plugin)>=0) {
-	    ((PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP));
+
+    /* Lauwenmark: Handle for plugin book writing (trigger) event */
+    if (execute_event(item, EVENT_TRIGGER,pl,NULL,msg,SCRIPT_FIX_ALL)!=0)
 	    return strlen(msg);
-	}
-    }
 
     buf[0] = 0;
     if(!book_overflow(item->msg,msg,BOOK_BUF)) { /* add msg string to book */
@@ -1140,13 +1154,18 @@ static int write_note(object *pl, object *item, char *msg, object *skill) {
 	} else {
 	    if (item->msg) free_string(item->msg);
 	    item->msg=add_string(buf); 
-	    /* This shouldn't be necessary - the object hasn't changed in any visible way */
-/*	    esrv_send_item(pl, item);*/
+            /* This shouldn't be necessary - the object hasn't changed in any
+             * visible way
+             */
+            /*	    esrv_send_item(pl, item);*/
 	}
-	new_draw_info_format(NDI_UNIQUE,0,pl, "You write in the %s.",query_short_name(item));
+        new_draw_info_format(NDI_UNIQUE,0,pl, "You write in the %s.",
+                             query_short_name(item));
 	return strlen(msg);
     } else
-	new_draw_info_format(NDI_UNIQUE,0,pl, "Your message won't fit in the %s!",query_short_name(item)); 
+        new_draw_info_format(NDI_UNIQUE,0,pl,
+                             "Your message won't fit in the %s!",
+                             query_short_name(item));
 
     return 0;
 }
@@ -1248,8 +1267,13 @@ static int write_scroll (object *pl, object *scroll, object *skill) {
 	newscroll->stats.exp = newscroll->value/5;
 
 	/* wait until finished manipulating the scroll before inserting it */
-	if (newscroll != scroll)
-	    newscroll=insert_ob_in_ob(newscroll,pl);
+	if (newscroll == scroll)
+    {
+        /* Remove to correctly merge with other items which may exist in inventory */
+        remove_ob(newscroll);
+        esrv_del_item(pl->contr,newscroll->count);
+    }
+	newscroll=insert_ob_in_ob(newscroll,pl);
 	esrv_send_item(pl, newscroll);
 	success = calc_skill_exp(pl,newscroll, skill);
 	if(!confused) success *= 2;
@@ -1278,9 +1302,9 @@ static int write_scroll (object *pl, object *scroll, object *skill) {
 }
 
 /* write_on_item() - wrapper for write_note and write_scroll */
-int write_on_item (object *pl,char *params, object *skill) {
+int write_on_item (object *pl,const char *params, object *skill) {
     object *item;
-    char *string=params;
+    const char *string=params;
     int msgtype;
     archetype *skat;
 
@@ -1340,7 +1364,7 @@ int write_on_item (object *pl,char *params, object *skill) {
  * 'throwable' (ie not applied cursed obj, worn, etc).
  */
 
-static object *find_throw_ob( object *op, char *request ) {
+static object *find_throw_ob( object *op, const char *request ) {
     object *tmp;
    
     if(!op) { /* safety */
@@ -1348,12 +1372,26 @@ static object *find_throw_ob( object *op, char *request ) {
 	return (object *) NULL;
     }
 
+    /* prefer marked item */
+    tmp = find_marked_object(op);
+    if (tmp != NULL) {
+        /* can't toss invisible or inv-locked items */
+        if (tmp->invisible || QUERY_FLAG(tmp, FLAG_INV_LOCKED)) {
+            tmp = NULL;
+        }
+    }
+
     /* look through the inventory */
-    for(tmp=op->inv;tmp;tmp=tmp->below) {
-       /* can't toss invisible or inv-locked items */
-	if(tmp->invisible||QUERY_FLAG(tmp,FLAG_INV_LOCKED)) continue;
-	if(!request || !strcmp(query_name(tmp),request) || !strcmp(tmp->name,request))
-	    break;
+    if (tmp == NULL) {
+        for (tmp = op->inv; tmp != NULL; tmp = tmp->below) {
+            /* can't toss invisible or inv-locked items */
+            if (tmp->invisible || QUERY_FLAG(tmp, FLAG_INV_LOCKED))
+                continue;
+            if (!request
+            || !strcmp(query_name(tmp), request)
+            || !strcmp(tmp->name, request))
+                break;
+        }
     }
  
     /* this should prevent us from throwing away
@@ -1423,9 +1461,11 @@ static int do_throw(object *op, object *part, object *toss_item, int dir, object
     object *throw_ob=toss_item, *left=NULL;
     tag_t left_tag;
     int eff_str = 0,maxc,str=op->stats.Str,dam=0;
-    int pause_f,weight_f=0;
-    event *evt;
+    int pause_f,weight_f=0, mflags;
     float str_factor=1.0,load_factor=1.0,item_factor=1.0;
+    mapstruct *m;
+    sint16  sx, sy;
+    tag_t tag;
 
     if(throw_ob==NULL) {
 	if(op->type==PLAYER) {
@@ -1480,11 +1520,13 @@ static int do_throw(object *op, object *part, object *toss_item, int dir, object
 #endif
 
     /* 3 things here prevent a throw, you aimed at your feet, you
-     * have no effective throwing strength, or you threw at a wall
-     */ 
-    if(!dir || (eff_str <= 1) ||
-	get_map_flags(part->map,NULL,
-		      part->x+freearr_x[dir],part->y+freearr_y[dir],NULL,NULL) & (P_WALL | P_OUT_OF_MAP)) {
+     * have no effective throwing strength, or you threw at something
+     * that flying objects can't get through.
+     */
+    mflags = get_map_flags(part->map,&m, part->x+freearr_x[dir],part->y+freearr_y[dir],&sx,&sy);
+
+    if(!dir || (eff_str <= 1) || (mflags & P_OUT_OF_MAP) ||
+       (GET_MAP_MOVE_BLOCK(m, sx, sy) & MOVE_FLY_LOW)) {
 
 	/* bounces off 'wall', and drops to feet */
 	remove_ob(throw_ob);
@@ -1639,37 +1681,15 @@ static int do_throw(object *op, object *part, object *toss_item, int dir, object
     throw_ob->speed_left = 0;
     throw_ob->map = part->map;
 
-    SET_FLAG(throw_ob, FLAG_FLYING);
-    SET_FLAG(throw_ob, FLAG_FLY_ON);
-    SET_FLAG(throw_ob, FLAG_WALK_ON);
+    throw_ob->move_type = MOVE_FLY_LOW;
+    throw_ob->move_on = MOVE_FLY_LOW | MOVE_WALK;
+
 #if 0
     /* need to put in a good sound for this */
     play_sound_map(op->map, op->x, op->y, SOUND_THROW_OBJ);
 #endif
-/* GROS - Now we can call the associated script_throw event (if any) */
-    if ((evt = find_event(throw_ob, EVENT_THROW)) != NULL)
-    {
-        CFParm CFP;
-        int k, l, m;
-        uint32 n;
-        k = EVENT_THROW;
-        l = SCRIPT_FIX_ACTIVATOR;
-        m = 0;
-        n = 0;
-        CFP.Value[0] = &k;
-        CFP.Value[1] = op;
-        CFP.Value[2] = throw_ob;
-        CFP.Value[3] = NULL;
-        CFP.Value[4] = NULL;
-        CFP.Value[5] = &n;
-        CFP.Value[6] = &m;
-        CFP.Value[7] = &m;
-        CFP.Value[8] = &l;
-        CFP.Value[9] = evt->hook;
-        CFP.Value[10]= evt->options;
-        if (findPlugin(evt->plugin)>=0)
-            ((PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP));
-    }
+/* Lauwenmark - Now we can call the associated script_throw event (if any) */
+    execute_event(throw_ob, EVENT_THROW,op,NULL,NULL,SCRIPT_FIX_ACTIVATOR);
 #ifdef DEBUG_THROW
     LOG(llevDebug," pause_f=%d \n",pause_f);
     LOG(llevDebug," %s stats: wc=%d dam=%d dist=%d spd=%f break=%d\n",
@@ -1677,12 +1697,14 @@ static int do_throw(object *op, object *part, object *toss_item, int dir, object
         throw_ob->last_sp,throw_ob->speed,throw_ob->stats.food);
     LOG(llevDebug,"inserting tossitem (%d) into map\n",throw_ob->count);
 #endif
+    tag = throw_ob->count;
     insert_ob_in_map(throw_ob,part->map,op,0);
-    move_arrow(throw_ob);
+    if (!was_destroyed (throw_ob, tag))
+        move_arrow(throw_ob);
     return 1;
 }
 
-int skill_throw (object *op, object *part, int dir, char *params, object *skill) {
+int skill_throw (object *op, object *part, int dir, const char *params, object *skill) {
     object *throw_ob;
 
     if(op->type==PLAYER) 
@@ -1692,4 +1714,3 @@ int skill_throw (object *op, object *part, int dir, char *params, object *skill)
 
     return do_throw(op,part, throw_ob,dir, skill);
 }
-
