@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_spell_attack_c =
- *   "$Id: spell_attack.c,v 1.20 2005/07/19 20:59:34 ryo_saeba Exp $";
+ *   "$Id: spell_attack.c,v 1.37 2006/05/25 17:49:36 akirschbaum Exp $";
  */
 
 
@@ -47,7 +47,7 @@
  * op is the spell object.
  */
 
-void check_spell_knockback(object *op) {
+static void check_spell_knockback(object *op) {
     object *tmp, *tmp2; /* object on the map */
     int weight_move;
 	int frictionmod=2; /*poor man's physics - multipy targets weight by this amount */
@@ -84,7 +84,7 @@ void check_spell_knockback(object *op) {
 	
 	 /* surface area? -tm */
 	
-	if(QUERY_FLAG(tmp, FLAG_FLYING))
+	if (tmp->move_type & MOVE_FLYING)
 		frictionmod = 1 ; /* flying objects loose the friction modifier */
 	
 	if(rndm(0, weight_move-1) > ((tmp->weight / num_sections) * frictionmod)) {  /* move it. */
@@ -111,9 +111,12 @@ void check_spell_knockback(object *op) {
  * is the first piece of the fork.
  */
 
-void forklightning(object *op, object *tmp) {
+static void forklightning(object *op, object *tmp) {
     int new_dir=1;  /* direction or -1 for left, +1 for right 0 if no new bolt */
     int t_dir; /* stores temporary dir calculation */
+    mapstruct *m;
+    sint16  sx,sy;
+    object *new_bolt;
 
     /* pick a fork direction.  tmp->stats.Con is the left bias
      * i.e., the chance in 100 of forking LEFT
@@ -127,31 +130,33 @@ void forklightning(object *op, object *tmp) {
     /* check the new dir for a wall and in the map*/
     t_dir = absdir(tmp->direction + new_dir);
 
-    if(get_map_flags(tmp->map,NULL, tmp->x + freearr_x[t_dir],tmp->y + freearr_y[t_dir],
-		     NULL, NULL) & (P_WALL | P_OUT_OF_MAP))
-	 new_dir = 0;
+    if(get_map_flags(tmp->map,&m, tmp->x + freearr_x[t_dir],tmp->y + freearr_y[t_dir],
+		     &sx, &sy) & P_OUT_OF_MAP)
+	return;
 
-    if(new_dir) { /* OK, we made a fork */
-	object *new_bolt = get_object();
+    if (OB_TYPE_MOVE_BLOCK(tmp, GET_MAP_MOVE_BLOCK(m, sx, sy)))
+	return;
 
-	copy_object(tmp,new_bolt);
+    /* OK, we made a fork */
+    new_bolt = get_object();
 
-	/* reduce chances of subsequent forking */
-	new_bolt->stats.Dex -= 10;  
-	tmp->stats.Dex -= 10;  /* less forks from main bolt too */
-	new_bolt->stats.Con += 25 * new_dir; /* adjust the left bias */
-	new_bolt->speed_left = -0.1;
-	new_bolt->direction = t_dir;
-	new_bolt->duration++;
-	new_bolt->x+=DIRX(new_bolt);
-	new_bolt->y+=DIRY(new_bolt);
-	new_bolt->stats.dam /= 2;  /* reduce daughter bolt damage */
-	new_bolt->stats.dam++;
-	tmp->stats.dam /= 2;  /* reduce father bolt damage */
-	tmp->stats.dam++;
-	new_bolt = insert_ob_in_map(new_bolt,op->map,op,0);
-	update_turn_face(new_bolt);
-    }
+    copy_object(tmp,new_bolt);
+
+    /* reduce chances of subsequent forking */
+    new_bolt->stats.Dex -= 10;  
+    tmp->stats.Dex -= 10;  /* less forks from main bolt too */
+    new_bolt->stats.Con += 25 * new_dir; /* adjust the left bias */
+    new_bolt->speed_left = -0.1;
+    new_bolt->direction = t_dir;
+    new_bolt->duration++;
+    new_bolt->x=sx;
+    new_bolt->y=sy;
+    new_bolt->stats.dam /= 2;  /* reduce daughter bolt damage */
+    new_bolt->stats.dam++;
+    tmp->stats.dam /= 2;  /* reduce father bolt damage */
+    tmp->stats.dam++;
+    new_bolt = insert_ob_in_map(new_bolt,m,op,0);
+    update_turn_face(new_bolt);
 }
 
 /* move_bolt: moves bolt 'op'.  Basically, it just advances a space,
@@ -169,7 +174,7 @@ void move_bolt(object *op) {
 	free_object(op);
 	return;
     }
-    hit_map(op,0,op->attacktype);
+    hit_map(op,0,op->attacktype,1);
 	
     if(!op->direction)
 	return;
@@ -189,7 +194,8 @@ void move_bolt(object *op) {
 	 * on the space.  So only call reflwall if we think the data it returns
 	 * will be useful.
 	 */
-	if ((mflags & P_WALL) || ((mflags & P_IS_ALIVE) && reflwall(m, x, y, op))) {
+	if (OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, x, y)) || 
+	  ((mflags & P_IS_ALIVE) && reflwall(m, x, y, op))) {
 
 	    if(!QUERY_FLAG(op, FLAG_REFLECTING))
 		return;
@@ -206,12 +212,21 @@ void move_bolt(object *op) {
 		op->direction=absdir(op->direction+4);
 	    else {
 		int left, right;
+		int mflags;
 
-		left = get_map_flags(op->map, NULL, op->x+freearr_x[absdir(op->direction-1)],
-			       op->y+freearr_y[absdir(op->direction-1)], NULL, NULL) & P_WALL;
+		/* Need to check for P_OUT_OF_MAP: if the bolt is tavelling
+		 * over a corner in a tiled map, it is possible that
+		 * op->direction is within an adjacent map but either
+		 * op->direction-1 or op->direction+1 does not exist.
+		 */
+		mflags = get_map_flags(op->map, &m, op->x+freearr_x[absdir(op->direction-1)],
+			       op->y+freearr_y[absdir(op->direction-1)], &x, &y);
 
-		right = get_map_flags(op->map,NULL, op->x+freearr_x[absdir(op->direction+1)],
-                              op->y+freearr_y[absdir(op->direction+1)], NULL, NULL) & P_WALL;
+		left = (mflags & P_OUT_OF_MAP) ? 0 : OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, x, y));
+
+		mflags = get_map_flags(op->map, &m, op->x+freearr_x[absdir(op->direction+1)],
+                              op->y+freearr_y[absdir(op->direction+1)], &x, &y);
+		right = (mflags & P_OUT_OF_MAP) ? 0 : OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, x, y));
 
 		if(left==right)
 		    op->direction=absdir(op->direction+4);
@@ -292,7 +307,7 @@ int fire_bolt(object *op,object *caster,int dir,object *spob, object *skill) {
 	free_object(tmp);
 	return 0;
     }
-    if (mflags & P_WALL) {
+    if (OB_TYPE_MOVE_BLOCK(tmp, GET_MAP_MOVE_BLOCK(tmp->map, tmp->x, tmp->y))) {
 	if(!QUERY_FLAG(tmp, FLAG_REFLECTING)) {
 	    free_object(tmp);
 	    return 0;
@@ -329,7 +344,7 @@ void explosion(object *op) {
 	free_object(op);
 	return;
     }
-    hit_map(op,0,op->attacktype);
+    hit_map(op,0,op->attacktype,0);
 
     if(op->range>0) {
 	for(i=1;i<9;i++) {
@@ -360,7 +375,7 @@ void explosion(object *op) {
  * poison cloud ball, etc.  op is the object to
  * explode.
  */
-void explode_bullet(object *op)
+static void explode_bullet(object *op)
 {
     tag_t op_tag = op->count;
     object *tmp, *owner;
@@ -374,7 +389,8 @@ void explode_bullet(object *op)
 
     if (op->env) {
 	object *env;
-	for (env = op; env->env != NULL; env = env->env) ;
+
+	env = object_get_env_recursive(op);
 	if (env->map == NULL || out_of_map (env->map, env->x, env->y)) {
 	    LOG (llevError, "BUG: explode_bullet(): env out of map\n");
 	    remove_ob (op);
@@ -393,7 +409,7 @@ void explode_bullet(object *op)
     }
 
     if (op->attacktype) {
-	hit_map (op, 0, op->attacktype);
+	hit_map (op, 0, op->attacktype, 1);
 	if (was_destroyed (op, op_tag))
 	    return;
     }
@@ -438,8 +454,7 @@ void explode_bullet(object *op)
 	tmp->stats.sp = op->direction;
 
     /* Prevent recursion */
-    CLEAR_FLAG (op, FLAG_WALK_ON);
-    CLEAR_FLAG (op, FLAG_FLY_ON);
+    op->move_on = 0;
 
     insert_ob_in_map(tmp, op->map, op, 0);
     /* remove the firebullet */
@@ -460,10 +475,12 @@ void check_bullet(object *op)
     tag_t op_tag = op->count, tmp_tag;
     object *tmp;
     int dam, mflags;
+    mapstruct *m;
+    sint16 sx, sy;
 
-    mflags = get_map_flags(op->map, NULL, op->x, op->y, NULL, NULL);
+    mflags = get_map_flags(op->map,&m, op->x, op->y, &sx, &sy);
 
-    if ( ! (mflags & (P_BLOCKED)))
+    if (!(mflags & P_IS_ALIVE) &&  !OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, sx, sy)))
         return;
 
     if (op->other_arch) {
@@ -479,7 +496,7 @@ void check_bullet(object *op)
     {
         if (QUERY_FLAG (tmp, FLAG_ALIVE)) {
             tmp_tag = tmp->count;
-            dam = hit_player (tmp, op->stats.dam, op, op->attacktype);
+            dam = hit_player (tmp, op->stats.dam, op, op->attacktype, 1);
             if (was_destroyed (op, op_tag) || ! was_destroyed (tmp, tmp_tag)
                 || (op->stats.dam -= dam) < 0)
             {
@@ -494,11 +511,10 @@ void check_bullet(object *op)
 }
 
 
-/* move_fired_arch: This should really be called move_bullet,
- * because it is only really applicable to bullets, and not
- * all fired arches (eg, bolts).
- * Basically, we move 'op' one square, and if it hits something,
- * call check_fired_arch
+/* Basically, we move 'op' one square, and if it hits something,
+ * call check_bullet.
+ * This function is only applicable to bullets, but not to all
+ * fired arches (eg, bolts).
  */
 
 void move_bullet(object *op)
@@ -541,7 +557,7 @@ void move_bullet(object *op)
         return;
     }
 
-    if ( ! op->direction || (mflags & P_WALL)) {
+    if ( ! op->direction || OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, new_x, new_y))) {
         if (op->other_arch) {
             explode_bullet (op);
         } else {
@@ -616,7 +632,7 @@ int fire_bullet(object *op,object *caster,int dir,object *spob) {
 	free_object(tmp);
 	return 0;
     }
-    if (mflags & P_WALL) {
+    if (OB_TYPE_MOVE_BLOCK(tmp, GET_MAP_MOVE_BLOCK(tmp->map, tmp->x, tmp->y))) {
 	if(!QUERY_FLAG(tmp, FLAG_REFLECTING)) {
 	    free_object(tmp);
 	    return 0;
@@ -643,7 +659,7 @@ int fire_bullet(object *op,object *caster,int dir,object *spob) {
 
 
 /* drops an object based on what is in the cone's "other_arch" */
-void cone_drop(object *op) {
+static void cone_drop(object *op) {
     object *new_ob = arch_to_object(op->other_arch);
 
     new_ob->x = op->x;
@@ -677,7 +693,7 @@ void move_cone(object *op) {
 
     /* lava saves it's life, but not yours  :) */
     if (QUERY_FLAG(op, FLAG_LIFESAVE)) {
-	hit_map(op,0,op->attacktype);
+	hit_map(op,0,op->attacktype,0);
 	return;
     }
 
@@ -694,7 +710,7 @@ void move_cone(object *op) {
 #endif
 
     tag = op->count;
-    hit_map(op,0,op->attacktype);
+    hit_map(op,0,op->attacktype,0);
 
     /* Check to see if we should push anything.
      * Spell objects with weight push whatever they encounter to some
@@ -751,6 +767,9 @@ int cast_cone(object *op, object *caster,int dir, object *spell)
 {
     object *tmp;
     int i,success=0,range_min= -1,range_max=1;
+    mapstruct *m;
+    sint16  sx, sy;
+    MoveType	movetype;
 
     if (!spell->other_arch) return 0;
 
@@ -766,6 +785,12 @@ int cast_cone(object *op, object *caster,int dir, object *spell)
 	range_max=8;
     }
 
+    /* Need to know what the movetype of the object we are about
+     * to create is, so we can know if the space we are about to
+     * insert it into is blocked.
+     */
+    movetype = spell->other_arch->clone.move_type;
+
     for(i=range_min;i<=range_max;i++) {
 	sint16 x,y, d;
 
@@ -793,7 +818,10 @@ int cast_cone(object *op, object *caster,int dir, object *spell)
 	x = op->x+freearr_x[d];
 	y = op->y+freearr_y[d];
 
-	if(get_map_flags(op->map,NULL, x,y, NULL, NULL) & (P_WALL | P_OUT_OF_MAP))
+	if(get_map_flags(op->map, &m, x,y, &sx, &sy) & P_OUT_OF_MAP)
+	    continue;
+
+	if ((movetype & GET_MAP_MOVE_BLOCK(m, sx, sy)) == movetype)
 	    continue;
 
 	success=1;
@@ -801,7 +829,8 @@ int cast_cone(object *op, object *caster,int dir, object *spell)
 	set_owner(tmp,op);
 	set_spell_skill(op, caster, spell, tmp);
 	tmp->level = caster_level (caster, spell);
-	tmp->x=x,tmp->y=y;
+	tmp->x = sx;
+	tmp->y = sy;
 	tmp->attacktype=spell->attacktype;
 
 	/* holy word stuff */                
@@ -837,17 +866,16 @@ int cast_cone(object *op, object *caster,int dir, object *spell)
 	}
 
 
-	if ( ! QUERY_FLAG (tmp, FLAG_FLYING))
+	if ( !(tmp->move_type & MOVE_FLY_LOW))
 	    LOG (llevDebug, "cast_cone(): arch %s doesn't have flying 1\n",
 		 spell->other_arch->name);
 
-	if (( ! QUERY_FLAG (tmp, FLAG_WALK_ON) || ! QUERY_FLAG (tmp, FLAG_FLY_ON))
-	    && tmp->stats.dam) {
+	if (!tmp->move_on && tmp->stats.dam) {
 	    LOG (llevDebug, 
-		 "cast_cone(): arch %s doesn't have walk_on 1 and fly_on 1\n", 
+		 "cast_cone(): arch %s doesn't have move_on set\n", 
 		 spell->other_arch->name);
 	}
-	insert_ob_in_map(tmp,op->map,op,0);
+	insert_ob_in_map(tmp,m,op,0);
 
 	/* This is used for tracking spells so that one effect doesn't hit
 	 * a single space too many times.
@@ -877,7 +905,7 @@ void animate_bomb(object *op) {
     if(op->state!=NUM_ANIMATIONS(op)-1)
 	return;
 
-    for(env=op;env->env!=NULL;env=env->env);
+    env = object_get_env_recursive(op);
 
     if (op->env) {
         if (env->map == NULL)
@@ -928,10 +956,12 @@ void animate_bomb(object *op) {
 int create_bomb(object *op,object *caster,int dir, object *spell) {
 
     object *tmp;
-
+    int mflags;
     sint16 dx=op->x+freearr_x[dir], dy=op->y+freearr_y[dir];
+    mapstruct *m;
 
-    if(get_map_flags(op->map,NULL, dx,dy, NULL,NULL) & (P_OUT_OF_MAP | P_WALL)) {
+    mflags = get_map_flags(op->map, &m, dx,dy, &dx,&dy);
+    if ((mflags & P_OUT_OF_MAP) || (GET_MAP_MOVE_BLOCK(m, dx, dy) & MOVE_WALK)) {
 	new_draw_info(NDI_UNIQUE, 0,op,"There is something in the way.");
 	return 0;
     }
@@ -947,7 +977,7 @@ int create_bomb(object *op,object *caster,int dir, object *spell) {
     set_spell_skill(op, caster, spell, tmp);
     tmp->x=dx;
     tmp->y=dy;
-    insert_ob_in_map(tmp,op->map,op,0);
+    insert_ob_in_map(tmp,m,op,0);
     return 1;
 }
 
@@ -967,7 +997,7 @@ int create_bomb(object *op,object *caster,int dir, object *spell) {
  * this info is used for blocked magic/unholy spaces.
  */
  
-object *get_pointed_target(object *op, int dir, int range, int type) {
+static object *get_pointed_target(object *op, int dir, int range, int type) {
     object *target;
     sint16 x,y;
     int dist, mflags;
@@ -981,9 +1011,11 @@ object *get_pointed_target(object *op, int dir, int range, int type) {
 	mp = op->map;
 	mflags = get_map_flags(op->map, &mp, x, y, &x, &y);
 
-	if (mflags & (P_OUT_OF_MAP | P_WALL)) return NULL;
+	if (mflags & P_OUT_OF_MAP)  return NULL;
 	if ((type & SPELL_MANA) && (mflags & P_NO_MAGIC)) return NULL;
 	if ((type & SPELL_GRACE) && (mflags & P_NO_CLERIC)) return NULL;
+	if (GET_MAP_MOVE_BLOCK(mp, x, y) & MOVE_FLY_LOW) return NULL;
+
 	if (mflags & P_IS_ALIVE) {
 	    for(target=get_map_ob(mp,x,y); target; target=target->above) {
 		if(QUERY_FLAG(target->head?target->head:target,FLAG_MONSTER)) {
@@ -1102,6 +1134,7 @@ void move_missile(object *op) {
     int i, mflags;
     object *owner;
     sint16 new_x, new_y;
+    mapstruct *m;
 
     if (op->range-- <=0) {
 	remove_ob(op);
@@ -1124,11 +1157,12 @@ void move_missile(object *op) {
     new_x = op->x + DIRX(op);
     new_y = op->y + DIRY(op);
 
-    mflags = get_map_flags(op->map, NULL, new_x, new_y, NULL, NULL);
+    mflags = get_map_flags(op->map, &m, new_x, new_y, &new_x, &new_y);
 
-    if (mflags & P_BLOCKED) {
+    if (!(mflags & P_OUT_OF_MAP) &&
+	((mflags & P_IS_ALIVE) || OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, new_x, new_y)))) {
 	tag_t tag = op->count;
-	hit_map (op, op->direction, AT_MAGIC);
+	hit_map (op, op->direction, AT_MAGIC, 1);
 	/* Basically, missile only hits one thing then goes away.
 	 * we need to remove it if someone hasn't already done so.
 	 */
@@ -1140,12 +1174,13 @@ void move_missile(object *op) {
     }
 
     remove_ob(op);
-    if ( ! op->direction || (mflags & (P_WALL | P_OUT_OF_MAP))) {
+    if ( ! op->direction || (mflags & P_OUT_OF_MAP)) {
 	free_object(op);
 	return;
     }
     op->x = new_x;
     op->y = new_y;
+    op->map = m;
     i=spell_find_dir(op->map, op->x, op->y, get_owner(op));
     if(i > 0 && i != op->direction){
 	op->direction=i;
@@ -1164,14 +1199,14 @@ void move_missile(object *op) {
  * give them the capability to have an inventory. b.t.
  */
 
-int make_object_glow(object *op, int radius, int time) {
+static int make_object_glow(object *op, int radius, int time) {
     object *tmp;
 
     /* some things are unaffected... */
     if(op->path_denied&PATH_LIGHT) 
 	return 0;
 
-    tmp=get_archetype(FORCE_NAME);
+    tmp=create_archetype(FORCE_NAME);
     tmp->speed = 0.01;
     tmp->stats.food = time;
     SET_FLAG(tmp, FLAG_IS_USED_UP);
@@ -1202,7 +1237,7 @@ int cast_destruction(object *op, object *caster, object *spell_ob) {
     sint16 sx,sy;
     mapstruct *m;
     object *tmp;
-    char *skill;
+    const char *skill;
 
     range = spell_ob->range + SP_level_range_adjust(caster, spell_ob);
     dam = spell_ob->stats.dam + SP_level_dam_adjust(caster, spell_ob);
@@ -1239,7 +1274,7 @@ int cast_destruction(object *op, object *caster, object *spell_ob) {
 		    if ((friendly && !QUERY_FLAG(tmp, FLAG_FRIENDLY) && tmp->type!=PLAYER) ||
 		       (!friendly && (QUERY_FLAG(tmp, FLAG_FRIENDLY) || tmp->type==PLAYER))) {
 			if (spell_ob->subtype == SP_DESTRUCTION) {
-			    hit_player(tmp,dam,op,spell_ob->attacktype);
+			    hit_player(tmp,dam,op,spell_ob->attacktype,0);
 			    if (spell_ob->other_arch) {
 				tmp = arch_to_object(spell_ob->other_arch);
 				tmp->x = sx;
@@ -1299,7 +1334,7 @@ int cast_curse(object *op, object *caster, object *spell_ob, int dir) {
     }
 
     if(force==NULL) {
-	force=get_archetype(FORCE_NAME);
+	force=create_archetype(FORCE_NAME);
 	force->subtype = FORCE_CHANGE_ABILITY;
 	free_string(force->name);
 	if (spell_ob->race) 
@@ -1365,7 +1400,7 @@ int mood_change(object *op, object *caster, object *spell) {
     int done_one, range, mflags, level, at, best_at;
     sint16 x, y, nx, ny;
     mapstruct	*m;
-    char *race;
+    const char *race;
 
     /* We precompute some values here so that we don't have to keep
      * doing it over and over again.
@@ -1480,9 +1515,9 @@ int mood_change(object *op, object *caster, object *spell) {
         set_owner(head, op);
 		set_spell_skill(op, caster, spell, head);
 		add_friendly_object(head);
-		head->move_type = PETMOVE;
+		head->attack_movement = PETMOVE;
 		done_one = 1;
-		change_exp(op, head->stats.exp / 2, spell->skill, SK_EXP_ADD_SKILL);
+		change_exp(op, head->stats.exp / 2, head->skill, SK_EXP_ADD_SKILL);
 		head->stats.exp = 0;
 	    }
 
@@ -1508,7 +1543,7 @@ int mood_change(object *op, object *caster, object *spell) {
 
 void move_ball_spell(object *op) {
     int i,j,dam_save,dir, mflags;
-    sint16 nx,ny;
+    sint16 nx,ny, hx, hy;
     object *owner;
     mapstruct *m;
 
@@ -1536,7 +1571,8 @@ void move_ball_spell(object *op) {
 
 	nx = op->x + freearr_x[tmpdir];
 	ny = op->y + freearr_y[tmpdir];
-	if ( ! (get_map_flags(op->map, NULL, nx, ny, NULL, NULL) & (P_WALL | P_OUT_OF_MAP))) {
+	if ( ! (get_map_flags(op->map, &m, nx, ny, &nx, &ny) & P_OUT_OF_MAP) &&
+	    !(OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, nx, ny)))) {
 	    dir = tmpdir;
 	    break;
 	}
@@ -1544,12 +1580,13 @@ void move_ball_spell(object *op) {
     if (dir == 0) {
 	nx = op->x;
 	ny = op->y;
+	m = op->map;
     }
 
     remove_ob(op);
     op->y=ny;
     op->x=nx;
-    insert_ob_in_map(op,op->map,op,0);
+    insert_ob_in_map(op,m,op,0);
 	 
     dam_save = op->stats.dam;  /* save the original dam: we do halfdam on 
                                 surrounding squares */
@@ -1559,7 +1596,6 @@ void move_ball_spell(object *op) {
      * the surround spaces.
      */
     for(j=0;j<9;j++) {
-	sint16 hx,hy;  /* hit these squares */
 	object *new_ob;
 
 	hx = nx+freearr_x[j];
@@ -1574,12 +1610,14 @@ void move_ball_spell(object *op) {
 	 * of the map either.
 	 */
 
-	if((mflags & P_IS_ALIVE) && (!owner || owner->x!=hx || owner->y!=hy || owner->map != m)) {
+	if((mflags & P_IS_ALIVE) && (!owner || owner->x!=hx || owner->y!=hy || !on_same_map(owner,op))) {
 	    if(j) op->stats.dam = dam_save/2;
-	    hit_map(op,j,op->attacktype);
+	    hit_map(op,j,op->attacktype,1);
 
 	}
-	if(!(mflags & P_WALL) && op->other_arch) { /* insert the other arch */
+
+	/* insert the other arch */
+	if(op->other_arch && !(OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, hx, hy)))) {
 	    new_ob = arch_to_object(op->other_arch);
 	    new_ob->x = hx;
 	    new_ob->y = hy;
@@ -1594,7 +1632,8 @@ void move_ball_spell(object *op) {
 
     if(i>=0) { /* we have a preferred direction!  */
 	/* pick another direction if the preferred dir is blocked. */
-	if(get_map_flags(op->map,NULL, nx + freearr_x[i], ny + freearr_y[i],NULL,NULL) & P_WALL) {
+	if (get_map_flags(op->map,&m, nx + freearr_x[i], ny + freearr_y[i],&hx,&hy) & P_OUT_OF_MAP ||
+	   OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, hx, hy))) {
 	    i= absdir(i + rndm(0, 2) -1);  /* -1, 0, +1 */
 	}
 	op->direction=i;
@@ -1616,8 +1655,11 @@ void move_swarm_spell(object *op)
     static int diagonal_adjust[10] = { -3, -2, -2, -1, 0, 0, 1, 2, 2, 3 };
     sint16 target_x, target_y, origin_x, origin_y;
     int basedir, adjustdir;
+    mapstruct *m;
+    object *owner;
 
-    if(op->duration == 0 || get_owner (op) == NULL) {
+    owner = get_owner(op);
+    if(op->duration == 0 || owner == NULL) {
 	remove_ob(op);
 	free_object(op);
 	return;
@@ -1660,13 +1702,13 @@ void move_swarm_spell(object *op)
      */
     
     if (op->spell && op->spell->type == SPELL && 
-	!(get_map_flags(op->map, NULL, target_x, target_y, NULL, NULL) & (P_WALL | P_OUT_OF_MAP))) {
+	!(get_map_flags(op->map, &m, target_x, target_y, &target_x, &target_y) & P_OUT_OF_MAP)) {
 
 	/* Bullet spells have a bunch more customization that needs to be done */
 	if (op->spell->subtype == SP_BULLET)
-	    fire_bullet(op, op, basedir, op->spell);
+	    fire_bullet(owner, op, basedir, op->spell);
 	else if (op->spell->subtype == SP_MAGIC_MISSILE)
-	    fire_arch_from_position (op, op, origin_x, origin_y, basedir, op->spell);
+	    fire_arch_from_position (owner, op, origin_x, origin_y, basedir, op->spell);
     }
 }
 
@@ -1692,7 +1734,7 @@ int fire_swarm (object *op, object *caster, object *spell, int dir)
 
     if (!spell->other_arch) return 0;
 
-    tmp=get_archetype(SWARM_SPELL);
+    tmp=create_archetype(SWARM_SPELL);
     tmp->x=op->x;
     tmp->y=op->y;	    
     set_owner(tmp,op);       /* needed so that if swarm elements kill, caster gets xp.*/
@@ -1750,13 +1792,13 @@ int cast_light(object *op,object *caster,object *spell, int dir) {
 	    if(QUERY_FLAG(target,FLAG_MONSTER)) {
 		/* oky doky. got a target monster. Lets make a blinding attack */
 		if(target->head) target = target->head;
-		(void) hit_player(target,dam,op,spell->attacktype);
+		(void) hit_player(target,dam,op,spell->attacktype,1);
 		return 1; /* one success only! */
 	    }
     }
 
     /* no live target, perhaps a wall is in the way? */
-    if(mflags & P_BLOCKED) {
+    if (OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, x, y))) {
 	new_draw_info(NDI_UNIQUE, 0,op,"Something is in the way.");
 	return 0;
     }
@@ -1818,8 +1860,8 @@ int cast_cause_disease(object *op, object *caster, object *spell, int dir) {
 
 	if (mflags & P_OUT_OF_MAP) return 0;
 
-	/* don't go through walls */
-	if (mflags & P_NO_PASS) return 0;
+	/* don't go through walls - presume diseases are airborne */
+	if (GET_MAP_MOVE_BLOCK(m, x, y) &  MOVE_FLY_LOW) return 0;
 
 	/* Only bother looking on this space if there is something living here */
 	if (mflags & P_IS_ALIVE) {
@@ -1881,7 +1923,7 @@ int cast_cause_disease(object *op, object *caster, object *spell, int dir) {
 			 new_draw_info_format(NDI_UNIQUE, 0, op, "You inflict %s on %s!",disease->name,walk->name);
 
 			 free_object(disease); /* don't need this one anymore */
-			 flash=get_archetype(ARCH_DETECT_MAGIC);
+			 flash=create_archetype(ARCH_DETECT_MAGIC);
 			 flash->x = x;
 			 flash->y = y;
 			 flash->map = walk->map;
