@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_spell_effect_c =
- *   "$Id: spell_effect.c,v 1.131 2005/08/10 13:38:31 ryo_saeba Exp $";
+ *   "$Id: spell_effect.c,v 1.155 2006/05/13 05:18:44 qal21 Exp $";
  */
 
 
@@ -36,7 +36,8 @@
 #include <spells.h>
 #include <sounds.h>
 
-/* cast_magic_storm: This is really used mostly for spell
+/**
+ * This is really used mostly for spell
  * fumbles at the like.  tmp is the object to propogate.
  * op is what is casting this.
  */
@@ -67,7 +68,7 @@ int recharge(object *op, object *caster, object *spell_ob) {
     int ncharges;
 
     wand = find_marked_object(op);
-    if(wand == NULL) {
+    if(wand == NULL || wand->type != WAND) {
 	new_draw_info(NDI_UNIQUE, 0, op, "You need to mark the wand you want to recharge.");
 	return 0;
     }
@@ -78,7 +79,7 @@ int recharge(object *op, object *caster, object *spell_ob) {
 	esrv_del_item(op->contr, wand->count);
 	remove_ob(wand);
 	free_object(wand);
-	tmp = get_archetype("fireball");
+	tmp = create_archetype("fireball");
 	tmp->stats.dam = (spell_ob->stats.dam + SP_level_dam_adjust(caster, spell_ob)) / 10;
 	if (!tmp->stats.dam) tmp->stats.dam = 1;
 	tmp->stats.hp = tmp->stats.dam / 2;
@@ -123,12 +124,13 @@ int recharge(object *op, object *caster, object *spell_ob) {
  * an item of proper value is generated.
  */
 
-/* polymorph_living - takes a living object (op) and turns it into 
+/**
+ * Takes a living object (op) and turns it into 
  * another monster of some sort.  Currently, we only deal with single
  * space monsters.
  */
 
-void polymorph_living(object *op) {
+static void polymorph_living(object *op) {
     archetype *at;
     int nr = 0, x = op->x, y = op->y, numat=0, choice,friendly;
     mapstruct *map = op->map;
@@ -189,7 +191,7 @@ void polymorph_living(object *op) {
 	set_owner(op,owner);
     if (friendly) {
 	SET_FLAG(op, FLAG_FRIENDLY);
-	op->move_type = PETMOVE;
+	op->attack_movement = PETMOVE;
 	add_friendly_object(op);
     }
 
@@ -212,13 +214,14 @@ void polymorph_living(object *op) {
 }
 
 
-/* polymorph_melt Destroys item from polymorph failure 
+/**
+ * Destroys item from polymorph failure 
  * who is the caster of the polymorph, op is the
  * object destroyed.  We should probably do something
  * more clever ala nethack - create an iron golem or
  * something.
  */
-void polymorph_melt(object *who, object *op)
+static void polymorph_melt(object *who, object *op)
 {
     /* Not unique */
     new_draw_info_format(NDI_GREY, 0, who,
@@ -230,10 +233,11 @@ void polymorph_melt(object *who, object *op)
     return;
 }
 
-/* polymorph_item - changes an item to another item of similar type.
+/**
+ * Changes an item to another item of similar type.
  * who is the caster of spell, op is the object to be changed.
  */
-void polymorph_item(object *who, object *op) {
+static void polymorph_item(object *who, object *op) {
     archetype *at;
     int max_value, difficulty, tries=0,choice, charges=op->stats.food,numat=0;
     object *new_ob;
@@ -311,7 +315,7 @@ void polymorph_item(object *who, object *op) {
     insert_ob_in_map(new_ob,who->map,new_ob,INS_NO_MERGE | INS_NO_WALK_ON);
 }
 
-/* polymorh - caster who has hit object op. */
+/** Caster who has hit object op. */
 void polymorph(object *op, object *who) {
 
     int tmp;
@@ -341,7 +345,7 @@ void polymorph(object *op, object *who) {
      */
     if(op->type == 0 || op->arch == NULL || 
        QUERY_FLAG(op,FLAG_NO_PICK) 
-       || QUERY_FLAG(op, FLAG_NO_PASS) || op->type == TREASURE)
+       || op->move_block || op->type == TREASURE)
 	return;
 
     tmp = rndm(0, 7);
@@ -350,7 +354,8 @@ void polymorph(object *op, object *who) {
 }
 
 
-/* cast_polymorph -
+/**
+ * cast_polymorph -
  * object *op is the player/monster
  * caster is object that cast it.
  * spell_ob is the actually spell object.
@@ -374,7 +379,10 @@ int cast_polymorph(object *op, object *caster, object *spell_ob, int dir) {
 	m = op->map;
 	mflags = get_map_flags(m, &m, x, y, &x, &y);
 
-	if (mflags & (P_WALL | P_NO_MAGIC))
+	if (mflags & (P_NO_MAGIC | P_OUT_OF_MAP))
+	    break;
+
+	if (GET_MAP_MOVE_BLOCK(m, x, y) & MOVE_FLY_LOW)
 	    break;
 
 	/* Get the top most object */
@@ -401,7 +409,8 @@ int cast_polymorph(object *op, object *caster, object *spell_ob, int dir) {
 
 
 
-/* Create a missile (nonmagic - magic +4). Will either create bolts or arrows
+/**
+ * Create a missile (nonmagic - magic +4). Will either create bolts or arrows
  * based on whether a crossbow or bow is equiped. If neither, it defaults to
  * arrows.
  * Sets the plus based on the casters level. It is also settable with the
@@ -412,10 +421,10 @@ int cast_polymorph(object *op, object *caster, object *spell_ob, int dir) {
  * create nonnmagic arrows, or even -1, etc...
  */
 
-int cast_create_missile(object *op, object *caster,object *spell, int dir, char *stringarg)
+int cast_create_missile(object *op, object *caster,object *spell, int dir, const char *stringarg)
 {
     int missile_plus=0, bonus_plus=0;
-    char *missile_name;
+    const char *missile_name;
     object *tmp, *missile;
     tag_t tag;
 
@@ -433,7 +442,7 @@ int cast_create_missile(object *op, object *caster,object *spell, int dir, char
 	    missile_name);
 	return 0;
     }
-    missile = get_archetype(missile_name);
+    missile = create_archetype(missile_name);
 
     if (stringarg) {
 	/* If it starts with a letter, presume it is a description */
@@ -492,9 +501,11 @@ int cast_create_missile(object *op, object *caster,object *spell, int dir, char
 }
 
 
-/*  allows the choice of what sort of food object to make.
- *  If stringarg is NULL, it will create food dependent on level  --PeterM*/
-int cast_create_food(object *op,object *caster, object *spell_ob, int dir, char *stringarg)
+/**
+ * allows the choice of what sort of food object to make.
+ *  If stringarg is NULL, it will create food dependent on level  --PeterM
+ */
+int cast_create_food(object *op,object *caster, object *spell_ob, int dir, const char *stringarg)
 {
     int food_value;
     archetype *at=NULL;
@@ -504,10 +515,11 @@ int cast_create_food(object *op,object *caster, object *spell_ob, int dir, char
               + 50 * SP_level_duration_adjust(caster,spell_ob);
 
     if(stringarg) {
-	at=find_archetype_by_object_name(stringarg);
-	if (at==NULL || ((at->clone.type != FOOD && at->clone.type != DRINK)
-	    || (at->clone.stats.food > food_value)))
-		stringarg = NULL;
+	at = find_archetype_by_object_type_name(FOOD, stringarg);
+	if (at == NULL)
+	    at = find_archetype_by_object_type_name(DRINK, stringarg);
+	if (at == NULL || at->clone.stats.food > food_value)
+	    stringarg = NULL;
     }
   
     if(!stringarg) {
@@ -572,7 +584,7 @@ int probe(object *op, object *caster, object *spell_ob, int dir) {
 
 	if (mflags & P_OUT_OF_MAP) break;
 
-	if (mflags & P_NO_MAGIC) {
+	if (!QUERY_FLAG(op, FLAG_WIZCAST) && (mflags & P_NO_MAGIC)) {
 	    new_draw_info(NDI_UNIQUE, 0,op,"Something blocks your magic.");
 	    return 0;
 	}
@@ -592,7 +604,8 @@ int probe(object *op, object *caster, object *spell_ob, int dir) {
 }
 
 
-/* This checks to see if 'pl' is invisible to 'mon'.
+/**
+ * This checks to see if 'pl' is invisible to 'mon'.
  * does race check, undead check, etc
  * Returns TRUE if mon can't see pl, false
  * otherwise.  This doesn't check range, walls, etc.  It
@@ -623,7 +636,8 @@ int makes_invisible_to(object *pl, object *mon)
     }
 }
 
-/* Makes the player or character invisible.
+/**
+ * Makes the player or character invisible.
  * Note the spells to 'stack', but perhaps in odd ways.
  * the duration for all is cumulative.
  * In terms of invis undead/normal invis, it is the last one cast that
@@ -674,7 +688,8 @@ int cast_invisible(object *op, object *caster, object *spell_ob) {
     return 1;
 }
 
-/* earth to dust spell.  Basically destroys earthwalsl in the area.
+/**
+ * Basically destroys earthwalls in the area.
  */
 int cast_earth_to_dust(object *op,object *caster, object *spell_ob) {
     object *tmp, *next;
@@ -695,26 +710,33 @@ int cast_earth_to_dust(object *op,object *caster, object *spell_ob) {
 	    mflags = get_map_flags(m, &m, sx, sy, &sx, &sy);
 
 	    if (mflags & P_OUT_OF_MAP) continue;
-	    /* If the space doesn't block, no wall here to remove */
-	    if (mflags & P_BLOCKED) {
+
+	    /* If the space doesn't block, no wall here to remove
+	     * Don't care too much what it blocks - this allows for
+	     * any sort of earthwall/airwall/waterwall, etc
+	     * type effects.
+	     */
+	    if (GET_MAP_MOVE_BLOCK(m, sx, sy)) {
 		for(tmp=get_map_ob(m, sx, sy);tmp!=NULL;tmp=next) {
 		    next=tmp->above;
 		    if(tmp&&QUERY_FLAG(tmp, FLAG_TEAR_DOWN))
-			hit_player(tmp,9998,op,AT_PHYSICAL);
+			hit_player(tmp,9998,op,AT_PHYSICAL,0);
 		}
 	    }
 	}
     return 1;
 }
 
-
+/**
+ * Handles the actual word of recalling. Called when force in player inventory expires.
+ */
 void execute_word_of_recall(object *op) {
     object *wor=op;
     while(op!=NULL && op->type!=PLAYER)
 	op=op->env;
 
     if(op!=NULL && op->map) {
-	if (get_map_flags(op->map, NULL, op->x, op->y, NULL, NULL) & P_NO_CLERIC)
+	if ((get_map_flags(op->map, NULL, op->x, op->y, NULL, NULL) & P_NO_CLERIC) && (!QUERY_FLAG(op,FLAG_WIZCAST)))
 	    new_draw_info(NDI_UNIQUE, 0,op,"You feel something fizzle inside you.");
 	else
 	    enter_exit(op,wor);
@@ -723,7 +745,8 @@ void execute_word_of_recall(object *op) {
     free_object(wor);
 }
 
-/* Word of recall causes the player to return 'home'.
+/**
+ * Word of recall causes the player to return 'home'.
  * we put a force into the player object, so that there is a 
  * time delay effect.
  */
@@ -734,10 +757,16 @@ int cast_word_of_recall(object *op, object *caster, object *spell_ob) {
     if(op->type!=PLAYER)
 	return 0;
 
-    dummy=get_archetype(FORCE_NAME);
+    if (find_obj_by_type_subtype(op,SPELL_EFFECT, SP_WORD_OF_RECALL))
+    {
+        new_draw_info(NDI_UNIQUE, 0, op, "You feel a force starting to build up inside you." );
+        return 1;
+    }
+
+    dummy=create_archetype(FORCE_NAME);
     if(dummy == NULL){
 	new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
-	LOG(llevError,"cast_word_of_recall: get_archetype(force) failed!\n");
+	LOG(llevError,"cast_word_of_recall: create_archetype(force) failed!\n");
 	return 0;
     }
     time = spell_ob->duration - SP_level_duration_adjust(caster, spell_ob);
@@ -765,7 +794,7 @@ int cast_word_of_recall(object *op, object *caster, object *spell_ob) {
     return 1;
 }
 
-/* cast_wonder
+/**
  * wonder is really just a spell that will likely cast another
  * spell.
  */
@@ -817,7 +846,7 @@ int perceive_self(object *op) {
 	new_draw_info(NDI_UNIQUE, 0,op,"You have:");
 	new_draw_info(NDI_UNIQUE, 0,op,cp);
 	if (tmp!=NULL) {
-	    for (i=0; i<7; i++) {
+	    for (i=0; i<NUM_STATS; i++) {
 		if (get_attr_value(&tmp->stats, i)<0) {
 		    new_draw_info_format(NDI_UNIQUE, 0,op, 
 			 "Your %s is depleted by %d", statname[i],
@@ -844,8 +873,7 @@ int perceive_self(object *op) {
     return 1;
 }
 
-/* int cast_create_town_portal (object *op, object *caster, int dir)
- *
+/**
  * This function cast the spell of town portal for op
  *
  * The spell operates in two passes. During the first one a place
@@ -880,6 +908,12 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
 	return 0;
     }
 
+    /* Check to see if the player is on a transport */
+    if (op->contr && op->contr->transport) {
+	new_draw_info(NDI_UNIQUE | NDI_NAVY, 0,op,"You need to exit the transport to cast that.\n");
+        return 0;
+    }
+
     /* The first thing to do is to check if we have a marked destination
      * dummy is used to make a check inventory for the force
      */
@@ -924,7 +958,7 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
      */
 
     /* First step: killing existing town portals */
-    dummy=get_archetype(spell->race);
+    dummy=create_archetype(spell->race);
     if(dummy == NULL){
 	new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
 	LOG(llevError,"get_object failed (force) in cast_create_town_portal for %s!\n",op->name);
@@ -1005,7 +1039,7 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
      */
 
     snprintf (portal_name,1024,"%s's portal to %s",op->name,force->name);
-    dummy=get_archetype(spell->slaying); /*The portal*/
+    dummy=create_archetype(spell->slaying); /*The portal*/
     if(dummy == NULL) {
 	new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
 	LOG(llevError,"get_object failed (perm_magic_portal) in cast_create_town_portal for %s!\n",op->name);
@@ -1024,7 +1058,7 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
      * object, so on future castings, we can know that he has an active
      * town portal.
      */
-    tmp=get_archetype(spell->race);
+    tmp=create_archetype(spell->race);
     if(tmp == NULL){
 	new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
 	LOG(llevError,"get_object failed (force) in cast_create_town_portal for %s!\n",op->name);
@@ -1043,7 +1077,7 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
      * where this portal goes to.
      */
     snprintf (portal_name,1024,"%s's portal to %s",op->name,op->map->path);
-    dummy=get_archetype (spell->slaying); /*The portal*/
+    dummy=create_archetype (spell->slaying); /*The portal*/
     if(dummy == NULL) {
 	new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
 	LOG(llevError,"get_object failed (perm_magic_portal) in cast_create_town_portal for %s!\n",op->name);
@@ -1063,7 +1097,7 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
     /* Now we create another town portal marker that
      * points back to the one we just made
      */
-    tmp=get_archetype(spell->race);
+    tmp=create_archetype(spell->race);
     if(tmp == NULL){
 	new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
 	LOG(llevError,"get_object failed (force) in cast_create_town_portal for %s!\n",op->name);
@@ -1084,7 +1118,8 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
 }
 
 
-/* This creates magic walls.  Really, it can create most any object,
+/**
+ * This creates magic walls.  Really, it can create most any object,
  * within some reason.
  */
 
@@ -1093,7 +1128,7 @@ int magic_wall(object *op,object *caster,int dir,object *spell_ob) {
     int i,posblocked,negblocked, maxrange;
     sint16 x, y;
     mapstruct *m;
-    char *name;
+    const char *name;
     archetype *at;
 
     if(!dir) {
@@ -1106,8 +1141,9 @@ int magic_wall(object *op,object *caster,int dir,object *spell_ob) {
     }
     m = op->map;
 
-    if ((QUERY_FLAG(spell_ob, FLAG_NO_PASS) || x != op->x || y != op->y) &&
-	get_map_flags(m, &m, x, y, &x, &y) & (P_OUT_OF_MAP | P_BLOCKED)) {
+    if ((spell_ob->move_block || x != op->x || y != op->y) &&
+      (get_map_flags(m, &m, x, y, &x, &y) & (P_OUT_OF_MAP|P_IS_ALIVE) ||
+      ((spell_ob->move_block & GET_MAP_MOVE_BLOCK(m, x, y)) == spell_ob->move_block))) {
 	new_draw_info(NDI_UNIQUE, 0,op,"Something is in the way.");
 	return 0;
     }
@@ -1193,7 +1229,9 @@ int magic_wall(object *op,object *caster,int dir,object *spell_ob) {
 	y = tmp->y+i*freearr_y[dir2];
 	m = tmp->map;
 
-	if(!(get_map_flags(m, &m, x, y, &x, &y) & (P_OUT_OF_MAP | P_BLOCKED)) && !posblocked) {
+	if(!(get_map_flags(m, &m, x, y, &x, &y) & (P_OUT_OF_MAP|P_IS_ALIVE)) &&
+	  ((spell_ob->move_block & GET_MAP_MOVE_BLOCK(m, x, y)) != spell_ob->move_block) &&
+	  !posblocked) {
 	    tmp2 = get_object();
 	    copy_object(tmp,tmp2);
 	    tmp2->x = x;
@@ -1209,7 +1247,9 @@ int magic_wall(object *op,object *caster,int dir,object *spell_ob) {
 	y = tmp->y-i*freearr_y[dir2];
 	m = tmp->map;
 
-	if(!(get_map_flags(m, &m, x, y, &x, &y) & (P_OUT_OF_MAP | P_BLOCKED)) && !negblocked) {
+	if(!(get_map_flags(m, &m, x, y, &x, &y) & (P_OUT_OF_MAP|P_IS_ALIVE)) &&
+	  ((spell_ob->move_block & GET_MAP_MOVE_BLOCK(m, x, y)) != spell_ob->move_block) &&
+	  !negblocked) {
 	    tmp2 = get_object();
 	    copy_object(tmp,tmp2);
 	    tmp2->x = x;
@@ -1227,7 +1267,10 @@ int magic_wall(object *op,object *caster,int dir,object *spell_ob) {
 }
 
 int dimension_door(object *op,object *caster, object *spob, int dir) {
-    int dist, maxdist, mflags;
+    uint32 dist, maxdist;
+    int  mflags;
+    mapstruct *m;
+    sint16  sx, sy;
 
     if(op->type!=PLAYER)
 	return 0;
@@ -1250,12 +1293,14 @@ int dimension_door(object *op,object *caster, object *spob, int dir) {
 	}
 
 	for(dist=0;dist<op->contr->count; dist++) {
-	    mflags = get_map_flags(op->map, NULL, 
+	    mflags = get_map_flags(op->map, &m, 
 		op->x+freearr_x[dir]*(dist+1), op->y+freearr_y[dir]*(dist+1),
-		NULL, NULL);
+		&sx, &sy);
 
-	    if ((mflags & (P_NO_MAGIC | P_OUT_OF_MAP)) ||
-		((mflags & P_NO_PASS) && (mflags & P_BLOCKSVIEW))) break;
+	    if (mflags & (P_NO_MAGIC | P_OUT_OF_MAP)) break;
+
+	    if ((mflags & P_BLOCKSVIEW) && 
+	      OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, sx, sy))) break;
 	}
 
 	if(dist<op->contr->count) {
@@ -1272,9 +1317,10 @@ int dimension_door(object *op,object *caster, object *spob, int dir) {
 	 * lots of other maps that protect areas with no magic, but the
 	 * areas themselves don't contain no magic spaces.
 	 */
-	if(get_map_flags(op->map, NULL, 
-	    op->x+freearr_x[dir]*dist, op->y+freearr_y[dir]*dist,
-	    NULL, NULL) & P_BLOCKED) {
+	/* This call here is really just to normalize the coordinates */
+	mflags = get_map_flags(op->map, &m,op->x+freearr_x[dir]*dist, op->y+freearr_y[dir]*dist,
+	    &sx, &sy);
+	if (mflags&P_IS_ALIVE || OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, sx, sy))) {
 		new_draw_info(NDI_UNIQUE, 0,op,"You cast your spell, but nothing happens.\n");
 		return 1; /* Maybe the penalty should be more severe... */
 	}
@@ -1285,26 +1331,29 @@ int dimension_door(object *op,object *caster, object *spob, int dir) {
 	 */
 
 	for(dist=0; dist < maxdist; dist++) {
-	    mflags = get_map_flags(op->map, NULL, 
+	    mflags = get_map_flags(op->map, &m, 
 		op->x+freearr_x[dir] * (dist+1), 
 	        op->y+freearr_y[dir] * (dist+1),
-	        NULL, NULL);
-	    if ((mflags & (P_NO_MAGIC | P_OUT_OF_MAP)) ||
-		((mflags & P_NO_PASS) && (mflags & P_BLOCKSVIEW))) {
-		break;
-	    }
+	        &sx, &sy);
+
+	    if (mflags & (P_NO_MAGIC | P_OUT_OF_MAP)) break;
+
+	    if ((mflags & P_BLOCKSVIEW) && 
+	      OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, sx, sy))) break;
+
 	}
 
 	/* If the destination is blocked, keep backing up until we
 	 * find a place for the player.
 	 */
-	for(;dist>0; dist--)
-	    if ((get_map_flags(op->map,NULL, 
-	      op->x+freearr_x[dir]*dist,
-	      op->y+freearr_y[dir]*dist,
-	      NULL, NULL) & (P_BLOCKED | P_OUT_OF_MAP)) ==0) 
-		break;
+	for(;dist>0; dist--) {
+	    if (get_map_flags(op->map, &m,op->x+freearr_x[dir]*dist, op->y+freearr_y[dir]*dist,
+			  &sx, &sy) & (P_OUT_OF_MAP|P_IS_ALIVE)) continue;
 
+
+	    if (!OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, sx, sy))) break;
+
+	}
 	if(!dist) {
 	    new_draw_info(NDI_UNIQUE, 0,op,"Your spell failed!\n");
 	    return 0;
@@ -1319,13 +1368,14 @@ int dimension_door(object *op,object *caster, object *spob, int dir) {
         return 1;
 
     if (op->type == PLAYER)
-	MapNewmapCmd(op->contr);
+        map_newmap_cmd(op->contr);
     op->speed_left= -FABS(op->speed)*5; /* Freeze them for a short while */
     return 1;
 }
 
 
-/* cast_heal: Heals something.
+/**
+ * Heals something.
  * op is the caster.
  * dir is the direction he is casting it in.
  * spell is the spell object.
@@ -1428,11 +1478,12 @@ int cast_heal(object *op,object *caster, object *spell, int dir) {
 }
 
 
-/* This is used for the spells that gain stats.  There are no spells
+/**
+ * This is used for the spells that gain stats.  There are no spells
  * right now that icnrease wis/int/pow on a temp basis, so no
  * good comments for those.
  */
-static char *no_gain_msgs[NUM_STATS] = {
+static const char* const no_gain_msgs[NUM_STATS] = {
 "You grow no stronger.",
 "You grow no more agile.",
 "You don't feel any healthier.",
@@ -1442,7 +1493,7 @@ static char *no_gain_msgs[NUM_STATS] = {
 "no pow"
 };
 
-int cast_change_ability(object *op,object *caster,object *spell_ob, int dir) {
+int cast_change_ability(object *op,object *caster,object *spell_ob, int dir, int silent) {
     object *tmp, *tmp2=NULL;
     object *force=NULL;
     int i;
@@ -1464,15 +1515,16 @@ int cast_change_ability(object *op,object *caster,object *spell_ob, int dir) {
 		break;
 	    }
 	    else if (spell_ob->race && spell_ob->race == tmp2->name) {
-		new_draw_info_format(NDI_UNIQUE, 0, op,
-		    "You can not cast %s while %s is in effect",
-		    spell_ob->name, tmp2->name_pl);
-		return 0;
+            if ( !silent )
+		        new_draw_info_format(NDI_UNIQUE, 0, op,
+		            "You can not cast %s while %s is in effect",
+		            spell_ob->name, tmp2->name_pl);
+		    return 0;
 	    }
 	}
     }
     if(force==NULL) {
-	force=get_archetype(FORCE_NAME);
+	force=create_archetype(FORCE_NAME);
 	force->subtype = FORCE_CHANGE_ABILITY;
 	free_string(force->name);
 	if (spell_ob->race) 
@@ -1528,9 +1580,8 @@ int cast_change_ability(object *op,object *caster,object *spell_ob, int dir) {
 	    }
 	}
     }
-  
-    if (QUERY_FLAG(spell_ob, FLAG_FLYING))
-	SET_FLAG(force, FLAG_FLYING);
+
+    force->move_type = spell_ob->move_type;
 
     if (QUERY_FLAG(spell_ob, FLAG_SEE_IN_DARK))
 	SET_FLAG(force, FLAG_SEE_IN_DARK);
@@ -1555,7 +1606,8 @@ int cast_change_ability(object *op,object *caster,object *spell_ob, int dir) {
     return 1;
 }
 
-/* This used to be part of cast_change_ability, but it really didn't make
+/**
+ * This used to be part of cast_change_ability, but it really didn't make
  * a lot of sense, since most of the values it derives are from the god
  * of the caster.
  */
@@ -1587,7 +1639,7 @@ int cast_bless(object *op,object *caster,object *spell_ob, int dir) {
 	}
     }
     if(force==NULL) {
-	force=get_archetype(FORCE_NAME);
+	force=create_archetype(FORCE_NAME);
 	force->subtype = FORCE_CHANGE_ABILITY;
 	free_string(force->name);
 	if (spell_ob->race) 
@@ -1674,6 +1726,7 @@ int cast_bless(object *op,object *caster,object *spell_ob, int dir) {
  * is finished.
  */
 static object *small, *large;
+static uint64 small_value, large_value;
 
 static void alchemy_object(object *obj, int *small_nuggets,
 	 int *large_nuggets, int *weight)
@@ -1698,10 +1751,10 @@ static void alchemy_object(object *obj, int *small_nuggets,
     if ((obj->value>0) && rndm(0, 29)) {
 	int count;
 
-	count = value / large->value;
+	count = value / large_value;
 	*large_nuggets += count;
-	value -= (uint64)count * (uint64)large->value;
-	count = value / small->value;
+	value -= (uint64)count * large_value;
+	count = value / small_value;
 	*small_nuggets += count;
     }
 
@@ -1709,10 +1762,10 @@ static void alchemy_object(object *obj, int *small_nuggets,
      * of large nuggets is not evenly divisable by the small nugget
      * value, take off an extra small_nugget (Assuming small_nuggets!=0)
      */
-    if (*small_nuggets * small->value >= large->value) {
+    if (*small_nuggets * small_value >= large_value) {
 	(*large_nuggets)++;
-	*small_nuggets -= large->value / small->value;
-	if (*small_nuggets && large->value % small->value)
+	*small_nuggets -= large_value / small_value;
+	if (*small_nuggets && large_value % small_value)
 		(*small_nuggets)--;
     }
     weight += obj->weight;
@@ -1765,8 +1818,10 @@ int alchemy(object *op, object *caster, object *spell_ob)
      */
     weight_max = spell_ob->duration + +SP_level_duration_adjust(caster,spell_ob);
     weight_max *= 1000;
-    small=get_archetype("smallnugget"),
-    large=get_archetype("largenugget");
+    small=create_archetype("smallnugget"),
+    large=create_archetype("largenugget");
+    small_value = query_cost(small, NULL, F_TRUE);
+    large_value = query_cost(large, NULL, F_TRUE);
 
     for(y= op->y-1;y<=op->y+1;y++) {
 	for(x= op->x-1;x<=op->x+1;x++) {
@@ -1777,7 +1832,14 @@ int alchemy(object *op, object *caster, object *spell_ob)
 
 	    mflags = get_map_flags(mp, &mp, nx, ny, &nx, &ny);
 
-	    if(mflags & (P_OUT_OF_MAP | P_WALL | P_NO_MAGIC))
+	    if(mflags & (P_OUT_OF_MAP | P_NO_MAGIC))
+		continue;
+
+	    /* Treat alchemy a little differently - most spell effects
+	     * use fly as the movement type - for alchemy, consider it
+	     * ground level effect.
+	     */
+	    if (GET_MAP_MOVE_BLOCK(mp, nx, ny) & MOVE_WALK)
 		continue;
 
 	    small_nuggets=0;
@@ -1828,7 +1890,8 @@ int alchemy(object *op, object *caster, object *spell_ob)
 }
 
 
-/* This function removes the cursed/damned status on equipped
+/**
+ * This function removes the cursed/damned status on equipped
  * items.
  */
 int remove_curse(object *op, object *caster, object *spell) {
@@ -1867,8 +1930,7 @@ int remove_curse(object *op, object *caster, object *spell) {
     return success;
 }
 
-/* Identifies objects in the players inventory/on the ground */
-
+/** Identifies objects in the players inventory/on the ground */
 int cast_identify(object *op, object *caster, object *spell) {
     object *tmp;
     int success = 0, num_ident;
@@ -2079,8 +2141,51 @@ int cast_detection(object *op, object *caster, object *spell, object *skill) {
 }
 
 
+/**
+ * Checks if victim has overcharged mana. caster_level is the caster's (skill)
+ * level whos spell did cause the overcharge.
+ */
+static void charge_mana_effect(object *victim, int caster_level)
+{
 
-/* cast_transfer
+    /* Prevent explosions for objects without mana. Without this check, doors
+     * will explode, too.
+     */
+    if (victim->stats.maxsp <= 0)
+        return;
+
+    new_draw_info(NDI_UNIQUE, 0, victim, "You feel energy course through you.");
+
+    if (victim->stats.sp >= victim->stats.maxsp*2) {
+        object *tmp;
+
+        new_draw_info(NDI_UNIQUE, 0, victim, "Your head explodes!");
+
+        /* Explodes a fireball centered at player */
+        tmp = create_archetype(EXPLODING_FIREBALL);
+        tmp->dam_modifier = random_roll(1, caster_level, victim, PREFER_LOW)/5+1;
+        tmp->stats.maxhp = random_roll(1, caster_level, victim, PREFER_LOW)/10+2;
+        tmp->x = victim->x;
+        tmp->y = victim->y;
+        insert_ob_in_map(tmp, victim->map, NULL, 0);
+        victim->stats.sp = 2*victim->stats.maxsp;
+    }
+    else if (victim->stats.sp >= victim->stats.maxsp*1.88) {
+        new_draw_info(NDI_UNIQUE, NDI_ORANGE, victim, "You feel like your head is going to explode.");
+    }
+    else if (victim->stats.sp >= victim->stats.maxsp*1.66) {
+        new_draw_info(NDI_UNIQUE, 0, victim, "You get a splitting headache!");
+    }
+    else if (victim->stats.sp >= victim->stats.maxsp*1.5) {
+        new_draw_info(NDI_UNIQUE, 0, victim, "Chaos fills your world.");
+        confuse_player(victim, victim, 99);
+    }
+    else if (victim->stats.sp >= victim->stats.maxsp*1.25) {
+        new_draw_info(NDI_UNIQUE, 0, victim, "You start hearing voices.");
+    }
+}
+
+/**
  * This spell transfers sp from the player to another person.
  * We let the target go above their normal maximum SP.
  */
@@ -2099,7 +2204,7 @@ int cast_transfer(object *op,object *caster, object *spell, int dir) {
 
     if (!(mflags & P_OUT_OF_MAP) && mflags & P_IS_ALIVE) {
 	for(plyr=get_map_ob(m, x, y); plyr!=NULL; plyr=plyr->above)
-	    if(QUERY_FLAG(plyr, FLAG_ALIVE))
+	    if (plyr != op && QUERY_FLAG(plyr, FLAG_ALIVE))
 		break;
     }
 
@@ -2109,7 +2214,7 @@ int cast_transfer(object *op,object *caster, object *spell, int dir) {
      */
     if(plyr==NULL)
         for(plyr=get_map_ob(op->map,op->x,op->y); plyr!=NULL; plyr=plyr->above)
-	    if(QUERY_FLAG(plyr,FLAG_ALIVE))
+	    if (plyr != op && QUERY_FLAG(plyr, FLAG_ALIVE))
 		break;
 
     if (!plyr) {
@@ -2119,49 +2224,26 @@ int cast_transfer(object *op,object *caster, object *spell, int dir) {
     /* give sp */
     if(spell->stats.dam > 0) {
 	plyr->stats.sp += spell->stats.dam + SP_level_dam_adjust(caster, spell);
-
-	new_draw_info(NDI_UNIQUE, 0,plyr,"You feel energy course through you.");
-
-	if(plyr->stats.sp>=plyr->stats.maxsp*2) {
-	    object *tmp;
-
-            new_draw_info(NDI_UNIQUE, 0,plyr,"Your head explodes!");
-
-	    /* Explodes a fireball centered at player */
-	    tmp = get_archetype(EXPLODING_FIREBALL);
-	    tmp->dam_modifier=random_roll(1, op->level, op, PREFER_LOW)/5+1;
-	    tmp->stats.maxhp=random_roll(1, op->level, op, PREFER_LOW)/10+2;
-	    tmp->x = plyr->x;
-	    tmp->y = plyr->y;
-	    insert_ob_in_map(tmp, plyr->map, NULL, 0);
-	    plyr->stats.sp = 2*plyr->stats.maxsp;
-        }
-        else if(plyr->stats.sp>=plyr->stats.maxsp*1.88)
-            new_draw_info(NDI_UNIQUE, NDI_ORANGE,plyr,"You feel like your head is going to explode.");
-        else if(plyr->stats.sp>=plyr->stats.maxsp*1.66)
-            new_draw_info(NDI_UNIQUE, 0,plyr, "You get a splitting headache!");
-        else if(plyr->stats.sp>=plyr->stats.maxsp*1.5) {
-            new_draw_info(NDI_UNIQUE, 0,plyr,"Chaos fills your world.");
-	    confuse_player(op,op,99);
-	}
-        else if(plyr->stats.sp>=plyr->stats.maxsp*1.25)
-            new_draw_info(NDI_UNIQUE, 0,plyr,"You start hearing voices.");
+	charge_mana_effect(plyr, caster_level(caster, spell));
         return 1;
     }
     /* suck sp away.  Can't suck sp from yourself */
     else if (op != plyr) {
 	/* old dragin magic used floats.  easier to just use ints and divide by 100 */
 
-	int rate = -spell->stats.dam + SP_level_dam_adjust(caster, spell), sucked=0;
+	int rate = -spell->stats.dam + SP_level_dam_adjust(caster, spell), sucked;
 
 	if (rate > 95) rate=95;
 
 	sucked = (plyr->stats.sp * rate) / 100;
 	plyr->stats.sp -= sucked;
-	/* Player doesn't get full credit */
 	if (QUERY_FLAG(op, FLAG_ALIVE)) {
+	    /* Player doesn't get full credit */
 	    sucked = (sucked * rate) / 100;
 	    op->stats.sp += sucked;
+	    if (sucked > 0) {
+		charge_mana_effect(op, caster_level(caster, spell));
+	    }
 	}
 	return 1;
     }
@@ -2169,7 +2251,8 @@ int cast_transfer(object *op,object *caster, object *spell, int dir) {
 }
 
 
-/* counterspell:  nullifies spell effects.
+/**
+ * Nullifies spell effects.
  * op is the counterspell object, dir is the direction
  * it was cast in.
  * Basically, if the object has a magic attacktype,
@@ -2238,7 +2321,7 @@ void counterspell(object *op,int dir)
 
 
 
-/* cast_consecrate() - a spell to make an altar your god's */
+/** A spell to make an altar your god's */
 int cast_consecrate(object *op, object *caster, object *spell) {
     char buf[MAX_BUF];
 
@@ -2276,7 +2359,7 @@ int cast_consecrate(object *op, object *caster, object *spell) {
     return 0;
 }
 
-/* animate_weapon - 
+/**
  * Generalization of staff_to_snake.  Makes a golem out of the caster's weapon.
  * The golem is based on the archetype specified, modified by the caster's level
  * and the attributes of the weapon.  The weapon is inserted in the golem's 
@@ -2318,7 +2401,8 @@ int animate_weapon(object *op,object *caster,object *spell, int dir) {
     y = op->y+freearr_y[dir];
 
     /* if there's no place to put the golem, abort */
-    if((dir==-1) || (get_map_flags(m, &m, x, y, &x, &y) & (P_BLOCKED | P_OUT_OF_MAP))) {
+    if((dir==-1) || (get_map_flags(m, &m, x, y, &x, &y) &  P_OUT_OF_MAP) ||
+      ((spell->other_arch->clone.move_type & GET_MAP_MOVE_BLOCK(m, x, y)) == spell->other_arch->clone.move_type)) {
 	new_draw_info(NDI_UNIQUE, 0,op,"There is something in the way.");
 	return 0;
     }
@@ -2472,9 +2556,7 @@ int animate_weapon(object *op,object *caster,object *spell, int dir) {
     return 1;
 }
 
-/* cast_daylight() - changes the map darkness level *lower* */
-
-/* cast_change_map_lightlevel: Was cast_daylight/nightfall.
+/**
  * This changes the light level for the entire map.
  */
 
@@ -2497,7 +2579,8 @@ int cast_change_map_lightlevel( object *op, object *caster, object *spell ) {
 
 
 
-/* create an aura spell object and put it in the player's inventory.
+/**
+ * Create an aura spell object and put it in the player's inventory.
  * as usual, op is player, caster is the object casting the spell,
  * spell is the spell object itself.
  */
@@ -2529,7 +2612,8 @@ int create_aura(object *op, object *caster, object *spell)
 }
 
 
-/* move aura function.  An aura is a part of someone's inventory,
+/**
+ * An aura is a part of someone's inventory,
  * which he carries with him, but which acts on the map immediately
  * around him.
  * Aura parameters:
@@ -2541,6 +2625,7 @@ int create_aura(object *op, object *caster, object *spell)
 void move_aura(object *aura) {
     int i, mflags;
     object *env;
+    mapstruct *m;
 
     /* auras belong in inventories */
     env = aura->env;
@@ -2573,16 +2658,22 @@ void move_aura(object *aura) {
 	sint16 nx, ny;
 	nx = aura->x + freearr_x[i];
 	ny = aura->y + freearr_y[i];
-	mflags = get_map_flags(env->map, NULL, nx, ny, NULL, NULL);
-	if ((mflags & (P_WALL | P_OUT_OF_MAP)) == 0) {
-	    hit_map(aura,i,aura->attacktype);
+	mflags = get_map_flags(env->map, &m, nx, ny, &nx, &ny);
+
+	/* Consider the movement tyep of the person with the aura as
+	 * movement type of the aura.  Eg, if the player is flying, the aura
+	 * is flying also, if player is walking, it is on the ground, etc.
+	 */
+	if (!(mflags & P_OUT_OF_MAP) && !(OB_TYPE_MOVE_BLOCK(env, GET_MAP_MOVE_BLOCK(m, nx, ny)))) {
+	    hit_map(aura,i,aura->attacktype,0);
+
 	    if(aura->other_arch) {
 		object *new_ob;
 
 		new_ob = arch_to_object(aura->other_arch);
 		new_ob->x = nx;
 		new_ob->y = ny;
-		insert_ob_in_map(new_ob,env->map,aura,0);
+		insert_ob_in_map(new_ob,m,aura,0);
 	    }
 	}
     }
@@ -2591,7 +2682,8 @@ void move_aura(object *aura) {
     insert_ob_in_ob(aura, env);
 }
 
-/* moves the peacemaker spell.
+/**
+ * moves the peacemaker spell.
  * op is the piece object.
  */      
 
@@ -2624,7 +2716,7 @@ void move_peacemaker(object *op) {
 	    victim->stats.grace = 0;
 	    victim->stats.Pow = 0;
 #endif
-	    victim->move_type = RANDO2;
+	    victim->attack_movement = RANDO2;
 	    SET_FLAG(victim,FLAG_UNAGGRESSIVE);
 	    SET_FLAG(victim,FLAG_RUN_AWAY);
 	    SET_FLAG(victim,FLAG_RANDOM_MOVE);
@@ -2637,11 +2729,12 @@ void move_peacemaker(object *op) {
 }   
       
 
-/* This writes a rune that contains the appropriate message.
+/**
+ * This writes a rune that contains the appropriate message.
  * There really isn't any adjustments we make.
  */
 
-int write_mark(object *op, object *spell, char *msg) {
+int write_mark(object *op, object *spell, const char *msg) {
     char rune[HUGE_BUF];
     object *tmp;
 
@@ -2667,4 +2760,3 @@ int write_mark(object *op, object *spell, char *msg) {
     insert_ob_in_map(tmp, op->map, op, INS_BELOW_ORIGINATOR);
     return 1;
 }
-
