@@ -1,13 +1,13 @@
 /*
  * static char *rcsid_spell_util_c =
- *   "$Id: spell_util.c,v 1.93 2005/07/15 08:02:00 ryo_saeba Exp $";
+ *   "$Id: spell_util.c,v 1.106 2006/06/05 06:28:56 mwedel Exp $";
  */
 
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2001 Mark Wedel & Crossfire Development Team
+    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -43,7 +43,7 @@ extern char *spell_mapping[];
  * the spell must be of this skill, it can be NULL in which case all
  * matching spells are used.
  */
-object *find_random_spell_in_ob(object *ob, char *skill)
+object *find_random_spell_in_ob(object *ob, const char *skill)
 {
     int k=0,s;
     object *tmp;
@@ -88,7 +88,7 @@ void set_spell_skill(object *op, object *caster, object *spob, object *dest)
  * all the spells to catch simple errors.  To use it all the time
  * will result in it spitting out messages that aren't really errors.
  */
-void init_spells() {
+void init_spells(void) {
 #ifdef SPELL_DEBUG
     static int init_spells_done = 0;
     int i;
@@ -126,7 +126,7 @@ void init_spells() {
  * not sure what this would be used for, as the data seems pretty
  * minimal, but easy enough to keep around.
  */
-void dump_spells()
+void dump_spells(void)
 {
     archetype *at;
 
@@ -167,7 +167,7 @@ void spell_effect (object *spob, int x, int y, mapstruct *map,
  * spell is the spell object.
  * Returns modified level.
  */
-int min_casting_level(object *caster, object *spell)
+static int min_casting_level(object *caster, object *spell)
 {
     int new_level;
 
@@ -233,7 +233,7 @@ int caster_level(object *caster, object *spell)
  * mana.  In that case, we return which ever value is higher.
  */
 
-int SP_level_spellpoint_cost(object *caster, object *spell, int flags)
+sint16 SP_level_spellpoint_cost(object *caster, object *spell, int flags)
 {
     int sp, grace, level = caster_level(caster, spell);
 
@@ -413,8 +413,12 @@ int reflwall(mapstruct *m,int x,int y, object *sp_op) {
  */
 int cast_create_obj(object *op,object *caster,object *new_op, int dir)
 {
+    mapstruct *m;
+    sint16  sx, sy;
+
     if(dir &&
-       (get_map_flags(op->map,NULL, op->x+freearr_x[dir],op->y+freearr_y[dir], NULL, NULL) & (P_BLOCKED | P_OUT_OF_MAP))) {
+      ((get_map_flags(op->map, &m, op->x+freearr_x[dir],op->y+freearr_y[dir], &sx, &sy) & P_OUT_OF_MAP) ||
+      OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, sx, sy)))) {
 	new_draw_info(NDI_UNIQUE, 0,op,"Something is in the way.");
 	new_draw_info(NDI_UNIQUE, 0,op,"You cast it at your feet.");
 	dir = 0;
@@ -447,7 +451,7 @@ int ok_to_put_more(mapstruct *m,sint16 x,sint16 y,object *op,int immune_stop) {
 
     if (mflags & P_OUT_OF_MAP) return 0;
 
-    if(mflags & P_WALL) return 0;
+    if (OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(mp, x, y))) return 0;
 
     for(tmp=get_map_ob(mp,x,y);tmp!=NULL;tmp=tmp->above) {
 	/* If there is a counterspell on the space, and this
@@ -509,16 +513,19 @@ int fire_arch_from_position (object *op, object *caster, sint16 x, sint16 y,
     if (mflags & P_OUT_OF_MAP) {
 	return 0;
     }
-    if (mflags & P_NO_PASS) {
-	new_draw_info(NDI_UNIQUE, 0, op, "You can't cast the spell on top of a wall!\n");
-	return 0;
-    }
 
     tmp=arch_to_object(spell->other_arch);
 
     if(tmp==NULL)
 	return 0;
 
+    if (OB_TYPE_MOVE_BLOCK(tmp, GET_MAP_MOVE_BLOCK(m, x, y))) {
+	new_draw_info(NDI_UNIQUE, 0, op, "You can't cast the spell on top of a wall!\n");
+	free_object(tmp);
+	return 0;
+    }
+
+
 
     tmp->stats.dam=spell->stats.dam+SP_level_dam_adjust(caster,spell);
     tmp->duration=spell->duration+SP_level_duration_adjust(caster,spell);
@@ -677,7 +684,7 @@ int spell_find_dir(mapstruct *m, int x, int y, object *exclude) {
  * also deals with multipart monsters properly.
  */
 
-void put_a_monster(object *op,char *monstername) {
+static void put_a_monster(object *op,const char *monstername) {
     object *tmp,*head=NULL,*prev=NULL;
     archetype *at;
     int dir;
@@ -690,7 +697,7 @@ void put_a_monster(object *op,char *monstername) {
      * first we check the closest square for free squares 
      */
 
-    dir=find_first_free_spot(at,op->map,op->x,op->y);
+    dir=find_first_free_spot(&at->clone,op->map,op->x,op->y);
     if(dir!=-1) {
 	/* This is basically grabbed for generate monster.  Fixed 971225 to
 	 * insert multipart monsters properly
@@ -715,7 +722,7 @@ void put_a_monster(object *op,char *monstername) {
 	insert_ob_in_map(head,op->map,op,0);
 
 	/* thought it'd be cool to insert a burnout, too.*/
-	tmp=get_archetype("burnout");
+	tmp=create_archetype("burnout");
 	tmp->map = op->map;
 	tmp->x=op->x+freearr_x[dir];
 	tmp->y=op->y+freearr_y[dir];
@@ -737,7 +744,7 @@ void put_a_monster(object *op,char *monstername) {
  * players part (alchemy, reincarnation, etc)
  */
 
-int summon_hostile_monsters(object *op,int n,char *monstername){
+int summon_hostile_monsters(object *op,int n,const char *monstername){
     int i;
     for(i=0;i<n;i++)
 	put_a_monster(op,monstername);
@@ -812,9 +819,9 @@ void shuffle_attack(object *op,int change_face)
  * power is how much grace the spell would normally take to cast.
  */
 
-void prayer_failure(object *op, int failure,int power)
+static void prayer_failure(object *op, int failure, int power)
 {  
-    char *godname;
+    const char *godname;
     object *tmp;
 
     if(!strcmp((godname=determine_god(op)),"none")) godname="Your spirit";
@@ -822,7 +829,7 @@ void prayer_failure(object *op, int failure,int power)
     if(failure<= -20 && failure > -40) /* wonder */
     {
 	new_draw_info_format(NDI_UNIQUE, 0,op,"%s gives a sign to renew your faith.",godname);
-	tmp = get_archetype(SPELL_WONDER);
+	tmp = create_archetype(SPELL_WONDER);
 	cast_cone(op,op,0,tmp);
 	free_object(tmp);
     }
@@ -840,7 +847,7 @@ void prayer_failure(object *op, int failure,int power)
     }
     else if (failure <= -150) /* blast the immediate area */
     { 
-	tmp = get_archetype(GOD_POWER);
+	tmp = create_archetype(GOD_POWER);
 	new_draw_info_format(NDI_UNIQUE, 0,op,"%s smites you!",godname);
 	cast_magic_storm(op,tmp, power);
     }
@@ -865,7 +872,7 @@ void spell_failure(object *op, int failure,int power, object *skill)
     if (failure<=-20 && failure > -40) /* wonder */
     {
 	new_draw_info(NDI_UNIQUE, 0,op,"Your spell causes an unexpected effect.");
-	tmp = get_archetype(SPELL_WONDER);
+	tmp = create_archetype(SPELL_WONDER);
 	cast_cone(op,op,0,tmp);
 	free_object(tmp);
     }
@@ -886,11 +893,11 @@ void spell_failure(object *op, int failure,int power, object *skill)
 	/* Safety check to make sure we don't get any mana storms in scorn */
 	if (get_map_flags(op->map, NULL, op->x, op->y, NULL, NULL) & P_NO_MAGIC) {
 	    new_draw_info(NDI_UNIQUE, 0, op, "The magic warps and you are turned inside out!");
-	    hit_player(op,9998,op,AT_INTERNAL);
+	    hit_player(op,9998,op,AT_INTERNAL,1);
 
 	} else {
 	    new_draw_info(NDI_UNIQUE, 0,op,"You lose control of the mana!  The uncontrolled magic blasts you!");
-	    tmp=get_archetype(LOOSE_MANA);
+	    tmp=create_archetype(LOOSE_MANA);
 	    tmp->level=skill->level;
 	    tmp->x=op->x;
 	    tmp->y=op->y;
@@ -907,7 +914,7 @@ void spell_failure(object *op, int failure,int power, object *skill)
     }
 }
 
-int cast_party_spell(object *op, object *caster,int dir,object *spell_ob, char *stringarg)
+static int cast_party_spell(object *op, object *caster,int dir,object *spell_ob, char *stringarg)
     {
     int success;
     player *pl;
@@ -923,13 +930,13 @@ int cast_party_spell(object *op, object *caster,int dir,object *spell_ob, char *
     /* Always cast spell on caster */
     success = cast_spell( op, caster, dir, spell, stringarg );
 
-    if ( caster->contr->party_number == -1 )
+    if ( caster->contr->party == NULL )
         {
         remove_ob( spell );
         return success;
         }
     for( pl=first_player; pl!=NULL; pl=pl->next )
-        if( ( pl->ob->contr->party_number == caster->contr->party_number ) && ( on_same_map( pl->ob, caster ) ) )
+        if( ( pl->ob->contr->party == caster->contr->party ) && ( on_same_map( pl->ob, caster ) ) )
             {
             cast_spell( pl->ob, caster, pl->ob->facing, spell, stringarg );
             }
@@ -967,11 +974,11 @@ int cast_party_spell(object *op, object *caster,int dir,object *spell_ob, char *
 
 int cast_spell(object *op, object *caster,int dir,object *spell_ob, char *stringarg) {
 
-    char *godname;
+    const char *godname;
     int success=0,mflags, cast_level=0, old_shoottype;
     object *skill=NULL;
 
-    if (op->contr) old_shoottype = op->contr->shoottype;
+    old_shoottype = op->contr ? op->contr->shoottype : 0;
 
     if (!spell_ob) {
 	LOG(llevError,"cast_spell: null spell object passed\n");
@@ -1075,7 +1082,7 @@ int cast_spell(object *op, object *caster,int dir,object *spell_ob, char *string
      * objects on the space - presume that they know what they are
      * doing.
      */
-    if (spell_ob->type == SPELL && caster->type != POTION &&
+    if (spell_ob->type == SPELL && caster->type != POTION && !QUERY_FLAG(op, FLAG_WIZCAST) &&
 	(QUERY_FLAG(caster, FLAG_ALIVE) || QUERY_FLAG(op, FLAG_ALIVE)) &&
 	(((mflags & P_NO_MAGIC) && spell_ob->stats.sp) ||
 	   ((mflags & P_NO_CLERIC) && spell_ob->stats.grace))) {
@@ -1271,7 +1278,7 @@ int cast_spell(object *op, object *caster,int dir,object *spell_ob, char *string
 	    break;
 
 	case SP_CHANGE_ABILITY:
-	    success = cast_change_ability(op,caster,spell_ob, dir);
+	    success = cast_change_ability(op,caster,spell_ob, dir, 0);
 	    break;
 
 	case SP_BLESS:
@@ -1283,7 +1290,7 @@ int cast_spell(object *op, object *caster,int dir,object *spell_ob, char *string
 	    break;
 
 	case SP_SUMMON_MONSTER:
-	    success = summon_object(op,caster,spell_ob, dir);
+	    success = summon_object(op,caster,spell_ob, dir,stringarg);
 	    break;
 
 	case SP_CHARGING:
@@ -1462,10 +1469,15 @@ void move_spell_effect(object *op) {
     }
 }
 
+#if 0
+/* check_spell_effect is no longer used - should perhaps be removed.
+ * MSW 2006-06-02
+ */
+
 /* this checks to see if something special should happen if 
  * something runs into the object.
  */
-void check_spell_effect(object *op) {
+static void check_spell_effect(object *op) {
 
     switch (op->subtype) {
 	case SP_BOLT:
@@ -1478,6 +1490,8 @@ void check_spell_effect(object *op) {
     }
 
 }
+#endif
+
 
 /* This is called by move_apply.  Basically, if someone
  * moves onto a spell effect and the walk_on or fly_on flags
@@ -1489,13 +1503,13 @@ void apply_spell_effect(object *spell, object *victim)
     switch (spell->subtype) {
 	case SP_CONE:
 	    if (QUERY_FLAG(victim, FLAG_ALIVE) && spell->speed && spell->attacktype)
-		hit_player(victim, spell->stats.dam, spell, spell->attacktype);
+		hit_player(victim, spell->stats.dam, spell, spell->attacktype, 0);
 	    break;
 
 	case SP_MAGIC_MISSILE:
 	    if (QUERY_FLAG (victim, FLAG_ALIVE)) {
 		tag_t spell_tag = spell->count;
-		hit_player (victim, spell->stats.dam, spell, spell->attacktype);
+		hit_player (victim, spell->stats.dam, spell, spell->attacktype, 1);
 		if ( ! was_destroyed (spell, spell_tag)) {
 		    remove_ob (spell);
 		    free_object (spell);
@@ -1505,10 +1519,9 @@ void apply_spell_effect(object *spell, object *victim)
 
 	case SP_MOVING_BALL:
 	    if (QUERY_FLAG (victim, FLAG_ALIVE))
-		hit_player (victim, spell->stats.dam, spell, spell->attacktype);
+		hit_player (victim, spell->stats.dam, spell, spell->attacktype, 1);
 	    else if (victim->material || victim->materialname)
 		save_throw_object (victim, spell->attacktype, spell);
 	    break;
     }
 }
-
