@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_swap_c =
- *    "$Id: swap.c,v 1.14 2003/03/03 05:13:08 mwedel Exp $";
+ *    "$Id: swap.c,v 1.18 2006/05/05 09:26:36 ryo_saeba Exp $";
  */
 
 /*
@@ -35,7 +35,7 @@
 /* This writes out information on all the temporary maps.  It is called by
  * swap_map below.
  */
-static void write_map_log() 
+static void write_map_log(void) 
 {
     FILE *fp;
     mapstruct *map;
@@ -67,7 +67,7 @@ static void write_map_log()
     fclose(fp);
 }
 
-void read_map_log() 
+void read_map_log(void) 
 {
     FILE *fp;
     mapstruct *map;
@@ -125,8 +125,6 @@ void read_map_log()
 
 void swap_map(mapstruct *map) {
     player *pl;
-    int evtid;
-    CFParm CFP;
 
     if(map->in_memory != MAP_IN_MEMORY) {
 	LOG(llevError,"Tried to swap out map which was not in memory.\n");
@@ -153,11 +151,8 @@ void swap_map(mapstruct *map) {
 	mapstruct *oldmap = map;
 
 	LOG(llevDebug,"Resetting map %s.\n",map->path);
-	/* GROS : Here we handle the MAPRESET global event */
-	evtid = EVENT_MAPRESET;
-	CFP.Value[0] = (void *)(&evtid);
-	CFP.Value[1] = (void *)(map->path);
-	GlobalEvent(&CFP);
+        /* Lauwenmark : Here we handle the MAPRESET global event */
+        execute_global_event(EVENT_MAPRESET, map->path);
 	map = map->next;
 	delete_map(oldmap);
 	return;
@@ -178,7 +173,7 @@ void swap_map(mapstruct *map) {
 	write_map_log();
 }
 
-void check_active_maps() {
+void check_active_maps(void) {
     mapstruct *map, *next;
 
     for(map=first_map;map!=NULL;map=next) {
@@ -200,7 +195,7 @@ void check_active_maps() {
  * map_least_timeout() returns the map with the lowest timeout variable (not 0)
  */
 
-mapstruct *map_least_timeout(char *except_level) {
+static mapstruct *map_least_timeout(char *except_level) {
   mapstruct *map, *chosen=NULL;
   int timeout = MAP_MAXTIMEOUT + 1;
   for(map = first_map;map != NULL; map = map->next)
@@ -260,12 +255,10 @@ int players_on_map(mapstruct *m, int show_all) {
  * they are visited.
  * This is very useful if the tmp-disk is very full.
  */
-void flush_old_maps() {
+void flush_old_maps(void) {
 
     mapstruct *m, *oldmap;
     long sec;
-    int evtid;
-    CFParm CFP;
     sec = seconds();
 
     m= first_map;
@@ -288,7 +281,7 @@ void flush_old_maps() {
 	 * the functions calling it may not expect the map list to change
 	 * underneath them.
 	 */
-	if (m->unique && m->in_memory == MAP_SWAPPED) {
+	if ((m->unique || m->template) && m->in_memory == MAP_SWAPPED) {
 	    LOG(llevDebug,"Resetting map %s.\n",m->path);
 	    oldmap = m;
 	    m = m->next;
@@ -300,11 +293,8 @@ void flush_old_maps() {
 	}
 	else {
 	    LOG(llevDebug,"Resetting map %s.\n",m->path);
-	    /* GROS : Here we handle the MAPRESET global event */
-	    evtid = EVENT_MAPRESET;
-	    CFP.Value[0] = (void *)(&evtid);
-	    CFP.Value[1] = (void *)(m->path);
-	    GlobalEvent(&CFP);
+            /* Lauwenmark : Here we handle the MAPRESET global event */
+            execute_global_event(EVENT_MAPRESET, m->path);
 	clean_tmp_map(m);
 	oldmap = m;
 	m = m->next;
