@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_time_c =
- *    "$Id: time.c,v 1.71 2005/07/19 20:59:35 ryo_saeba Exp $";
+ *    "$Id: time.c,v 1.89 2006/05/05 09:26:36 ryo_saeba Exp $";
  */
 
 /*
@@ -37,7 +37,7 @@
 #include <sproto.h>
 #endif
 
-/* The following removes doors.  The functions check to see if similar 
+/** The following removes doors.  The functions check to see if similar 
  * doors are next to the one that is being removed, and if so, set it
  * so those will be removed shortly (in a cascade like fashion.)
  */
@@ -62,6 +62,9 @@ void remove_door(object *op) {
   free_object(op);
 }
 
+/**
+ * Same as remove_door but for locked doors.
+ */
 void remove_door2(object *op) {
   int i;
   object *tmp;
@@ -83,10 +86,10 @@ void remove_door2(object *op) {
   free_object(op);
 }
 
-/* Will generate a monster according to content
+/** Will generate a monster according to content
  * of generator.
  */
-void generate_monster_inv(object *gen) {
+static void generate_monster_inv(object *gen) {
     int i;
     object *op,*head=NULL;
 
@@ -109,10 +112,10 @@ void generate_monster_inv(object *gen) {
     qty=rndm(0,qty-1);
     for (op=gen->inv;qty;qty--)
         op=op->below;
-    i=find_free_spot(op->arch,gen->map,gen->x,gen->y,1,9);
+    i=find_free_spot(op,gen->map,gen->x,gen->y,1,9);
     if (i==-1)
         return;
-    head=ObjectCreateClone(op);
+    head=object_create_clone(op);
     CLEAR_FLAG(head, FLAG_IS_A_TEMPLATE);
     unflag_inv (head,FLAG_IS_A_TEMPLATE);
     if (rndm(0, 9))
@@ -124,7 +127,7 @@ void generate_monster_inv(object *gen) {
                       gen->map->difficulty,0);
 }
 
-void generate_monster_arch(object *gen) {
+static void generate_monster_arch(object *gen) {
     int i;
     object *op,*head=NULL,*prev=NULL;
     archetype *at=gen->other_arch;
@@ -141,7 +144,7 @@ void generate_monster_arch(object *gen) {
 	LOG(llevError,"Generator (%s) not on a map?\n", gen->name);
 	return;
     }
-    i=find_free_spot(at,gen->map,gen->x,gen->y,1,9);
+    i=find_free_spot(&at->clone,gen->map,gen->x,gen->y,1,9);
     if (i==-1) return;
     while(at!=NULL) {
 	op=arch_to_object(at);
@@ -164,7 +167,7 @@ void generate_monster_arch(object *gen) {
     }
 }
 
-void generate_monster(object *gen) {
+static void generate_monster(object *gen) {
 
     if(GENERATE_SPEED(gen)&&rndm(0, GENERATE_SPEED(gen)-1))
         return;
@@ -175,7 +178,7 @@ void generate_monster(object *gen) {
 
 }
 
-void remove_force(object *op) {
+static void remove_force(object *op) {
     if (--op->duration > 0) return;
 
     switch (op->subtype) {
@@ -196,7 +199,7 @@ void remove_force(object *op) {
     free_object(op);
 }
 
-void remove_blindness(object *op) {
+static void remove_blindness(object *op) {
   if(--op->stats.food > 0)
     return;
   CLEAR_FLAG(op, FLAG_APPLIED);
@@ -208,7 +211,7 @@ void remove_blindness(object *op) {
   free_object(op);
 }
 
-void poison_more(object *op) {
+static void poison_more(object *op) {
   if(op->env==NULL||!QUERY_FLAG(op->env,FLAG_ALIVE)||op->env->stats.hp<0) {
     remove_ob(op);
     free_object(op);
@@ -233,11 +236,11 @@ void poison_more(object *op) {
   }
   (void)hit_player(op->env,
                    op->stats.dam,
-                   op,AT_INTERNAL);
+                   op,AT_INTERNAL,1);
 }
 
 
-void move_gate(object *op) { /* 1 = going down, 0 = goind up */
+static void move_gate(object *op) { /* 1 = going down, 0 = goind up */
     object *tmp;
 
     if(op->stats.wc < 0 || (int)op->stats.wc  >= NUM_ANIMATIONS(op)) {
@@ -260,7 +263,7 @@ void move_gate(object *op) { /* 1 = going down, 0 = goind up */
 	    }
 	}
 	if((int)op->stats.wc < (NUM_ANIMATIONS(op)/2+1)) {
-	    CLEAR_FLAG(op, FLAG_NO_PASS);
+	    op->move_block = 0;
 	    CLEAR_FLAG(op, FLAG_BLOCKSVIEW);
 	    update_all_los(op->map, op->x, op->y);
 	}
@@ -318,7 +321,7 @@ void move_gate(object *op) { /* 1 = going down, 0 = goind up */
 
 	    if(tmp!=NULL) {
 		if(QUERY_FLAG(tmp, FLAG_ALIVE)) {
-		    hit_player(tmp, random_roll(1, op->stats.dam, tmp, PREFER_LOW), op, AT_PHYSICAL);
+		    hit_player(tmp, random_roll(1, op->stats.dam, tmp, PREFER_LOW), op, AT_PHYSICAL, 1);
 		    if(tmp->type==PLAYER) 
 			new_draw_info_format(NDI_UNIQUE, 0, tmp,
 					     "You are crushed by the %s!",op->name);
@@ -331,7 +334,7 @@ void move_gate(object *op) { /* 1 = going down, 0 = goind up */
 			&& (!QUERY_FLAG(tmp, FLAG_NO_PICK)
 			   ||QUERY_FLAG(tmp,FLAG_CAN_ROLL))) {
 		    /* If it has speed, it should move itself, otherwise: */
-		    int i=find_free_spot(tmp->arch,op->map,op->x,op->y,1,9);
+		    int i=find_free_spot(tmp,op->map,op->x,op->y,1,9);
 
 		    /* If there is a free spot, move the object someplace */
 		    if (i!=-1) {
@@ -353,7 +356,7 @@ void move_gate(object *op) { /* 1 = going down, 0 = goind up */
 	    if(tmp) {
 		    op->stats.food=1;
 	    } else {
-		SET_FLAG(op, FLAG_NO_PASS);    /* The coast is clear, block the way */
+		op->move_block = MOVE_ALL;
 		if(!op->arch->clone.stats.ac)
 		    SET_FLAG(op, FLAG_BLOCKSVIEW);
 		update_all_los(op->map, op->x, op->y);
@@ -365,11 +368,11 @@ void move_gate(object *op) { /* 1 = going down, 0 = goind up */
     } /* gate is going up */
 }
 
-/*  hp      : how long door is open/closed
+/**  hp      : how long door is open/closed
  *  maxhp   : initial value for hp
  *  sp      : 1 = open, 0 = close
  */
-void move_timed_gate(object *op)
+static void move_timed_gate(object *op)
 {
   int v = op->value;
 
@@ -388,14 +391,13 @@ void move_timed_gate(object *op)
   }
 }
 
-/*  slaying:    name of the thing the detector is to look for
+/**  slaying:    name of the thing the detector is to look for
  *	 speed:      frequency of 'glances'
  *	 connected:  connected value of detector
  *  sp:         1 if detection sets buttons
  *              -1 if detection unsets buttons
  */
-
-void move_detector(object *op) 
+static void move_detector(object *op) 
 {
     object *tmp;
     int last = op->value;
@@ -441,7 +443,7 @@ void move_detector(object *op)
 }
 
 
-void animate_trigger (object *op)
+static void animate_trigger(object *op)
 {
   if((unsigned char)++op->stats.wc >= NUM_ANIMATIONS(op)) {
     op->stats.wc = 0;
@@ -452,7 +454,7 @@ void animate_trigger (object *op)
   }
 }
 
-void move_hole(object *op) { /* 1 = opening, 0 = closing */
+static void move_hole(object *op) { /* 1 = opening, 0 = closing */
     object *next,*tmp;
 
     if(op->value) { /* We're opening */
@@ -460,7 +462,9 @@ void move_hole(object *op) { /* 1 = opening, 0 = closing */
 	    op->stats.wc=0;
 	    op->speed = 0;
 	    update_ob_speed(op);
-	    SET_FLAG(op, FLAG_WALK_ON);
+
+	    /* Hard coding this makes sense for holes I suppose */
+	    op->move_on = MOVE_WALK;
 	    for (tmp=op->above; tmp!=NULL; tmp=next) {
 		next=tmp->above;
 		move_apply(op,tmp,tmp);
@@ -471,7 +475,8 @@ void move_hole(object *op) { /* 1 = opening, 0 = closing */
 	return;
     }
     /* We're closing */
-    CLEAR_FLAG(op, FLAG_WALK_ON);
+    op->move_on = 0;
+
     op->stats.wc++;
     if((int)op->stats.wc >= NUM_ANIMATIONS(op))
 	op->stats.wc=NUM_ANIMATIONS(op)-1;
@@ -485,7 +490,7 @@ void move_hole(object *op) { /* 1 = opening, 0 = closing */
 }
 
 
-/* stop_item() returns a pointer to the stopped object.  The stopped object
+/** stop_item() returns a pointer to the stopped object.  The stopped object
  * may or may not have been removed from maps or inventories.  It will not
  * have been merged with other items.
  *
@@ -525,7 +530,7 @@ object *stop_item (object *op)
     }
 }
 
-/* fix_stopped_item() - put stopped item where stop_item() had found it.
+/** fix_stopped_item() - put stopped item where stop_item() had found it.
  * Inserts item into the old map, or merges it if it already is on the map.
  *
  * 'map' must be the value of op->map before stop_item() was called.
@@ -551,9 +556,8 @@ object *fix_stopped_arrow (object *op)
     }
 
     op->direction=0;
-    CLEAR_FLAG(op, FLAG_WALK_ON);
-    CLEAR_FLAG(op, FLAG_FLY_ON);
-    CLEAR_FLAG(op, FLAG_FLYING);
+    op->move_on=0;
+    op->move_type=0;
     op->speed = 0;
     update_ob_speed(op);
     op->stats.wc = op->stats.sp;
@@ -572,7 +576,7 @@ object *fix_stopped_arrow (object *op)
     } else
 	op->slaying = NULL;
 
-    /* Reset these to zero, so that CAN_MERGE will work properly */
+    /* Reset these to zero, so that can_merge will work properly */
     op->spellarg = NULL;
     op->stats.sp = 0;
     op->stats.hp = 0;
@@ -584,7 +588,7 @@ object *fix_stopped_arrow (object *op)
     return op;
 }
 
-/* stop_arrow() - what to do when a non-living flying object
+/** stop_arrow() - what to do when a non-living flying object
  * has to stop. Sept 96 - I added in thrown object code in
  * here too. -b.t.
  *
@@ -594,31 +598,8 @@ object *fix_stopped_arrow (object *op)
 
 static void stop_arrow (object *op)
 {
-    event *evt;
-    /* GROS: Handle for plugin stop event */
-    if ((evt = find_event(op, EVENT_STOP)) != NULL)
-    {
-        CFParm CFP;
-        int k, l, m;
-        uint32 n;
-        k = EVENT_STOP;
-        l = SCRIPT_FIX_NOTHING;
-        m = 0;
-        n = 0;
-        CFP.Value[0] = &k;
-        CFP.Value[1] = NULL;
-        CFP.Value[2] = op;
-        CFP.Value[3] = NULL;
-        CFP.Value[4] = NULL;
-        CFP.Value[5] = &n;
-        CFP.Value[6] = &m;
-        CFP.Value[7] = &m;
-        CFP.Value[8] = &l;
-        CFP.Value[9] = evt->hook;
-        CFP.Value[10]= evt->options;
-        if (findPlugin(evt->plugin)>=0)
-            ((PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP));
-    }
+    /* Lauwenmark: Handle for plugin stop event */
+    execute_event(op, EVENT_STOP,NULL,NULL,NULL,SCRIPT_FIX_NOTHING);
     if (op->inv) {
 	object *payload = op->inv;
 	remove_ob (payload);
@@ -633,7 +614,7 @@ static void stop_arrow (object *op)
     }
 }
 
-/* Move an arrow along its course.  op is the arrow or thrown object.
+/** Move an arrow along its course.  op is the arrow or thrown object.
  */
 
 void move_arrow(object *op) {
@@ -732,7 +713,9 @@ void move_arrow(object *op) {
     } /* if there is something alive on this space */
 
 
-    if (mflags & P_WALL) {
+    if (OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, new_x, new_y))) {
+	int retry=0;
+
 	/* if the object doesn't reflect, stop the arrow from moving
 	 * note that this code will now catch cases where a monster is
 	 * on a wall but has reflecting - the arrow won't reflect.
@@ -742,22 +725,35 @@ void move_arrow(object *op) {
 	if(!QUERY_FLAG(op, FLAG_REFLECTING) || !(rndm(0, 19))) {
 	    stop_arrow (op);
 	    return;
-	} else {    /* object is reflected */
+	} else {
 	    /* If one of the major directions (n,s,e,w), just reverse it */
 	    if(op->direction&1) {
 		op->direction=absdir(op->direction+4);
-	    } else {
-		/* The below is just logic for figuring out what direction
-		 * the object should now take.
+		retry=1;
+	    }
+	    /* There were two blocks with identical code -
+	     * use this retry here to make this one block
+	     * that did the same thing.
+	     */
+	    while (retry<2) {
+		int left, right, mflags;
+		mapstruct *m1;
+		sint16	x1, y1;
+
+		retry++;
+
+		/* Need to check for P_OUT_OF_MAP: if the arrow is tavelling
+		 * over a corner in a tiled map, it is possible that
+		 * op->direction is within an adjacent map but either
+		 * op->direction-1 or op->direction+1 does not exist.
 		 */
-	
-		int left, right;
+		mflags = get_map_flags(op->map,&m1, op->x+freearr_x[absdir(op->direction-1)],
+		       op->y+freearr_y[absdir(op->direction-1)], &x1, &y1);
+		left = (mflags & P_OUT_OF_MAP) ? 0 : OB_TYPE_MOVE_BLOCK(op, (GET_MAP_MOVE_BLOCK(m1, x1, y1)));
 
-
-		left= get_map_flags(op->map,NULL, op->x+freearr_x[absdir(op->direction-1)],
-		       op->y+freearr_y[absdir(op->direction-1)], NULL, NULL) & P_WALL;
-		right=get_map_flags(op->map,NULL, op->x+freearr_x[absdir(op->direction+1)],
-		   op->y+freearr_y[absdir(op->direction+1)], NULL, NULL) & P_WALL;
+		mflags = get_map_flags(op->map,&m1, op->x+freearr_x[absdir(op->direction+1)],
+		   op->y+freearr_y[absdir(op->direction+1)], &x1, &y1);
+		right = (mflags & P_OUT_OF_MAP) ? 0 : OB_TYPE_MOVE_BLOCK(op, (GET_MAP_MOVE_BLOCK(m1, x1, y1)));
 
 		if(left==right)
 		    op->direction=absdir(op->direction+4);
@@ -765,24 +761,22 @@ void move_arrow(object *op) {
 		    op->direction=absdir(op->direction+2);
 		else if(right)
 		    op->direction=absdir(op->direction-2);
+
+		mflags = get_map_flags(op->map,&m1, op->x+DIRX(op),op->y+DIRY(op), &x1, &y1);
+
+		/* If this space is not out of the map and not blocked, valid space -
+		 * don't need to retry again.
+		 */
+		if (!(mflags & P_OUT_OF_MAP) && 
+		  !OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m1, x1, y1))) break;
+
 	    }
-	    /* Is the new direction also a wall?  If show, shuffle again */
-	    if(get_map_flags(op->map,NULL, op->x+DIRX(op),op->y+DIRY(op), NULL, NULL) & P_WALL) {
-		int left, right;
-
-		left= get_map_flags(op->map,NULL, op->x+freearr_x[absdir(op->direction-1)],
-			 op->y+freearr_y[absdir(op->direction-1)], NULL, NULL) & P_WALL;
-		right = get_map_flags(op->map,NULL, op->x+freearr_x[absdir(op->direction+1)],
-		     op->y+freearr_y[absdir(op->direction+1)], NULL, NULL) & P_WALL;
-
-		if(!left)
-		    op->direction=absdir(op->direction-1);
-		else if(!right)
-		    op->direction=absdir(op->direction+1);
-		else {		/* is this possible? */
-		    stop_arrow (op);
-		    return;
-		}
+	    /* Couldn't find a direction to move the arrow to - just
+	     * top it from moving.
+	     */
+	    if (retry==2) {
+		stop_arrow (op);
+		return;
 	    }
 	    /* update object image for new facing */
 	    /* many thrown objects *don't* have more than one face */
@@ -803,11 +797,11 @@ void move_arrow(object *op) {
     insert_ob_in_map (op, m, op,0);
 }
 
-/* This routine doesnt seem to work for "inanimate" objects that
+/** This routine doesnt seem to work for "inanimate" objects that
  * are being carried, ie a held torch leaps from your hands!. 
  * Modified this routine to allow held objects. b.t. */
 
-void change_object(object *op) { /* Doesn`t handle linked objs yet */
+static void change_object(object *op) { /* Doesn`t handle linked objs yet */
   object *tmp,*env,*pl;
   int i,j;
 
@@ -840,7 +834,7 @@ void change_object(object *op) { /* Doesn`t handle linked objs yet */
 	    esrv_send_item(pl, tmp);
 	}
     } else {
-        j=find_first_free_spot(tmp->arch,op->map,op->x,op->y);
+        j=find_first_free_spot(tmp,op->map,op->x,op->y);
 	if (j==-1)  /* No free spot */
 	    free_object(tmp);
 	else {
@@ -854,7 +848,6 @@ void change_object(object *op) { /* Doesn`t handle linked objs yet */
 
 void move_teleporter(object *op) {
     object *tmp, *head=op;
-    event *evt;
 
     /* if this is a multipart teleporter, handle the other parts
      * The check for speed isn't strictly needed - basically, if
@@ -871,39 +864,14 @@ void move_teleporter(object *op) {
 	if (!QUERY_FLAG(tmp, FLAG_IS_FLOOR)) break;
 
     /* If nothing above us to move, nothing to do */
-    if (!tmp) return;
+    if (!tmp || QUERY_FLAG(tmp, FLAG_WIZPASS)) return;
 
     if(EXIT_PATH(head)) {
-	if(op->above->type==PLAYER) {
-	    /* GROS: Handle for plugin TRIGGER event */
-	    if ((evt = find_event(op, EVENT_TRIGGER)) != NULL) {
-		CFParm CFP;
-		CFParm* CFR;
-		int k, l, m;
-		uint32 n;
-		int rtn_script = 0;
-		m = 0;
-		n = 0;
-		k = EVENT_TRIGGER;
-		l = SCRIPT_FIX_ALL;
-		CFP.Value[0] = &k;
-		CFP.Value[1] = op;
-		CFP.Value[2] = op->above;
-		CFP.Value[3] = NULL;
-		CFP.Value[4] = NULL;
-		CFP.Value[5] = &n;
-		CFP.Value[6] = &m;
-		CFP.Value[7] = &m;
-		CFP.Value[8] = &l;
-		CFP.Value[9] = evt->hook;
-		CFP.Value[10]= evt->options;
-		if (findPlugin(evt->plugin)>=0) {
-		    CFR = (PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP);
-		    rtn_script = *(int *)(CFR->Value[0]);
-		}
-		if (rtn_script!=0) return;
-	    }
-	    enter_exit(op->above, head);
+	if(tmp->type==PLAYER) {
+            /* Lauwenmark: Handle for plugin TRIGGER event */
+            if (execute_event(op, EVENT_TRIGGER, tmp, NULL, NULL, SCRIPT_FIX_ALL) != 0)
+                return;
+	    enter_exit(tmp, head);
 	}
 	else
 	    /* Currently only players can transfer maps */
@@ -916,83 +884,29 @@ void move_teleporter(object *op) {
 	    free_object(head);
 	    return;
 	}
-	/* GROS: Handle for plugin TRIGGER event */
-	if ((evt = find_event(op, EVENT_TRIGGER)) != NULL)
-	{
-		CFParm CFP;
-		CFParm* CFR;
-		int k, l, m;
-		uint32 n;
-		int rtn_script = 0;
-		m = 0;
-		n = 0;
-		k = EVENT_TRIGGER;
-		l = SCRIPT_FIX_ALL;
-		CFP.Value[0] = &k;
-		CFP.Value[1] = op;
-		CFP.Value[2] = op->above;
-		CFP.Value[3] = NULL;
-		CFP.Value[4] = NULL;
-		CFP.Value[5] = &n;
-		CFP.Value[6] = &m;
-		CFP.Value[7] = &m;
-		CFP.Value[8] = &l;
-		CFP.Value[9] = evt->hook;
-		CFP.Value[10]= evt->options;
-		if (findPlugin(evt->plugin)>=0)
-		{
-			CFR = (PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP);
-			rtn_script = *(int *)(CFR->Value[0]);
-		}
-		if (rtn_script!=0) return;
-	}
+        /* Lauwenmark: Handle for plugin TRIGGER event */
+        if (execute_event(op, EVENT_TRIGGER, tmp, NULL, NULL, SCRIPT_FIX_ALL) != 0)
+        return;
 	transfer_ob(tmp,EXIT_X(head),EXIT_Y(head),0,head);
     }
     else {
 	/* Random teleporter */
-	/* GROS: Handle for plugin TRIGGER event */
-        if ((evt = find_event(op, EVENT_TRIGGER)) != NULL) {
-	    CFParm CFP;
-	    CFParm* CFR;
-	    int k, l, m;
-	    uint32 n;
-	    int rtn_script = 0;
-	    m = 0;
-	    n = 0;
-	    k = EVENT_TRIGGER;
-	    l = SCRIPT_FIX_ALL;
-	    CFP.Value[0] = &k;
-	    CFP.Value[1] = op;
-	    CFP.Value[2] = op->above;
-	    CFP.Value[3] = NULL;
-	    CFP.Value[4] = NULL;
-	    CFP.Value[5] = &n;
-	    CFP.Value[6] = &m;
-	    CFP.Value[7] = &m;
-	    CFP.Value[8] = &l;
-	    CFP.Value[9] = evt->hook;
-	    CFP.Value[10]= evt->options;
-	    if (findPlugin(evt->plugin)>=0) {
-		CFR = (PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP);
-		rtn_script = *(int *)(CFR->Value[0]);
-	    }
-	    if (rtn_script!=0) return;
-	}
+        /* Lauwenmark: Handle for plugin TRIGGER event */
+        if (execute_event(op, EVENT_TRIGGER, tmp, NULL, NULL, SCRIPT_FIX_ALL) != 0)
+            return;
 	teleport(head, TELEPORTER, tmp);
     }
 }
 
 
-/*  This object will teleport someone to a different map
-    and will also apply changes to the player from its inventory.
-    This was invented for giving classes, but there's no reason it
-    can't be generalized.
-*/
-
+/**  This object will teleport someone to a different map
+  *  and will also apply changes to the player from its inventory.
+  *  This was invented for giving classes, but there's no reason it
+  *  can't be generalized.
+  */
 void move_player_changer(object *op) {
     object *player;
     object *walk;
-    event *evt;
     char c;
 
     if (!op->above || !EXIT_PATH(op)) return;
@@ -1001,36 +915,9 @@ void move_player_changer(object *op) {
      * needs to be on top.
      */
     if(op->above->type==PLAYER) {
-	/* GROS: Handle for plugin TRIGGER event */
-	if ((evt = find_event(op, EVENT_TRIGGER)) != NULL) {
-	    CFParm CFP;
-	    CFParm* CFR;
-	    int k, l, m;
-	    uint32 n;
-	    int rtn_script = 0;
-
-	    m = 0;
-	    n = 0;
-	    k = EVENT_TRIGGER;
-	    l = SCRIPT_FIX_NOTHING;
-	    CFP.Value[0] = &k;
-	    CFP.Value[1] = op;
-	    CFP.Value[2] = op->above;
-	    CFP.Value[3] = NULL;
-	    CFP.Value[4] = NULL;
-	    CFP.Value[5] = &n;
-	    CFP.Value[6] = &m;
-	    CFP.Value[7] = &m;
-	    CFP.Value[8] = &l;
-	    CFP.Value[9] = evt->hook;
-	    CFP.Value[10]= evt->options;
-	    if (findPlugin(evt->plugin)>=0) {
-		CFR = (PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP);
-		rtn_script = *(int *)(CFR->Value[0]);
-	    }
-	    if (rtn_script!=0) return;
-	} /* if event tied to this */
-
+        /* Lauwenmark: Handle for plugin TRIGGER event */
+        if (execute_event(op, EVENT_TRIGGER,op->above,NULL,NULL,SCRIPT_FIX_NOTHING)!=0)
+            return;
 	player=op->above;
 	for(walk=op->inv;walk!=NULL;walk=walk->below)
 	    apply_changes_to_player(player,walk);
@@ -1047,7 +934,8 @@ void move_player_changer(object *op) {
 	    player->contr->bed_y = EXIT_Y(op);
 	}
 	else
-	    LOG(llevDebug, "WARNING: destination '%s' in player_changer must be an absolute path!\n",
+            LOG(llevDebug,
+                "WARNING: destination '%s' in player_changer must be an absolute path!\n",
 		EXIT_PATH(op));
 	
 	enter_exit(op->above,op);
@@ -1055,7 +943,8 @@ void move_player_changer(object *op) {
     }
 }
 
-/* firewalls fire other spells.
+/**
+ * firewalls fire other spells.
  * The direction of the wall is stored in op->stats.sp.
  * walls can have hp, so they can be torn down.
  */
@@ -1077,16 +966,16 @@ void move_firewall(object *op) {
 }
 
 
-/*  move_player_mover:  this function takes a "player mover" as an
+/**
+ * move_player_mover:  this function takes a "player mover" as an
  * argument, and performs the function of a player mover, which is:
-
+ *
  * a player mover finds any players that are sitting on it.  It
  * moves them in the op->stats.sp direction.  speed is how often it'll move.
  * If attacktype is nonzero it will paralyze the player.  If lifesave is set,
  * it'll dissapear after hp+1 moves.  If hp is set and attacktype is set,
  * it'll paralyze the victim for hp*his speed/op->speed
-
-*/
+ */
 void move_player_mover(object *op) {
     object *victim, *nextmover;
     int dir = op->stats.sp;
@@ -1097,7 +986,8 @@ void move_player_mover(object *op) {
     if (!dir) dir=rndm(1, 8);
 
     for(victim=get_map_ob(op->map,op->x,op->y); victim !=NULL; victim=victim->above) {
-	if(QUERY_FLAG(victim, FLAG_ALIVE)&& (!(QUERY_FLAG(victim,FLAG_FLYING))||op->stats.maxhp)) {
+	if(QUERY_FLAG(victim, FLAG_ALIVE) && !QUERY_FLAG(victim, FLAG_WIZPASS) && 
+	   (victim->move_type & op->move_type || !victim->move_type)) {
 
 	    if (victim->head) victim = victim->head;
 
@@ -1114,6 +1004,8 @@ void move_player_mover(object *op) {
 		    m->path, op->x, op->y);
 		return ;
 	    }
+	    
+	    if (should_director_abort(op, victim)) return ;
 
 	    for(nextmover=get_map_ob(m,nx, ny); nextmover !=NULL; nextmover=nextmover->above) {
 		if(nextmover->type == PLAYERMOVER) 
@@ -1156,7 +1048,7 @@ void move_player_mover(object *op) {
     }
 }
 
-/*
+/**
  * Will duplicate a specified object placed on top of it.
  * connected: what will trigger it.
  * level: multiplier.  0 to destroy.
@@ -1170,18 +1062,22 @@ void move_duplicator(object *op) {
 	return;
     for (tmp=op->above; tmp != NULL; tmp=tmp->above) {
 	if (strcmp(op->other_arch->name, tmp->arch->name) == 0) {
-	    if (op->level == 0) {
+	    if (op->level <= 0) {
 		remove_ob(tmp);
 		free_object(tmp);
 	    } else {
-		tmp->nrof *= op->level;
+		uint64 new_nrof = (uint64)tmp->nrof*op->level;
+		if (new_nrof >= 1UL<<31)
+		    new_nrof = 1UL<<31;
+		tmp->nrof = new_nrof;
 	    }
 	    break;
 	}
     }
 }
 
-/* move_creator (by peterm) 
+/**
+ * move_creator (by peterm) 
  * it has the creator object create it's other_arch right on top of it.
  * connected:  what will trigger it
  * hp:  how many times it may create before stopping
@@ -1194,119 +1090,127 @@ void move_duplicator(object *op) {
  * more concern with large objects, most notably a part being placed
  * outside of the map which would cause the server to crash
 */
+void move_creator(object *creator) {
+    object *new_ob;
 
-void move_creator(object *op) {
-    object *tmp,*head=NULL,*prev=NULL;
-    archetype *at=op->other_arch;
-
-    op->stats.hp--;
-    if(op->stats.hp < 0 && !QUERY_FLAG(op,FLAG_LIFESAVE)) {
-	op->stats.hp=-1;
+    if(!QUERY_FLAG(creator, FLAG_LIFESAVE) && --creator->stats.hp < 0) {
+	creator->stats.hp=-1;
 	return;
     }
-    if (!at) {
-	LOG(llevError,"move_creator: Creator doesn't have other arch set: %s (%s, %d, %d)\n",
-	    op->name?op->name:"(null)", op->map->path, op->x, op->y);
-	return;
-    }
-    /* Make sure this multipart object fits */
-    if (at->more && arch_blocked(at, op->map, op->x, op->y))
-	return;
 
-    while(at!=NULL) {
-	tmp=arch_to_object(at);
-	tmp->x=op->x+at->clone.x;
-	tmp->y=op->y+at->clone.y;
-	tmp->map=op->map;
-	tmp->level=op->level;
+    if (creator->inv != NULL) {
+	object *ob;
+	int i;
+	object *ob_to_copy;
 
-	if(head!=NULL)
-	    tmp->head=head,prev->more=tmp;
-
-	insert_ob_in_map(tmp,op->map,op,0);
-	if (QUERY_FLAG(tmp, FLAG_FREED)) return;
-
-	if(head==NULL)
-	    head=tmp;
-	prev=tmp;
-	at=at->more;
-
-	if(op->slaying) {
-	    if (tmp->name) free_string (tmp->name);
-	    if (tmp->title) free_string (tmp->title);
-	    tmp->name = add_string(op->slaying);
-	    tmp->title = add_string(op->slaying);
+	/* select random object from inventory to copy */
+	ob_to_copy = creator->inv;
+	for (ob = creator->inv->below, i = 1; ob != NULL; ob = ob->below, i++) {
+	    if (rndm(0, i) == 0) {
+		ob_to_copy = ob;
+	    }
 	}
+	new_ob = object_create_clone(ob_to_copy);
+	CLEAR_FLAG(new_ob, FLAG_IS_A_TEMPLATE);
+	unflag_inv(new_ob, FLAG_IS_A_TEMPLATE);
+    } else {
+	if (creator->other_arch == NULL) {
+	    LOG(llevError,"move_creator: Creator doesn't have other arch set: %s (%s, %d, %d)\n", creator->name ? creator->name : "(null)", creator->map->path, creator->x, creator->y);
+	    return;
+	}
+
+	new_ob = object_create_arch(creator->other_arch);
+	fix_generated_item(new_ob, creator, 0, 0, GT_MINIMAL);
+    }
+
+    /* Make sure this multipart object fits */
+    if (new_ob->arch->more && ob_blocked(new_ob, creator->map, creator->x, creator->y)) {
+	free_object(new_ob);
+	return;
+    }
+
+    insert_ob_in_map_at(new_ob, creator->map, creator, 0, creator->x, creator->y);
+    if (QUERY_FLAG(new_ob, FLAG_FREED))
+	return;
+
+    if (creator->slaying) {
+	FREE_AND_COPY(new_ob->name, creator->slaying);
+	FREE_AND_COPY(new_ob->title, creator->slaying);
     }
 }
 
-/* move_marker --peterm@soda.csua.berkeley.edu
-   when moved, a marker will search for a player sitting above
-   it, and insert an invisible, weightless force into him
-   with a specific code as the slaying field.
-   At that time, it writes the contents of its own message
-   field to the player.  The marker will decrement hp to
-   0 and then delete itself every time it grants a mark.
-   unless hp was zero to start with, in which case it is infinite.*/
-
+/**
+ * move_marker --peterm@soda.csua.berkeley.edu
+ * when moved, a marker will search for a player sitting above
+ * it, and insert an invisible, weightless force into him
+ * with a specific code as the slaying field.
+ * At that time, it writes the contents of its own message
+ * field to the player.  The marker will decrement hp to
+ * 0 and then delete itself every time it grants a mark.
+ * unless hp was zero to start with, in which case it is infinite.*/
 void move_marker(object *op) {
-  object *tmp,*tmp2;
+    object *tmp,*tmp2;
   
-  for(tmp=get_map_ob(op->map,op->x,op->y);tmp!=NULL;tmp=tmp->above) {
-	 
+    for(tmp=get_map_ob(op->map,op->x,op->y);tmp!=NULL;tmp=tmp->above) {
+	if(tmp->type == PLAYER) { /* we've got someone to MARK */
 
-    if(tmp->type == PLAYER && quest_marker_compatible( op, tmp ) ) { /* we've got someone to MARK */
+	    if ( quest_on_activate(op, tmp->contr) )
+		return;
 
-		/* remove an old force with a slaying field == op->name */
-      for(tmp2=tmp->inv;tmp2 !=NULL; tmp2=tmp2->below) {
-		  if(tmp2->type == FORCE && tmp2->slaying && !strcmp(tmp2->slaying,op->name)) break;
-      }
-		if(tmp2) {
-		  remove_ob(tmp2);
-		  free_object(tmp2);
-		}
+	    /* remove an old force with a slaying field == op->name */
+	    for(tmp2=tmp->inv;tmp2 !=NULL; tmp2=tmp2->below) {
+		if(tmp2->type == FORCE && tmp2->slaying && !strcmp(tmp2->slaying,op->name)) break;
+	    }
 
-      /* cycle through his inventory to look for the MARK we want to place */
-      for(tmp2=tmp->inv;tmp2 !=NULL; tmp2=tmp2->below) {
-		  if(tmp2->type == FORCE && tmp2->slaying && !strcmp(tmp2->slaying,op->slaying)) break;
-      }
+	    if(tmp2) {
+		remove_ob(tmp2);
+		free_object(tmp2);
+	    }
+
+	    /* cycle through his inventory to look for the MARK we want to 
+	     * place 
+	     */
+	    for(tmp2=tmp->inv;tmp2 !=NULL; tmp2=tmp2->below) {
+		if(tmp2->type == FORCE && tmp2->slaying && !strcmp(tmp2->slaying,op->slaying)) break;
+	    }
       
-      /* if we didn't find our own MARK */
-      if(tmp2==NULL) {
-	         
-		  object *force = get_archetype(FORCE_NAME);
-		  force->speed = 0;
-		  if(op->stats.food) {
-			 force->speed = 0.01;
-			 force->speed_left = -op->stats.food;
-		  }
-		  update_ob_speed (force);
-		  /* put in the lock code */
-		  force->slaying = add_string(op->slaying);
-          if ( op->lore )
-              force->lore = add_string( op->lore );
-          quest_clear_markers( op, tmp );
-		  insert_ob_in_ob(force,tmp);
-		  if(op->msg)
+	    /* if we didn't find our own MARK */
+	    if(tmp2==NULL) {
+		object *force = create_archetype(FORCE_NAME);
+
+		force->speed = 0;
+		if(op->stats.food) {
+		    force->speed = 0.01;
+		    force->speed_left = -op->stats.food;
+		}
+		update_ob_speed (force);
+		/* put in the lock code */
+		force->slaying = add_string(op->slaying);
+
+		if ( op->lore )
+		    force->lore = add_string( op->lore );
+
+		insert_ob_in_ob(force,tmp);
+		if(op->msg)
 		    new_draw_info(NDI_UNIQUE|NDI_NAVY,0,tmp,op->msg);
-		  if(op->stats.hp > 0) { 
+
+		if(op->stats.hp > 0) { 
 		    op->stats.hp--;
 		    if(op->stats.hp==0) {
-		      /* marker expires--granted mark number limit */
-		      remove_ob(op);
-		      free_object(op);
-		      return;
+			/* marker expires--granted mark number limit */
+			remove_ob(op);
+			free_object(op);
+			return;
 		    }
-		  }
-      }
-
-    }
-
-  }
+		}
+	    } /* if tmp2 == NULL */
+	} /* if tmp->type == PLAYER */
+    } /* For all objects on this space */
 }
  
 int process_object(object *op) {
-    event *evt;
+    if (QUERY_FLAG(op, FLAG_IS_A_TEMPLATE))
+	return 0;
 
     if(QUERY_FLAG(op, FLAG_MONSTER))
 	if(move_monster(op) || QUERY_FLAG(op, FLAG_FREED)) 
@@ -1343,31 +1247,18 @@ int process_object(object *op) {
 	}
 	return 1;
     }
-    /* GROS: Handle for plugin time event */
-    if ((evt = find_event(op, EVENT_TIME)) != NULL)
-    {
-	CFParm CFP;
-	int k, l, m;
-	uint32 n;
-	k = EVENT_TIME;
-	l = SCRIPT_FIX_NOTHING;
-	m = 0;
-	n = 0;
-	CFP.Value[0] = &k;
-	CFP.Value[1] = NULL;
-	CFP.Value[2] = op;
-	CFP.Value[3] = NULL;
-	CFP.Value[4] = NULL;
-	CFP.Value[5] = &n;
-	CFP.Value[6] = &m;
-	CFP.Value[7] = &m;
-	CFP.Value[8] = &l;
-	CFP.Value[9] = evt->hook;
-	CFP.Value[10]= evt->options;
-	if (findPlugin(evt->plugin)>=0)
-	    ((PlugList[findPlugin(evt->plugin)].eventfunc) (&CFP));
-    }
+    /* Lauwenmark: Handle for plugin time event */
+    execute_event(op, EVENT_TIME,NULL,NULL,NULL,SCRIPT_FIX_NOTHING);
     switch(op->type) {
+	case TRANSPORT:
+	    /* Transports are directed by players - thus, there
+	     * speed is reduced when the player moves them about.
+	     * So give them back there speed here, since process_objects()
+	     * has decremented it.
+	     */
+	    if (op->speed_left < 0.0) op->speed_left += 1.0;
+	    return 1;
+
 	case SPELL_EFFECT:
 	    move_spell_effect(op);
 	    return 1;
@@ -1424,7 +1315,7 @@ int process_object(object *op) {
 	    return 0;
 
 	case EARTHWALL:
-	    hit_player(op, 2, op, AT_PHYSICAL);
+	    hit_player(op, 2, op, AT_PHYSICAL, 1);
 	    return 0;
 
 	case FIREWALL:
