@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_init_c =
- *    "$Id: image.c,v 1.8 2005/07/14 09:18:57 akirschbaum Exp $";
+ *    "$Id: image.c,v 1.12 2006/06/05 06:28:57 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2001 Mark Wedel
+    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -29,7 +29,7 @@
 /** \file
  * Image related communication
  *
- *  \date 2003-12-02
+ *  \date 2006-03-18
  *
  * This file deals with the image related communication to the
  * client.  I've located all the functions in this file - this
@@ -47,11 +47,11 @@
 #define MAX_FACE_SETS	20  /**< Maximum number of image sets the program will handle */
 
 /** Information about one image */
-typedef struct FaceInfo {
+typedef struct face_info {
   uint8 *data;		    /**< image data */
   uint16 datalen;	    /**< length of the xpm data */
   uint32 checksum;	    /**< Checksum of face data */
-} FaceInfo;
+} face_info;
 
 /** Information about one face set */
 typedef struct {
@@ -61,10 +61,10 @@ typedef struct {
     char    *size;
     char    *extension;
     char    *comment;
-    FaceInfo	*faces; /**< images in this faceset */
-} FaceSets;
+    face_info	*faces; /**< images in this faceset */
+} face_sets;
 
-static FaceSets facesets[MAX_FACE_SETS];    /**< All facesets */
+static face_sets facesets[MAX_FACE_SETS];    /**< All facesets */
 
 /**
  * Checks specified faceset is valid
@@ -79,7 +79,7 @@ int is_valid_faceset(int fsn)
 /**
  * Frees all faceset information
  */
-void free_socket_images()
+void free_socket_images(void)
 {
     int num,q;
 
@@ -173,7 +173,7 @@ static void check_faceset_fallback(int faceset, int togo)
  * to do such.
  */
 
-void read_client_images()
+void read_client_images(void)
 {
     char filename[400];
     char buf[HUGE_BUF];
@@ -223,7 +223,7 @@ void read_client_images()
     for (fileno=0; fileno<MAX_FACE_SETS; fileno++) {
 	/* if prefix is not set, this is not used */
 	if (!facesets[fileno].prefix) continue;
-	facesets[fileno].faces = calloc(nrofpixmaps, sizeof(FaceInfo));
+	facesets[fileno].faces = calloc(nrofpixmaps, sizeof(face_info));
 
 	sprintf(filename,"%s/crossfire.%d",settings.datadir, fileno);
 	LOG(llevDebug,"Loading image file %s\n", filename);
@@ -275,10 +275,9 @@ void read_client_images()
 /**
  * Client tells us what type of faces it wants.  Also sets
  * the caching attribute.
- *
  */
 
-void SetFaceMode(char *buf, int len, NewSocket *ns)
+void set_face_mode_cmd(char *buf, int len, socket_struct *ns)
 {
     char tmp[256];
 
@@ -290,7 +289,7 @@ void SetFaceMode(char *buf, int len, NewSocket *ns)
 	sprintf(tmp,"drawinfo %d %s", NDI_RED,"Warning - send unsupported face mode.  Will use Png");
 	Write_String_To_Socket(ns, tmp, strlen(tmp));
 #ifdef ESRV_DEBUG
-	LOG(llevDebug,"SetFaceMode: Invalid mode from client: %d\n", mode);
+	LOG(llevDebug,"set_face_mode_cmd: Invalid mode from client: %d\n", mode);
 #endif
     }
     if (mask) {
@@ -304,7 +303,7 @@ void SetFaceMode(char *buf, int len, NewSocket *ns)
  * caching images.
  */
 
-void SendFaceCmd(char *buff, int len, NewSocket *ns)
+void send_face_cmd(char *buff, int len, socket_struct *ns)
 {
     long tmpnum = atoi(buff);
     short facenum=tmpnum & 0xffff;
@@ -322,7 +321,7 @@ void SendFaceCmd(char *buff, int len, NewSocket *ns)
  * we look at the facecache, and if set, send the image name.
  */
 
-void esrv_send_face(NewSocket *ns,short face_num, int nocache)
+void esrv_send_face(socket_struct *ns,short face_num, int nocache)
 {
     SockList sl;
     char fallback;
@@ -332,7 +331,7 @@ void esrv_send_face(NewSocket *ns,short face_num, int nocache)
 	return;
     }
 
-    sl.buf = malloc(MAXSOCKBUF);
+    sl.buf = malloc(MAXSOCKSENDBUF);
     fallback = get_face_fallback(ns->faceset, face_num);
 
     if (facesets[fallback].faces[face_num].data == NULL) {
@@ -348,7 +347,7 @@ void esrv_send_face(NewSocket *ns,short face_num, int nocache)
 	else
 	    strcpy((char*)sl.buf, "face ");
 
-	sl.len=strlen(sl.buf);
+	sl.len=strlen((char*)sl.buf);
 	SockList_AddShort(&sl, face_num);
 	if (ns->image2)
 	    SockList_AddChar(&sl, fallback);
@@ -383,22 +382,22 @@ void esrv_send_face(NewSocket *ns,short face_num, int nocache)
  * if you want further detail.
  */
 
-void send_image_info(NewSocket *ns, char *params)
+void send_image_info(socket_struct *ns, char *params)
 {
     SockList sl;
     int i;
 
-    sl.buf = malloc(MAXSOCKBUF);
+    sl.buf = malloc(MAXSOCKSENDBUF);
 
-    sprintf(sl.buf,"replyinfo image_info\n%d\n%d\n", nrofpixmaps-1, bmaps_checksum);
+    sprintf((char*)sl.buf,"replyinfo image_info\n%d\n%d\n", nrofpixmaps-1, bmaps_checksum);
     for (i=0; i<MAX_FACE_SETS; i++) {
 	if (facesets[i].prefix) {
-	    sprintf(sl.buf + strlen(sl.buf), "%d:%s:%s:%d:%s:%s:%s",
+	    sprintf((char*)sl.buf + strlen((char*)sl.buf), "%d:%s:%s:%d:%s:%s:%s",
 		    i,  facesets[i].prefix, facesets[i].fullname, facesets[i].fallback,
 		    facesets[i].size, facesets[i].extension, facesets[i].comment);
 	}
     }
-    sl.len = strlen(sl.buf);
+    sl.len = strlen((char*)sl.buf);
     Send_With_Handling(ns, &sl);
     free(sl.buf);
 }
@@ -412,7 +411,7 @@ void send_image_info(NewSocket *ns, char *params)
  *  - checksum
  *  - name
  */
-void send_image_sums(NewSocket *ns, char *params)
+void send_image_sums(socket_struct *ns, char *params)
 {
     int start, stop;
     short i;
@@ -420,7 +419,7 @@ void send_image_sums(NewSocket *ns, char *params)
     char *cp, buf[MAX_BUF];
     SockList sl;
 
-    sl.buf = malloc(MAXSOCKBUF);
+    sl.buf = malloc(MAXSOCKSENDBUF);
 
     start = atoi(params);
     for (cp = params; *cp != '\0'; cp++)
@@ -432,11 +431,18 @@ void send_image_sums(NewSocket *ns, char *params)
 	cs_write_string(ns, buf, strlen(buf));
 	return;
     }
-    sprintf(sl.buf,"replyinfo image_sums %d %d ", start, stop);
+    sprintf((char*)sl.buf,"replyinfo image_sums %d %d ", start, stop);
 
-    sl.len = strlen(sl.buf);
+    sl.len = strlen((char*)sl.buf);
 
     for (i=start; i<=stop; i++) {
+	if (sl.len+2+4+1+1+strlen(new_faces[i].name)+1 > MAXSOCKSENDBUF) {
+	    LOG(llevError, "send_image_sums: buffer overflow, rejecting range %d..%d\n", start, stop);
+	    sprintf(buf, "replyinfo image_sums %d %d", start, stop);
+	    cs_write_string(ns, buf, strlen(buf));
+	    return;
+	}
+
 	SockList_AddShort(&sl, i);
 	ns->faces_sent[i] |= NS_FACESENT_FACE;
 
@@ -446,19 +452,10 @@ void send_image_sums(NewSocket *ns, char *params)
 
 	qq = strlen(new_faces[i].name);
 	SockList_AddChar(&sl, ( char )( qq + 1 ));
-	strcpy(sl.buf + sl.len, new_faces[i].name);
+	strcpy((char*)sl.buf + sl.len, new_faces[i].name);
 	sl.len += qq;
 	SockList_AddChar(&sl, 0);
     }
-    /* It would make more sense to catch this pre-emptively in the code above.
-     * however, if this really happens, we probably just want to cut down the
-     * size to less than 1000, since that is what we claim the protocol would
-     * support.
-     */
-    if (sl.len > MAXSOCKBUF) {
-	LOG(llevError,"send_image_send: buffer overrun, %d > %d\n", sl.len, MAXSOCKBUF);
-	abort();
-    }
     Send_With_Handling(ns, &sl);
     free(sl.buf);
 }
