@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_sock_info_c =
- *   "$Id: info.c,v 1.24 2005/05/29 17:35:54 tchize Exp $";
+ *   "$Id: info.c,v 1.29 2006/05/21 10:35:10 akirschbaum Exp $";
  */
 
 /*
@@ -48,7 +48,7 @@
  * parameter.  the esrv_drawinfo functions should probably be
  * replaced with this, just using black as the color.
  */
-static void esrv_print_msg(NewSocket *ns,int color, const char *str)
+static void esrv_print_msg(socket_struct *ns,int color, const char *str)
 {
     char buf[HUGE_BUF];
 
@@ -70,7 +70,7 @@ static void esrv_print_msg(NewSocket *ns,int color, const char *str)
  * intro   Intro message to send with main message if client does not support the message type
  * message The main message
  */
-static void esrv_print_ext_msg(NewSocket *ns,int color,uint8 type, uint8 subtype, const char *message)
+static void esrv_print_ext_msg(socket_struct *ns,int color,uint8 type, uint8 subtype, const char *message)
 {
     char buf[HUGE_BUF];
     snprintf(buf,HUGE_BUF, "drawextinfo %d %hhu %hhu %s", color, type, subtype, message);
@@ -90,7 +90,7 @@ static void esrv_print_ext_msg(NewSocket *ns,int color,uint8 type, uint8 subtype
  * Else sends message to player via esrv_print_msg
  */
 
-static void print_message(int colr, object *pl,const char *tmp) {
+static void print_message(int colr, const object *pl, const char *tmp) {
 
   if(tmp == (char *) NULL) {
     tmp="[NULL]";
@@ -112,7 +112,7 @@ static void print_message(int colr, object *pl,const char *tmp) {
  * and clears the string.
  */
 
-void flush_output_element(object *pl, Output_Buf *outputs)
+void flush_output_element(const object *pl, Output_Buf *outputs)
 {
     char tbuf[MAX_BUF];
 
@@ -139,7 +139,7 @@ void flush_output_element(object *pl, Output_Buf *outputs)
  * and adds message to queue.
  */
 
-void check_output_buffers(object *pl, const char *buf)
+static void check_output_buffers(const object *pl, const char *buf)
 {
     int i, oldest=0;
 
@@ -194,7 +194,7 @@ void check_output_buffers(object *pl, const char *buf)
  *
  */
 
-void new_draw_info(int flags,int pri, object *pl, const char *buf)
+void new_draw_info(int flags, int pri, const object *pl, const char *buf)
 {
 
     if (flags & NDI_ALL) {
@@ -240,7 +240,7 @@ void new_draw_info(int flags,int pri, object *pl, const char *buf)
  * client/server bandwidth (client could keep track of various strings
  */
 
-void new_draw_info_format(int flags, int pri,object *pl, const char *format, ...)
+void new_draw_info_format(int flags, int pri, const object *pl, const char *format, ...)
 {
     char buf[HUGE_BUF];
 
@@ -256,7 +256,7 @@ void new_draw_info_format(int flags, int pri,object *pl, const char *format, ...
 
 
 void draw_ext_info(
-        int flags, int pri, object *pl, uint8 type, 
+        int flags, int pri, const object *pl, uint8 type, 
         uint8 subtype, const char* message, const char* oldmessage){
             
     if(!pl || (pl->type!=PLAYER) || (pl->contr==NULL))
@@ -279,9 +279,9 @@ void draw_ext_info(
 }
 
 void draw_ext_info_format(
-        int flags, int pri, object *pl, uint8 type, 
+        int flags, int pri, const object *pl, uint8 type, 
         uint8 subtype, const char* old_format, 
-        char* new_format, ...){
+        const char* new_format, ...){
             
     char buf[HUGE_BUF];
 	if(!pl || (pl->type!=PLAYER) || (pl->contr==NULL))
@@ -309,7 +309,7 @@ void draw_ext_info_format(
  * Writes to everyone on the map *except* op.  This is useful for emotions.
  */
 
-void new_info_map_except(int color, mapstruct *map, object *op, const char *str) {
+void new_info_map_except(int color, const mapstruct *map, const object *op, const char *str) {
     player *pl;
 
     for(pl = first_player; pl != NULL; pl = pl->next)
@@ -322,7 +322,7 @@ void new_info_map_except(int color, mapstruct *map, object *op, const char *str)
  * Writes to everyone on the map except op1 and op2
  */
 
-void new_info_map_except2(int color, mapstruct *map, object *op1, object *op2,
+void new_info_map_except2(int color, const mapstruct *map, const object *op1, const object *op2,
 			  const char *str) {
     player *pl;
 
@@ -337,7 +337,7 @@ void new_info_map_except2(int color, mapstruct *map, object *op1, object *op2,
  * Writes to everyone on the specified map
  */
 
-void new_info_map(int color, mapstruct *map, const char *str) {
+void new_info_map(int color, const mapstruct *map, const char *str) {
     player *pl;
 
     for(pl = first_player; pl != NULL; pl = pl->next)
@@ -359,7 +359,7 @@ void clear_win_info(object *op)
 /**
  * Get player's current range attack in obuf.
  */
-void rangetostring(object *pl,char *obuf)
+void rangetostring(const object *pl,char *obuf)
 {
     switch(pl->contr->shoottype) {
 	case range_none:
@@ -432,7 +432,7 @@ void rangetostring(object *pl,char *obuf)
 /**
  * Sets player title.
  */
-void set_title(object *pl, char *buf)
+void set_title(const object *pl, char *buf)
 {
     /* Eneq(@csd.uu.se): Let players define their own titles. */
     if (pl->contr->own_title[0]=='\0')
@@ -455,10 +455,11 @@ void set_title(object *pl, char *buf)
  */
 static void magic_mapping_mark_recursive(object *pl, char *map_mark, int px, int py)
 {
-    int x, y, dx, dy,mflags;
+    int x, y, dx, dy,mflags, l;
     sint16 nx, ny;
     mapstruct *mp;
     New_Face *f;
+    object *ob;
 
     for (dx = -1; dx <= 1; dx++) {
 	for (dy = -1; dy <= 1; dy++) {
@@ -475,17 +476,19 @@ static void magic_mapping_mark_recursive(object *pl, char *map_mark, int px, int
 	    if (mflags & P_OUT_OF_MAP) continue;
 
 	    if (map_mark[MAGIC_MAP_HALF + x + MAGIC_MAP_SIZE* (MAGIC_MAP_HALF + y)] == 0) {
-		f= GET_MAP_FACE(mp, nx, ny, 0);
-		if (f == blank_face)
-		    f= GET_MAP_FACE(mp, nx, ny, 1);
-		if (f == blank_face)
-		    f= GET_MAP_FACE(mp, nx, ny, 2);
-
+		for (l=0; l < MAP_LAYERS; l++) {
+		    ob = GET_MAP_FACE_OBJ(mp, nx, ny, l);
+		    if (ob && !ob->invisible && ob->face != blank_face) break;
+		}
+		if (ob) f = ob->face;
+		else f = blank_face;
+		
 		/* Should probably have P_NO_MAGIC here also, but then shops don't
 		 * work.
 		 */
 		if (mflags & P_BLOCKSVIEW)
-		    map_mark[MAGIC_MAP_HALF + x + MAGIC_MAP_SIZE* (MAGIC_MAP_HALF + y)] = FACE_WALL | (f?f->magicmap:0);
+		    map_mark[MAGIC_MAP_HALF + x + MAGIC_MAP_SIZE* (MAGIC_MAP_HALF + y)] = 
+			FACE_WALL | (f?f->magicmap:0);
 		else {
 		    map_mark[MAGIC_MAP_HALF + x + MAGIC_MAP_SIZE* (MAGIC_MAP_HALF + y)] = FACE_FLOOR | (f?f->magicmap:0);
 		    magic_mapping_mark_recursive(pl, map_mark, x, y);
@@ -514,10 +517,11 @@ static void magic_mapping_mark_recursive(object *pl, char *map_mark, int px, int
 
 void magic_mapping_mark(object *pl, char *map_mark, int strength)
 {
-    int x, y, mflags;
+    int x, y, mflags, l;
     sint16 nx, ny;
     mapstruct *mp;
     New_Face *f;
+    object *ob;
 
     for (x = -strength; x <strength; x++) {
 	for (y = -strength; y <strength; y++) {
@@ -528,11 +532,12 @@ void magic_mapping_mark(object *pl, char *map_mark, int strength)
 	    if (mflags & P_OUT_OF_MAP)
 		continue;
 	    else {
-		f= GET_MAP_FACE(mp, nx, ny, 0);
-		if (f == blank_face)
-		    f= GET_MAP_FACE(mp, nx, ny, 1);
-		if (f == blank_face)
-		    f= GET_MAP_FACE(mp, nx, ny, 2);
+		for (l=0; l < MAP_LAYERS; l++) {
+		    ob = GET_MAP_FACE_OBJ(mp, nx, ny, l);
+		    if (ob && !ob->invisible && ob->face != blank_face) break;
+		}
+		if (ob) f = ob->face;
+		else f = blank_face;
 	    }
 
 	    if (mflags & P_BLOCKSVIEW)
@@ -590,8 +595,8 @@ void draw_magic_map(object *pl)
 	}
     }
 
-    sl.buf=malloc(MAXSOCKBUF);
-    snprintf((char*)sl.buf, MAXSOCKBUF, "magicmap %d %d %d %d ", (xmax-xmin+1), (ymax-ymin+1),
+    sl.buf=malloc(MAXSOCKSENDBUF);
+    snprintf((char*)sl.buf, MAXSOCKSENDBUF, "magicmap %d %d %d %d ", (xmax-xmin+1), (ymax-ymin+1),
 	    MAGIC_MAP_HALF - xmin, MAGIC_MAP_HALF - ymin);
     sl.len=strlen((char*)sl.buf);
     
@@ -611,7 +616,7 @@ void draw_magic_map(object *pl)
  * Send a kill log record to sockets
  */
 
-void Log_Kill(const char *Who,
+void log_kill(const char *Who,
             const char *What, int WhatType,
             const char *With, int WithType)
 {
