@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_init_c =
- *    "$Id: init.c,v 1.37 2005/07/22 20:41:25 ryo_saeba Exp $";
+ *    "$Id: init.c,v 1.52 2006/06/05 06:28:57 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2001 Mark Wedel
+    Copyright (C) 2006 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -60,29 +60,29 @@
 #include <newserver.h>
 
 Socket_Info socket_info;
-NewSocket *init_sockets;
+socket_struct* init_sockets;
 
 /**
  * Initializes a connection. Really, it just sets up the data structure,
  * socket setup is handled elsewhere.  We do send a version to the
  * client.
  */
-void InitConnection(NewSocket *ns, uint32 from)
+void init_connection(socket_struct *ns, const char *from_ip)
 {
     SockList sl;
     unsigned char buf[256];
     int	bufsize=65535;	/*Supposed absolute upper limit */
     int oldbufsize;
-    int buflen=sizeof(int);
+    socklen_t buflen=sizeof(int);
 
 #ifdef WIN32 /* ***WIN32 SOCKET: init win32 non blocking socket */
 	int temp = 1;	
 
 	if(ioctlsocket(ns->fd, FIONBIO , &temp) == -1)
-		LOG(llevError,"InitConnection:  Error on ioctlsocket.\n");
+		LOG(llevError,"init_connection:  Error on ioctlsocket.\n");
 #else 
     if (fcntl(ns->fd, F_SETFL, O_NONBLOCK)==-1) {
-		LOG(llevError,"InitConnection:  Error on fcntl.\n");
+		LOG(llevError,"init_connection:  Error on fcntl.\n");
     }
 #endif /* end win32 */
 
@@ -93,7 +93,7 @@ void InitConnection(NewSocket *ns, uint32 from)
 	LOG(llevDebug, "Default buffer size was %d bytes, will reset it to %d\n", oldbufsize, bufsize);
 #endif
 	if(setsockopt(ns->fd,SOL_SOCKET,SO_SNDBUF, (char*)&bufsize, sizeof(&bufsize))) {
-	    LOG(llevError,"InitConnection: setsockopt unable to set output buf size to %d\n", bufsize);
+	    LOG(llevError,"init_connection: setsockopt unable to set output buf size to %d\n", bufsize);
 	}
     }
     buflen=sizeof(oldbufsize);
@@ -106,9 +106,8 @@ void InitConnection(NewSocket *ns, uint32 from)
     ns->facecache = 0;
     ns->image2 = 0;
     ns->sound = 0;
-    ns->ext2 = 0;
-    ns->ext_title_flag = 1;
     ns->exp64 = 0;
+    ns->monitor_spells = 0;
     ns->mapmode = Map0Cmd;
     ns->darkness = 1;
     ns->status = Ns_Add;
@@ -124,6 +123,9 @@ void InitConnection(NewSocket *ns, uint32 from)
     ns->update_look = 0;
     ns->has_readable_type = 0;
     ns->supported_readables = 0;
+    ns->monitor_spells = 0;
+    ns->tick=0;
+    ns->is_bot = 0;
 
     /* we should really do some checking here - if total clients overflows
      * we need to do something more intelligent, because client id's will start
@@ -131,16 +133,18 @@ void InitConnection(NewSocket *ns, uint32 from)
      * just open and close connections could get this total up.
      */
     ns->inbuf.len=0;
-    ns->inbuf.buf=malloc( sizeof( unsigned char ) * MAXSOCKBUF );
+    ns->inbuf.buf=malloc(MAXSOCKRECVBUF);
     /* Basic initialization. Needed because we do a check in
-     * HandleClient for oldsocketmode without checking the
+     * handle_client for oldsocketmode without checking the
      * length of data.
      */
-    memset( ns->inbuf.buf, 0, sizeof( unsigned char ) * MAXSOCKBUF );
+    memset(ns->inbuf.buf, 0, MAXSOCKRECVBUF);
     memset(&ns->lastmap,0,sizeof(struct Map));
-    memset(&ns->faces_sent,0,sizeof(ns->faces_sent));
+    memset(ns->faces_sent, 0, ns->faces_sent_len*sizeof(*ns->faces_sent));
     memset(&ns->anims_sent,0,sizeof(ns->anims_sent));
     memset(&ns->stats,0,sizeof(struct statsinfo));
+    ns->map_scroll_x=0;
+    ns->map_scroll_y=0;
     /* Do this so we don't send a face command for the client for
      * this face.  Face 0 is sent to the client to say clear
      * face information.
@@ -150,11 +154,10 @@ void InitConnection(NewSocket *ns, uint32 from)
     ns->outputbuffer.start=0;
     ns->outputbuffer.len=0;
     ns->can_write=1;
+    ns->password_fails = 0;
 
     ns->sent_scroll=0;
-    sprintf((char*)buf,"%d.%d.%d.%d",
-          (from>>24)&255, (from>>16)&255, (from>>8)&255, from&255);
-    ns->host=strdup_local((char*)buf);
+    ns->host=strdup_local(from_ip);
     sprintf((char*)buf, "version %d %d %s\n", VERSION_CS,VERSION_SC, VERSION_INFO);
     sl.buf=buf;
     sl.len=strlen((char*)buf);
@@ -169,7 +172,7 @@ void InitConnection(NewSocket *ns, uint32 from)
 
 
 /** This sets up the socket and reads all the image information into memory. */
-void init_ericserver()
+void init_ericserver(void)
 {
     struct sockaddr_in	insock;
     struct protoent  *protox;
@@ -208,7 +211,8 @@ void init_ericserver()
 
     LOG(llevDebug,"Initialize new client/server data\n");
     socket_info.nconns = 1;
-    init_sockets = malloc(sizeof(NewSocket));
+    init_sockets = malloc(sizeof(socket_struct));
+    init_sockets[0].faces_sent = NULL; /* unused */
     socket_info.allocated_sockets=1;
 
     protox = getprotobyname("tcp");
@@ -218,8 +222,7 @@ void init_ericserver()
     }
     init_sockets[0].fd = socket(PF_INET, SOCK_STREAM, protox->p_proto);
     if (init_sockets[0].fd == -1) {
-	perror("Error create new client server socket.");
-	LOG(llevError, "Error creating socket on port\n");
+	LOG(llevError, "Cannot create socket: %s\n", strerror_local(errno));
 	exit(-1);
     }
     insock.sin_family = AF_INET;
@@ -230,15 +233,14 @@ void init_ericserver()
     linger_opt.l_linger = 0;
     if(setsockopt(init_sockets[0].fd,SOL_SOCKET,SO_LINGER,(char *) &linger_opt,
        sizeof(struct linger))) {
-	perror("error on setsockopt LINGER");
-	LOG(llevError, "Error on setsockopt LINGER\n");
+	LOG(llevError, "Cannot setsockopt(SO_LINGER): %s\n", strerror_local(errno));
     }
 /* Would be nice to have an autoconf check for this.  It appears that
  * these functions are both using the same calling syntax, just one
  * of them needs extra valus passed.
  */
 #if defined(__osf__) || defined(hpux) || defined(sgi) || defined(NeXT) || \
-        defined(__sun__) || defined(linux) || defined(SVR4) || \
+        defined(__sun__) || defined(__linux__) || defined(SVR4) || \
         defined(__FreeBSD__) || defined(__OpenBSD__) || \
         defined(WIN32) /* ---win32 add this here */  || \
         defined(__GNU__) /* HURD */
@@ -250,20 +252,17 @@ void init_ericserver()
 #endif
 
 	if(setsockopt(init_sockets[0].fd,SOL_SOCKET,SO_REUSEADDR, &tmp, sizeof(tmp))) {
-	    perror("error on setsockopt REUSEADDR");
-	    LOG(llevError,"error on setsockopt REUSEADDR\n");
+	    LOG(llevError, "Cannot setsockopt(SO_REUSEADDR): %s\n", strerror_local(errno));
 	}
     }
 #else
     if(setsockopt(init_sockets[0].fd,SOL_SOCKET,SO_REUSEADDR,(char *)NULL,0)) {
-	perror("error on setsockopt REUSEADDR");
-	LOG(llevError,"error on setsockopt REUSEADDR\n");
+	LOG(llevError, "Cannot setsockopt(SO_REUSEADDR): %s\n", strerror_local(errno));
     }
 #endif
 
     if (bind(init_sockets[0].fd,(struct sockaddr *)&insock,sizeof(insock)) == (-1)) {
-	perror("error on bind command");
-	LOG(llevError,"error on bind command\n");
+	LOG(llevError, "Cannot bind socket to port %d: %s\n", ntohs(insock.sin_port), strerror_local(errno));
 #ifdef WIN32 /* ***win32: close() -> closesocket() */
 	shutdown(init_sockets[0].fd,SD_BOTH);
 	closesocket(init_sockets[0].fd);
@@ -273,8 +272,7 @@ void init_ericserver()
 	exit(-1);
     }
     if (listen(init_sockets[0].fd,5) == (-1))  {
-	perror("error on listen");
-	LOG(llevError,"error on listen\n");
+	LOG(llevError, "Cannot listen on socket: %s\n", strerror_local(errno));
 #ifdef WIN32 /* ***win32: close() -> closesocket() */
 	shutdown(init_sockets[0].fd,SD_BOTH);
 	closesocket(init_sockets[0].fd);
@@ -295,7 +293,7 @@ void init_ericserver()
  ******************************************************************************/
 
 /** Free's all the memory that ericserver allocates. */
-void free_all_newserver()
+void free_all_newserver(void)
 {  
     LOG(llevDebug,"Freeing all new client/server information.\n");
     free_socket_images();
@@ -309,7 +307,7 @@ void free_all_newserver()
  * update the list
  */
 
-void free_newsocket(NewSocket *ns)
+void free_newsocket(socket_struct *ns)
 {
 #ifdef WIN32 /* ***win32: closesocket in windows style */
 	shutdown(ns->fd,SD_BOTH);
