@@ -1,7 +1,7 @@
 
 /*
  * static char *rcsid_item_c =
- *    "$Id: item.c,v 1.32 2005/07/14 09:18:57 akirschbaum Exp $";
+ *    "$Id: item.c,v 1.41 2006/06/01 20:55:54 akirschbaum Exp $";
  */
 
 /*
@@ -62,7 +62,7 @@
  * single byte in length.  If the data is longer than that byte, it is
  * truncated approprately.
  */
-inline void add_stringlen_to_sockbuf(const char *buf, SockList *sl)
+static inline void add_stringlen_to_sockbuf(const char *buf, SockList *sl)
 {
     int len;
 
@@ -77,7 +77,7 @@ inline void add_stringlen_to_sockbuf(const char *buf, SockList *sl)
  *  This is a similar to query_name, but returns flags
  *  to be sended to client. 
  */
-unsigned int query_flags (object *op)
+static unsigned int query_flags(const object *op)
 {
     unsigned int flags = 0;
 
@@ -133,6 +133,71 @@ unsigned int query_flags (object *op)
     return flags;
 }
 
+/**
+ * Used in the send_look to put object head into SockList
+ * sl for socket ns.  Need socket to know if we need to send
+ * animation of face to the client.
+ */
+static void add_object_to_socklist(socket_struct *ns, SockList *sl, object *head)
+{
+    int flags, len, anim_speed;
+    char item_n[MAX_BUF];
+    const char *item_p;
+
+    flags = query_flags (head);
+    if (QUERY_FLAG(head, FLAG_NO_PICK))
+	flags |=  F_NOPICK;
+
+    if (!(ns->faces_sent[head->face->number] & NS_FACESENT_FACE))
+	esrv_send_face(ns, head->face->number,0);
+
+    if (QUERY_FLAG(head,FLAG_ANIMATE) && !ns->anims_sent[head->animation_id])
+	esrv_send_animation(ns, head->animation_id);
+
+    SockList_AddInt(sl, head->count);
+    SockList_AddInt(sl, flags);
+    SockList_AddInt(sl, QUERY_FLAG(head, FLAG_NO_PICK) ? -1 : WEIGHT(head));
+    SockList_AddInt(sl, head->face->number);
+
+    if (!head->custom_name) {
+	strncpy(item_n,query_base_name(head, 0),127);
+	item_n[127]=0;
+	len=strlen(item_n);
+	item_p=query_base_name(head, 1);
+    } else {
+	strncpy(item_n,head->custom_name,127);
+	item_n[127]=0;
+	len=strlen(item_n);
+	item_p=head->custom_name;
+    }
+    strncpy(item_n+len+1, item_p, 127);
+    item_n[254]=0;
+    len += strlen(item_n+1+len) + 1;
+    SockList_AddChar(sl, (char ) len);
+    memcpy(sl->buf+sl->len, item_n, len);
+    sl->len += len;
+
+    SockList_AddShort(sl,head->animation_id);
+    anim_speed=0;
+    if (QUERY_FLAG(head,FLAG_ANIMATE)) {
+	if (head->anim_speed) anim_speed=head->anim_speed;
+	else {
+	    if (FABS(head->speed)<0.001) anim_speed=255;
+	    else if (FABS(head->speed)>=1.0) anim_speed=1;
+	    else anim_speed = (int) (1.0/FABS(head->speed));
+	}
+	if (anim_speed>255) anim_speed=255;
+    }
+    SockList_AddChar(sl, (char) anim_speed);
+    SockList_AddInt(sl, head->nrof);
+
+    if (ns->itemcmd == 2)
+	SockList_AddShort(sl, head->client_type);
+
+    SET_FLAG(head, FLAG_CLIENT_SENT);
+}
+
+
 /**
  * Send the look window.  Don't need to do animations here 
  * This sends all the faces to the client, not just updates.  This is
@@ -142,9 +207,9 @@ unsigned int query_flags (object *op)
 void esrv_draw_look(object *pl)
 {
     object *tmp, *last;
-    int flags, got_one=0,anim_speed, start_look=0, end_look=0, len;
+    int got_one=0,start_look=0, end_look=0;
     SockList sl;
-    char buf[MAX_BUF], *item_p,item_n[MAX_BUF];
+    char buf[MAX_BUF];
 
     if (!pl->contr->socket.update_look) {
 	LOG(llevDebug,"esrv_draw_look called when update_look was not set\n");
@@ -157,10 +222,12 @@ void esrv_draw_look(object *pl)
        pl->map->in_memory != MAP_IN_MEMORY || out_of_map(pl->map,pl->x,pl->y))
 	    return;
 
-    for (tmp=get_map_ob(pl->map,pl->x,pl->y); tmp && tmp->above;tmp=tmp->above)
-	; 
+    if (pl->contr->transport)
+	for (tmp=pl->contr->transport->inv; tmp && tmp->above;tmp=tmp->above) ;
+    else
+	for (tmp=get_map_ob(pl->map,pl->x,pl->y); tmp && tmp->above;tmp=tmp->above) ;
 
-    sl.buf=malloc(MAXSOCKBUF);
+    sl.buf=malloc(MAXSOCKSENDBUF);
 
     Write_String_To_Socket(&pl->contr->socket, "delinv 0", strlen("delinv 0"));
     sprintf((char*)sl.buf,"item%d ", pl->contr->socket.itemcmd);
@@ -185,7 +252,14 @@ void esrv_draw_look(object *pl)
 	    SockList_AddShort(&sl, 0);
     }
 
+    if (pl->contr->transport) {
+	add_object_to_socklist(&pl->contr->socket, &sl, pl->contr->transport);
+	got_one++;
+    }
+
     for (last=NULL; tmp!=last; tmp=tmp->below) {
+	object *head;
+
 	if (QUERY_FLAG(tmp, FLAG_IS_FLOOR) && !last) {
 	    last = tmp->below;  /* assumes double floor mode */
 	    if (last && QUERY_FLAG(last, FLAG_IS_FLOOR))
@@ -211,69 +285,20 @@ void esrv_draw_look(object *pl)
 		    SockList_AddShort(&sl, 0);
 		break;
 	    }
+	    if (tmp->head) head = tmp->head;
+	    else head = tmp;
 
-	    flags = query_flags (tmp);
-	    if (QUERY_FLAG(tmp, FLAG_NO_PICK))
-		flags |=  F_NOPICK;
-	    if (!(pl->contr->socket.faces_sent[tmp->face->number] & NS_FACESENT_FACE))
-		esrv_send_face(&pl->contr->socket, tmp->face->number,0);
-
-	    if (QUERY_FLAG(tmp,FLAG_ANIMATE) && 
-			   !pl->contr->socket.anims_sent[tmp->animation_id])
-		esrv_send_animation(&pl->contr->socket, tmp->animation_id);
-
-	    SockList_AddInt(&sl, tmp->count);
-	    SockList_AddInt(&sl, flags);
-	    SockList_AddInt(&sl, QUERY_FLAG(tmp, FLAG_NO_PICK) ? -1 : WEIGHT(tmp));
-	    SockList_AddInt(&sl, tmp->face->number);
-
-      if (!tmp->custom_name) {
-	      strncpy(item_n,query_base_name(tmp, 0),127);
-	      item_n[127]=0;
-	      len=strlen(item_n);
-	      item_p=query_base_name(tmp, 1);
-      }
-      else {
-        strncpy(item_n,tmp->custom_name,127);
-	      item_n[127]=0;
-	      len=strlen(item_n);
-	      item_p=tmp->custom_name;
-      }
-	    strncpy(item_n+len+1, item_p, 127);
-	    item_n[254]=0;
-	    len += strlen(item_n+1+len) + 1;
-	    SockList_AddChar(&sl, (char ) len);
-	    memcpy(sl.buf+sl.len, item_n, len);
-	    sl.len += len;
-
-	    SockList_AddShort(&sl,tmp->animation_id);
-	    anim_speed=0;
-	    if (QUERY_FLAG(tmp,FLAG_ANIMATE)) {
-		if (tmp->anim_speed) anim_speed=tmp->anim_speed;
-		else {
-		    if (FABS(tmp->speed)<0.001) anim_speed=255;
-		    else if (FABS(tmp->speed)>=1.0) anim_speed=1;
-		    else anim_speed = (int) (1.0/FABS(tmp->speed));
-		}
-		if (anim_speed>255) anim_speed=255;
-	    }
-	    SockList_AddChar(&sl, (char) anim_speed);
-	    SockList_AddInt(&sl, tmp->nrof);
-
-	    if (pl->contr->socket.itemcmd == 2)
-		SockList_AddShort(&sl, tmp->client_type);
-
-	    SET_FLAG(tmp, FLAG_CLIENT_SENT);
+	    add_object_to_socklist(&pl->contr->socket, &sl, head);
 	    got_one++;
 
-	    if (sl.len > (MAXSOCKBUF-MAXITEMLEN)) {
+	    if (sl.len > (MAXSOCKSENDBUF-MAXITEMLEN)) {
 		Send_With_Handling(&pl->contr->socket, &sl);
 		sprintf((char*)sl.buf,"item%d ", pl->contr->socket.itemcmd);
 		sl.len=strlen((char*)sl.buf);
 		SockList_AddInt(&sl, 0);
 		got_one=0;
 	    }
-	}
+	} /* If LOOK_OBJ() */
     }
     if (got_one)
 	Send_With_Handling(&pl->contr->socket, &sl);
@@ -287,11 +312,10 @@ void esrv_draw_look(object *pl)
 void esrv_send_inventory(object *pl, object *op)
 {
     object *tmp;
-    int flags, got_one=0, anim_speed, len;
+    int got_one=0;
     SockList sl;
-    char item_n[MAX_BUF], *item_p;
     
-    sl.buf=malloc(MAXSOCKBUF);
+    sl.buf=malloc(MAXSOCKSENDBUF);
 
     sprintf((char*)sl.buf,"delinv %d", op->count);
     sl.len=strlen((char*)sl.buf);
@@ -303,69 +327,28 @@ void esrv_send_inventory(object *pl, object *op)
     SockList_AddInt(&sl, op->count);
     
     for (tmp=op->inv; tmp; tmp=tmp->below) {
-	if (LOOK_OBJ(tmp)) {
-	    flags = query_flags (tmp);
-	    if (QUERY_FLAG(tmp, FLAG_NO_PICK))
-		flags |=  F_NOPICK;
-	    if (!(pl->contr->socket.faces_sent[tmp->face->number] & NS_FACESENT_FACE))
-		esrv_send_face(&pl->contr->socket, tmp->face->number,0);
-	    if (QUERY_FLAG(tmp,FLAG_ANIMATE) && 
-			   !pl->contr->socket.anims_sent[tmp->animation_id])
-		esrv_send_animation(&pl->contr->socket, tmp->animation_id);
-	    SockList_AddInt(&sl, tmp->count);
-	    SockList_AddInt(&sl, flags);
-	    SockList_AddInt(&sl, QUERY_FLAG(tmp, FLAG_NO_PICK) ? -1 : WEIGHT(tmp));
-	    SockList_AddInt(&sl, tmp->face->number);
-        
-      if (!tmp->custom_name) {
-	      strncpy(item_n,query_base_name(tmp, 0),127);
-	      item_n[127]=0;
-	      len=strlen(item_n);
-	      item_p=query_base_name(tmp, 1);
-      }
-      else {
-        strncpy(item_n,tmp->custom_name,127);
-	      item_n[127]=0;
-	      len=strlen(item_n);
-	      item_p=tmp->custom_name;
-      }
-	    strncpy(item_n+len+1, item_p, 127);
-	    item_n[254]=0;
-	    len += strlen(item_n+1+len) + 1;
-	    SockList_AddChar(&sl, (char) len);
-	    memcpy(sl.buf+sl.len, item_n, len);
-	    sl.len += len;
+	object *head;
 
-	    SockList_AddShort(&sl,tmp->animation_id);
-	    anim_speed=0;
-	    if (QUERY_FLAG(tmp,FLAG_ANIMATE)) {
-		if (tmp->anim_speed) anim_speed=tmp->anim_speed;
-		else {
-		    if (FABS(tmp->speed)<0.001) anim_speed=255;
-		    else if (FABS(tmp->speed)>=1.0) anim_speed=1;
-		    else anim_speed = (int) (1.0/FABS(tmp->speed));
-		}
-		if (anim_speed>255) anim_speed=255;
-	    }
-	    SockList_AddChar(&sl, (char)anim_speed);
-	    SockList_AddInt(&sl, tmp->nrof);
-	    if (pl->contr->socket.itemcmd == 2)
-		SockList_AddShort(&sl, tmp->client_type);
-	    SET_FLAG(tmp, FLAG_CLIENT_SENT);
+	if (tmp->head) head = tmp->head;
+	else head = tmp;
+
+	if (LOOK_OBJ(head)) {
+	    add_object_to_socklist(&pl->contr->socket, &sl, head);
+        
 	    got_one++;
 
 	    /* IT is possible for players to accumulate a huge amount of
 	     * items (especially with some of the bags out there) to
 	     * overflow the buffer.  IF so, send multiple item commands.
 	     */
-	    if (sl.len > (MAXSOCKBUF-MAXITEMLEN)) {
+	    if (sl.len > (MAXSOCKSENDBUF-MAXITEMLEN)) {
 		Send_With_Handling(&pl->contr->socket, &sl);
 		sprintf((char*)sl.buf,"item%d ", pl->contr->socket.itemcmd);
 		sl.len=strlen((char*)sl.buf);
 		SockList_AddInt(&sl, op->count);
 		got_one=0;
 	    }
-	}
+	} /* If LOOK_OBJ() */
     }
     if (got_one)
 	Send_With_Handling(&pl->contr->socket, &sl);
@@ -400,12 +383,15 @@ void esrv_update_item(int flags, object *pl, object *op)
 	 */
 	LOG(llevDebug,"We have not sent item %s (%d)\n", op->name, op->count);
     }
-    sl.buf=malloc(MAXSOCKBUF);
+    sl.buf=malloc(MAXSOCKSENDBUF);
 
     strcpy((char*)sl.buf,"upditem ");
     sl.len=strlen((char*)sl.buf);
 
     SockList_AddChar(&sl, (char) flags);
+
+    if (op->head) op=op->head;
+
     SockList_AddInt(&sl, op->count);
 
     if (flags & UPD_LOCATION)
@@ -416,7 +402,13 @@ void esrv_update_item(int flags, object *pl, object *op)
 
     if (flags & UPD_WEIGHT) {
 	sint32 weight = WEIGHT(op);
-	SockList_AddInt(&sl, weight);
+
+	/* TRANSPORTS are odd - they sort of look like containers, yet can't be
+	 * picked up.  So we don't to send the weight, as it is odd that you see
+	 * weight sometimes and not other (the draw_look won't send it
+	 * for example.
+	 */
+	SockList_AddInt(&sl,  QUERY_FLAG(op, FLAG_NO_PICK) ? -1 : weight);
 	if (pl == op) {
 	    op->contr->last_weight = weight;
 	}
@@ -429,20 +421,21 @@ void esrv_update_item(int flags, object *pl, object *op)
     }
     if (flags & UPD_NAME) {
 	int len;
-	char *item_p, item_n[MAX_BUF];
+	const char *item_p;
+    char item_n[MAX_BUF];
 
-  if (!op->custom_name) {
-	  strncpy(item_n,query_base_name(op, 0),127);
-	  item_n[127]=0;
-	  len=strlen(item_n);
-	  item_p=query_base_name(op, 1);
-  }
-  else {
-    strncpy(item_n,op->custom_name,127);
-	  item_n[127]=0;
-	  len=strlen(item_n);
-	  item_p=op->custom_name;
-  }
+	if (!op->custom_name) {
+	    strncpy(item_n,query_base_name(op, 0),127);
+	    item_n[127]=0;
+	    len=strlen(item_n);
+	    item_p=query_base_name(op, 1);
+	}
+	else {
+	    strncpy(item_n,op->custom_name,127);
+	    item_n[127]=0;
+	    len=strlen(item_n);
+	    item_p=op->custom_name;
+	}
 
 	strncpy(item_n+len+1, item_p, 127);
 	item_n[254]=0;
@@ -479,9 +472,7 @@ void esrv_update_item(int flags, object *pl, object *op)
  */
 void esrv_send_item(object *pl, object*op)
 {
-    int anim_speed, len;
     SockList sl;
-    char item_n[MAX_BUF], *item_p;
     
     /* If this is not the player object, do some more checks */
     if (op!=pl) {
@@ -497,57 +488,16 @@ void esrv_send_item(object *pl, object*op)
 	}
     }
 
-    sl.buf=malloc(MAXSOCKBUF);
+    sl.buf=malloc(MAXSOCKSENDBUF);
 
     sprintf((char*)sl.buf,"item%d ", pl->contr->socket.itemcmd);
     sl.len=strlen((char*)sl.buf);
 
+    if (op->head) op=op->head;
+
     SockList_AddInt(&sl, op->env? op->env->count:0);
 
-    if (!(pl->contr->socket.faces_sent[op->face->number] & NS_FACESENT_FACE))
-	esrv_send_face(&pl->contr->socket, op->face->number,0);
-    if (op->env && QUERY_FLAG(op,FLAG_ANIMATE) &&
-	   !pl->contr->socket.anims_sent[op->animation_id])
-	esrv_send_animation(&pl->contr->socket, op->animation_id);
-
-    SockList_AddInt(&sl, op->count);
-    SockList_AddInt(&sl, query_flags(op));
-    SockList_AddInt(&sl, WEIGHT(op));
-    SockList_AddInt(&sl, op->face->number);
-    
-    if(!op->custom_name) {
-      strncpy(item_n,query_base_name(op, 0),127);
-      item_n[127]=0;
-      len=strlen(item_n);
-      item_p=query_base_name(op, 1);
-    } else {
-      strncpy(item_n,op->custom_name,127);
-	    item_n[127]=0;
-	    len=strlen(item_n);
-	    item_p=op->custom_name;
-    }
-    strncpy(item_n+len+1, item_p, 127);
-    item_n[254]=0;
-    len += strlen(item_n+1+len) + 1;
-    SockList_AddChar(&sl, (char)len);
-    memcpy(sl.buf+sl.len, item_n, len);
-    sl.len += len;
-
-    SockList_AddShort(&sl,op->animation_id);
-    anim_speed=0;
-    if (QUERY_FLAG(op,FLAG_ANIMATE)) {
-	if (op->anim_speed) anim_speed=op->anim_speed;
-        else {
-	    if (FABS(op->speed)<0.001) anim_speed=255;
-            else if (FABS(op->speed)>=1.0) anim_speed=1;
-            else anim_speed = (int) (1.0/FABS(op->speed));
-	}
-        if (anim_speed>255) anim_speed=255;
-    }
-    SockList_AddChar(&sl, (char)anim_speed);
-    SockList_AddInt(&sl, op->nrof);
-    if (pl->contr->socket.itemcmd == 2)
-	SockList_AddShort(&sl, op->client_type);
+    add_object_to_socklist(&pl->contr->socket, &sl, op);
 
     Send_With_Handling(&pl->contr->socket, &sl);
     SET_FLAG(op, FLAG_CLIENT_SENT);
@@ -563,7 +513,7 @@ void esrv_del_item(player *pl, int tag)
 {
     SockList sl;
 
-    sl.buf=malloc(MAXSOCKBUF);
+    sl.buf=malloc(MAXSOCKSENDBUF);
 
     strcpy((char*)sl.buf,"delitem ");
     sl.len=strlen((char*)sl.buf);
@@ -585,7 +535,7 @@ void esrv_del_item(player *pl, int tag)
  * pointer, or null if it can't be found.
  */
 
-object *esrv_get_ob_from_count(object *pl, tag_t count)
+static object *esrv_get_ob_from_count(object *pl, tag_t count)
 {
     object *op, *tmp;
 
@@ -601,24 +551,32 @@ object *esrv_get_ob_from_count(object *pl, tag_t count)
 		    return tmp;
 
     for(op = get_map_ob (pl->map, pl->x, pl->y); op; op = op->above)
-	if (op->count == count)
+	if (op->head != NULL && op->head->count == count)
+	    return op;
+	else if (op->count == count)
 	    return op;
 	else if (op->type == CONTAINER && pl->container == op)
 	    for(tmp = op->inv; tmp; tmp = tmp->below)
 		if (tmp->count == count)
 		    return tmp;
+
+    if (pl->contr->transport) {
+	for(tmp = pl->contr->transport->inv; tmp; tmp = tmp->below)
+	    if (tmp->count == count)
+		return tmp;
+    }
     return NULL;
 }
 
 
 /** Client wants to examine some object.  So lets do so. */
-void ExamineCmd(char *buf, int len,player *pl)
+void examine_cmd(char *buf, int len,player *pl)
 {
     long tag = atoi(buf);
     object *op = esrv_get_ob_from_count(pl->ob, tag);
 
     if (!op) {
-	LOG(llevDebug, "Player '%s' tried examine the unknown object (%ld)\n",
+	LOG(llevDebug, "Player '%s' tried to examine the unknown object (%ld)\n",
 	    pl->ob->name, tag);
 	return;
     }
@@ -626,7 +584,7 @@ void ExamineCmd(char *buf, int len,player *pl)
 }
 
 /** Client wants to apply some object.  Lets do so. */
-void ApplyCmd(char *buf, int len,player *pl)
+void apply_cmd(char *buf, int len,player *pl)
 {
     uint32 tag = atoi(buf);
     object *op = esrv_get_ob_from_count(pl->ob, tag);
@@ -644,7 +602,7 @@ void ApplyCmd(char *buf, int len,player *pl)
     }
 
     if (!op) {
-	LOG(llevDebug, "Player '%s' tried apply the unknown object (%d)\n",
+	LOG(llevDebug, "Player '%s' tried to apply the unknown object (%d)\n",
 	  pl->ob->name, tag);
 	return;
     }
@@ -652,10 +610,11 @@ void ApplyCmd(char *buf, int len,player *pl)
 }
 
 /** Client wants to apply some object.  Lets do so. */
-void LockItem(uint8 *data, int len,player *pl)
+void lock_item_cmd(uint8 *data, int len,player *pl)
 {
     int flag, tag;
     object *op;
+    object *tmp;
 
     flag = data[0];
     tag = GetInt_String(data+1);
@@ -669,11 +628,20 @@ void LockItem(uint8 *data, int len,player *pl)
 	CLEAR_FLAG(op,FLAG_INV_LOCKED);
     else
 	SET_FLAG(op,FLAG_INV_LOCKED);
-    esrv_update_item(UPD_FLAGS, pl->ob, op);
+
+    tmp = merge_ob(op, NULL);
+    if (tmp == NULL) {
+	/* object was not merged */
+	esrv_update_item(UPD_FLAGS, pl->ob, op);
+    } else {
+	/* object was merged into tmp */
+	esrv_del_item(pl, tag);
+	esrv_send_item(pl->ob, tmp);
+    }
 }
 
 /** Client wants to apply some object.  Lets do so. */
-void MarkItem(uint8 *data, int len,player *pl)
+void mark_item_cmd(uint8 *data, int len,player *pl)
 {
     int tag;
     object *op;
@@ -691,7 +659,7 @@ void MarkItem(uint8 *data, int len,player *pl)
 
 
 /**
- * look_at prints items on the specified square.
+ * Prints items on the specified square.
  *
  * [ removed EARTHWALL check and added check for containers inventory.
  *   Tero.Haatanen@lut.fi ]
@@ -751,7 +719,7 @@ void look_at(object *op,int dx,int dy) {
 
 
 /** Client wants to look at some object.  Lets do so. */
-void LookAt(char *buf, int len,player *pl)
+void look_at_cmd(char *buf, int len,player *pl)
 {
     int dx, dy;
     char *cp;
@@ -777,13 +745,16 @@ void esrv_move_object (object *pl, tag_t to, tag_t tag, long nrof)
 
     op = esrv_get_ob_from_count(pl, tag);
     if (!op) {
-	LOG(llevDebug, "Player '%s' tried to move the unknown object (%ld)\n",
+	LOG(llevDebug, "Player '%s' tried to move an unknown object (%ld)\n",
 	    pl->name, tag);
 	return;
     }
 
-    if (!to) {	/* drop it to the ground */
-/*	printf ("Drop it on the ground.\n");*/
+    /* If on a transport, you don't drop to the ground - you drop to the
+     * transport.
+     */
+    if (!to && !pl->contr->transport) {	/* drop it to the ground */
+/*	LOG(llevDebug, "Drop it on the ground.\n");*/
 
 	if (op->map && !op->env) {
 /*	    LOG(llevDebug,"Dropping object to ground that is already on ground\n");*/
@@ -813,20 +784,27 @@ void esrv_move_object (object *pl, tag_t to, tag_t tag, long nrof)
 	return ;
     }
     /* If not dropped or picked up, we are putting it into a sack */
-    env = esrv_get_ob_from_count(pl, to);
-    if (!env) {
-      LOG(llevDebug, 
-	  "Player '%s' tried to move object to the unknown location (%d)\n",
-	  pl->name, to);
-      return;
-    }
-    /* put_object_in_sack presumes that necessary sanity checking
-     * has already been done (eg, it can be picked up and fits in
-     * in a sack, so check for those things.  We should also check
-     * an make sure env is in fact a container for that matter.
-     */
-    if (env->type == CONTAINER && can_pick(pl, op) && sack_can_hold(pl, env, op, nrof)) {
-	put_object_in_sack (pl, env, op, nrof);
+    if (pl->contr->transport) {
+	if (can_pick(pl, op) && transport_can_hold(pl->contr->transport, op, nrof)) {
+	    put_object_in_sack (pl, pl->contr->transport, op, nrof);
+	}
+    } else {
+	env = esrv_get_ob_from_count(pl, to);
+	if (!env) {
+	    LOG(llevDebug, 
+		"Player '%s' tried to move object to the unknown location (%d)\n",
+		pl->name, to);
+	    return;
+	}
+	/* put_object_in_sack presumes that necessary sanity checking
+	 * has already been done (eg, it can be picked up and fits in
+	 * in a sack, so check for those things.  We should also check
+	 * an make sure env is in fact a container for that matter.
+	 */
+	if (env->type == CONTAINER
+	    && can_pick(pl, op) && sack_can_hold(pl, env, op, nrof)) {
+	    put_object_in_sack (pl, env, op, nrof);
+	}
     }
 }
 
