@@ -1,13 +1,13 @@
 
 /*
  * static char *rcsid_loop_c =
- *    "$Id: loop.c,v 1.29 2005/06/23 05:18:38 mwedel Exp $";
+ *    "$Id: loop.c,v 1.46 2006/06/05 06:28:57 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2002-2003 Mark Wedel & The Crossfire Development Team
+    Copyright (C) 2006 Mark Wedel & The Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -61,6 +61,7 @@
 #include <arpa/inet.h>
 #endif
 
+#include <loader.h>
 #include <newserver.h>
 
 /*****************************************************************************
@@ -76,24 +77,24 @@
  * we end up having 2 tables.
  */
 
-typedef void (*func_uint8_int_ns) (char*, int, NewSocket *);
+typedef void (*func_uint8_int_ns) (char*, int, socket_struct *);
 
-struct NsCmdMapping {
-    char *cmdname;
-    func_uint8_int_ns  cmdproc;
+struct client_cmd_mapping {
+    const char *cmdname;
+    func_uint8_int_ns cmdproc;
 };
 
 typedef void (*func_uint8_int_pl)(char*, int, player *);
-struct PlCmdMapping {
-    char *cmdname;
+struct player_cmd_mapping {
+    const char *cmdname;
     func_uint8_int_pl cmdproc;
     uint8   flag;
 };
 
 /**
- * Dispatch table for the server.
+ * Dispatch tables for the server.
  *
- * CmdMapping is the dispatch table for the server, used in HandleClient,
+ * CmdMapping is the dispatch table for the server, used in handle_client,
  * which gets called when the client has input.  All commands called here 
  * use the same parameter form (char* data, int len, int clientnum.
  * We do implicit casts, because the data that is being passed is
@@ -103,41 +104,42 @@ struct PlCmdMapping {
  * flag is 1 if the player must be in the playing state to issue the
  * command, 0 if they can issue it at any time.
  */
-static struct PlCmdMapping plcommands[] = {
-    { "examine",	ExamineCmd,	1},
-    { "apply",		ApplyCmd,	1},
-    { "move",		MoveCmd,	1},
-    { "reply",		ReplyCmd,	0},
-    { "command",	PlayerCmd,	1},
-    { "ncom",		(func_uint8_int_pl)NewPlayerCmd, 1},
-    { "lookat",		LookAt,		1},
-    { "lock",		(func_uint8_int_pl)LockItem,	1},
-    { "mark",		(func_uint8_int_pl)MarkItem,	1},
-    { "mapredraw",	MapRedrawCmd,	0},	/* Added: phil */
-    { NULL, NULL}	/* terminator */
+/** Commands sent by the player during the game play itself */
+static struct player_cmd_mapping player_commands[] = {
+    { "examine",	examine_cmd,	1},
+    { "apply",		apply_cmd,	1},
+    { "move",		move_cmd,	1},
+    { "reply",		reply_cmd,	0},
+    { "command",	player_cmd,	1},
+    { "ncom",		(func_uint8_int_pl)new_player_cmd, 1},
+    { "lookat",		look_at_cmd,		1},
+    { "lock",		(func_uint8_int_pl)lock_item_cmd,	1},
+    { "mark",		(func_uint8_int_pl)mark_item_cmd,	1},
+    { "mapredraw",	map_redraw_cmd,	0},	/* Added: phil */
+    { NULL, NULL, 0}	/* terminator */
 };
 
-/** Face-related commands */
-static struct NsCmdMapping nscommands[] = {
-    { "addme",		AddMeCmd },
-    { "askface",	SendFaceCmd},	/* Added: phil */
-    { "requestinfo",	RequestInfo},
-    { "setfacemode",	SetFaceMode},
-    { "setsound",	SetSound},
-    { "setup",		SetUp},
-    { "version",	VersionCmd },
-    { "toggleextendedinfos", ToggleExtendedInfos}, /*Added: tchize*/
-    { "toggleextendedtext", ToggleExtendedText},   /*Added: tchize*/
-    { "asksmooth", AskSmooth},   /*Added: tchize (smoothing technologies)*/ 
+/** Commands sent directly by client, when connecting */
+static struct client_cmd_mapping client_commands[] = {
+    { "addme",		add_me_cmd },
+    { "askface",	send_face_cmd},	/* Added: phil */
+    { "requestinfo",	request_info_cmd},
+    { "setfacemode",	set_face_mode_cmd},
+    { "setsound",	set_sound_cmd},
+    { "setup",		set_up_cmd},
+    { "version",	version_cmd },
+    { "toggleextendedinfos", toggle_extended_infos_cmd}, /*Added: tchize*/
+    { "toggleextendedtext", toggle_extended_text_cmd},   /*Added: tchize*/
+    { "asksmooth", ask_smooth_cmd},   /*Added: tchize (smoothing technologies)*/ 
     { NULL, NULL}	/* terminator (I, II & III)*/
 };
 
 /**
- * RequestInfo is sort of a meta command. There is some specific
+ * request_info_cmd is sort of a meta command. There is some specific
  * request of information, but we call other functions to provide
  * that information.
  */
-void RequestInfo(char *buf, int len, NewSocket *ns)
+void request_info_cmd(char *buf, int len, socket_struct *ns)
 {
     char    *params=NULL, *cp;
     /* No match */
@@ -163,16 +165,17 @@ void RequestInfo(char *buf, int len, NewSocket *ns)
     if (!strcmp(buf, "image_info")) send_image_info(ns, params);
     else if (!strcmp(buf,"image_sums")) send_image_sums(ns, params);
     else if (!strcmp(buf,"skill_info")) send_skill_info(ns, params);
+    else if (!strcmp(buf,"spell_paths")) send_spell_paths(ns, params);
     else Write_String_To_Socket(ns, bigbuf, len);
 }
 
 /**
  * Handles old socket format.
  */
-void Handle_Oldsocket(NewSocket *ns)
+void handle_oldsocket(socket_struct *ns)
 {
     int stat,i;
-    CommFunc	command;
+    command_function	command;
     char buf[MAX_BUF],*cp;
     object ob;
     player pl;
@@ -182,6 +185,11 @@ void Handle_Oldsocket(NewSocket *ns)
      * or no more characters to read.
      */
     do {
+	if (ns->inbuf.len >= MAXSOCKRECVBUF-1) {
+	    ns->status = Ns_Dead;
+	    LOG(llevDebug, "Old input socket sent too much data without newline\n");
+	    return;
+	}
 #ifdef WIN32 /* ***win32: change oldsocket read() to recv() */
 		stat = recv(ns->fd, ns->inbuf.buf + ns->inbuf.len, 1,0);
 
@@ -193,21 +201,16 @@ void Handle_Oldsocket(NewSocket *ns)
 
 	if (stat<0 && errno != EAGAIN && errno !=EWOULDBLOCK) {
 #endif	
-	    perror("Handle_Oldsocket got an error.");
+	    LOG(llevError, "Cannot read from socket: %s\n", strerror_local(errno));
 	    ns->status = Ns_Dead;
 	    return;
 	}
 	if (stat == 0) return;
-	if (stat == MAXSOCKBUF-1) {
-	    ns->status = Ns_Dead;
-	    LOG(llevDebug, "Old input socket sent too much data without newline\n");
-	    return;
-	}
     } while (ns->inbuf.buf[ns->inbuf.len++]!='\n');
 
     ns->inbuf.buf[ns->inbuf.len]=0;
 
-    cp = strchr(ns->inbuf.buf, ' ');
+    cp = strchr((char*)ns->inbuf.buf, ' ');
     if (cp) {
 	/* Replace the space with a null, skip any more spaces */
 	*cp++=0;
@@ -223,25 +226,25 @@ void Handle_Oldsocket(NewSocket *ns)
 
     /* If just a return, don't do anything */
     if (ns->inbuf.buf[0] == 0) return;
-    if (!strcasecmp(ns->inbuf.buf,"quit")) {
+    if (!strcasecmp((char*)ns->inbuf.buf,"quit")) {
 	ns->status = Ns_Dead;
 	return;
     }
-    if (!strcasecmp(ns->inbuf.buf, "listen")) {
-	if (ns->comment) free(ns->comment);
+    if (!strcasecmp((char*)ns->inbuf.buf, "listen")) {
 	if (cp) {
-	    char *buf="Socket switched to listen mode\n";
+	    const char *buf="Socket switched to listen mode\n";
 
+	    free(ns->comment);
 	    ns->comment = strdup_local(cp);
 	    ns->old_mode = Old_Listen;
 	    cs_write_string(ns, buf, strlen(buf));
 	} else {
-	    char *buf="Need to supply a comment/url to listen\n";
+	    const char *buf="Need to supply a comment/url to listen\n";
 	    cs_write_string(ns, buf, strlen(buf));
 	}
 	return;
     }
-    if (!strcasecmp(ns->inbuf.buf, "name")) {
+    if (!strcasecmp((char*)ns->inbuf.buf, "name")) {
 	char *cp1=NULL;
 	if (cp) cp1= strchr(cp, ' ');
 	if (cp1) {
@@ -249,31 +252,45 @@ void Handle_Oldsocket(NewSocket *ns)
 	    while (isspace(*cp1)) cp1++;
 	}
 	if (!cp || !cp1) {
-	    char *buf="Need to provide a name/password to name\n";
+	    const char *buf="Need to provide a name/password to name\n";
 	    cs_write_string(ns, buf, strlen(buf));
 	    return;
 	}
 
-	if (!verify_player(cp, cp1)) {
-	    char *buf="Welcome back\n";
-	    if (ns->comment) free(ns->comment);
+	if (verify_player(cp, cp1)==0) {
+	    const char *buf="Welcome back\n";
+	    free(ns->comment);
 	    ns->comment = strdup_local(cp);
 	    ns->old_mode = Old_Player;
 	    cs_write_string(ns, buf, strlen(buf));
 	}
+	else if (verify_player(cp, cp1)==2) {
+	    ns->password_fails++;
+	    if (ns->password_fails >= MAX_PASSWORD_FAILURES) {
+		const char *buf="You failed to log in too many times, you will now be kicked.\n";
+		LOG(llevInfo, "A player connecting from %s in oldsocketmode has been dropped for password failure\n",
+		    ns->host); 
+		cs_write_string(ns, buf, strlen(buf));
+		ns->status = Ns_Dead;
+	    }
+	    else {
+		const char *buf="Could not login you in.  Check your name and password.\n";
+		cs_write_string(ns, buf, strlen(buf));
+	    }
+	}	
 	else {
-	    char *buf="Could not login you in.  Check your name and password.\n";
+	    const char *buf="Could not login you in.  Check your name and password.\n";
 	    cs_write_string(ns, buf, strlen(buf));
 	}
 	return;
     }
 
-    command = find_oldsocket_command(ns->inbuf.buf);
+    command = find_oldsocket_command((char*)ns->inbuf.buf);
     if (!command && ns->old_mode==Old_Player) {
-	command = find_oldsocket_command2(ns->inbuf.buf);
+	command = find_oldsocket_command2((char*)ns->inbuf.buf);
     }
     if (!command) {
-	sprintf(buf,"Could not find command: %s\n", ns->inbuf.buf);
+	snprintf(buf, sizeof(buf), "Could not find command: %s\n", ns->inbuf.buf);
 	cs_write_string(ns, buf, strlen(buf));
 	return;
     }
@@ -299,14 +316,14 @@ void Handle_Oldsocket(NewSocket *ns)
 /**
  * Handle client input.
  *
- * HandleClient is actually not named really well - we only get here once
+ * handle_client is actually not named really well - we only get here once
  * there is input, so we don't do exception or other stuff here.
  * sock is the output socket information.  pl is the player associated
  * with this socket, null if no player (one of the init_sockets for just
  * starting a connection)
  */
 
-void HandleClient(NewSocket *ns, player *pl)
+void handle_client(socket_struct *ns, player *pl)
 {
     int len=0,i;
     unsigned char *data;
@@ -321,15 +338,15 @@ void HandleClient(NewSocket *ns, player *pl)
 	}
 	    
 	if (ns->status == Ns_Old) {
-	    Handle_Oldsocket(ns);
+	    handle_oldsocket(ns);
 	    return;
 	}
-	i=SockList_ReadPacket(ns->fd, &ns->inbuf, MAXSOCKBUF-1);
+	i=SockList_ReadPacket(ns->fd, &ns->inbuf, MAXSOCKRECVBUF-1);
 	/* Special hack - let the user switch to old mode if in the Ns_Add
 	 * phase.  Don't demand they add in the special length bytes
 	 */
 	if (ns->status == Ns_Add) {
-	    if (!strncasecmp(ns->inbuf.buf,"oldsocketmode", 13)) {
+	    if (!strncasecmp((char*)ns->inbuf.buf,"oldsocketmode", 13)) {
 		ns->status = Ns_Old;
 		ns->inbuf.len=0;
 		cs_write_string(ns, "Switched to old socket mode\n", 28);
@@ -340,7 +357,7 @@ void HandleClient(NewSocket *ns, player *pl)
 
 	if (i<0) {
 #ifdef ESRV_DEBUG
-	    LOG(llevDebug,"HandleClient: Read error on connection player %s\n", (pl?pl->ob->name:"None"));
+	    LOG(llevDebug,"handle_client: Read error on connection player %s\n", (pl?pl->ob->name:"None"));
 #endif
 	    /* Caller will take care of cleaning this up */
 	    ns->status =Ns_Dead;
@@ -349,6 +366,8 @@ void HandleClient(NewSocket *ns, player *pl)
 	/* Still dont have a full packet */
 	if (i==0) return;
 
+	ns->inbuf.buf[ns->inbuf.len]='\0';  /* Terminate buffer - useful for string data */
+
 	/* First, break out beginning word.  There are at least
 	 * a few commands that do not have any paremeters.  If
 	 * we get such a command, don't worry about trying
@@ -362,10 +381,9 @@ void HandleClient(NewSocket *ns, player *pl)
 	}
 	else len=0;
 
-	ns->inbuf.buf[ns->inbuf.len]='\0';  /* Terminate buffer - useful for string data */
-	for (i=0; nscommands[i].cmdname !=NULL; i++) {
-	    if (strcmp((char*)ns->inbuf.buf+2,nscommands[i].cmdname)==0) {
-		nscommands[i].cmdproc((char*)data,len,ns);
+	for (i=0; client_commands[i].cmdname !=NULL; i++) {
+	    if (strcmp((char*)ns->inbuf.buf+2,client_commands[i].cmdname)==0) {
+		client_commands[i].cmdproc((char*)data,len,ns);
 		ns->inbuf.len=0;
 		return;
 	    }
@@ -377,10 +395,10 @@ void HandleClient(NewSocket *ns, player *pl)
 	 * at the map causes a crash.  If the command is valid, but
 	 * one they can't use, we still swallow it up.
 	 */
-	if (pl) for (i=0; plcommands[i].cmdname !=NULL; i++) {
-	    if (strcmp((char*)ns->inbuf.buf+2,plcommands[i].cmdname)==0) {
-		if (pl->state == ST_PLAYING || plcommands[i].flag == 0)
-		    plcommands[i].cmdproc((char*)data,len,pl);
+	if (pl) for (i=0; player_commands[i].cmdname !=NULL; i++) {
+	    if (strcmp((char*)ns->inbuf.buf+2,player_commands[i].cmdname)==0) {
+		if (pl->state == ST_PLAYING || player_commands[i].flag == 0)
+		    player_commands[i].cmdproc((char*)data,len,pl);
 		ns->inbuf.len=0;
 		return;
 	    }
@@ -434,7 +452,7 @@ void watchdog(void)
 extern unsigned long todtick;
 
 /** Waits for new connection */
-static void block_until_new_connection()
+static void block_until_new_connection(void)
 {
 
     struct timeval Timeout;
@@ -496,12 +514,12 @@ static void block_until_new_connection()
  * There are 2 lists we need to look through - init_sockets is a list
  * 
  */
-void doeric_server()
+void doeric_server(void)
 {
     int i, pollret;
     fd_set tmp_read, tmp_exceptions, tmp_write;
     struct sockaddr_in addr;
-    int addrlen=sizeof(struct sockaddr);
+    socklen_t addrlen=sizeof(struct sockaddr);
     player *pl, *next;
 
 #ifdef CS_LOGSTATS
@@ -562,8 +580,7 @@ void doeric_server()
 		    &tmp_exceptions, &socket_info.timeout);
 
     if (pollret==-1) {
-	perror("doeric_server: error on select");
-	LOG(llevError,"doeric_server: error on select\n");
+	LOG(llevError, "select failed: %s\n", strerror_local(errno));
 	return;
     }
 
@@ -579,10 +596,13 @@ void doeric_server()
 #endif
 	/* If this is the case, all sockets currently in used */
 	if (socket_info.allocated_sockets <= socket_info.nconns) {
-	    init_sockets = realloc(init_sockets,sizeof(NewSocket)*(socket_info.nconns+1));
+	    init_sockets = realloc(init_sockets,sizeof(socket_struct)*(socket_info.nconns+1));
 	    if (!init_sockets) fatal(OUT_OF_MEMORY);
 	    newsocknum = socket_info.allocated_sockets;
 	    socket_info.allocated_sockets++;
+	    init_sockets[newsocknum].faces_sent_len = nrofpixmaps;
+	    init_sockets[newsocknum].faces_sent = malloc(nrofpixmaps*sizeof(*init_sockets[newsocknum].faces_sent));
+	    if (!init_sockets[newsocknum].faces_sent) fatal(OUT_OF_MEMORY);
 	    init_sockets[newsocknum].status = Ns_Avail;
 	}
 	else {
@@ -596,12 +616,27 @@ void doeric_server()
 	}
 	init_sockets[newsocknum].fd=accept(init_sockets[0].fd, (struct sockaddr *)&addr, &addrlen);
 	if (init_sockets[newsocknum].fd==-1) {
-	    perror("doeric_server: error on accept");
-	    LOG(llevError,"doeric_server: error on accept\n");
+	    LOG(llevError, "accept failed: %s\n", strerror_local(errno));
 	}
 	else {
-	    InitConnection(&init_sockets[newsocknum],ntohl(addr.sin_addr.s_addr));
-	    socket_info.nconns++;
+	    char buf[MAX_BUF];
+	    long ip;
+	    socket_struct *ns;
+
+	    ns = &init_sockets[newsocknum];
+
+	    ip = ntohl(addr.sin_addr.s_addr);
+	    sprintf(buf, "%ld.%ld.%ld.%ld", (ip>>24)&255, (ip>>16)&255, (ip>>8)&255, ip&255);
+
+	    if (checkbanned(NULL, buf)) {
+		LOG(llevInfo, "Banned host tried to connect: [%s]\n", buf);
+		close(init_sockets[newsocknum].fd);
+		init_sockets[newsocknum].fd = -1;
+	    }
+	    else {
+		init_connection(ns, buf);
+		socket_info.nconns++;
+	    }
 	}
     }
 
@@ -615,7 +650,7 @@ void doeric_server()
 	    continue;
 	}
 	if (FD_ISSET(init_sockets[i].fd, &tmp_read)) {
-	    HandleClient(&init_sockets[i], NULL);
+        handle_client(&init_sockets[i], NULL);
 	}
 	if (FD_ISSET(init_sockets[i].fd, &tmp_write)) {
 	    init_sockets[i].can_write=1;
@@ -653,7 +688,7 @@ void doeric_server()
 	    final_free_player(pl);
 	}
 	else {
-	    HandleClient(&pl->socket, pl);
+	    handle_client(&pl->socket, pl);
 	    /* If the player has left the game, then the socket status
 	     * will be set to this be the leave function.  We don't
 	     * need to call leave again, as it has already been called
@@ -673,14 +708,17 @@ void doeric_server()
 		 * sending them whenever they change, and probably just as useful
 		 */
 		esrv_update_stats(pl);
-		if (pl->last_weight != (uint32)-1 && pl->last_weight != WEIGHT(pl->ob)) {
+		if (pl->last_weight != -1 && pl->last_weight != WEIGHT(pl->ob)) {
 		    esrv_update_item(UPD_WEIGHT, pl->ob, pl->ob);
 		    if(pl->last_weight != WEIGHT(pl->ob))
 			LOG(llevError, "esrv_update_item(UPD_WEIGHT) did not set player weight: is %lu, should be %lu\n", (unsigned long)pl->last_weight, WEIGHT(pl->ob));
 		}
-		if (pl->ob->map && pl->ob->map->in_memory==MAP_IN_MEMORY)
-		    draw_client_map(pl->ob);
+		/* draw_client_map does sanity checking that map is
+		 * valid, so don't do it here.
+		 */
+		draw_client_map(pl->ob);
 		if (pl->socket.update_look) esrv_draw_look(pl->ob);
+		if (pl->socket.tick) send_tick(pl);
 	    }
 	}
     }
