@@ -1,7 +1,7 @@
 
 /*
  * static char *rcsid_sockets_c =
- *    "$Id: lowlevel.c,v 1.10 2005/05/03 19:31:28 ryo_saeba Exp $";
+ *    "$Id: lowlevel.c,v 1.16 2006/05/21 10:35:11 akirschbaum Exp $";
  */
 
 /*
@@ -88,15 +88,16 @@ void SockList_AddInt64(SockList *sl, uint64 data)
     sl->buf[sl->len++] =( char )(  data & 0xff );
 }
 
-/* Basically does the reverse of SockList_AddInt, but on
+/**
+ * Basically does the reverse of SockList_AddInt, but on
  * strings instead.  Same for the GetShort, but for 16 bits.
  */
-int GetInt_String(unsigned char *data)
+int GetInt_String(const unsigned char *data)
 {
     return ((data[0]<<24) + (data[1]<<16) + (data[2]<<8) + data[3]);
 }
 
-short GetShort_String(unsigned char *data) {
+short GetShort_String(const unsigned char *data) {
     return ((data[0]<<8)+data[1]);
 }
 
@@ -108,10 +109,10 @@ short GetShort_String(unsigned char *data) {
 
 /**
  * This reads from fd and puts the data in sl.  We return true if we think
- * we have a full packet, 0 if we have a partial packet.  The only processing
- * we do is remove the intial size value.  len (As passed) is the size of the
- * buffer allocated in the socklist.  We make the assumption the buffer is
- * at least 2 bytes long.
+ * we have a full packet, 0 if we have a partial packet, or -1 if an error
+ * occurred.  The only processing we do is remove the initial size value.  len
+ * (As passed) is the size of the buffer allocated in the socklist.  We make
+ * the assumption the buffer is at least 2 bytes long.
  */
  
 int SockList_ReadPacket(int fd, SockList *sl, int len)
@@ -144,15 +145,13 @@ int SockList_ReadPacket(int fd, SockList *sl, int len)
 			LOG(llevDebug,"Connection closed by client\n");
 		else
 		{
-			perror("ReadPacket got an error."); /* ***win32 <- for what is this instead of log? */
 			LOG(llevDebug,"ReadPacket got error %d, returning 0\n",WSAGetLastError());
 		}
 		return -1;	/* kick this user! */
 	    }
 #else
 	if (errno != EAGAIN && errno !=EWOULDBLOCK) {
-		perror("ReadPacket got an error.");
-		LOG(llevDebug,"ReadPacket got error %d, returning 0\n",errno);
+		LOG(llevDebug, "ReadPacket got error %s, returning 0\n", strerror_local(errno));
 	    }
 #endif
 	    return 0;	/*Error */
@@ -201,15 +200,13 @@ int SockList_ReadPacket(int fd, SockList *sl, int len)
 			LOG(llevDebug,"Connection closed by client\n");
 		else
 		{
-			perror("ReadPacket got an error.");
 			LOG(llevDebug,"ReadPacket got error %d, returning 0\n",WSAGetLastError());
 		}
 		return -1;	/* kick this user! */
 	    }
 #else
 	if (errno != EAGAIN && errno !=EWOULDBLOCK) {
-		perror("ReadPacket got an error.");
-		LOG(llevDebug,"ReadPacket got error %d, returning 0\n",errno);
+		LOG(llevDebug, "ReadPacket got error %s, returning 0\n", strerror_local(errno));
 	    }
 #endif
 	    return 0;	/*Error */
@@ -243,7 +240,7 @@ int SockList_ReadPacket(int fd, SockList *sl, int len)
  * data, and len is the number of bytes to add.
  */
 
-static void add_to_buffer(NewSocket *ns, unsigned char *buf, int len)
+static void add_to_buffer(socket_struct *ns, const unsigned char *buf, int len)
 {
     int avail, end;
 
@@ -284,7 +281,7 @@ static void add_to_buffer(NewSocket *ns, unsigned char *buf, int len)
  * When the socket is clear to write, and we have backlogged data, this
  * is called to write it out.
  */
-void write_socket_buffer(NewSocket *ns)
+void write_socket_buffer(socket_struct *ns)
 {
     int amt, max;
 
@@ -343,10 +340,10 @@ void write_socket_buffer(NewSocket *ns)
  * of bytes to write.  IT doesn't return anything - rather, it
  * updates the ns structure if we get an  error.
  */
-void Write_To_Socket(NewSocket *ns, unsigned char *buf, int len)
+static void Write_To_Socket(socket_struct *ns, const unsigned char *buf, int len)
 {
     int amt=0;
-    unsigned char *pos=buf;
+    const unsigned char *pos=buf;
 
     if (ns->status == Ns_Dead || !buf) {
 	LOG(llevDebug,"Write_To_Socket called with dead socket\n");
@@ -409,12 +406,12 @@ void Write_To_Socket(NewSocket *ns, unsigned char *buf, int len)
  * Takes a string of data, and writes it out to the socket. A very handy
  * shortcut function.
  */
-void cs_write_string(NewSocket *ns, const char *buf, int len)
+void cs_write_string(socket_struct *ns, const char *buf, int len)
 {
     SockList sl;
 
     sl.len = len;
-    sl.buf = (unsigned char*)buf;
+    sl.buf = (unsigned char *)buf;
     Send_With_Handling(ns, &sl);
 }
 
@@ -425,16 +422,16 @@ void cs_write_string(NewSocket *ns, const char *buf, int len)
  * The only difference in this function is that we take a SockList
  *, and we prepend the length information.
  */
-void Send_With_Handling(NewSocket *ns,SockList  *msg)
+void Send_With_Handling(socket_struct *ns, const SockList *msg)
 {
-    unsigned char sbuf[4];
+    unsigned char sbuf[2];
 
     if (ns->status == Ns_Dead || !msg)
 	return;
 
-    if (msg->len >= MAXSOCKBUF) {
-	LOG(llevError,"Trying to send a buffer beyond properly size, len =%d\n",
-	    msg->len);
+    if (msg->len > MAXSOCKSENDBUF) {
+	LOG(llevError,"Trying to send a buffer beyond properly size, len %d > max len %d\n",
+	    msg->len, MAXSOCKSENDBUF);
 	/* Almost certainly we've overflowed a buffer, so quite now to make
 	 * it easier to debug.
 	 */
@@ -451,12 +448,12 @@ void Send_With_Handling(NewSocket *ns,SockList  *msg)
  * Takes a string of data, and writes it out to the socket. A very handy
  * shortcut function.
  */
-void Write_String_To_Socket(NewSocket *ns, char *buf, int len)
+void Write_String_To_Socket(socket_struct *ns, const char *buf, int len)
 {
     SockList sl;
 
     sl.len = len;
-    sl.buf = (uint8*)buf;
+    sl.buf = (unsigned char *)buf;
     Send_With_Handling(ns, &sl);
 }
 
@@ -476,7 +473,7 @@ CS_Stats cst_tot, cst_lst;
 /**
  * Writes out the gathered stats.  We clear cst_lst.
  */
-void write_cs_stats()
+void write_cs_stats(void)
 {
     time_t now=time(NULL);
 
