@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_metaserver_c =
- *    "$Id: metaserver.c,v 1.16 2003/12/02 18:51:44 ryo_saeba Exp $";
+ *    "$Id: metaserver.c,v 1.18 2006/05/20 12:36:48 ryo_saeba Exp $";
  */
 
 /*
@@ -54,7 +54,7 @@ static struct sockaddr_in sock;
  * set to -1.  We use this instead of messing with the settings.meta_on so that
  * that can be examined to at least see what the user was trying to do.
  */
-void metaserver_init()
+void metaserver_init(void)
 {
 
 #ifdef WIN32 /* ***win32 metaserver_init(): init win32 socket */
@@ -117,7 +117,7 @@ void metaserver_init()
 /**
  * Updates our info in the metaserver
  */
-void metaserver_update()
+void metaserver_update(void)
 {
     char data[MAX_BUF], num_players=0;
     player *pl;
@@ -130,7 +130,7 @@ void metaserver_update()
      * costly.
      */
     for (pl=first_player; pl!=NULL; pl=pl->next) {
-	if (!pl->hidden) num_players++;
+        if (!pl->hidden && (pl->socket.is_bot == 0)) num_players++;
     }
 
     sprintf(data,"%s|%d|%s|%s|%d|%d|%ld", settings.meta_host, num_players, VERSION, 
