@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_sound_c =
- *   "$Id: sounds.c,v 1.5 2003/12/14 16:28:47 ryo_saeba Exp $";
+ *   "$Id: sounds.c,v 1.7 2006/05/21 10:35:11 akirschbaum Exp $";
  */
 
 /* Send bug reports to Raphael Quinet (quinet@montefiore.ulg.ac.be) */
@@ -39,7 +39,7 @@ void play_sound_player_only(player *pl, short soundnum,  sint8 x, sint8 y)
     }
     else soundtype=SOUND_NORMAL;
 
-    sl.buf=malloc(MAXSOCKBUF);
+    sl.buf=malloc(MAXSOCKSENDBUF);
     strcpy((char*)sl.buf, "sound ");
     sl.len=strlen((char*)sl.buf);
     SockList_AddChar(&sl, x);
@@ -53,7 +53,7 @@ void play_sound_player_only(player *pl, short soundnum,  sint8 x, sint8 y)
 #define POW2(x) ((x) * (x))
 
 /** Plays some sound on map at x,y.  */
-void play_sound_map(mapstruct *map, int x, int y, short sound_num)
+void play_sound_map(const mapstruct *map, int x, int y, short sound_num)
 {
     player *pl;
 
